/*
 * 	lightbox.js - lightbox for Photoblogger skin
 *	jslint browser: true
 */
 
;(function($, $window, $document, $body, undefined) {	
	'use strict';
	// Requires laza.util, laza.swipe and laza.transform
	
	$.fn.lightbox = function(album, settings) {
		
		if (typeof album === UNDEF) {
			return this;
		}
		
		settings = $.extend({}, $.fn.lightbox.defaults, settings);

		var self = $(this),
			text = getTranslations($.fn.lightbox.text),
			thumbs = $([]),
			thumb = $(),
			main = $(),
			image = $(),
			media = $(),
			caption = $(),
			el,
			isImage = true,
			isVideo = false,
			isAudio = false,
			isOther = false,
			isExternal = false,
			isVr = false,
			isPdf = false,
			isPanorama = false,
			swiped = false,
			dontRemove = false,
			slideshow = null,
			fitTimeout = null,
			controlsTimeout = null,
			resizeTimeout = null,
			autoPanoTimeout = null,
			
			curr = -1,
			loadCounter = 0,
			loadTimer,
			direction = 0,
			mainPadding,
			fitPadding = settings['fitPadding'], 
			
			// Elements
			lightbox,
			lb_overlay,
			lb_activity,
			
			// Controls
			lb_btn_left,
			lb_close,
			lb_zoom,
			lb_play,
			lb_pause,
			lb_btn_right,
			
			// Caption
			lb_caption,
			
			ns = self.data('llb_ns'),
			extraSizes = settings.hasOwnProperty('extraSizes')? settings.extraSizes.split(/,\s*/) : false,
			
			backgroundAudioRoot = $('[data-audioplayer]'),
			backgroundAudioOn = backgroundAudioRoot.length && !backgroundAudioRoot.data('paused'),

			cleanup = function(ns) {	
					// cleaning up the old event handlers
					$window.add(document).off('.' + ns);
					// Removing the lightbox :: you can only have one lightbox per thumb set
					$('#' + ns).remove();
				},
			
			closeModal = function(id) {
					lightbox.find('.modal').trigger('destroy');
				},
			
			// Reacalculating image position and size upon window resize
			
			fitImage = function(initial) {
				
					clearTimeout(fitTimeout);
					clearTimeout(resizeTimeout);
					
					if (!main || !main.length) {
						return;
					}
					
					var w, 
						h,
						lbw = lightbox.width(),
						lbh = lightbox.height(),
						lbah,							// Available height
						ratio,
						fillRatio,
						ratioX,
						ratioY,
						isPano = false,
						canZoomIn,
						canZoomOut,
						img = image[0],
						defaultPoster = image.hasClass('default');
					
					if (typeof mainPadding === UNDEF) {
						// Picture padding: border
						mainPadding = parseInt(main.css('paddingTop'));
					}
					
					if (typeof fitPadding === UNDEF) {
						// Image margin defined by "fitRatio" - Photoblogger
						fitPadding = Math.round((1 - settings.fitRatio) * Math.min(lbw, lbh) / 2);
					}
					
					if (!(w = main.data('oWidth')) || !(h = main.data('oHeight'))) {
						
						// Has no dimensions saved yet
						
						var im = thumbs.eq(curr).data(J.OBJ);
						
						if (isExternal && im.hasOwnProperty(J.EXTERNAL)) {
							// External content
							var s = im[J.EXTERNAL]['size'];
							if (s) {
								// Has manually specified size
								s = s.split('x');
								w = s[0];
								h = s[1] || Math.round(s[0] *.75);
							} else {
								s = guessDimensions(im[J.EXTERNAL]['cont']);
								if (s) {
									// Has explicitly given size in the HTML code
									w = s[0];
									h = s[1];
								} else {
									// No info
									if (im[J.EXTERNAL]['cont'].includes('vimeo.com') ||
										im[J.EXTERNAL]['cont'].includes('youtube.com')) {
										// In case of YouTube or Vimeo with 720p as default
										w = 1280;
										if ((s = im[J.EXTERNAL]['cont'].match(/.*style="padding(-bottom)?:\s?([\d\.]+)%/)) && 
											s && (s.length > 1)) {
											// Responsive embedding
											h = Math.round((w * parseFloat(s[2])) / 100);
										} else {
											h = 720;
										}
									} else {
										w = im[J.IMAGE][J.WIDTH];
										h = im[J.IMAGE][J.HEIGHT];
									}
								}
							}
						} else if (isAudio && defaultPoster) {
							// Audio with poster image
							w = settings.defaultPosterSize[0];
							h = settings.defaultPosterSize[1];
						} else if (isVideo) {
							// Video
							w = im[J.VIDEO][J.WIDTH];
							h = im[J.VIDEO][J.HEIGHT];
						} else if (isVr || isPdf) {
							// Has no dimensions, use full screen
							w = lbw;
							h = lbh;
						} else {
							// Normal images and all fallbacks
							w = im[J.IMAGE][J.WIDTH];
							h = im[J.IMAGE][J.HEIGHT];
							if (settings.linkOriginals && settings.hiDpi) {
								w /= 2;
								h /= 2;
							}
						}
					}
					
					// Saving original width and height
					
					main.data({
							'oWidth': 		w,
							'oHeight': 		h
						});
					
					// Discounting padding and image border
					
					lbw -= 2 * (fitPadding + mainPadding);
					lbh -= 2 * (fitPadding + mainPadding);
					
					// Discounting caption height
					
					if (settings.fitImages && settings.fitAboveCaption && !slideshow && caption.length && caption.is(':visible')) {
						lbah = lbh - Math.ceil(caption.outerHeight());
					} else {
						lbah = lbh;
					}
						
					if (isVr || isPdf) {							
						// Full screen content
						
						w = Math.min(lbw, w);
						h = Math.min(lbah, h);
						
						ratio = 1;
						
						canZoomIn = canZoomOut = false;
						
					} else if (isExternal) {
						
						ratioX = Math.min(lbw / w, settings.maxZoom);
						ratioY = Math.min(lbah / h, settings.maxZoom);
						
						// Can scale up or max 1:1
						ratio = Math.min(ratioX, ratioY);
						
						w *= ratio;
						h *= ratio;
						
						canZoomIn = w < lbw && h < lbh;
						canZoomOut = !canZoomIn && ratio > 1;
						
					} else if (isAudio && defaultPoster) {			
						// Audio with poster image
						
						w = Math.min(settings.defaultPosterSize[0], lbw);
						h = Math.min(settings.defaultPosterSize[1], lbah);
						
						ratio = 1;
						
						canZoomIn = canZoomOut = false;
						
					} else {										
						// Image, Audio, Video, Other
						
						ratioX = Math.min(lbw / w, settings.maxZoom);
						ratioY = Math.min(lbah / h, settings.maxZoom);
						
						// Fit to screen or 1:1 ?
						
						if (settings.fitImages) {
							// Fit images to screen (normal and panorama)
							
							// Fit both or vertically only?
							ratio = settings.fitBoth? Math.min(ratioX, ratioY) : ratioY;
							
							// Can scale up or max 1:1
							ratio = Math.min(settings.scaleUp? settings.maxZoom : 1, ratio);
							
							// Panorama if "fit vertical" mode and wide enough
							isPano = 	isImage 
										&& 
										(	
											(
												settings.autoPano 
												&& 
												!settings.fitBoth 
												&& 
												w * ratio >= lbw * settings.panoramaAutoDetect
											)
											||
											(
												isPanorama
												&&
												w * ratio > lbw
											)
										);
							
							w = Math.round(w * ratio);
							h = Math.round(h * ratio);
							
							canZoomIn = !isPano && ratio < 1 || isPano && !settings.fitBoth;
							
						} else {
							
							// 1:1 or pano
							ratio = 1;
							
							// Checking panorama
							if (w > lbw || h > lbah) {
							
								// Fit for the largest AR (fill screen) to test panorama
								fillRatio = Math.max(ratioX, ratioY);
								
								// Testing panorama
								isPano = 	isImage 
											&& 
											(	
												(
													// panorama set by user
													isPanorama
													&& 
													(
														// hangs over the edges
														w * fillRatio > lbw 
														|| 
														h * fillRatio > lbah
													)
												)
												||
												(
													// ... or auto pano
													settings.autoPano
													&&
													(
														(	
															// ignoring those that doesn't qualify as panorama
															w / h > 2
															||
															w / h < .5625
														)
														&& 
														(	
															// hangs over the edges least 1.5 times the width or height
															(w * fillRatio / lbw) >= settings.panoramaAutoDetect 
															|| 
															(h * fillRatio / lbah) >= settings.panoramaAutoDetect
														)
													)
												)
											);
											
								if (isPano) {
									// display in fill or 1:1 mode?
									if (settings.fitBoth) {
										ratio = (w > h)? ratioY : ratioX;
										w = Math.round(w * ratio);
										h = Math.round(h * ratio);
									}
									
									canZoomIn = false;
									
								} else {
									
									canZoomIn = settings.scaleUp && ratio < 1;
									
								}
								
							} else {
								
								canZoomIn = settings.scaleUp && w < lbw && h < lbah;
								
							}
							
							/*
							if (HIDPI && ratio <= 1 && !settings.scaleUp) {
								w = Math.round(w / 2);
								h = Math.round(h / 2);
								ratio *= 2;
							}
							*/
						}
						
						canZoomOut = ratio > 1 || w > lbw || h > (lbah - (settings.fitAboveCaption? caption.outerHeight() : 0));
					}
					
					// Sizing
					
					if (initial === true) {
						
						main.css({
								left: 		fitPadding + Math.round((lbw - w) / 2),
								top: 		fitPadding + Math.round((lbah - h) / 2),
								width: 		w + 2 * mainPadding,
								height: 	h + 2 * mainPadding,
								opacity: 	0
							}).show();
							
					} else {
						
						main.stop(true, false).animate({
								left: 		fitPadding + Math.round((lbw - w) / 2),
								top: 		fitPadding + Math.round((lbah - h) / 2),
								width: 		w + 2 * mainPadding,
								height: 	h + 2 * mainPadding
							}, 500, function() {
								// Reset center
								if (!isPano) {
									$(this).translateX(0, 500);
								}
							});
						
					}
					
					// Auto panorama attach or detach
					
					if (isImage) {
						
						if (isPano) {
							
							autoPanoTimeout = setTimeout(function() {
									if (isPano) {
										main.autopano({ 
												direction: 	direction 
											});
									} else {
										main.trigger('autopanoRemove');
									}
								}, 550);
								// Delay by transTime = 500ms, necessary to allow time to finish zoom, otherwise pano calculates with wrong dimensions 
							
						} else {
							
							clearTimeout(autoPanoTimeout);
							main.trigger('autopanoRemove');
							
						}
					}
					
					/*
					log('fit:'+lightbox.width() + ':' + lightbox.height());
					log('window:'+$window.width() + ':' + $window.height());
					*/
					
					$.cookie('fitImages', settings.fitImages? (ratio > 1) : (ratio < 1));
					
					if (canZoomIn) {
						lb_zoom.removeClass('icon-contract').addClass('icon-expand');
					} else {
						lb_zoom.removeClass('icon-expand').addClass('icon-contract');
					}
					
					lb_zoom.toggleClass('invisible', !canZoomIn && !canZoomOut);
							
				},
			
			// Removing the old image and caption
			
			removeOldContent = function() {
				
					var oldMain = lightbox.find('.' + settings.lightboxMainClass),
						oldCaption = lightbox.find('.' + settings.lightboxCaptionClass);
						
					if (oldMain.length) {
						
						oldMain.trigger('autopanoRemove').off('.' + ns);
						
						if (settings.muteBackgroundAudio && backgroundAudioOn) {
							if (isAudio || isVideo) {
								backgroundAudioRoot.trigger('fadeOutPlayer');
							} else {
								backgroundAudioRoot.trigger('fadeInPlayer');
							}
						}
						
						// Removing the old image
						if (!swiped) {
							oldMain.translateXAndFade(-100 * direction, 0, settings.speed, function() {
								oldMain.remove();	
							});
						}
						
						oldMain.off('.' + ns);
					}
					
					if (oldCaption.length) {
						
						// Removing the old caption
						oldCaption.find('.buttons a').trigger('removeTooltip');
						
						if (oldCaption.length > 1) {
							oldCaption.eq(-1).prevAll('.' + settings.lightboxCaptionClass).remove();
							oldCaption = lightbox.find('.' + settings.lightboxCaptionClass);
						}
						
						oldCaption.opacity(0, settings.speed, function() {
							oldCaption.remove();	
						});
					}
				},
			
			// Reading image type
			
			getImageType = function(item) {
					isVideo = isAudio = isOther = isImage = isVr = isPdf = false;
					isExternal = item.hasOwnProperty(J.EXTERNAL);
					isVr = !isExternal && settings.use360Player && item.hasOwnProperty(J.PROJECTIONTYPE) && item[J.PROJECTIONTYPE] === 'equirectangular';
					isPanorama = item.hasOwnProperty(J.PANORAMA) && item[J.PANORAMA];
					if (!isExternal && !isVr) {
						switch (item[J.CATEGORY]) {
							case 'video':
								isVideo = true;
								break;
							case 'audio':
								isAudio = true;
								break;
							case 'other':
								isOther = true;
								isPdf = album.getExtension(item).toLowerCase() === 'pdf';
								break;
							default:
								isImage = true;
						}
					}
				},
			
			// Updates rating (if changed externally)
				
			updateRating = function(r) {
					
					if (curr >= 0) {
						if (typeof r === UNDEF) {
							r = thumbs.eq(curr).data(J.OBJ)[J.VISITORRATING] || 0;
						}
					
						lightbox.find('.' + settings.lightboxCaptionClass + ' .ratingbox span')
							.removeClass('r0 r1 r2 r3 r4 r5')
							.addClass('r' + r);
					}
				},
										
			// Creating caption and buttons
			
			prepareCaption = function(item) {
					
					caption = $('<div>', {
							'class': 	settings.lightboxCaptionClass + ' scrollable ' + (slideshow? ' slideshow' : '')
						})
						.hide()
						.appendTo(lightbox);
					
					var buttons = $('<div>', {
								'class': 	'buttons'
							}),
						useLabels = settings.buttonLabels && lightbox.width() >= 640;
					
					if (settings.useRating) {
						
						var r = item[J.RATING] || 0;
						
						if (settings.visitorRating || r) {
							var s;
						
							el = $('<div>', {
									'class':	'ratingbox'
								}).appendTo(buttons);
								
							el.append($('<label>', {
									html:		text.rating
								}));
								
							s = $('<span>', {
									'class':	'rating',
									html:		STARS
								})
								.appendTo(el);
								
							if (settings.visitorRating) {
								
								if (item.hasOwnProperty(J.VISITORRATING)) {
									// Already exists
									r = item[J.VISITORRATING];
								} else {
									if (!settings.useJalbumRating) {
										// Ignore jAlbum rating
										r = 0;
									}
									// Save the default
									item[J.VISITORRATING] = r;
								}
										
								s.addClass('r' + r + (settings.useJalbumRating? '' : ' icon-close'))
									.on('click.' + ns, function(e) {
											var el = $(this),
												r = Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left - parseFloat(el.css('paddingLeft'))) / 16), 5); 
												//Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left + 4) / el.width()) * 5, 5);
												
											el.removeClass('r0 r1 r2 r3 r4 r5').addClass('r' + r);
											item[J.VISITORRATING] = r;
											
											// Propagate to thumbnails
											self.trigger('refreshRating', thumbs[curr]);
												
											return false;
										});
							} else {
								// Unalterable rating from jAlbum
								s.addClass('r' + r);
							}
						}
					}
					
					// Extra sizes
					
					if (extraSizes && item[J.CATEGORY] === 'image') {
						el = ($('<div>', {
								'class': 	'download icon-download'
							})).appendTo(buttons);
						
						var p = album.getImagePath(item);
						
						for (var i = 0; i < extraSizes.length; i++) {
							el.append($('<a>', {
									text: 		extraSizes[i],
									href:  		p.replace('slides/', 'dl/' + extraSizes[i] + '/'),
									download: 	''
								}));
						}
					}
					
					// Share buttons
					
					if ((DEBUG || !LOCAL) && settings.share && settings.showShare) {
						
						/*var el = $('<div>', {
									'class': 	'share-cont social-links text-center'
								}).prependTo(buttons);
						*/
						/*
						if (useLabels) {
							el.append($('<small>', {
									text:	text.shareOn
								}));
						}
						*/
						buttons.renderShares({
								sites: 			settings.share,
								title: 			item[J.TITLE] || item[J.NAME],
								description: 	item[J.COMMENT],
								image: 			album.getAbsoluteItemPath(item)
							});
					}
					
					// Print button
					
					if (settings.printImage && item[J.CATEGORY] === 'image' && !settings.rightClickProtect) {
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-printer'
							}).appendTo(buttons);
						
						if (useLabels) {
							el.html('<span>' + text.print + '</span>')
								.data('tooltip', text.printLabel);
						} else {
							el.data('tooltip', text.print);
						}
						
						el.on('click', function(e) {
								printImage((item[J.ORIGINAL] && settings.showDownload)? item[J.ORIGINAL][J.PATH] : item[J.IMAGE][J.PATH],
									item[J.TITLE] || '',
									item[J.THUMBCAPTION] || ''
								);
							});
					}
					
					// Download button
					
					if (settings.showDownload && (isImage || isPdf || settings.allowDownloadOthers)) {
						var href = album.getOriginalPath(item);
						
						if (!href && settings.allowDownloadScaled) {
							href = album.getImagePath(item);
						}
						
						if (href) {
							el = $('<a>', {
									'class': 	settings.buttonClass + ' icon-download',
									download: 	'',
									href: 		href
								}).appendTo(buttons);
								
							if (useLabels) {
								el.html('<span>' + text.download + '</span>');
							} else {
								el.data('tooltip', text.download);
							}
						}
					}
					
					// PayPal button
					
					if (settings.hasOwnProperty('shop') && album.hasShop(item)) {
						var txt = album.getPriceRange(item);
						
						txt = text.addCart + (txt? (' <b>' + txt + '</b>') : '');

						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-shopping-cart-add'
							}).on('click', function(e) {
								settings.shop.root.trigger('addItems', item);
							}).appendTo(buttons);
						
						if (useLabels) {
							el.html('<span>' + txt + '</span>')
								.data('tooltip', text.shopLabel);
						} else {
							el.data('tooltip', txt);
						}
						
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-shopping-cart'
							}).on('click', function(e) {
								settings.shop.root.trigger('showCart');
							}).appendTo(buttons);
					
						if (useLabels) {
							el.html('<span>' + text.viewCart + '</span>')
								.data('tooltip', text.viewCartLabel);
						} else {
							el.data('tooltip', text.viewCart);
						}
						
					}
					
					// Feedback button
					
					if (settings.hasOwnProperty('feedback')) {
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-email-send'
							}).on('click', function(e) {
								settings.feedback.root.trigger('addItems', item);
							}).appendTo(buttons);
					
						if (useLabels) {
							el.html('<span>' + text.addComment + '</span>')
								.data('tooltip', text.feedbackLabel);
						} else {
							el.data('tooltip', text.addComment);
						}
						
					}
					
					// Fotomoto button
					
					if (!LOCAL && typeof FOTOMOTO !== UNDEF && settings.fotomoto) {
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-fotomoto'
							}).on('click', function(e) {
								FOTOMOTO.API.showWindow(10, 
									album.getOriginalPath(item) || album.getItemPath(item)
								);
							}).appendTo(buttons);
					
						if (useLabels) {
							el.html('<span>' + text.fotomotoBtn + '</span>')
								.data('tooltip', text.fotomotoTooltip);
						} else {
							el.data('tooltip', text.fotomotoBtn);
						}
					}
					
					// Photo data
					
					if (item[J.PHOTODATA]) {
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-camera'
							}).appendTo(buttons);
					
						if (useLabels) {
							el.html('<span>' + text.metaBtn + '</span>')
								.data('tooltip', text.metaLabel);
						} else {
							el.data('tooltip', text.metaBtn);
						}
						
						el.on('click', function() {
								lightbox.modal($('<div>', {
										'class':	'photodata',
										html: 		item[J.PHOTODATA]
									}), {
										'class':	'small',
										title: 		text.metaBtn
									});
								
								return false;
							});
					}
					
					// Regions
					
					if (settings.showRegions && item[J.REGIONS]) {
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-user'
							}).appendTo(buttons);
					
						if (useLabels) {
							el.html('<span>' + (settings.regionsBtn || text.regionsBtn) + '</span>')
								.data('tooltip', text.regionsLabel);
						} else {
							el.data('tooltip', (settings.regionsBtn || text.regionsBtn));
						}
						
						el.on('click', function() {
								var el = lightbox.find('.' + settings.regionsClass);
								
								if (el.length) {
									el.remove();
								} else {
									var reg = JSON.parse(item[J.REGIONS]);
									
									el = $('<div>', {
										'class':	settings.regionsClass
									}).appendTo(main);
									
									for (var i = 0, r; i < reg.length; i++) {
										r = reg[i].split(';');
										el.append($('<a>').css({
											left:		(100 * parseFloat(r[1])) + '%',
											top:		(100 * parseFloat(r[2])) + '%',
											width:		(100 * parseFloat(r[3])) + '%',
											height:		(100 * parseFloat(r[4])) + '%'
										}).append('<span>' + r[0] + '</span>'));
									}
								}
												
								return false;
							});
					}
					
					// Map
					
					if (!LOCAL && settings.showMap && item[J.LOCATION]) {
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-location'
							}).appendTo(buttons);
					
						if (useLabels) {
							el.html('<span>' + text.mapBtn + '</span>')
								.data('tooltip', text.mapLabel);
						} else {
							el.data('tooltip', text.mapBtn);
						}
						
						el.on('click', function() {
							var target = $('<div>', {
										'class': 	'map-cont'
									});
							
							target.height($window.height() * settings.mapHeight);
							lightbox.modal(target, {
									'class': 	'no-padding'
								});
							
							setTimeout(function() {
								target.addMap({
										location: 			item[J.LOCATION],
										type:				settings.mapType,
										zoom:				settings.mapZoom,
										enableScrollWheel:	true,
										fitBounds:			false,
										onTypeChanged:		function(type) {
																	settings.mapType = type;
																},
										onZoomChanged:		function(zoom) {
																	settings.mapZoom = zoom;
																}
									});
							}, 100);
							
							return false;
						});
					}
					
					// Mostphotos
					
					if (!LOCAL && item[J.MOSTPHOTOS]) {
						el = $('<a>', {
								'class': 	settings.buttonClass + ' icon-shopping-cart',
								target: 	'_blank',
								href: 		'https://mostphotos.com/' + item[J.MOSTPHOTOS]
							}).appendTo(buttons);
						
						if (useLabels) {
							el.html('<span>' + text.mostphotosBtn + '</span>')
								.data('tooltip', text.mostphotosLabel);
						} else {
							el.data('tooltip', text.mostphotosBtn);
						}
					}
					
					if (settings.showNumbers) {
						buttons.prepend($('<span>', {
								'class':	'numbers',
								'html':		'<big>' + (curr + 1) + '</big><small>' + thumbs.length + '</small>'
							}));
					}
					
					if (item[J.SOUNDCLIP]) {
						buttons.append($('<audio>', {
								'class':	'soundclip',
								src:		item[J.SOUNDCLIP],
								controls:	'controls'
							}));
					}
					
					if (item[J.IMAGECAPTION]) {
						caption.append($('<div>', {
								'class': 	'caption',
								html: 		item[J.IMAGECAPTION]
							}));
						caption.find('[data-tooltip]').addTooltip();
					}
					
					if (!buttons.is(':empty')) {
						caption.addClass(settings.hasbuttonsClass);
						caption.append(buttons);
						buttons.children('a').addTooltip();
					}
					
					caption.toggle(!slideshow || caption.children('.caption').length > 0);
					
					if (caption.is(':empty')) {
						lightbox.removeClass(settings.captionVisibleClass);
						caption.remove();
					} else {
						lightbox.addClass(settings.captionVisibleClass);
						caption.show();
					}
						
				},	
			
			// Preloading the next image (direction sensitive)
			
			preloadNext = function() {
					var nextItem;
					
					if (direction < 0 && curr > 0) {
						nextItem = thumbs.eq(curr - 1).data(J.OBJ);
					} else if (curr < thumbs.length - 2) {
						nextItem = thumbs.eq(curr + 1).data(J.OBJ);
					}
					
					if (nextItem) {
						if (nextItem[J.CATEGORY] === 'image') {
							var next = new Image();
							next.src = album.getImagePath(nextItem);
						}
					}
				},
			
			// Image clicked :: toggle controls or audio/video controls
			
			imageClicked = function(e) {
					var touched = $('html').data('whatinput') === 'touch' || e.type === 'touchend';
						/*
						playBtnClick = function(m) {
								var offs = main.offset();
								return 	Math.abs(m.clientWidth / 2 + offs.left - e.originalEvent.changedTouches[0].clientX) < 40 &&
										Math.abs(m.clientHeight / 2 + offs.top - e.originalEvent.changedTouches[0].clientY) < 40;
							};
						*/
					//log(e.type);
								
					if (isVideo || isAudio) {
						
						var m = media[0];
							
						if (m.paused) {
							m.play();
							if (VEND !== 'ms') {
								m.controls = false;
							}
							if (touched) {
								toggleControls();
								if (lightbox.hasClass(settings.captionVisibleClass)) {
									hideCaption();
								}
							}
						} else {
							m.pause();
							if (VEND !== 'ms') {
								m.controls = true;
							}
							if (touched) {
								showControls();
								showCaption();
							}
						}
						
					} else if (isImage) {
						
						if (touched) {
							toggleControls();
							if (!lightbox.hasClass(settings.captionVisibleClass)) {
								showCaption();
							}
						} else if (settings.clickForNext) {
							// image navigation
							if (((e.pageX || e.originalEvent.pageX) - main.position().left) > (main.width() / 2)) {
								nextImage();
							} else {
								previousImage();
							}
						}
					}
				},
			
			// The image is loaded
			
			imageReady = function() {
				
					loadCounter--;
					
					if (loadCounter > 0) {
						return;
					}
									
					if (DEBUG) {
						var d = new Date();
						if (isExternal || isVr) {
							console.log((isExternal? 'External content' : '360 player') + ' loaded: ' + (d - loadTimer) + 'ms');
						} else if (image.length) {
							console.log(((isVideo || isAudio)? 'Media' : 'Image') + ' [' + curr + '] loaded: ' + (d - loadTimer) + 'ms src="' + image[0].src + '"');
							if (typeof image[0] === UNDEF || typeof media[0] === UNDEF) {
								console.log('Premature ' + (typeof image[0] === UNDEF)? ('loadImage.done(' + image[0].src + ')') : ('loadMedia.done('+ media[0].src + ')'));
							}
						}
					}
					
					main.hide().css('opacity', 0);
					
					// Moving 100px away from center to prepare for moving back
					main.translateX(100 * direction);
					
					if ((isImage || isVideo || isAudio) && settings.rightClickProtect) {
						media.on('contextmenu', function(e) {
								e.preventDefault()
								return false;
							});
					}
												
					if (isVideo || isAudio) {
						media.attr({
							autoplay: 	settings.videoAuto,
							loop:		settings.videoLoop
						});
						media[0].volume = settings.volume;
						media.on('volumechange.' + ns, function() {
								settings.volume = this.volume;
							});
					}
					
					// Leaving time for CSS transform to settle down
					setTimeout(function() {
							
						fitImage(true);
						
						lb_activity.hide();
						
						// Moving back to center
						main.show().translateXAndFade(0, 1, settings.speed);
						
						// Preparing the image after transition
						setTimeout(function() {
								
							if (!isVr && !isPdf && (!isExternal || !main.children('iframe').length)) {
								// Swipe handler
								main.swipe({
										onSwipeStart:	function() {
																$(this).trigger('autopanoStop');
															},
										
										onSwipedLeft: 	function() {
																swiped = true;
																nextImage();
															},
														
										onSwipedRight: 	function() {
																swiped = true;
																previousImage();
															},
														
										onFinished: 	function() {
																if (!dontRemove) {
																	$(this).trigger('removeSwipe');
																	$(this).remove();
																}
															},
														
										onClick:		function(e) {
																if (lightbox.data('panomove')) {
																	$(this).trigger('autopanoStop');
																} else {
																	imageClicked(e);
																}
															}
									});
							}
							/*
							if (settings.autoPano && isImage && image.width() > lightbox.width()) {
								main.autopano({
										autoStart:		true
									});
							}
							*/
							if (settings.onLoadEnd !== false) {
								settings.onLoadEnd(thumb);
							}
							
							if (settings.preloadNext) {
								preloadNext();
							}
							
						}, settings.speed / 2);
						
					}, 50);
					
					// Continuing slideshow
					if (slideshow) {
						if (isAudio || isVideo) {
							suspendAuto();
							media[0].onended = function() {
								resumeAuto();
							};
						} else {
							clearTimeout(slideshow);
							slideshow = setTimeout(nextImage, settings.slideshowDelay);
							if (lb_pause) {
								lb_pause.find('.progress').show();
							}
						}
					} else {
						stopAuto(true);
						if (settings.autohideControls) {
							hideControlsLater();
						}
					}
				},
			
			// Main entry point to load a new image
			
			loadImage = function(n) {
				
					if (typeof n !== 'number') {
						n = thumbs.index(n);
					}
					
					if (n < 0 || n >= thumbs.length) {
						// Out of bounds move
						
						dontRemove = true;
						
						if (n < 0) {
							
							switch (settings.afterLast) {
								case 'donothing':
									n = 0;
									break;
									
								case 'startover':
									n = thumbs.length - 1;
									break;
									
								case 'onelevelup': 
									if (settings.level) {
										window.location.href = '../' + settings.indexName;
									}
									n = 0;
									break;
									
								case 'nextfolder':
									if (settings.baseSet && settings.previousFoldersLast) {
										window.location.href = settings.previousFoldersLast;
									}
									n = 0;
									break;
									
								case 'nextindex':
									if (settings.baseSet && settings.previousFolderPath) {
										window.location.href = settings.previousFolderPath;
									}
									n = 0;
									break;
									
								default:
									n = 0;
									quitLightbox();
									
							}
							
						} else {
							
							switch (settings.afterLast) {
								
								case 'donothing':
									n = thumbs.length - 1;
									break;
									
								case 'startover':
									n = 0;
									break;
									
								case 'onelevelup': 
									if (settings.level) {
										window.location.href = '../' + settings.indexName;
									}
									n = thumbs.length - 1;
									break;
									
								case 'nextfolder':
									if (settings.baseSet && settings.nextFoldersFirst) {
										window.location.href = settings.nextFoldersFirst;
									}
									n = thumbs.length - 1;
									break;
									
								case 'nextindex':
									if (settings.baseSet && settings.nextFolderPath) {
										window.location.href = settings.nextFolderPath;
									}
									n = thumbs.length - 1;
									break;
									
								case 'ask':
									
									var buttons = new Array();
									
									if (swiped) {
										main.translateXAndFade(0, 1, settings.speed * 2);
										swiped = false;
									}
									
									n = thumbs.length - 1;
						
									if (thumbs.length > 1) {
										// Start over
										buttons.push({
												t: 	text.startOver,
												c:	'icon-loop',
												h: 	function(lb) { 
														loadImage(0);
													}
											});
									}
									
									if (settings.level) {
										// Up one level
										buttons.push({
												t: 	settings.level? text.upOneLevel : (settings.homepageLinkText || text.backToHome), 
												c:	'icon-one-level-up',	
												h: 	function() { 
														window.location.href = '../' + settings.indexName;
													}
											});
									}
									
									// Back to thumbnails
									buttons.push({
											t: 	text.backToIndex,
											c:	'icon-arrow-up',
											h: 	function() { 
													quitLightbox();
												}
										});
									
									if (settings.baseSet) {
										
										if (settings.nextFoldersFirst) {
											// Go to next folder
											buttons.push({
													t: 	text.nextFolder,
													c:	'icon-arrow-right',
													h: 	function() {
															window.location.href = settings.nextFoldersFirst;
														}
												});
										}
										
										if (settings.nextFolderPath) {
											// Go to next index
											buttons.push({
													t: 	text.nextIndex,
													c:	'icon-thumbnails',
													h: 	function() {
															window.location.href = settings.nextFolderPath;
														}
												});
										}
									}
							
									lightbox.modal($('<p>', {
											'class':	'text-center',
											text: 		text.atLastPageQuestion
										}), buttons, {
											onClose:	function() {
															loadImage(thumbs.length - 1);
														},
											'class': 	'secondary',
											title: 		text.atLastPage
										});
									
									return;
										
								default:
									// Back to index
									n = slideshow? 0 : (thumbs.length - 1);
									stopAuto();
									quitLightbox();
							}
						}
						
						if (swiped) {
							main.translateXAndFade(0, 1, settings.speed * 2);
							swiped = false;
						}
						
					} else {
						dontRemove = false;
					}
					
					if (settings.useFullScreen && !isFullscreen()) {
						requestFullscreen();
					}
						
					// Direction is calculated if the current thumb exists
					if (lightbox.is(':visible')) {
						// Lightbox is on
						if ((Math.abs(n - curr) >= thumbs.length - 1) && settings.afterLast === 'startover') {
							// Flip over
							direction = 1;
						} else {
							direction = (curr > n)? -1 : ((curr < n)? 1 : 0);
						}
						
						if (curr === n) {
							// the requested image is already on screen :: nothing to do
							return;
						}
						
						if (settings.autoHideControls) {
							hideControlsLater();
						}
						
					} else {
						// We're on the index page :: show lightbox
						$body.add($('html')).css('overflow', 'hidden');
						$body.addClass(settings.lightboxOnClass);
						direction = 0;
						
						if (settings.useRating) {
							updateRating();
						}
						
						lightbox.show();
						backgroundAudioOn = backgroundAudioRoot.length && !backgroundAudioRoot.data('paused');
						
						if (settings.autoStart) {
							startAuto();
						}
					}
					
					closeModal();
					
					var item;
					
					curr = n;
					thumb = thumbs.eq(curr);
					item = thumb.data(J.OBJ);
					
					if (!item) {
						console.log('Fatal error: image (' + curr + ') is missing from the database! (Upload data1.json again!)');
						return;
					}
					
					lb_activity.show();
					if (lb_pause) {
						lb_pause.find('.progress').hide();
					}
					
					getImageType(item);
					
					if (settings.onLoadStart !== false) {
						settings.onLoadStart(thumb);
					}
					
					setTimeout(function() {
						
						// Delayed image loading
											
						removeOldContent(direction);
						swiped = false;
	
						loadCounter = 1;
						loadTimer = new Date();
						
							
						main = $('<div>', {
								'class': 	'lightbox-main ' + (isExternal? 'external' : (isVr? 'vr' : (isPdf? 'pdf' : item[J.CATEGORY])))
							})
							.hide()
							.appendTo(lightbox);
							
						if (isImage) {
							
							// Image
							
							media = image = $('<img>')
								.one('load.' + ns, function(e) {
									imageReady();
								})
								.attr('src', album.getImagePath(item))
								.appendTo(main);
							
							if (DEBUG) {
								console.log('Loading image [' + curr + '] src="' + album.getImagePath(item) + '"');
							}
							
							if (image[0].complete && image[0].naturalWidth > 0) {
								// From cache
								image.off('load.' + ns);
								imageReady();
							}
							
						} else if (isAudio || isVideo) {
							
							if (isVideo) {
								
								// Video
								media = image = $('<video>', {
											preload: 		'auto',
											controls: 		'controls',					// album.getVideoDuration(item) > 1500,
											controlsList:	'nodownload',
											poster: 		album.getPosterPath(item)
										});
								
							} else {
								
								var src = album.getPosterPath(item);
									
								// Audio
								loadCounter = 2;
								
								image = $('<img>', {
										'class': 	'poster' + (src.endsWith('poster.png')? ' default' : '')
									}).one('load', function(e) {
										imageReady();
									})
									.attr('src', src)
									.appendTo(main);
								
								media = $('<audio>', {
										preload: 		'auto',
										controls: 		'true',
										controlsList:	'nofullscreen nodownload'
									});
							}
							
							media.one('loadedmetadata', function(e) {
										imageReady();
									})
								.attr('src', album.getSourcePath(item))
								.appendTo(main);
							
							if (settings.muteBackgroundAudio && backgroundAudioRoot.length) {
								media.on('playing', mediaPlaying)
									.on('paused', mediaPaused)
									.on('ended', mediaEnded);
									
								backgroundAudioOn = !backgroundAudioRoot.data('paused');
							}
							
						} else if (isExternal) {
							
							var cont = item[J.EXTERNAL]['cont'],
								size = item[J.EXTERNAL]['size'];
								
							if (size) {
								size = size.split('x');
								main.data({
									oWidth:		parseInt(size[0]),
									oHeight:	parseInt(size[1])
								})
							}
								
							main.one('DOMReady', function(e) {
									imageReady();
								});
							
							if (cont.match(/^https?\:\/\//i) || cont.match(/\.html?$/i) || cont.match(/^\.\.\//)) {
								main.addClass('iframe')
									.append($('<iframe>', { 
											width: 				'100%',
											height: 			'100%',
											src: 				cont,
											frameborder: 		0,
											allowfullscreen: 	'allowfullscreen'
										}));
							} else {
								if (cont.match(/^<iframe/i)) {
									main.addClass('iframe');
								}
								main.append(cont);
							}
							
							// Considered ready after 200ms or the "ready" event, whatever comes first 
							setTimeout(imageReady, 200);
							
						} else if (isVr) {
							
							var w = Math.round(lightbox.width() * settings.fitRatio),
								h = Math.round(lightbox.height() * settings.fitRatio);
								
							main.css({
										width:		w,
										height:		h
									})
								.attr('id', 'vr' + curr)
								.addClass('vr')
								.show()
								.one('DOMReady', imageReady);
							
							// Photosphere viewer by Jeremy Heleine
							if (typeof PhotoSphereViewer !== UNDEF) {
								setTimeout(function() {
										var psv = new PhotoSphereViewer({
												panorama: 			album.getItemPath(item),
												container: 			main[0],
												usexmpdata:			false,
												zoom_level:			30,
												loading_html:		'<div class="lightbox-loading"><div></div></div>',
												navbar: 			true,
												navbar_style: 		{	
																		autorotateThickness:	2,
																		zoomRangeThickness: 	2,
																		zoomRangeDisk:			12,
																		fullscreenThickness: 	2,
																		backgroundColor: 		'rgba(17, 17, 17, 0.35)'
																	},
											});
									}, settings.speed);
							} else {
								console.log('Fatal Error: Missing "photo-sphere-viewer.min.js"!');
							}
							
							// Considered ready after 200ms or the "ready" event, whatever comes first 
							setTimeout(imageReady, 200);
							
						} else {
							
							// Other
							var target = main;
							
							if (isPdf) {
								
								if (!HASPDFVIEWER || ISIOSDEVICE) {
									
									target = $('<iframe>', {
											src:		'https://docs.google.com/viewer?url=' + window.location.href.getDir() + album.getSourcePath(item) + '&embedded=true'		
										})
										.appendTo(main);
										
								} else {							
									target = $('<object>', {
											type:		'application/pdf'
										})
										.attr('data', album.getSourcePath(item))
										.appendTo(main);
								}
							}
							
							$('<a>', {
									href: 		album.getSourcePath(item),
									target: 	'_blank'
								})
								.append($('<img>', {
									'class': 	'other'
								})
								.one('load', function(e) {
									imageReady();
								})
								.attr('src', album.getImagePath(item)))
								.appendTo(target);
							
							$('<p>', {
									'class': 	'click-hint',
									'html': 	text.clickToOpen
								})
								.appendTo(target);
						}	
						
						prepareCaption(item);
																
					}, settings.speed / 4);
				},
			
			// Hiding the lightbox overlay
			
			quitLightbox = function() {
					
					stopAuto(true);
					
					$body.add($('html')).css('overflow', '')
					$body.removeClass(settings.lightboxOnClass);
					
					if (settings.muteBackgroundAudio && backgroundAudioOn) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
					
					if (!main.length) {
						if (settings.onClose !== false) {
							settings.onClose(thumb);
						}
					} else {
						
						if ((isAudio || isVideo) && media) {
							var sv = media[0].volume,
								tm = settings.speed / (sv * 50),
								fade = function() {
										if (media) {
											var v = Math.max(media[0].volume - 0.02, 0);
											if (v > 0.005) {
												media[0].volume = v;
												setTimeout(fade, tm);
											} else {
												media[0].pause();
											}
										}
									};
							
							media.off('.' + ns);
							fade();
						}
						
						lightbox.fadeOut(settings.speed, function() {
							if (settings.useFullScreen) {
								requestFullscreen();
							}
							if (main.length) {
								main.remove();
								main = $();
							}
							if (caption.length) {
								caption.remove();
								caption = $();
							}
							//inProgress = false;
							if (settings.onClose !== false) {
								settings.onClose(thumb);
							}
							
							$body.hideAllTooltips();
						});
					}
				},
			
			// Removing the lightbox completely
			
			removeLightbox = function() {
					stopAuto(true);
					// removing event handlers
					thumbs.off('.' + ns);
					$document.add($window).add(lightbox).off('.' + ns);
					$body.add($('html')).css('overflow', '')
					$body.removeClass(settings.lightboxOnClass);
					
					if (!main.length) {
						// No image
						lightbox.remove();
					} else {
						// Fade out
						main.animate({ 
								'opacity': 	0 
							}, settings.speed, function() {
								lightbox.remove();
							});
					}
				},
			
			// Starting auto slideshow
			
			startAuto = function(keepMusic) {
					
					clearTimeout(slideshow);
					slideshow = setTimeout(nextImage, settings.slideshowDelay / 4);
					
					if (lb_play) {
						lb_play.hide();
						lb_pause.show();
					}
					lightbox.addClass(settings.playingClass);
					
					if (settings.autohideControls) {
						hideControlsLater();
					}
					
					caption.addClass('slideshow');
					
					if (!(keepMusic === true) && settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
					
					if (settings.onSlideshowStart !== false) {
						settings.onSlideshowStart(thumb);
					}
				},
			
			resumeAuto = function() {
					
					if (lb_play) {
						lb_play.hide();
						lb_pause.show();
					}
					
					if (settings.autohideControls) {
						hideControlsLater();
					}
					
					lightbox.addClass(settings.playingClass);
					slideshow = setTimeout(nextImage, settings.slideshowDelay / 4);
					
					if (settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
				},
	
			stopAuto = function(keepMusic) {
				
					if (settings.onSlideshowPause !== false) {
						settings.onSlideshowPause(thumb);
					}
					
					if (!(keepMusic === true) && settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeOutPlayer');
					}
					
					if (lb_play) {
						lb_play.show();
						lb_pause.hide();
					}
					clearTimeout(slideshow);
					slideshow = null;
					
					showArrows();
					showControls();
					caption.removeClass('slideshow');
					
				},
			
			suspendAuto = function() {
					if (settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeOutPlayer');
					}
					clearTimeout(slideshow);
					slideshow = null;
					//showControls();
				},
				
			mediaPlaying = function() {
				
					if (settings.muteBackgroundAudio) {
						if (backgroundAudioOn = !backgroundAudioRoot.data('paused')) {
							backgroundAudioRoot.trigger('fadeOutPlayer');
						}
					}
				},
				
			mediaPaused = function() {
					if (settings.muteBackgroundAudio && backgroundAudioOn) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
				},
				
			mediaEnded = function() {
					backgroundAudioOn = false;
				},
			
			toggleZoom = function() {
					settings.fitImages = !settings.fitImages;
					//savePrefs();
					fitImage();
				},
			
			showControls = function() {
					lb_close.add(lb_zoom).show();
				},
			
			hideControls = function() {
					lb_close.add(lb_zoom).fadeOut(500);
					$body.hideAllTooltips();
				},
			
			hideControlsLater = function() {
					clearTimeout(controlsTimeout);
					controlsTimeout = setTimeout(function() {
							hideArrows();
							hideControls();
							caption.addClass('slideshow');
						}, slideshow? Math.min(settings.slideshowDelay / 2, 2000) : 2000);
				},
			
			showArrows = function() {
					lb_btn_left.add(lb_btn_right).show();
				},
			
			hideArrows = function() {
					lb_btn_left.add(lb_btn_right).fadeOut(500);
				},
			
			previousImage = function() {
					loadImage(curr - 1);
				},
			
			nextImage = function() {
					loadImage(curr + 1);
				},
				
			// Setting up the structure
			
			initLightbox = function(ns) {
				
					lightbox = $('<div>', {
							id: 		ns,
							'class': 	'lightbox'
						}).hide().appendTo('body');
					
					// Darken background
					lb_overlay = $('<div>', {
							'class': 	'lightbox-overlay'
						}).appendTo(lightbox);
					
					// Activity indicator
					lb_activity = $('<div>', {
							'class': 	'lightbox-loading'
						}).append('<div>').appendTo(lightbox);
					
					// Close button
					lb_close = $('<button>', {
							type: 		'button',
							'class': 	'lightbox-btn close',
							'title': 	text.close
						})
						.on('click.' + ns, function(){ 
							quitLightbox(); 
							return false; 
						})
						.appendTo(lightbox);
					
					// Zoom toggle button
					lb_zoom = $('<button>', {
							type: 		'button',
							'class': 	'lightbox-icon zoom icon-expand',
							'title': 	text.toggleZoom
						})
						.on('click.' + ns, function(){
							toggleZoom(); 
							return false; 
						})
						.appendTo(lightbox);
										
					// Play button
					lb_play = $('<button>', {
							type: 		'button',
							'class': 	'lightbox-icon play icon-play',
							'title': 	text.slideshow
						})
						.on('click.' + ns, function(){ 
							startAuto(); 
							return false; 
						})
						.appendTo(lightbox);
		
					// Pause button
					lb_pause = $('<button>', {
							type: 		'button',
							'class': 	'lightbox-icon pause icon-pause',
							'title': 	text.pause
						})
						.append($('<span>', {
							'class': 	'progress'
						}))
						.hide()
						.on('click.' + ns, function(){ 
							stopAuto(); 
							return false; 
						})
						.appendTo(lightbox);
					
					// Buttons
					lb_btn_left = thumbs.length?
						$('<button>', {
								type: 		'button',
								'class': 	'lightbox-btn arrow-left'
							})
							.on('click.' + ns, function(){
								previousImage();
								return false;
							})
							.appendTo(lightbox)
						: 
						$();
						
					lb_btn_right = thumbs.length?
						$('<button>', {
								type: 		'button',
								'class': 	'lightbox-btn arrow-right'
							})
							.on('click.' + ns, function(){
								nextImage();
								return false;
							})
							.appendTo(lightbox)
						: 
						$();
												
					// Adding tooltips
					lightbox.find('button').addTooltip();
		
				};
			
		// Initializing Lightbox
					
		if (ns) {
			// should never happen, but still living lightbox attached to thumb elements
			cleanup(ns);
		}
		
		// Creating new namespace
		self.data('llb_ns', ns = 'llb_' + Math.floor(Math.random() * 10000));		

		// Finding thumbs
		thumbs = self.find(settings.delegate);
		
		// Initializing controls
		initLightbox();	
		
		// Setting up events
				
		// Resize
		$window.on('resize.' + ns /* + ' orientationchange.' + ns*/, function() {
				clearTimeout(resizeTimeout);
				// Executed only if 50ms has passed since the last resize event
				resizeTimeout = setTimeout(function() { 
					clearTimeout(resizeTimeout);
					if (main && main.length && !(isVr && isFullscreen())) {
						fitImage();
					}
				}, 50);
			});
		
		// Quit on document touch
		if (settings.quitOnDocClick) {
			lb_overlay.on('click.' + ns, function(e) {
					
					if (settings.autohideControls && controlsHidden()) {
						// Toggle controls back
						showControls();
						return false;
					}
					
					if (main.length && !$(e.target).is(main)) {
						// Quit
						quitLightbox();
					}
				});
		}
		
		// On mouse-driven devices
		
		if (settings.autohideControls) {
			
			// Auto hide controls
			$document.on('mousemove.' + ns, function() {
					showControls();
					hideControlsLater();
				});
		}
						
		// Keyboard
		if (settings.enableKeyboard) {
			$document.on('keyup.' + ns, function(e) {
				if (!main.length || lightbox.children('.modal:visible').length || document.activeElement && (document.activeElement.nodeName === 'INPUT' || document.activeElement.nodeName === 'TEXTAREA')) {
					return true;
				}
				
				e.preventDefault();
				
				switch (e.keyCode) {
					
					case 27:
						quitLightbox();
						break;
					
					case 33:
					case 37:
						previousImage();
						break;
						
					case 34:
					case 39:
						nextImage();
						break;
						
					case 97:
					case 35:
						loadImage(thumbs.length - 1);
						break;
						
					case 103:
					case 36:
						loadImage(0);
						break;
					
					case 106:
					case 179:
						if (slideshow) {
							stopAuto();
						} else {
							startAuto();
						}
						break;
						
					case 107:
						toggleZoom();
						break;
						
					case 32:
						if (isVideo || isAudio) {
							if (media[0].paused) {
								media[0].play();
							} else {
								media[0].pause();
							}
						} else {
							if (slideshow) {
								stopAuto();
							} else {
								startAuto();
							}
						}
						break;
						
					default:
						return true;
										
				}
				
				return false;
			});
		}
		
		// Mouse wheel
		if (settings.enableMouseWheel) {
			var wheelTimeout = null;
			lightbox.on('mousewheel.' + ns + ' DOMMouseScroll.' + ns, function(e) {
					clearTimeout(wheelTimeout);
					if (	!main.length || 
							$(e.target).closest('.' + settings.lightboxCaptionClass).length ||
							main.hasClass('pdf') || 
							lightbox.children('.modal:visible').length
						) {
						return true;
					}
					wheelTimeout = setTimeout((e.originalEvent.wheelDelta > 0 || e.originalEvent.detail < 0)? previousImage : nextImage, 100); 
					return false;
				});
		}
		
		// Events that can be triggered from outside
		
		// External trigger to switch image
		self.on('lightboxLoad.' + ns, function(e, n, auto) {
					stopAuto(true);
					
					if (typeof n === 'number') {
						n = Math.minMax(0, n, thumbs.length);
					} else if (typeof n === UNDEF) {
						n = 0;
					} else {
						n = thumbs.index(n);
					}
					
					if (settings.onStart !== false) {
						settings.onStart((typeof n === 'number')? thumbs.eq(n) : th);
					}
					
					loadImage(n);
					
					if (auto) {
						if (typeof auto === 'number') {
							setTimeout(startAuto, auto);
						} else {
							startAuto();
						}
					}
				})
			// External trigger to quit = does not remove, just hides the lightbox
			.on('lightboxQuit.' + ns, function() {
					quitLightbox();
					return false;
				})
			// Removing the lightbox completely
			.on('lightboxRemove.' + ns, function() {
					removeLightbox();
					return false;
				})
			// Stopping auto hide controls
			.on('stopAutoHide.' + ns, function() {
					clearTimeout(controlsTimeout);
				})
			// Returning the current lightbox container
			.on('lightboxContainer.' + ns, function() {
					return lightbox;
				})
			// External trigger to go to the previous image
			.on('lightboxNext.' + ns, function() {
					stopAuto();
					nextImage();
					return false;
				})
			// External trigger to go to next image
			.on('lightboxPrevious.' + ns, function() {
					stopAuto();
					previousImage();
					return false;
				});
				
		// Ready event
		
		if (settings.hasOwnProperty('onReady') && typeof settings.onReady === 'function') {
			settings.onReady(thumb);
		}
		
		// Auto start
		
		if (settings.autoStart && settings.baseSet) {
			startAuto();
		}

		return this;
	};
	
	$.fn.lightbox.defaults = {
			delegate:						'.card.lbable',
			lightboxMainClass:				'lightbox-main',
			lightboxCaptionClass:			'lightbox-caption',
			lightboxOnClass:				'lightbox-on',
			buttonClass:					'button',
			activeClass:					'active',
			scrollClass:					'scroll',
			regionsClass:					'regions',
			captionVisibleClass:			'caption-visible',
			thumbsVisibleClass:				'thumbs-visible',
			hasbuttonsClass:				'hasbuttons',
			immerseClass:					'immerse',
			slideshowDelay:					4000,
			hideControlsDelay:				2500,
			fitImages:						true,
			fitBoth:						true,
			fitAboveCaption:				true,
			fitRatio:						0.94,
			maxZoom:						1.4,
			scaleUp:						false,
			baseSet:						true,
			autohideControls:				false,
			autoStart:						false,
			clickForNext:					true,
			backgroundAudioSlideshowControl:false,
			muteBackgroundAudio:			true,
			use360Player:					true,
			useFullScreen:					false,
			afterLast:						'donothing',
			mapHeight:						0.8,
			videoAuto:						false,
			videoLoop:						false,
			autoPano:						false,
			volume:							0.5,
			rightClickProtect: 				false,
			useRating:						false,
			jalbumRating:					false,
			visitorRating:					false,
			buttonLabels:					true,
			showNumbers:					false,
			showShare:						false,
			allowDownloadOthers:			false,
			allowDownloadScaled:			false,
			showDownload:					false,
			showMap:						false,
			showRegions:					false,
			printImage:						false,
			speed: 							400,
			panoramaAutoDetect:				1.5,
			panoramaTreshold:				[ 2.5, 2.5 ],
			defaultPosterSize:				[ 628, 360 ],
			preloadNext:					true,
			enableKeyboard: 				true,
			enableMouseWheel:				true,
			quitOnEnd:						true,
			quitOnDocClick: 				true,
			onStart:						false,
			onClose:						false,
			onLoadStart:					false,
			onLoadEnd:						false,
			onAfterLast:					false,
			onBeforeFirst:					false,
			onReady:						false,
			onSlideshowStart:				false,
			onSlideshowPause:				false
		};
	
	$.fn.lightbox.text = {
			startOver:						'Start over',
			upOneLevel:						'Up one level',
			backToHome:						'Back to home',
			backToIndex:					'Back to index page',
			nextFolder:						'Next folder',
			nextIndex:						'Next index page',
			atLastPageQuestion:				'Where to go next?',
			atLastPage:						'At last page',
			atFirstPage:					'At first page',
			close:							'Close',
			slideshow:						'Slideshow',
			pause:							'Pause',
			rating:							'Rating',
			toggleZoom:						'Toggle zoom',
			download: 						'Download',
			print:							'Print',
			printLabel:						'Print out this photo on your printer',
			mapBtn: 						'Map',
			mapLabel:						'Show the photo location on map',
			fotomotoBtn:					'Buy / Share',
			fotomotoLabel:					'Buy prints or digital files, share, send free eCards',
			mostphotosBtn:					'Purchase',
			mostphotosLabel:				'Download this image from <b>mostphotos.com</b>!',
			regionsBtn:						'People',
			regionsLabel:					'Show tagged people', 
			share:							'Share',
			shareLabel:						'Share this photo over social sites',
			shareOn:						'Share on',
			shopBtn: 						'Buy',
			shopLabel:						'Add this item to the shopping cart',
			viewCartLabel:					'View shopping cart',
			feedbackLabel:					'View feedback window',
			metaBtn: 						'Photo data',
			metaLabel:						'Display photographic (Exif/Iptc) data',
			viewCart:						'View cart',
			addCart:						'Add to cart',
			addComment:						'Add comment',
			clickToOpen:					'Click to open this document with the associated viewer!'
		};

	
})(jQuery, jQuery(window), jQuery(document), jQuery('body'));
