/* 
 *	Rendering share buttons
 */

;(function($) {
	'use strict';
		
	$.fn.renderShares = function(settings) {
		
			settings = $.extend({}, $.fn.renderShares.defaults, settings);
			
			var self = 				$(this),
				text = 				getTranslations({
											share: 			'Share',
											shareOn: 		'Share on',
											checkThisOut: 	'Found this page',
											email:			'Email',
											slideshow:		'Slideshow',
											copy:			'Copy',
											copied:			'Copied',
											localWarning: 	'Can\'t share local albums. Please upload your album first!'
										}),
				call = 				encodeURIComponent(settings.callAction || text['checkThisOut']),
				label = 			settings.label || text['share'],
				pageExt =			settings.indexName.getExt(),
				
				copyUrl = function(e) {
						var url = this.querySelector('input[name=url]'),
							save = url.value,
							succ;
						
						if (url) {
							url.focus();
							url.setSelectionRange(0, save.length);
							
							try {
								succ = document.execCommand('copy');
							} catch(err) {
								succ = false;
							}
							
							if (succ) {
								url.value = text['copied'] + '...';
								url.classList.add('copied');
								setTimeout(function() {
										url.value = save;
										url.classList.remove('copied');
									}, 500);
							}
						}
						
						return false;
					},
					
				changeUrl = function(e) {
						var opt = this.checked,
							option = this.name,
							url = this.parentNode.querySelector('input[name=url]');
						
						if (opt) {
							if (url.value.indexOf('#') >= 0) {
								if (url.value.indexOf(new RegExp('[\#&]' + option)) === -1) {
									url.value += '&' + option;
								}
							} else {
								url.value += '#' + option;
							}
						} else {
							url.value = url.value.replace(new RegExp('[\#&]' + option, 'g'), '');
						}
					},
				
				updateLinks = function(e, opt) {
					
						if (typeof opt === UNDEF) {
							opt = {};
						}
						
						if (!opt.hasOwnProperty('href')) {
							opt.href = window.location.href;
						}
						
						opt = $.extend({}, settings, opt);
						
						// Fallbacks 
						
						if (!opt.hasOwnProperty('pageExt')) {
							opt.pageExt = opt.indexName.getExt()
						}
						
						if (opt.hasOwnProperty('description')) {
							opt.description = opt.description.stripHTML();
						} else {
							var c = $(opt.currCardHook + ' .caption');
							if (c.length) {
								opt.description = c.text();
							} else {
								opt.description = $('meta[name=description]').attr('content') || '';
							}
						}
						
						if (opt.hasOwnProperty('title')) {
							opt.title = opt.title.stripHTML();
						} else {
							var c = $(opt.currCardHook + ' img, ' + opt.currCardHook + ' video');
							if (c.length) {
								opt.title = c.attr('alt') || c.attr('src').getFile();
							} else {
								opt.title = $('meta[name=title]').attr('content') || $('title').text() || '';
							}
						}
						
						if (!opt.hasOwnProperty('image')) {
							var c = $(opt.currCardHook + ' img, ' + opt.currCardHook + ' video');
							if (c.length) {
								opt.image = c.attr('src');
							} else {
								c = $('link[rel=image_src]');
								opt.image = (c.length)? c.attr('href') : (window.location.href.getDir() + 'folderthumb.jpg');
							} 
						} 
						
						if (!opt.image.startsWith('http')) {
							opt.image = window.location.href.getDir() + opt.image;
						}
						
						// Encoded versions
						var href = 			encodeURIComponent(opt.href),
							slideHref =		(opt.href.indexOf('#img=') > 0)? opt.href.replace(settings.indexName, '').replace('#img=', 'slides/').replaceExt(opt.pageExt) : opt.href,
							title = 		encodeURIComponent(opt.title),
							description = 	encodeURIComponent(opt.description),
							image = 		encodeURIComponent(opt.image),
							link;
						
						self.find('.' + settings.className + ' a').each(function() {
							var a = $(this);
							
							switch (a.data('share')) {
							
								case 'facebook':
									// FB has removed the custom sharing option :(
									/*
									if (opt.href.indexOf('#') > 0) {
										return;
									}
									*/
									
									if (typeof FB === UNDEF) {
										link = 'https://www.facebook.com/sharer.php?s=100&p%5Burl%5D=' + opt.href + '&p%5Bimages%5D%5B0%5D=' + opt.image + '&p%5Btitle%5D=' + title;
									} else {
										a.off('.share').on('click.share', function() {
											FB.ui({
													method: 		'feed',
													link: 			slideHref,
													caption: 		opt.title,
													description: 	opt.description,
													picture: 		opt.image
												});
											return false;
										});
										return;
									}
									break;
								
								case 'twitter':
									link = 'https://twitter.com/home?status=' + title + ': ' + href;
									break;
									
								case 'tumblr':
									link = 'https://www.tumblr.com/share/link?url=' + href + '&name=' + title; 
									break;
									
								case 'pinterest':
									link = 'https://pinterest.com/pin/create/button/?url=' + href + '&media=' + image + '&description=' + title;
									break;
								
								case 'linkedin':
									link = 'https://www.linkedin.com/shareArticle?mini=true&url=' + href + '&title=' + title + '&summary=' + description;
									break;
									
								case 'digg':
									link = 'https://digg.com/submit?url=' + opt.href;
									break;
									
								case 'stumbleupon':
									link = 'https://www.stumbleupon.com/submit?url=' + opt.href + '&title=' + title;
									break;
									
								case 'reddit':
									link = 'https://www.reddit.com/submit?url=' + opt.href;
									break;
									
								case 'email':
									link = 'mailto:?subject=' + call + '&body=' + title + '%0D%0A' + description + '%0D%0A' + encodeURI(opt.href);
									break;
									
								case 'link':
									var tt = $('<form class="copy-url">' +
											'<input name="url" type="text" readonly value="' + opt.href + '">' +
											'<input name="slideshow" id="slideshowcb" type="checkbox">' +
											'<label for="slideshowcb">' + text['slideshow'] + '</label>' +
											' <button type="submit" class="button icon-copy"> ' + text['copy'] + '</button>' +
										'</form>');
									
									tt.on('submit', copyUrl);
									tt.find('[name=slideshow]').on('change', changeUrl);
									
									a.trigger('destroyTooltip').addTooltip(tt, {
											stay:	3000,
											pos: 	settings.pos 
										});
									
									return;
									
								default:
									return;
									
							}
						
							a.attr({
									href: 		link,
									target: 	'_blank'
								});
							
						});
					},
			
				createButtons = function() {
						
						var buttons = 	settings.buttons.split(','),
							href = 		encodeURIComponent(window.location.href.split('#')[0]),
							title = 	encodeURIComponent(settings.title || $('meta[name=title]').attr('content') || $('title').text()),
							image = 	settings.image? 
											(window.location.href.getDir() + encodeURIComponent(settings.image)) : 
											$('link[rel=image_src]').attr('href'),
							cont = 		$('<div>', {
												'class': 	settings.buttonsClassName
											}).appendTo(self);
						
						for (var i = 0; i < buttons.length; i++) {
							
							switch (buttons[i]) {
								
								case 'facebook':
									cont.append('<div class="likebtn fb-like" data-href="' + href + '" data-layout="button_count" data-action="like" data-size="large" data-show-faces="false" data-share="false"></div>');
									break;
								
								case 'twitter':
									cont.append('<a class="likebtn twitter-share-button icon-twitter" href="https://twitter.com/intent/tweet?text=' + title + '&url=' + href + '" data-size="large" target="_blank"> Tweet</a>');
									break;
	
								case 'pinterest':
									cont.append('<div class="likebtn"><a data-pin-do="buttonPin" data-pin-count="beside" data-pin-tall="true" data-pin-save="true" href="https://www.pinterest.com/pin/create/button/?url=' + href + '&media=' + image + '&description=' + title + '" data-pin-do="buttonPin"></a></div>');
									break;
							}
						}
					},
				
				createLinks = function(cont) {
						var n,
							cont = $('<div>', {
									'class': 	settings.className
								}).appendTo(self),
							sites = settings.sites.split(',');
						
						for (var i = 0, a; i < sites.length; i++) {
							
							n = sites[i];
							if ('facebook,twitter,tumblr,pinterest,linkedin,digg,stumbleupon,reddit,email,link'.indexOf(n) >= 0) {
								a = $('<a>', {
										'class': 	settings.buttonClassName + ' icon-' + n.unCamelCase(),
										rel:		'noopener',
										text:		' ' + (settings.buttonLabels? n.capitalize() : '')
									})
									.data('share', n)
									.appendTo(cont);
								
								if (n === 'email') {
									a.data('tooltip', text['email']);
								} else if (n !== 'link') {
									a.data('tooltip', text['shareOn'] + ' ' + n.capitalize());
								}
							}
						}
						
						updateLinks();
						
						cont.children('a').not('.icon-link').addTooltip({ 
								pos: 	settings.pos 
							});
					};
		
		
			createLinks();
			
			if (settings.hasOwnProperty('buttons') && !LOCAL) {
				createButtons();
			}
			
			if (self.closest('.modal').length) {
				self.find('.' + settings.className + ' a').on('click', function() {
						$(this).closest('.modal').trigger('close');
						return true;
					});
			}
			
			self.on('updateLinks', updateLinks);
					
			return this;
		};
	
	$.fn.renderShares.defaults = {
			buttonLabels:			false,		
			className: 				'shares',
			buttonsClassName:		'buttons',
			buttonClassName:		'btn',
			sites: 					'facebook,twitter,tumblr',
			indexName:				'index.html',
			currCardHook:			'.lightbox .curr',
			pos:					[1,2,1,0]
		};

})(jQuery);