/* 
 *	main.js - the skin's javascript functions
 */
	
;(function($, $window, $document, $body, undefined) {
	'use strict';
				
	/********************************************************
	 *
	 *		Implementing select functions on images
	 *
	 ********************************************************/
	
	$.fn.selectable = function(options) {
		
			options = $.extend({
							cardClass:				'card',
							thumbClass:				'thumb',
							checkboxClass: 			'checkbox',
							checkedClass:			'checked',
							checkmarkClass:			'icon-checkmark',
							hasSelectedClass:		'has-selected',
							allSelectedClass:		'all-selected',
							//noneSelected:			null,
							//anySelected:			null,
							selectionChange:		null
						}, options);
			
			var self = $(this).eq(0),
				cards,
				
				update = function(n) {
					
						if (typeof options.selectionChange === FUNCTION) {
							options.selectionChange.call(n);
						}
						
						$body
							.toggleClass(options.hasSelectedClass, n > 0)
							.toggleClass(options.allSelectedClass, n === cards.length);
					},
					
				selectedCnt = function() {
						return cards.filter('.' + options.checkedClass).length;
					},
				
				getImages = function() {
						return options.selector? self.find(options.selector) : self.find('.' + options.cardClass);
					},
					
				selectCard = function(card) {
						card.addClass(options.checkedClass)
							.find('.' + options.checkboxClass)
							.addClass(options.checkmarkClass);
					},
					
				unselectCard = function(card) {
						card.removeClass(options.checkedClass)
							.find('.' + options.checkboxClass)
							.removeClass(options.checkmarkClass);
					},
					
				toggleCard = function(card) {
						if (card.hasClass(options.checkedClass)) {
							unselectCard(card);
						} else {
							selectCard(card);
						}
					},
			
				clicked = function(e) {
						var card = $(e.target).closest('.' + options.cardClass);
						var n = selectedCnt();
						
						if (card.hasClass(options.checkedClass)) {
							unselectCard(card);
							n -= 1;
							
							if (typeof options.selectionChange === FUNCTION) {
								options.selectionChange.call(n);
							}
							/*
							if (n === 0) {
								if (typeof options.noneSelected === FUNCTION) {
									options.noneSelected.call();
								}
							} else {
								if (typeof options.anySelected === FUNCTION) {
									options.anySelected.call(n);
								}
							}
							*/
						} else {
							
							selectCard(card);
							n += 1;
							
							
							if (typeof options.selectionChange === FUNCTION) {
								options.selectionChange.call(n);
							}
							/*
							if (typeof options.anySelected === FUNCTION) {
								options.anySelected.call(n);
							}
							*/
						}	
						
						update(n);
						return false;
					},
			
				selectAll = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							
							if (n < cards.length) {
								n = cards.length;
								cards.not('.' + options.checkedClass).each(function() {
										selectCard($(this));
									});
										
								if (typeof options.selectionChange === FUNCTION) {
									options.selectionChange.call(n);
								}
								/*
								if (typeof options.anySelected === FUNCTION) {
									options.anySelected.call(cards.length);
								}
								*/
							}
							update(n);
						}
					},
			
				selectNone = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							
							if (n > 0) {
								n = 0;
								cards.filter('.' + options.checkedClass).each(function() {
										unselectCard($(this));
									});
										
								if (typeof options.selectionChange === FUNCTION) {
									options.selectionChange.call(n);
								}
								/*
								if (typeof options.noneSelected === FUNCTION) {
									options.noneSelected.call();
								}
								*/
							}
							update(n);
						}
					},
					
				selectInverse = function() {
					
						if (self.is(':visible')) {
							var n = cards.length - selectedCnt();
							
							if (n === 0) {
								// All selected
								cards.each(function() {
										unselectCard($(this));
									});
								
								if (typeof options.selectionChange === FUNCTION) {
									options.selectionChange.call(0);
								}
								/*
								if (typeof options.noneSelected === FUNCTION) {
									options.noneSelected.call();
								}
								*/
							} else {
								// Some selected
								cards.each(function() {
										toggleCard($(this));
									});
								
								if (typeof options.selectionChange === FUNCTION) {
									options.selectionChange.call(n);
								}
								/*
								if (typeof options.anySelected === FUNCTION) {
									options.anySelected.call(n);
								}
								*/
							}
							update(n);
						}
					},
				
				selectRefresh = function() {
						var img, 
							cb;
							
						cards = getImages();
						
						cards.each(function() {
								
							if (!($(this).children('span.' + options.checkboxClass).length)) {
								
								img = $(this).children('.' + options.thumbClass).eq(0);
								
								if (img.length) {
									cb = $('<span>', {
											'class': 	options.checkboxClass
										}).insertAfter(img);
								} else {
									cb = $('<span>', {
											'class': 	options.checkboxClass
										}).prependTo($(this));
								}
								
								cb.on({
										click: 		clicked
									});
							}
							
						});
						
						if (typeof options.selectionChange === FUNCTION) {
							options.selectionChange.call(selectedCnt());
						}
						
					};
			
			selectRefresh();
			
			self.on({
					selectAll: 			selectAll,
					selectNone: 		selectNone,
					selectInverse: 		selectInverse,
					selectRefresh: 		selectRefresh
				});
					
			return this;
		};
			
		
	/********************************************************
	 *
	 *					The main skin code
	 *
	 ********************************************************/
	
	$.fn.skin = function(settings) {
		
			settings = $.extend({}, $.fn.skin.defaults, settings);
			
			var main				= 	$(this),								// Main; sidebar included
				mainCont			=	main.find('#main-cont'),				// Main column
				text 				= 	getTranslations({						// Translated texts
												foundNTimes:				'found {0} time(s)',
												notFound: 					'not found',
												search: 					'Search',
												newImages:					'New images',
												results:					'Results',
												reset:						'Reset',
												label: 						'Label',
												selectedItems:				'Selection',
												addCart:					'Add to cart',
												'return': 					'return',
												select:						'Select',
												sortBy:						'Sort by',
												sortedBy:					'Sorted by',
												ascending:					'ascending',
												descending:					'descending',
												multipleSelectHint:			'Use SHIFT to select range, CTRL for multiple entries',
												noRating:					'No rating',
												inTheLastDay:				'in the last day',
												inThePastNDays:				'in the past {0} days',
												sinceMyLastVisit:			'since my last visit',
												betweenDays:				'between {0} and {0}',
												onDay:						'on {0}',
												beforeDay:					'before {0}',
												afterDay:					'after {0}',
												imagesAdded:				'Images added',
												imagesModified:				'Images modified',
												imagesTaken:				'Images taken',
												'new':						'New',
												more:						'more',
												less:						'less'
											}),
			
				shopBox 			= 	$(),									// Whole Shopping section
				shopRoot 			=	$(),									// Root element for cart
				feedbackBox 		=	$(),									// Whole Feedback section
				feedbackRoot 		=	$(),									// Root element for the feedback tool
				mapRoot				=	$(),									// Root element for map
				album,															// The database
				lightbox,														// Lightbox element
				ns 					=	'pb_skin',								// Namespace
				lastHash 			= 	'',										// Storing last state
				pageExt				= 	settings.indexName.split('.')[1] || 'html',	// Index page extension
				initByHash 			= 	window.location.hash !== "",			// Initial URL contains hash
				cacheBuster,													// Refresh thumbnails?
				nows 				=	new Date() / 1000,						// Now in seconds
				currency 			=	'EUR',
				
				lazyloadFoldersTimeout = null,								// Lazyload folders tarcking
				lazyloadFoldersInProgress = false,
				
				// Determine card type
				
				cardType = function(card) {
						var s = card.closest('section');
						if (!s.length) {
							return 'image';
						} else if (s.hasClass(settings.folderClass)) {
							return 'folder';
						} else if (s.hasClass(settings.thumbsClass)) {
							return 'thumb';
						}
						
						return '';
					},
												
				/********************************************************
				 *
				 *				Managing buttons and panels
				 *
				 ********************************************************/
				 
				// Toggle select buttons: selected count, max count
				
				toggleSelectButtons = function(sc, mc) {
						var p = shopRoot.add(feedbackRoot);
												
						if (typeof mc === UNDEF || !mc) {
							p.find('.' + settings.selectBoxClass).hide();					
						} else {
							p.find('.' + settings.selectBoxClass).show();
							p.find('.' + settings.selectNoneClass + 
									',.' + settings.addSelectedClass + 
									',.' + settings.keepSelectedClass).toggleClass('disabled', typeof sc === UNDEF || sc === 0);
									
							p.find('.' + settings.selectAllClass).toggleClass('disabled', typeof mc !== UNDEF && sc === mc);
						}
					},
					
				// Toggle boxes according to item count
				toggleBoxes = function(mc) {
					
						if (mc) {
							shopBox.add(feedbackBox).show();
						} else {
							shopBox.add(feedbackBox).hide();
						}
					},
						
				/********************************************************
				 *
				 *					Managing overlays
				 *				e.g. after search or filter
				 *
				 ********************************************************/
				/*
					<div id="main-cont" class="large-9 medium-8 small-12 columns has-overlay"> <!-- fixed part -->
						<article class="cont base-overlay" >
							<section class="row folder-cont">
								<div class="folders"></div>
							</section>
							<section class="row ${thumbContClass}">
								<div class="cont $captionClasses $layoutClass">
								</div>
							</section>
						</article>
						<article class="cont overlay" id="${namespace}" data-oly_ns="${namespace}"> <!-- dynamically generated -->
							<header class="row">
								<div class="cont icon-${icon}">
									<h4 class="${overlayTitleClass}"> Title</h4>
									<a class="close" href>${return}</a>
								</div>
							</header>
							<section class="row ${thumbContClass}">
								<div class="cont $captionClasses $layoutClass">
								</div>
							</section>
						</article>
					</div>
				*/
				
				OLYTAG = 'article',
				
				// Creating header

				createOverlayHeader = function(o, options) {
						var hc 	= $('<div>', {
										'class':	settings.contentClass
									});
						
						if (options) {
							// Icon
							if (options.hasOwnProperty('icon')) {
								hc.addClass(options.icon);
							}
							
							// Title
							if (options.hasOwnProperty('title')) {
								hc.append($('<h' + settings.overlayHeadLevel + '>', {
										'class':	settings.overlayTitleClass,
										html: 		' ' + options.title
									}));
							}			
						}
						
						// Close button
						hc.append($('<a>', {
								'class': 	'close',
								href: 		'',
								text: 		text.return
							}).on('click', function() {
								removeOverlay();
								return false;
							}));
						
						// Adding as first element to overlay
						o.prepend($('<header>').append(hc));
					},
					
				// Creating an empty thumbnail container for overlays

				createThumbContainer = function(o) {
						var tc = o.children('section.' + settings.thumbContClass);
						
						if (!tc.length) {
							tc = $('<section>', {
									'class':	[ settings.rowClass, settings.thumbContClass ].join(' ')
								})
								.append($('<div>', {
									'class':	[ settings.thumbsClass, settings.thumbLayout, settings.captionClasses ].join(' ')
								}))
								.appendTo(o);
						}
						
						return tc;
					},
					
				// Adding new overlay
				
				addOverlay = function(options) {
						
						// Removing previous overlay 
						var	ns = 'oly_' + Math.floor(Math.random() * 10000),			// Namespace
							bo = getBaseOverlay(),										// Base overlay
							o = $('<' + OLYTAG + '>', {
									'class': 	settings.overlayClass
								}).attr('data-oly_ns', ns);
						
						// Class to signal the page has overlay 
						main.addClass(settings.hasOverlayClass);
						
						// Removing existing overlay(s) besides base
						removeExtraOverlays();
						
						if (bo.length) {
							// Hide Base
							bo.hideAllTooltips();
							bo.hide();
							// ... and its attached elements
							$('[rel=' + bo.data('oly_ns') + ']').hide();
							o = o.insertAfter(bo);
						} else {
							// No base overlay
							o = o.appendTo(mainCont);
						}
												
						// Save namespace ID
						o[0]['overlay'] = ns;
						
						// Creating overlay header
						if (typeof options !== UNDEF) {
							createOverlayHeader(o, options);
							if (options.hasOwnProperty('className')) {
								o.addClass(options['className']);
							}
						}
						
						// Ensure thumb container exists
						createThumbContainer(o);
						
						// Call "added" action if present
						main.trigger('overlayAdded', o);
						
						return o;
					},
					
				// Getting base overlay
				
				getBaseOverlay = function() {
				
						return mainCont.find(OLYTAG + '.' + settings.baseClass).eq(0);
					},
					
				// Getting extra overlays
				
				getExtraOverlays = function(visible) {
						
						return visible?
							mainCont.find(OLYTAG + '.' + settings.overlayClass + ':visible')
							:
							mainCont.find(OLYTAG + '.' + settings.overlayClass);
					},
										
				// Getting all overlays; base included
				
				getAllOverlays = function(visible) {
						return visible?
							mainCont.find(OLYTAG + '.' + settings.baseClass + ':visible, ' + OLYTAG + '.' + settings.overlayClass + ':visible')
							:
							mainCont.find(OLYTAG + '.' + settings.baseClass + ', ' + OLYTAG + '.' + settings.overlayClass);
					},
					
				// Getting active overlay; creating one if missing
				
				getOverlay = function() {
						var o = getAllOverlays(true).last();		// The latest added visible overlay
							
						if (!o.length) {
							o = addOverlay();
							createThumbContainer(o);
						}
						
						// Name space ID
						if (!o.data('oly_ns')) {
							var ns = 'oly_' +  + Math.floor(Math.random() * 10000);
							o.attr('data-oly_ns', ns);
						}
						
						return o;
					},
					
				// Get overlay Namespace
				
				getOverlayNS = function() {
						return getOverlay().data('oly_ns');
					},
			
				// Attaching related boxes to be removed with the overlay
				
				attachToOverlay = function(el) {
						var ns = getOverlayNS();
						
						if (typeof ns !== UNDEF) {
							el.each(function() {
									$(this).attr('rel', ns);
								});
						}
					},
					
				// Removing all extra overlays
				
				removeExtraOverlays = function() {
						var removeOly = function(o) {
									var ns = o.data('oly_ns');
									
									if (typeof ns !== UNDEF) {
										// Event handlers
										$window.off('.' + ns);
										// Attached elements
										$('[rel=' + ns + ']').remove();
									}
									
									o.trigger('lightboxRemove');
									o.remove();
								};
								
						getExtraOverlays().each(function() {
								removeOly($(this));
							});
					},
					
				// Removing currrent overlay; showing base
				
				removeOverlay = function() {
						var bo = getBaseOverlay();
						
						// Class to signal the page has no overlay 
						main.removeClass(settings.hasOverlayClass);
						
						// Removing existing overlay(s) besides base
						removeExtraOverlays();
						
						if (bo.length) {
							// Showing base overlay
							bo.show();
							// Attached elements
							$('[rel=' + bo.data('oly_ns') + ']').show();
							// Refreshing thumbnails
							bo.find('.' + settings.thumbsClass).trigger('refresh');
							
							// restore original URL
							removeParam(settings.indexName);
								
							//main.trigger('overlayRemoved', main.children('section.' + settings.thumbContClass + ':visible').eq(0));
							updateShares();
							toggleSelectButtons(0, getImages().filter('.' + settings.selectableClass).length); 						
						}
					},
				
				// Getting folders
				
				getFolders = function(o) {
					
						if (typeof o === UNDEF) {
							return getOverlay().find('.' + settings.folderClass);
						} else if (typeof o == 'string') {
							return getOverlay().find('.' + settings.folderClass + o);
						} else if (o.length) {
							return o.find('.' + settings.folderClass);
						}
						return $();
					},
													
				// Getting folders
				
				getFolderCont = function(o) {
					
						if (typeof o === UNDEF) {
							return getOverlay().find('.' + settings.folderContClass);
						} else if (typeof o == 'string') {
							return getOverlay().find('.' + settings.folderContClass + ' ' + o);
						} else if (o.length) {
							return o.find('.' + settings.folderContClass);
						}
						return $();
					},
										
				// Getting current thumbs array :: optionally selected subelements
				
				getImages = function(selector) {
						return getOverlay().find('section.' + settings.thumbContClass + ':visible .' + settings.thumbsClass + 
								(selector? (' ' + selector) : ''));
					},
				
				// Getting current thumbs array :: optionally selected subelements
				
				getBaseImages = function(selector) {
						return getBaseOverlay().find('section.' + settings.thumbContClass + ' .' + settings.thumbsClass + 
								(selector? (' ' + selector) : ''));
					},
				
				// Getting all the cards of the overlay (image + caption)
				
				getCards = function(o) {
							
						if (typeof o === UNDEF || !o.length) {
							o = getOverlay();
						}
						
						return o.find('section.' + settings.thumbContClass + ':visible .' + settings.cardClass);
					},
					
				// Getting attached lightbox
				
				getLightbox = function(o) {
					
						return (o || getOverlay()).find('.' + settings.lightboxClass);
					},
					
				// Loading an image in lightbox
				
				lightboxLoad = function(card, auto) {
					
						getOverlay().trigger('lightboxLoad', (typeof auto === UNDEF)? card : [ card, auto ]);
					},
					
				// Leaving lightbox
				
				lightboxQuit = function() {
					
						getOverlay().trigger('lightboxQuit');
					},
					
				// Getting thumbnail by file name
				
				getCardByName = function(name, base) {
						var imgs,
							t = $();
						
						imgs = (typeof base === UNDEF || !base)?
							getImages('.' + settings.thumbClass)
							:
							getBaseImages('.' + settings.thumbClass);
						
						name = transcodeJavaURIComponent(name);
						
						imgs.each(function() {
								// Link as data
								var l = $(this).data('href');
								
								if (l) {
									// Data
									l = l.substring(l.lastIndexOf('/') + 1);
								} else if ($(this).attr('href')) {
									// href attr
									l = $(this).attr('href').replace('#img=', '');
								}
								
								if ((l === name) || ((l.slice(-pageExt.length) === pageExt) && (l.split('.')[0] === name.split('.')[0]))) {
									t = $(this).parent();
									return false;
								}
							});
						
						return t;
					},
					
				// Getting selected items from current overlay
				
				getSelectedItems = function() {
					
						var r = [];
								
						getImages('.' + settings.thumbClass + '.' + settings.checkedClass).each(function() {
								r.push($(this).closest('.' + settings.cardClass).data(J.OBJ));
							});
						
						return r;
					},
				
				// Getting all items from current overlay
				
				getAllItems = function() {
					
						var r = [];
								
						getImages('.' + settings.thumbClass).each(function() {
								r.push($(this).closest('.' + settings.cardClass).data(J.OBJ));
							});
						
						return r;
					},
					
				// Getting all items from base overlay
				
				getBaseItems = function() {
					
						var r = [];
						
						getBaseOverlay().find('.' + settings.thumbsClass + ' .' + settings.thumbClass).each(function() {
								r.push($(this).closest('.' + settings.cardClass).data(J.OBJ));
							});
						
						return r;
					},
					
				// Adding one folder's thumb
				
				loadFolderThumb = function(thumb) {
						
						thumb.find('a[data-thumb]').each(function() {
							var t = $(this);
							if (!t.children('img').length) {
								t.append($('<img>', {
										alt: 		t.data('alt'),
										'class': 	settings.hideImageClass
									})
									// Onload action
									.one('load', function() {
										thumb.removeClass(settings.lazyloadClass);
										$(this).addClass(settings.showImageClass).removeClass(settings.hideImageClass);
									})
									.attr('src', t.data('thumb')));
							}
						});
					},
				
				// Loading visible folder thumbs
				
				lazyloadFolders = function(doneFn) {
						
						if (lazyloadFoldersInProgress) {
							clearTimeout(lazyloadFoldersTimeout);
							return;
						}
												
						if (!getBaseOverlay().is(':visible')) {
							// We're on an overlay
							return;
						}
						
						lazyloadFoldersInProgress = true;
						
						var folders = getFolders('.' + settings.lazyloadClass);
						
						if (folders.length) {
							var wh = $window.height(),
								scrollTop = $window.scrollTop();
						
							folders.each(function() {
								var t = $(this),
									tt = t.offset().top,
									th = t.height();
									
								if ((tt < scrollTop + 2 * wh) && (tt + th) > (scrollTop - wh)) {
									loadFolderThumb(t);
								}
								
								if (tt >= scrollTop + 2 * wh) {
									// Below fold
									return false;
								}
							});
							
							clearTimeout(lazyloadFoldersTimeout);
							lazyloadFoldersTimeout = setTimeout(lazyloadFolders, settings.scrollCheckInterval);
						
						} else {
							$window.off('.lazy');
						}
						
						lazyloadFoldersInProgress = false;
						
						if (typeof doneFn === FUNCTION) {
							doneFn.call(this);
						}
						
					},
		
				/********************************************************
				 *
				 *						Folder tree
				 *
				 ********************************************************/
				
				getFolderTree = function() {
					
						var getFolderObjects = function(folder) {
								if (folder.hasOwnProperty(J.FOLDERS) && folder[J.FOLDERS].length) {
									var ul, li, i, p, f = folder[J.FOLDERS];
									ul = $('<ul>');
									for (i = 0; i < f.length; i++) {
										p = album.getPath(f[i]);
										li = $('<li>').appendTo(ul);
										li.append($('<a>', {
												href: 	(p? (p + '/') : '') + settings.indexName,
												html: 	f[i][J.TITLE] || f[i][J.NAME] || ''
											})).append(getFolderObjects(f[i]));
										if (!p) {
											li.addClass('actual');
										}
									}
									return ul;
								}
								return $();
							},
							
							getHomeLink = function() {
								var p = settings.rootPath;
								return $('<a>', {
										'class': 	'home-link icon-home',
										href:		(p? (p + '/') : '') + settings.indexName,
										text:		' ' + album.getAlbumTitle()
									});
							};
							
						if (settings.folderTree.type === 'tree') {
							if (settings.level > 0) {
								return getHomeLink().add(getFolderObjects(album.getTree()));
							} else {
								return getFolderObjects(album.getTree());
							}
						} else {
							getFolderObjects(album.getCurrentFolder());
						}
					};
								
				/********************************************************
				 *
				 *					Image overlay rendering 
				 *			or initializing images in the base overlay
				 *
				 ********************************************************/
				/*
					<article class="overlay">
						<section class="thumb-cont">
							<div class="thumbnails">
								<div class="card lazyload image hascaption">
									<a class="thumb">
										<img src="" alt="" class="hide-image">
									</a>
									<div class="caption">
										<h6>Title</h6>
										<div class="comment">Comment</div>
									</div>
								</div>
							</div>
						</section>
					</article>
				*/
				
				$.fn.renderImages = function(items, complete, baseSet) {
					
					if (typeof items === UNDEF || !items.length || !Array.isArray(items)) {
						return this;
					}
					
					var overlay				= $(this).eq(0),
						ns 					= overlay.data('oly_ns'),								// Namespace
						thumbContainer		= $(),													// thumbnail container (for overlays)
						thumbnails			= $(),													// thumbnails wrap div
						cols 				= 0,													// Columns
						scrollTop 			= $window.scrollTop(),									// Scroll timeout
						lazyloadTimeout 	= null,													// Lazy load timeout
						
						// Rendering images
						
						initImages = function() {
								var cards = getCards(), 
									card,
									image,
									img,
									isLbable,
									isMedia,
									caption,
									s,
									d,
									cd = settings.hasOwnProperty('markNew')? (nows - settings.markNew.days * ONEDAY_S) : null,
									item;
								
								// Creating new elements for the items with no thumbs
								for (var i = 0; i < items.length; i++) {
									
									item = items[i];
									
									if (i < cards.length) {
										// Already exists in the HTML source
										card = cards.eq(i);
									} else {
										// Not yet exists in the HTML source: happens only on overlays
										card = $('<div>', {
												'class': 	[ settings.cardClass, settings.lazyloadClass, item[J.CATEGORY] ].join(' ')
											})
											.appendTo(thumbnails);
											
									}
									
									// Finding/creating image's <a> wrap
									image = card.children('.' + settings.thumbClass);
									if (!image.length) {
										image = $('<a>', {
												'class':	settings.thumbClass
											})
											.data('href', album.getItemPath(item))
											.appendTo(card);
									}
									
									// Data pointer
									card.data(J.OBJ, item);
									
									isLbable = album.isLightboxable(item);
									isMedia = item[J.CATEGORY] === 'video' && item[J.CATEGORY] === 'audio';
											
									// Creating structure
									if (isLbable) {
										
										card.addClass(settings.lbableClass);
										
										if (isMedia) {
											// Video or audio
										} else {
											
											// Finding/creating image
											img = image.find('img').eq(0);
											if (!img.length) {
												img = $('<img>', {
														alt:		album.getAlt(item)
													}).appendTo(image);
											}
										
											// Onload event
											img.addClass(settings.hideImageClass)
												.on('load.' + ns, function() {
														$(this).addClass(settings.showImageClass)
															.removeClass(settings.hideImageClass)
															.parents('.' + settings.cardClass).removeClass(settings.preloadClass);
													});
												
											// Cached image?
											if (img[0].complete) {
												img.trigger('load');
											}
											
											// No source yet?
											/*
											if (!img.attr('src')) {
												img.attr('src', album.getThumbPath(card.data(J.OBJ)));
											}
											*/
											// Adding width, height
											img.attr({
													width:		item[J.THUMB][J.WIDTH],
													height:		item[J.THUMB][J.HEIGHT]
												});
											
											// Adding extra classes
											if ((item[J.THUMB][J.HEIGHT] / item[J.THUMB][J.WIDTH]) < 0.5) {
												card.addClass(settings.panoClass);
											}
											
											// Masonry: adding width
											if (settings.thumbLayout === 'horizontal') {
												card.css({
														width:		Math.round(item[J.THUMB][J.WIDTH] *.9)
													});
											}
											
											// Icon thuumbnail?
											if (item[J.THUMB][J.PATH].match(/res\/\w+\.png$/)) {
												image.addClass('iconthumb');
											}
											
											// CLick event
											image.on('click.' + ns, imageClicked);
										}
									}
									
									// Caption
									if (s = item[J.THUMBCAPTION]) {
										
										caption = card.children('.' + settings.captionClass);
										
										if (!caption.length) {
											// No caption yet
											caption = $('<div>', {
													'class': 	settings.captionClass,
													html: 		s
												}).appendTo(card);
										}
										
										card.addClass(settings.hascaptionClass);
									}
											
									// Preparing for Lightbox
									if (isLbable) {
										
										if (shopRoot.length && album.hasShop(item) || feedbackRoot.length && settings.feedback) {
											// Select box
											image.addClass(settings.selectableClass);
											
											if (shopRoot.length) {
												d = $('<a>', {
															'class': 	'icon-shopping-cart add-cart',
															text: 		' ' + text.addCart
														})
													.on('click.' + ns, function(e) {
															var card = $(e.target).closest('.' + settings.cardClass);
															if (card.length) {
																shopRoot.trigger('addItems', card.data(J.OBJ));
															}
															return false;
														})
													.appendTo(image);
													
												d.addTooltip(album.getPriceRange(item));
											}
										}
										
										if (!baseSet && settings.linkToFolder) {
											// Folder link
											var p = album.getPath(item);
											
											if (p) {
												image.append($('<a>', {
														'class': 	'icon-folder folder-link',
														href:		 p + settings.indexName
													}));
											}
										}
										
										if (baseSet) {
											image.attr('href', '#img=' + item[J.PATH]);
										}
										
									} else {
										
										card.addClass(item[J.CATEGORY]);
										image.attr('href', album.getItemPath(item));
									}
									
									// Mark new images
									if (cd != null) {
										
										d = item[J.DATES];
										
										if (d && (d = d[settings.markNew.reference]) && d >= cd) {
											s = settings.markNew.text?
												$('<span>', {
														'class':	'new-image',
														text:		settings.markNew.text
													})
												:
												$('<span>', {
													'class':	'icon-new-fill new-image'
													});
											
											image.append(s);
												
											if (!TOUCHENABLED) {
												s.data('tooltip', (new Date(d * 1000)).toLocaleDateString(LOCALE)).addTooltip();
											}
										}
									}
									
									// Rating
									if (settings.useRating && isLbable) {
										
										var r = item[J.RATING] || 0;
										
										if (settings.visitorRating || r) {
											
											s = $('<span>', {
													'class':	'rating',
													html:		STARS
												})
												.appendTo(image);
												
											if (settings.visitorRating) {
												
												if (item.hasOwnProperty(J.VISITORRATING)) {
													// Already exists
													r = item[J.VISITORRATING];
												} else {
													if (!settings.useJalbumRating) {
														// Ignore jAlbum rating
														r = 0;
													}
													// Save the default
													item[J.VISITORRATING] = r;
												}
												
												s.addClass('r' + r + (settings.useJalbumRating? '' : ' icon-close'))
													.on('click.' + ns, function(e) {
															var el = $(this),
																r = Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left - parseFloat(el.css('paddingLeft'))) / 16), 5),
																item = el.closest('.' + settings.cardClass).data(J.OBJ); 
																//Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left + 4) / el.width()) * 5, 5);
																
															el.removeClass('r0 r1 r2 r3 r4 r5').addClass('r' + r);
															item[J.VISITORRATING] = r;
															
															if (!baseSet) {
																// Propagate to base overlay
																el = getCardByName(item[J.NAME], true);
																el.find('.rating')
																	.removeClass('r0 r1 r2 r3 r4 r5')
																	.addClass('r' + r);
															}
															
															return false;
														});
											} else {
												// Unalterable rating from jAlbum
												s.addClass('r' + r);
											}
										}
									}
										
									// Prevent copy
									if (settings.rightClickProtect) {
										image.on('contextmenu', function(e) {
												e.preventDefault()
												return false;
											});
									}
								}
								
								lazyloadTimeout = setTimeout(function() {
										
										lazyloadThumbs();
										
										if (settings.thumbLayout === 'vertical') {
											// Fixing vertical layout
											setTimeout(fixThumbs, 100, true);
										}
										
									}, settings.scrollCheckInterval / 2);

							},
							
										
						// Fixing vertical tiles
						
						fixTimeout = null,
						
						fixThumbs = function(init) {
								var sw = thumbContainer.width();
								
								// It's already fixed for this width?
								if (thumbnails.data('fixedfor') === sw) {
									return;
								}
								
								var thumbs = thumbnails.children('.' + settings.cardClass),
									firstThumb = thumbs.eq(0),
									left = [],
									top = [],
									th = [],
									ch = [],
									ex = [],
									bottom = 0,
									captionBelow = thumbnails.hasClass('below'),
									tw = firstThumb.width(),
									fixed = !!thumbnails.data('fixedfor'),
									belowFold = $body.scrollTop() + window.outerHeight,
									
									// Get the card's extra height
									getThumbMargin = function(thumb) {
											var props = ['marginBottom', 'paddingTop', 'paddingBottom', 'borderTopWidth', 'borderBottomWidth'],
												v,
												h = 0;
												
											for (var i = 0; i < props.length; i++) {
												v = getStyle(thumb, props[i]);
												if (v.endsWith('px')) {
													h += parseFloat(v);
												} else if (v.endsWith('em')) {
													h += parseFloat(v) * 16;
												} else if (v.endsWith('%')) {
													h += parseFloat(v) * sw / 100;
												}
											}
											
											return Math.round(h);
										},
													
									// Get thumbnail columns
									getCols = function() {
											var el = $('<div>', {
														'class':		settings.cardClass,
														visibility: 	'hidden',
														height: 		0
													}).appendTo(thumbnails),
												cols = Math.round(0.05 + thumbnails.width() / el.outerWidth(true)); 
												
											el.remove();
											
											return Math.max(1, cols);
										},
										
									// Top column
									getTopCol = function() {
											var tc = 0;
												
											for (var min = top[0], i = 1; i < top.length; i++) {
												if (top[i] < min) {
													min = top[i];
													tc = i;
												}
											}
											
											return tc;
										},
										
									// Setting 'auto' height
									prepare = function() {
										
											if (fixed) {
												thumbs.each(function(i) {
														this.children[0].style.height = 'auto';
													});
											}
											
											// Fixing container height so it won't repaint when making layers absolute pos
											thumbnails.css({
													height:			thumbnails.outerHeight()
												});
								
											requestAnimationFrame(collectData);
										},
									
									// Collecting thumb and caption heights 
									collectData = function() {
										
											var	data;
												
											thumbs.each(function(i) {
													data = $(this).data(J.OBJ);
													if (data) {
														th[i] = (data[J.THUMB][J.WIDTH] <= tw)?
																	// too small: fits height
																	data[J.THUMB][J.HEIGHT]
																	:
																	// fits both
																	(Math.floor(tw * data[J.THUMB][J.HEIGHT] / data[J.THUMB][J.WIDTH]) - 1);
													}
													
													if (captionBelow) {
														ex[i] = (this.childElementCount > 1)? this.children[1].scrollHeight : 0;
													}
														
												});
		
											requestAnimationFrame(setPositions);		
										},
											
									// Setting position: auto, left and top
									setPositions = function() {
											var	c,
												mr = parseFloat(getStyle(firstThumb[0], 'marginRight')),
												cw = firstThumb.outerWidth(true) * 100 / (sw + mr),
												mb = getThumbMargin(thumbs[0]),
												child;
												
											thumbs.each(function(i) {
													
													if (i < cols) {
														c = i;
														left[c] = i * cw;
														top[i] = 0;
													} else {
														c = getTopCol();
													}
														
													this.setAttribute('style', 'opacity:0; position:absolute; left:' + left[c] + '%; top:' + top[c] + 'px;');
													child = this.childNodes;
													child[0].style.height = th[i] + 'px';
													top[c] += (captionBelow? ex[i] : 0) + th[i] + mb;
													//console.log(i + ': top=' + top[c] + ' ext=' + ex + ' th=' + th[i] + ' mb=' + mb); 
													bottom = Math.max(bottom, top[c]);
												});
											
											requestAnimationFrame(finalize);
										},
										
									// Fixing container, triggering fade in
									finalize = function() {
										
											// Adjusting container height
											thumbnails.css({
													height:			bottom
												});
											
											// Remembering last fixed width
											thumbnails.data('fixedfor', sw);
											
											thumbs.each(function() {
													this.style.opacity = 1;
												});
											
											lazyloadThumbs();
										};
								
								cols = getCols();
								
								// Wait for all the first row images
								thumbs.slice(0, cols).find('img').waitAllImg(function() {
										requestAnimationFrame(prepare);
									});
								
							},
							
						// Refreshing layout e.g. after restoring the base layer
						
						refresh = function() {
								
								if (settings.thumbLayout === 'vertical') {
									fixThumbs();
								} 
								
								lazyloadThumbs();
							},
						
						// Loading visible thumbs
						
						lazyloadThumbs = function() {
							
								if (!thumbContainer.is(':visible')) {
									return;
								}
								
								lazyloadTimeout = clearTimeout(lazyloadTimeout);
									
								var st = $window.scrollTop();
								
								if (Math.abs(st - scrollTop) > 20) {
									// Scrolling in progress
									scrollTop = st;
									lazyloadTimeout = setTimeout(lazyloadThumbs, settings.scrollCheckInterval);
									return;
								}
								
								// Lazyload images
								var thumbs = getImages('.' + settings.lazyloadClass),
									wh = $window.height();
								
								thumbs.not(':empty').each(function() {
									var t = $(this),
										tt = t.offset().top,
										th = t.height();
										
									if ((tt + th) > (scrollTop - wh)) {
										
										if (tt < (scrollTop + 2 * wh)) {
											// 2 screens ahead
											loadThumb(t);
										} else {
											return false;
										}
									}
								});
								
								if (thumbs.length) {
									// perpetrating lazyload
									scrollTop = st;
									lazyloadTimeout = setTimeout(lazyloadThumbs, settings.scrollCheckInterval);
									
								} else {
									// No more left
									$window.off('scroll.' + ns);
								}
							},
						
						// Clicked image
						
						imageClicked = function(e) {
								if (e.target.nodeName === 'A' && ($(e.target).closest('.' + settings.commentClass).length || $(e.target).hasClass('folder-link'))) {
									// Link in comment or link to folder
									return true;
								}
								lightboxLoad($(e.target).closest('.' + settings.cardClass));
								return false;
							},
						
						// Adding one thumb
						
						loadThumb = function(card) {
								
								if (card.length) {
									var thumb = card.children('.' + settings.thumbClass).eq(0),
										img = thumb.children('img');
										
									if (!img.length) {
										thumb.append($('<img>', {
												'class': 	settings.hideImageClass
											}));
									}
									
									if (typeof img.attr('src') !== UNDEF && img.attr('src') !== false) {
										// IMG already has source
										thumb.addClass(settings.showImageClass).removeClass(settings.hideImageClass);
									} else {
										// Providing source and onload action
										img.one('load', function() {
											// Showing image on load
											$(this).addClass(settings.showImageClass).removeClass(settings.hideImageClass);
											//card.removeClass(settings.lazyloadClass);
										})
										.attr('src', album.getThumbPath(card.data(J.OBJ)));
									}
									
									card.removeClass(settings.lazyloadClass);
								}
													
							},
				
						// Marking active card
						
						setActiveCard = function(card) {
							
								if (card) {
									var t = 			$(card),
										item =			t.data(J.OBJ),
										wh = 			$window.height(),
										scrollTop = 	$window.scrollTop(),
										tt = 			t.offset().top,
										th = 			t.outerHeight();
									
									getCards().removeClass(settings.activeClass);
									t.addClass(settings.activeClass);
									
									if (album.hasLocation(item)) {
										mapRoot.trigger('setActive', item[J.NAME]);
									} else {
										mapRoot.trigger('resetMarkers');
									}
									
									// Moving into view
									if (tt + th > scrollTop + wh) {
										$('html, body').stop(true, false).animate({
												scrollTop: 		Math.round(tt - wh + th + 30) + 'px'
											});
									} else if (tt < scrollTop) {
										$('html, body').stop(true, false).animate({
												scrollTop: 		Math.round(Math.max(tt - 30, 0)) + 'px'
											});
									}
								}
							},
							
						// Refreshing the rating
						
						refreshRating = function(e, card, rating) {
							
								if (card) {
									var t = $(card),
										o = t.data(J.OBJ),
										r = ((typeof rating !== UNDEF)? rating : (settings.visitorRating? o[J.VISITORRATING] : o[J.RATING])) || 0;
										
									t.find('.rating')
										.removeClass('r0 r1 r2 r3 r4 r5')
										.addClass('r' + r);
										
									if (!baseSet) {
										var el = getCardByName(o[J.NAME], true);
										el.find('.rating')
											.removeClass('r0 r1 r2 r3 r4 r5')
											.addClass('r' + r);
									}
								}
								
							};
					
					// Creating new Unique ID if not yet exists
					if (!ns) {
						overlay.data('oly_ns', ns = 'oly_' + Math.floor(Math.random() * 10000));
					}
						
					thumbContainer = overlay.find('.' + settings.thumbContClass).eq(0);

					if (!thumbContainer.length) {
						thumbContainer = createThumbContainer(overlay);
					}
					
					thumbnails = thumbContainer.children(':first-child');
					
					thumbnails.after($('<div>', {
							'class': 	'progressbar'
						}));
						
					if (items.length > 2 && settings.thumbLayout === 'horizontal') {
						thumbnails.addClass('wrap');
					}
						
					initImages();
					
					// Externally triggered events
					
					if (settings.thumbLayout === 'vertical') {
						// Fixing vertical layout on screen change
						$window.on('resize.' + ns + ' orientationchange.' + ns, function(e) {
								clearTimeout(fixTimeout);
								fixTimeout = setTimeout(fixThumbs, 500);
								return true;
							});
					}
								
					// Moving the active element into view
					overlay.on('setactivecard', function(e, card) {
							setActiveCard(card);
						});
					
					// External trigger for fixing thumbnails and lazyload
					overlay.on('refresh', refresh);
					
					// Refreshing the rating
					overlay.on('refreshRating', refreshRating);
					
					// Done function
					if (typeof complete === FUNCTION) {
						if (typeof baseSet !== UNDEF) {
							complete.call(undefined, overlay, baseSet);
						} else {
							complete.call(undefined, overlay);
						}
					}
					
					overlay.find('.progressbar').remove();
					
					return this;
				};
		
		
			var	
				/********************************************************
				 *
				 *					Preparing images
				 *
				 ********************************************************/
					
				prepareImages = function(overlay, baseSet) {
					
						if (!album) {
							return;
						}
						
						var cards = getCards(overlay);
							
						if (cards.length > 1) {
							
							// Filtering and sort only if there are at least 2 items
							
							if ($('.filter-cont').length) {
								// Initializing Filters / Sort
								prepareFilterBox(baseSet);
							}
						}
						
						if (!cards.filter('.' + settings.lbableClass).length) {
							
							$('.' + settings.startshowClass).fadeOut();
							
						} else {
							
							if (album.hasShop() || settings.feedback) {
								
								var ci = cards.find('.' + settings.selectableClass);
								
								toggleSelectButtons(0, ci.length);
								
								if (ci.length) {
									// Make items selectable
									overlay.selectable({
											cardClass:			settings.selectableClass,
											checkedClass:		settings.checkedClass,
											selectionChange:	function() {
																		toggleSelectButtons(this, ci.length); 
																		shopRoot.add(feedbackRoot).trigger('itemsSelected', this);
																	}
										});
									
									
									shopBox.add(feedbackBox).find('.' + settings.keepSelectedClass).on('click.' + ns, function() {
											keepSelectedItems(text.selectedItems, function() {
													removeParam();
													updateShares();
												});
											
											return false;
										});
									
								}
							}
						
							/* Waiting for Derek's answer
							if (!LOCAL && typeof FOTOMOTO !== UNDEF && settings.fotomoto) {
								FOTOMOTO.API.checkinImages(getFotomotoImages(cards), function(images) {
										
									});
							}
							*/
							
							overlay.trigger('lightboxRemove');
							
							overlay.lightbox(album, {
									
									baseSet:		(typeof baseSet === UNDEF)? false : baseSet,
									
									onLoadStart:	function(card) {
															var item = card.data(J.OBJ);
															if (item) {
																addParam(settings.pageName || settings.indexName, {
																		img: 	album.getItemName(item)
																	}, item[J.TITLE]);
															}						
														},
														
									onLoadEnd: 		function(card) {
															overlay.trigger('setactivecard', card);
														},
													
									onClose:		function() {
															removeParam(settings.pageName || settings.indexName, 'img');
														},
													
									onReady:		function() {
															if (initByHash && window.location.hash) {
																initByHash = false;
																var param = readParam();
																
																if (param.hasOwnProperty('label')) {
																	// Label filter
																	getItemsByLabel(param.label, function() {
																		addParam(settings.indexName, {
																				label:			param.label,
																				search:			null,
																				newimages:		null,
																				date:			null
																			}, text.label + ': ' + param.label);
																	});
																} else if (param.hasOwnProperty('search')) {
																	// Search
																	getItemsBySearch(param.search, function() {
																		addParam(settings.indexName, {
																				search:			param.search,
																				label:			null,
																				newimages: 		null,
																				date:			null
																			}, text.search + ': ' + param.search);
																	});
																} else if (param.hasOwnProperty('newimages')) {
																	// Search new images
																	getItemsByDate(getSearchNewOptions(param.newimages), function() {
																		addParam(settings.indexName, {
																				newimages:		param.newimages,
																				search:			null,
																				label:			null,
																				date:			null
																			}, text.searchNew + ': ' + param.newimages);
																	});
																} else if (param.hasOwnProperty('date')) {
																	var options = getDateRangeOptions(param.date);
																	// Search images in date range
																	getItemsByDate(options, function() {
																		addParam(settings.indexName, {
																				date:			param.date,
																				newimages:		null,
																				search:			null,
																				label:			null
																			}, getDateReference(options.reference) + ': ' + param.date);
																	});
																} else if (param.hasOwnProperty('img')) {
																	var card = getCardByName(param.img);
																	if (card.length) {
																		if (param.hasOwnProperty('slideshow')) {
																			lightboxLoad(card, 2000);
																		} else {
																			lightboxLoad(card);
																		}
																	} else {
																		removeParam(settings.pageName || settings.indexName, 'img');
																	}
																} else if (param.hasOwnProperty('slideshow')) {
																	lightboxLoad(0, 2000);
																}
															}
															
															$('.' + settings.startshowClass).fadeIn().on('click', function() {
																	$(this).trigger('removeTooltip');
																	startAuto();
																	return false;
																});
														},
													
									onSlideshowStart:
													function() {
															if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
																$(settings.backgroundMusic.hook).trigger('startPlayer');
															}
														},
													
									onSlideshowPause:
													function() {
															if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
																$(settings.backgroundMusic.hook).trigger('pausePlayer');
															}
														}
								});
						}
					},
			
				// Hash change listener :: when the browser's back / forward buttons were pressed
				
				stateChange = function() {
						if (window.location.hash === lastHash) {
							return;
						}
						
						lastHash = window.location.hash;
						var param = readParam();
						
						if (param.hasOwnProperty('label')) {
							// Label filter
							getItemsByLabel(param.label, function() {
									setParam(settings.indexName, {
											label:			param.label
										}, text.label + ': ' + param.label);
									updateShares();
								});
						} else if (param.hasOwnProperty('search')) {
							// Search
							getItemsBySearch(param.search, function() {
									setParam(settings.indexName, {
											search:			param.search
										}, text.search + ': ' + param.search);
									updateShares();
								});
						} else if (param.hasOwnProperty('newimages')) {
							// New images
							getItemsByDate(getSearchNewOptions(param.newimages), function() {
									setParam(settings.indexName, {
											newimages: 	param.newimages
										}, text.newImages + ': ' + param.newimages);
									updateShares();	
								});
						} else if (param.hasOwnProperty('date')) {
							// Date range
							var options = getDateRangeOptions(param.date);
							getItemsByDate(options, function() {
									setParam(settings.indexName, {
											date: 		param.date
										}, getDateReference(options.reference) + ': ' + param.date);
									updateShares();	
								});
						} else if (param.hasOwnProperty('img')) {
							// Image (no search or label)
							var card = getCardByName(param.img);
							if (card.length) {
								lightboxLoad(card);
							} else {
								removeParam(settings.indexName, 'img');
							}
						} else {
							// Plain index page
							lightboxQuit()
							removeOverlay();
						}
						
					},
					
				// Starts slideshow
				
				startAuto = function(e) {
						var card = $();
						
						if (!settings.restartSlideshow) {
							card = getCards().filter('.' + settings.activeClass);
						}
						
						if (!card.length) {
							// Fall back to first
							card = getCards().eq(0);
						}
					
						lightboxLoad(card, 2000);
						
						return false;
					},
					
				/********************************************************
				 *
				 *		Functionalities that results in new overlay
				 *
				 ********************************************************/
				
				/********************************************************
				 *
				 *				Keeping only selected items
				 *
				 ********************************************************/
			
				keepSelectedItems = function(tx, ready) {
						var items = getSelectedItems();
						
						if (!items.length || items.length === getImages('.' + settings.lbableClass).length) {
							return;
						}
						
						var t = addOverlay({
									className:	'selected-items',
									icon:		'icon-checkbox-checked',
									title:		'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						setTimeout(function() {
								h.find('.progressbar').remove();
								h.append($('<small>', {
										text: 		text.foundNTimes.replace('{0}', items.length)
									}));
								t.renderImages(items, prepareImages);
								toggleSelectButtons(0, items.length);
								toggleBoxes(items.length);
								if (typeof ready === FUNCTION) {
									ready.call();
								}
							}, 50);
					},
					
				/********************************************************
				 *
				 *					Search and tag search
				 *
				 ********************************************************/
				 
				// Executing Search
				
				getItemsBySearch = function(tx, ready) {
						var t = addOverlay({
									className:	'search-results',
									icon:		'icon-search',
									title:		'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						setTimeout(function() {
							album.collectItems($.extend(settings.search, {
								exact:		false,
								terms:		tx,
								ready:		function() {
													h.find('.progressbar').remove();
													h.append($('<small>', {
															text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
													t.renderImages(this, prepareImages);
													toggleSelectButtons(0, this.length);
													toggleBoxes(this.length);
													if (typeof ready === FUNCTION) {
														ready.call();
													}
												}
							}));
						}, 50);
					},
		
				// Preparing Search functionality
				
				prepareSearch = function(f) {
					
						if (!f.length || !album) {
							return;
						}
						
						// Adding defaults if missing
						
						settings.search = $.extend({
							depth:		'tree',
							fields:		'creator,keywords,title,comment'
						}, settings.search);
						
						var startSearch = function(f) {
								
								var tx = f.find('input').eq(0).val();
								
								if (tx && tx.length > 1) {
									getItemsBySearch(tx, function() {
										setParam(settings.indexName, {
												search:		tx
											}, text.search + ': ' + tx);
										updateShares();
									});
								}
								return false;
							};
							
						f.find('.search-btn, button').on('click', function() {
							if (f.hasClass(settings.activeClass)) {
								startSearch(f);
								f.removeClass(settings.activeClass);
							} else {
								f.find('input').eq(0).focus();
								f.addClass(settings.activeClass);
							}
							return false;
						});
						
						f.on('submit', function(e) {
							e.preventDefault();
							startSearch(f);
							f.removeClass(settings.activeClass);
							return false;
						});
						
						f.find('input').on({
							'focus': 	function() { 
											f.addClass(settings.activeClass); 
										}
						});
					},
				
				// Preparing Tag Search functionality
				
				prepareTagSearch = function() {
					
						var f = $(settings.tagCloud.searchHook);
						
						if (!f.length || !album) {
							return;
						}
										
						var startSearch = function(f) {
								
								var tx = f.find('input').eq(0).val();
								
								if (tx && tx.length > 1) {
									getItemsByLabel(tx, function() {
										setParam(settings.indexName, {
												label: 	tx
											}, text.label + ': ' + tx);
										updateShares();
									});
								}
								return false;
							};
							
						f.find('.search-btn, button').on('click', function() {
							f.find('input').eq(0).focus();
							startSearch(f);
							return false;
						});
						
						f.on('submit', function(e) {
							e.preventDefault();
							startSearch(f);
							return false;
						});
						
					},
				
				/********************************************************
				 *
				 *						Tag cloud
				 *
				 ********************************************************/
				 
				// Do a label filtering
			
				getItemsByLabel = function(tx, caseSensitive, ready) {
						var t = addOverlay({
									className:		'tags-found',
									icon:			'icon-tag',
									title:			'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 'progressbar'
							}));
						
						if (t.offset().top < $window.scrollTop() || t.offset().top > ($window.scrollTop() + $window.height() - 100)) {
							$window.scrollTop(t.offset().top);
						}
						
						if (typeof caseSensitive === UNDEF) {
							caseSensitive = false;
						} else if (typeof caseSensitive === FUNCTION) {
							ready = caseSensitive;
							caseSensitive = false;
						}
						
						album.collectItems($.extend(settings.tagCloud, {
								exact:			settings.exactFields,
								caseSensitive:	caseSensitive,
								terms:			tx,
								ready: 			function() {
														h.find('.progressbar').remove();
														h.append($('<small>', {
															text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
														t.renderImages(this, prepareImages);
														toggleSelectButtons(0, this.length);
														toggleBoxes(this.length);
														if (typeof ready === FUNCTION) {
															ready.call();
														}
													}
							}));
					},
					
				// Collecting and displaying tags
			
				renderTagCloud = function(tags) {
					
						if (!tags.length) {
							return;
						}
						
						var t = $(settings.tagCloud.hook);
						
						if (!t.length) {
							return;
						}
						
						var tagClicked = function() {
								var tx = $(this).children('span').eq(0).text();
								
								if (tx && tx.length > 1) {
									getItemsByLabel(tx, true, function() {
										setParam(settings.indexName, {
												label: 		tx
											}, text.label + ': ' + tx);
										updateShares();
									});
								}
								
								return false;
							};
							
						for (var a, r, cnt, i = 0, l = tags.length; i < l; i++) {
							
							// create link element
							a = $('<a>', {
									'class': 	'tag'
								}).append($('<span>', {
									html: 		tags[i][0]
								}));
							a.on('click', tagClicked);
							
							// add count
							if (tags[i][1] > 1) {
								if (settings.tagCloud.fontVaries) {
									r = 0;
									cnt = Math.min(64, tags[i][1]);
									// calculate weight
									while (cnt > 1) {
										r++;
										cnt = cnt >> 1;
									}
									a.addClass('size-' + r);
								}
								a.append('&nbsp;').append($('<sup>', {
									text: 		tags[i][1]
								}));
							}
							
							t.append(a);
						}
						
						if (tags.length) {
							var d;
							
							if (t.outerHeight() > 175) {
								
								t.css({
									maxHeight: 150,
									overflow: 'hidden'
								});
								
								d = $('<div>', {
									'class': 'toggle-height'
								});
								
								d.append($('<a>', {
										'class': 	'icon-caret-down',
										text: 		' ' + text.more
									}).on('click', function() {
										var b = t.parent(),
											toggleHeight = function() {
												var h = t.prop('scrollHeight');
												
												b.css({
														height: 	'auto',
														overflow: 	'visible'
													});
												t.css({
														maxHeight: 	150,
													});
												setTimeout(function() {
														t.css({
																maxHeight: 	h + 20
															});
													}, 25);
												d.addClass(settings.activeClass);
											};
										
										b.css({
												height: 	b.height(),
												overflow: 	'hidden'
											});
										t.css({
												maxHeight: 	'none'
											});
										setTimeout(toggleHeight, 50);
									
								})).append($('<a>', {
										'class': 	'icon-caret-up',
										text: 		' ' + text.less
									}).on('click', function() {
										t.css({
												maxHeight: 	150
											});
										d.removeClass(settings.activeClass);
									}));
								
								t.after(d);
							}
						}
					},
				
				/********************************************************
				 *
				 *						Search new images
				 *
				 ********************************************************/
				 
				getSearchNewOptions = function(days) {
					
						return settings.hasOwnProperty('searchNew')?
							{
								reference:		settings.searchNew['reference'] || 'dateTaken',
								depth:			settings.searchNew['depth'] || 'subfolders',
								start:			nows / ONEDAY_S - days
							}
							:
							{
								reference:		'dateTaken',
								depth:			'subfolders',
								start:			nows / ONEDAY_S - days
							};
						
					},
					
				// Search for new images
				
				getDateRangeOptions = function(range) {
						var m = range.match(/^([amt])?([cts])?\:?(\d+)\-?(\d+)?/i),
							o = {
									range:		0,
									reference:	'dateTaken',
									depth:		'current'
								};
								
						if (m) {
							if (m[1]) {
								// Reference
								if (m[1] === 'a') {
									o.added = 'added';
								} else if (m[1] === 'm') {
									o.added = 'fileModified';
								} else if (m[1] === 't') {
									o.added = 'dateTaken';
								}
							}
							
							if (m[2]) {
								// Depth
								if (m[2] === 'c') {
									o.depth = 'current';
								} else if (m[2] === 't') {
									o.depth = 'tree';
								} else if (m[2] === 's') {
									o.depth = 'subfolders';
								}
							}
							
							if (m[4]) {
								// Range
								o.start = parseInt(m[3]) || 0;
								o.end = parseInt(m[4]);
							} else if (m[3]) {
								// Single day
								o.start = parseInt(m[3]);
								o.range = 1;
							}
						}
						
						return o;
					},
				
				getDateReference = function(r) {
					
						if (r === 'dateTaken') { 
							return text.imagesTaken;
						} else if (r === 'fileModified') {
							return text.imagesModified;
						}
						
						return text.imagesAdded;
					},
					
				getDateRangeTitle = function(o) {
						var t1 = getDateReference(o['reference']),
							t2,
							start,
							end,  
							today = Math.floor(new Date() / ONEDAY_MS),
							date = function(d) {
									return new Date(d * ONEDAY_MS).toLocaleDateString(LOCALE);
								};
						
						if (!o.hasOwnProperty('end') && !o.hasOwnProperty('start')) {
							t2 = getTimespan(o.range);
						} else if (!o.hasOwnProperty('end')) {
							if (o.hasOwnProperty('start')) {
								if (o.hasOwnProperty('range')) {
									if (o['range'] > 1) {
										t2 = text.betweenDays.template(date(o.start), date(o.start + o.range));
									} else {
										t2 = text.onDay.template(date(o.start));
									}
								} else {
									t2 = getTimespan(nows / ONEDAY_S - o.start);
									//t2 = text.afterDay.template(date(o.start));
								}
							}
						} else {
							t2 = text.beforeDay.template(date(o.end));
						}	
						
						return t1 + ' ' + t2;
					},
				
				getItemsByDate = function(options, ready) {
						var t = addOverlay({
									className:		'search-results',
									icon:			'icon-calendar',
									title:			getDateRangeTitle(options)
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 		'progressbar'
							}));
						
						setTimeout(function() {
								album.collectByDate($.extend(options, {
									ready:		function() {
														h.find('.progressbar').remove();
														h.append($('<small>', {
															text: this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
														t.renderImages(this, prepareImages);
														toggleSelectButtons(0, this.length);
														toggleBoxes(this.length);
														if (typeof ready === FUNCTION) {
															ready.call();
														}
													}
									}));
							}, 50);
					},
					
				// Search date range
				
				searchDateRange = function(range) {
					
						if (range) {
							
							var options = getDateRangeOptions(range);
							
							getItemsByDate(options, function() {
									setParam(settings.indexName, {
											date:		range
										}, getDateRangeTitle(options));
									updateShares();
								});
						}
						return false;
					},
				
				// Search new images
				
				searchNew = function(days) {
						
						if (days) {
							
							getItemsByDate($.extend(settings.searchNew, { 
									start: 		nows / ONEDAY_S - days 
								}), function() {
									setParam(settings.indexName, {
											newimages:		days
										}, getDateReference(settings.searchNew['reference']));
									updateShares();
								});
						}
						return false;
					},
				
				// Prepare Search new functionality
				
				prepareSearchNew = function(f) {
					
						if (!f.length || !album) {
							return;
						}
						
						// Setting up defaults if something is missing
						
						settings.searchNew = $.extend({
								days: 				'3,30,90',
								depth: 				'folder',
								reference: 			'dateTaken',
								sinceLastVisit:		true
							}, settings.searchNew);
						
						var	days = settings.searchNew.days.split(',');
						
						f.append($('<label>', {
								text: 	getDateReference(settings.searchNew.reference)
							}));
								
						for (var i = 0; i < days.length; i++) {
							f.append($('<a>', {
									'class': 	'small button',
									text: 		getTimespan(days[i])
								})
								.data('days', parseInt(days[i], 10))
								.on('click', function() {
									searchNew($(this).data('days'));
								}));
						}
								
						if (settings.searchNew.sinceLastVisit) {
							
							var start = $.cookie('lastVisit'),
								now = Math.round(new Date() / 1000);
							
							if (start && start < (now - ONEDAY_S)) {
								f.append($('<a>', {
										'class': 	'small button',
										text: 		text.sinceMyLastVisit
									})
									.on('click', function() {
										searchNew((now - start) / ODEDAY_S);
									}));
							}
							
							// Storing last visit date for 10 years
							$.cookie('lastVisit', now, 315360000);
						}
						
					},
								
					/********************************************************
					 *
					 *					Filtering & Sort
					 *
					 ********************************************************/
					
					// Dates are objects
					
					getDate = function(item, name) {
							if (item.hasOwnProperty(J.DATES)) {
								if (name === 'addedDate') {
									return item[J.DATES]['added'];
								} else if (name === 'originalDate') {
									return item[J.DATES]['dateTaken'];
								} else {
									return item[J.DATES]['fileModified'];
								}
							}
							
							return null;
						},
					
					// Some variables get remapped by the skin
					
					getMappedVal = function(item, name) {
						
							if (item.hasOwnProperty(name)) {
								// Generic case
								return item[name];
							}
							
							return null;
						},
						
					// Do filtering on current folder
					
					doFilter = function(e, ready) {
							var form = $(e.target).closest('form'),
								items = getAllItems(),
								cnt = items.length,
								tx = [];
							
							form.find('select:visible,input[type=range],input[type=date]').each(function() {
									var name = this.name,
										label = form.find('label[for=' + name + ']').text(),
										type = $(this).data('type'),
										value = $(this).val(),
										display = function(val) {
												if (name ===  'rating') {
													return Math.round(val);
													} else if (name === 'fileSize') {
														return niceByte(val);
												} else if (type === 'date') {
														return (new Date(val * 1000)).toLocaleDateString(LOCALE);
												} else if (type === 'numeric') {
													if ($(this).data('unit') === 'seconds') {
														return niceTime(val);
													} else if ($(this).data('unit') === 'byte') {
														return niceByte(val);
													} else if (val >= 10) {
														return Math.round(val);
													} else if (val >= 1) {
														return val.toFixed(1);
													} else if (val >= 0.1) {
														return val.toFixed(2);
													}
													return val;
												}
												return toCurrency(val, currency);
											};
										
									if (type === 'numeric' || type === 'date' || type === 'financial') {
											
										var low = $(this).data('low'),
											high = $(this).data('high');
											
										if (low > this.min || high < this.max) {
											// Only if the range is narrower than the whole range
											
											if (name.endsWith('Date')) {
												// Date
												for (var i = items.length - 1, val; i >= 0; i--) {
													if ((val = getDate(items[i], name)) !== null && 
														(val >= low && val <= high)) {
														continue;
													}
													items.splice(i, 1);
												}
											} else if (JCAMERA.indexOf(name) !== -1) {
												// Camera data fields
												if (name === 'exposureTime') {
													for (var i = items.length - 1, val; i >= 0; i--) {
														if (items[i].hasOwnProperty(J.CAMERA) &&
															items[i][J.CAMERA].hasOwnProperty(name) &&
															(val = getExposure(items[i][J.CAMERA][name])) &&
															(val >= low && val <= high)) {
															continue;
														}
														items.splice(i, 1);
													}
												} else if (name.startsWith('focalLength')) {
													for (var i = items.length - 1, val; i >= 0; i--) {
														if (items[i].hasOwnProperty(J.CAMERA) &&
															items[i][J.CAMERA].hasOwnProperty(name) &&
															!isNaN(val = parseInt(items[i][J.CAMERA][name])) && 
															(val >= low && val <= high)) {
															continue;
														}
														items.splice(i, 1);
													}
												} else {
													for (var i = items.length - 1, val; i >= 0; i--) {
														if (items[i].hasOwnProperty(J.CAMERA) &&
															items[i][J.CAMERA].hasOwnProperty(name) &&
															(val = items[i][J.CAMERA][name]) !== null &&
															(val >= low && val <= high)) {
															continue;
														}
														items.splice(i, 1);
													}
												}
											} else if (name === 'rating') {
												// Rating
												for (var i = items.length - 1, val, r = settings.visitorRating? J.VISITORRATING : J.RATING; i >= 0; i--) {
													val = items[i][r] || 0;
													if (val >= low && val <= high) {
														continue;
													}
													items.splice(i, 1);
												}
											} else {
												// Generic
												for (var i = items.length - 1, val; i >= 0; i--) {
													if ((val = items[i][name]) !== null && 
														(val >= low && val <= high)) {
														continue;
													}
													items.splice(i, 1);
												}
											}
										
											tx.push(label + '=' + display(low) + '&ndash;' + display(high));
										}
										
									} else if (this.nodeName === 'SELECT' && value !== null && value.length) {
										
										// Only if at least one option is selected
																			
										if (!Array.isArray(value)) {
											value = [ value ];
										}
										
										if (JCAMERA.indexOf(name) !== -1) {
											// Camera data :: no arrays expected
											for (var i = items.length - 1, val; i >= 0; i--) {
												if (items[i].hasOwnProperty(J.CAMERA) &&
													items[i][J.CAMERA].hasOwnProperty(name) &&
													(val = items[i][J.CAMERA][name]) !== null &&
													value.includes(val)) {
													continue;
												}
												items.splice(i, 1);
											}
											
										} else {
											// Generic case
											for (var i = items.length - 1, found, val; i >= 0; i--) {
												if (items[i].hasOwnProperty(name)) {
													val = items[i][name];
													if (Array.isArray(val)) {
														// Array e.g. keywords
														found = false;
														for (var j = 0; j < value.length; j++) {
															if (val.includes(value[j])) {
																console.log(items[i]['name'] + ': ' + val);  
																found = true;
																break;
															}
														}
														if (found) {
															continue;
														}
													} else if (value.includes(val)) {
														continue;
													}
												}
												items.splice(i, 1);
											}
										}
										
										// Header text addition
										if (this['multiple']) {
											tx.push(name + '=[' + value.join(',') + ']');  
										} else {
											tx.push(label + '=' + value);  
										}
									}
								});
							
							if (items.length < cnt) {
								
								// Only show overlay if at least one thumb was filtered out
								
								var t = addOverlay({
											className:	'filtered-items',
											//icon:		'icon-filter',
											title:		text.results
										}),
									h = t.find('header .' + settings.overlayTitleClass);
									
								h.addClass('filter')
									.append($('<span>', {
										'class': 	'progressbar'
									}));
								
								t.find('.close').on('click', resetBaseFilters);
								
								setTimeout(function() {
										h.find('.progressbar').remove();
										h.append($('<small>', {
												html: 		tx.join(', ') + ': ' + text.foundNTimes.replace('{0}', items.length)
											}));
										//h.after(sh);
										t.renderImages(items, prepareImages);
										toggleSelectButtons(0, items.length);
										toggleBoxes(items.length);
										if (typeof ready === FUNCTION) {
											ready.call();
										}
									}, 50);
							}
							
							return false;
						},
					
					// Do sorting on current set of thumbs
					
					doSort = function(e, ready) {
							var el = this.selectedOptions[0],
								name = this.value;
							
							if (!el || !name) {
								return true;
							}
							
							var items = getAllItems(),
								cnt = items.length,
								data = $(el).data('sort'),
								type = data['type'],
								asc = data['ascending'],
								tx = [],
								v1,
								v2;
								
							if (type === 'numeric' || type === 'financial' || type === 'date') {
								// Number
								
								if (name.endsWith('Date')) {
									// Date
									items = items.sort(function(i1, i2) {
											v1 = getDate(i1, name);
											v2 = getDate(i2, name);
											return asc? (v1 || 0) - (v2 || 0)
														:
														(v2 || 0) - (v1 || 0);
										});
									
								} else if (JCAMERA.indexOf(name) !== -1) {
									// Camera data
									if (name === 'exposureTime') {
										items = items.sort(function(i1, i2) {
												v1 = (i1.hasOwnProperty(J.CAMERA) && i1[J.CAMERA].hasOwnProperty(name))? 
														i1[J.CAMERA][name] : 0;
												v2 = (i2.hasOwnProperty(J.CAMERA) && i2[J.CAMERA].hasOwnProperty(name))? 
														i2[J.CAMERA][name] : 0;
												return asc? (getExposure(v1 || 0) - getExposure(v2 || 0)) 
															: 
															(getExposure(v2 || 0) - getExposure(v1 || 0));
											});
									} else {
										items = items.sort(function(i1, i2) {
												v1 = (i1.hasOwnProperty(J.CAMERA) && i1[J.CAMERA].hasOwnProperty(name))? 
														parseFloat(i1[J.CAMERA][name]) : 0;
												v2 = (i2.hasOwnProperty(J.CAMERA) && i2[J.CAMERA].hasOwnProperty(name))? 
														parseFloat(i2[J.CAMERA][name]) : 0;
												return asc? ((v1 || 0) - (v2 || 0)) 
															: 
															((v2 || 0) - (v1 || 0));
											});
									}
									
								} else if (name === 'rating') {
									// Rating
									var r = settings.visitorRating? J.VISITORRATING : J.RATING;
									items = items.sort(function(i1, i2) {
											return asc? (i1[r] || 0) - (i2[r] || 0)
														:
														(i2[r] || 0) - (i1[r] || 0);
										});
									
								} else {
									// Every other numeric
									items = items.sort(function(i1, i2) {
											if (i1.hasOwnProperty(name) && i2.hasOwnProperty(name)) {
												return asc?	parseFloat(i1[name]) - parseFloat(i2[name])
															:
															parseFloat(i2[name]) - parseFloat(i1[name]);
											}
											return asc? (i1.hasOwnProperty(name)? 1 : 0)
														:
														(i2.hasOwnProperty(name)? -1 : 0);
										});
								}
								
							} else {
								// String
								
								items = items.sort(function(i1, i2) {
									if (i1.hasOwnProperty(name) && i2.hasOwnProperty(name)) {
										return asc?	(i1[name] + '').localeCompare(i2[name] + '')
													:
													(i2[name] + '').localeCompare(i1[name] + '');
									}
									return asc? (i1.hasOwnProperty(name)? 1 : 0)
												:
												(i2.hasOwnProperty(name)? -1 : 0);
								});
								
							}
							
							var t = addOverlay({
										className:	'sorted-items',
										//icon:		'icon-ordering',
										title:		text.sortedBy + '<small>' + data.label + ' (' + text[data.ascending? 'ascending' : 'descending'] + ')</small>' 
									}),
							h = t.find('header .' + settings.overlayTitleClass);
								
							h.addClass('icon-ordering sort')
								.append($('<span>', {
									'class': 	'progressbar'
								}));
							
							t.find('.close').on('click', resetBaseSort);
							
							setTimeout(function() {
									h.find('.progressbar').remove();
									//h.after(fh);
									t.renderImages(items, prepareImages);
									toggleSelectButtons(0, items.length);
									toggleBoxes(items.length);
									if (typeof ready === FUNCTION) {
										ready.call();
									}
								}, 50);
							
							return false;
						},
					
				// Read filter ranges from current overlay
				
				getFilterRanges = function(items, filter) {
						
						if (!filter || !items.length) {
							return;
						}
						
						var setMinMax = function(f, val) {
									if (f.hasOwnProperty('min') && f.hasOwnProperty('min')) {
										if (f.min > val) {
											f.min = val;
										} else if (f.max < val) {
											f.max = val;
										}
									} else {
										f.min = f.max = val;
									}
								};
								
						for (var name in filter) {
							
							var f = filter[name], 
								type = f['type'];
							
							if (type === 'numeric' || type === 'date' || type === 'financial') {
								
								if (name.endsWith('Date')) {
									// Date
									for (var i = items.length - 1, val; i >= 0; i--) {
										if ((val = getDate(items[i], name)) !== null) {
											setMinMax(f, val);
										}
									}
								} else if (JCAMERA.indexOf(name) !== -1) {
									// Camera data fields
									if (name === 'exposureTime') {
										for (var i = items.length - 1, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(J.CAMERA) &&
												items[i][J.CAMERA].hasOwnProperty(name) &&
												(val = getExposure(items[i][J.CAMERA][name]))) {
												setMinMax(f, val);
											}
										}
									} else if (name.startsWith('focalLength')) {
										for (var i = items.length - 1, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(J.CAMERA) &&
												items[i][J.CAMERA].hasOwnProperty(name) &&
												!isNaN(val = parseInt(items[i][J.CAMERA][name]))) {
												setMinMax(f, val);
											}
										}
									} else {
										for (var i = items.length - 1, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(J.CAMERA) &&
												items[i][J.CAMERA].hasOwnProperty(name) &&
												(val = items[i][J.CAMERA][name]) !== null) {
												setMinMax(f, val);
											}
										}
									}
								} else if (name === 'rating') {
									// Rating
									for (var i = items.length - 1, val, r = settings.visitorRating? J.VISITORRATING : J.RATING; i >= 0; i--) {
										val = items[i][r] || 0;
										setMinMax(f, val);
									}
								} else {
									// Generic
									for (var i = items.length - 1, val; i >= 0; i--) {
										if ((val = items[i][name]) !== null) {
											setMinMax(f, val);
										}
									}
								}
								
							} else {
								// String
								if (!f.hasOwnProperty('values')) {
									f.values = [];
								}
								
								if (JCAMERA.indexOf(name) !== -1) {
									// Camera data :: no arrays expected
									for (var i = 0, val; i < items.length; i++) {
										if (items[i].hasOwnProperty(J.CAMERA) &&
											items[i][J.CAMERA].hasOwnProperty(name)) {
											pushNew(f.values, (items[i][J.CAMERA][name] + '').trim());
										}
									}
								} else {
									for (var i = 0, val; i < items.length; i++) {
										if (items[i].hasOwnProperty(name)) {
											pushNew(f.values, items[i][name]);
										}
									}
								}
							}
						}
						
						return filter;
					},
					
				// Enabling / disabling the reset button
				
				enableResetBtn = function(e) {
						var form = (typeof e === UNDEF || e.hasOwnProperty('target'))? $(this).closest('form') : e;
						form.find('button[type=reset]').removeClass('disabled');
					},
					
				disableResetBtn = function(e) {
						var form = (typeof e === UNDEF || e.hasOwnProperty('target'))? $(this).closest('form') : e;
						form.find('button[type=reset]').addClass('disabled');
					},
					
				// Reseting filters to defaults
				
				_resetFilters = function(form) {
					
						form.find('select').each(function() {
								$(this).prop('selectedIndex', this['multiple']? -1 : 0);
							});
						
						form.find('input[type=range]').each(function() {
								$(this).trigger('resetRange');
							});
						
						disableResetBtn(form);
						
					},
					
				// Reset filters after a form elemnt changed
				
				resetFilters = function(e) {
						_resetFilters($(e.target).closest('form'));
						return false;
					},
					
				// Reseting base filters to defaults
				
				resetBaseFilters = function() {
						resetBaseSort();
						_resetFilters($('#filters form.filter'));
					},
					
				// Reseting base filters to defaults
				
				resetBaseSort = function() {
						$('#filters form.sort select').prop('selectedIndex', 0);
					},
					
				// Preparing filters / sort
				
				prepareFilterBox = function(baseSet) {
					
						var items = getAllItems(),
							filters = album.getPropertyObject(album.getCurrentFolder(), J.FILTERS, true),
							sort = album.getPropertyObject(album.getCurrentFolder(), J.SORT, true),
							box = $(),
							cont,
							filterForm = $(),
							sortForm = $(),
							group,
							inp,
							lbl,
							type,
							buttons,
							btn;
							
						if ((filters || sort) && items.length > 1) {
							
							if (baseSet) {
								
								box = $('#filters');
								filterForm = box.find('form.filter');
								
							} else {
								
								box = $('<div>', {
										'class':		'box filter-cont',
										id:				'filters_' + getOverlayNS()
									}).insertBefore($('#filters').hide());
									
								box.append($('<h' + settings.overlayHeadLevel + '>', {
										'class':		'icon-' + (filters? 'filter' : 'ordering'),
										text:			$('#filters').find('.' + settings.overlayTitleClass).text()
									}));
								
								filterForm = $('<form>', {
										'class':		'filter'
									}).appendTo(box);
									
							}
							
							buttons = $('<div>', {
									'class':	'buttons'
								}).appendTo(filterForm);
								
							// Attaching box to current overlay
							attachToOverlay(box);
						
						} else {
							
							if (baseSet) {
								$('#filters').hide();
							}
							
							return;
						}
						
						if (filters) {
							
							// Getting filter range
							filters = getFilterRanges(items, filters);
							
							var m;
							
							for (var f in filters) {
								
								type = filters[f]['type'];
								
								group = $('<div>', {
										'class':		'group ' + ((type === 'multiple_string' || type === 'single_string')? 'select' : 'range') 
									}).insertBefore(buttons);
								
								if (filters[f]['label']) {
									lbl = $('<label>', {
											'for':			f,
											text:			filters[f]['label']
										}).appendTo(group);
								}
									
								if (type === 'multiple_string' || type === 'single_string') {
									
									if (f === 'rating') {
										filters[f].values = [ text.noRating, 1, 2, 3, 4, 5 ];
									}
									
									inp = $('<select>', {
											name:			f
										}).appendTo(group);
									
									if (type === 'multiple_string') {
										inp.attr('multiple', true);
										inp.addTooltip(text.multipleSelectHint);
									}
									
									if (!filters[f].hasOwnProperty('values') || !filters[f].values.length) { // || filters[f].values.length < 2) {
										
										group.remove();
										
									} else {
									
										if (type === 'single_string') {
											inp.prepend($('<option>', {
													text:			text.select,		// + ' ' + filters[f]['label'],
													disabled:		'',
													selected:		''
												}));
										}
										
										if (filters[f].values) {
											for (var i = 0; i < filters[f].values.length; i++) {
												inp.append($('<option>', {
														text:			filters[f].values[i]
													}));
											}
										}
										
										inp.on('change', enableResetBtn);
									}
									
								} else {
									
									inp = $('<input>', {
											name:		f,
											type:		'range'
										})
										.data('type', type)
										.appendTo(group);
									
									if (f === 'rating') {
										filters[f]['step'] = 1;
										filters[f]['min'] = 0;
										filters[f]['max'] = 5;
									}
									
									if (!filters[f].hasOwnProperty('min') || !filters[f].hasOwnProperty('max') || filters[f]['min'] === filters[f]['max']) {
										
										group.remove();
										
									} else {
										
										if (type === 'financial' && (m = filters[f]['label'].match(/\S+\s+[\(\[](\S+)[\)\]]/))) {
										
											filters[f]['currency'] = m[1];
										
										} else if (type === 'numeric') {
											
											if ('exposureTime.shutterSpeed'.indexOf(f) >= 0) {
												filters[f]['unit'] = 'seconds';
											} else if (f === 'fileSize') {
												filters[f]['unit'] = 'byte';
												filters[f]['step'] = 1;
											}
										}
										filters[f].onChanged = enableResetBtn;
										filters[f].onReseted = disableResetBtn;
										
										inp.rangeSlider(filters[f]);
									}
								}
									
								inp.attr('data-type', type || 'single_string');
							}
							
							// Adding buttons
							
							if ((inp = filterForm.find('input,select')).length) {
								
								filterForm.data('single', inp.length === 1 && inp[0].nodeName === 'SELECT' && !inp.attr('multiple'));
								
								if (!(baseSet && filterForm.data('single'))) {
									// Reset button
									btn = $('<button>', {
											type:			'reset',
											'class':		(baseSet? 'disabled secondary' : 'alert') + ' button',
											text:			' ' + text.reset
										}).appendTo(buttons);
									
										if (baseSet) {
											btn.on('click.' + ns, resetFilters);
										} else {
											btn.on('click.' + ns, function() {
													resetBaseFilters();
													removeOverlay();
												});
										}
								}
									
								if (filterForm.data('single')) {
									// Single combo box -> Update instantly on change
									
									inp.on('change.' + ns, doFilter);
									
								} else {
									// Multiple inputs or not single combo box -> Use 'Search' button
									btn = filterForm.find('button[type=submit]');
									
									if (!btn.length) {
										btn = $('<button>', {
												type:			'submit',
												'class':		'button',
												text:			' ' + text.search
											}).appendTo(buttons);
									}
									
									btn.on('click.' + ns, doFilter);
								}
								
								filterForm.on('submit.' + ns, doFilter);
							}
						}
						
						if (buttons.is(':empty')) {
							buttons.remove();
						}
						
						if (sort) {
							
							
							if (baseSet) {
								
								sortForm = box.find('form.sort');
								
							} else {
								
								sortForm = $('<form>', {
											'class':		'sort'
										}).appendTo(cont);
							}
							
							lbl = $('<label>', {
									'class':		'icon-ordering',
									'for':			'sortby',
									text:			' '
								}).appendTo(sortForm);
								
							inp = $('<select>', {
									'class':		'small',
									name:			'sortby'
								}).appendTo(sortForm);
							
							inp.prepend($('<option>', {
									text:			text.sortBy,
									disabled:		'',
									selected:		''
								}));
							
							for (var s in sort) {
								
								inp.append($('<option>', {
										html:			'&#8613; ' + sort[s].label,
										value:			s
									}).data('sort', {
										type:			sort[s].type,
										label:			sort[s].label,
										ascending:		true
									}));
								
								inp.append($('<option>', {
										html:			'&#8615; ' + sort[s].label,
										value:			s
									}).data('sort', {
										type:			sort[s].type,
										label:			sort[s].label,
										asc:			false
									}));
							}
						
							inp.on('change.' + ns, doSort);
						}
						
						if ((!sortForm.length || sortForm.is(':empty')) && (!filterForm.length || filterForm.is(':empty'))) {
							box.hide();
						}
					},
								
				/********************************************************
				 *
				 *						Controls
				 *
				 ********************************************************/
				
				/********************************************************
				 *
				 *			Initialization of misc functions
				 *
				 ********************************************************/
										
				// Right click blocking
				
				avoidRightClick = function() {
						getImages().add(getFolders()).on('contextmenu', 'img', function(e) {
								e.preventDefault()
								return false;
							});
					},
					
				// Initializing background music
				
				prepareBackgroundMusic = function(player) {
					
						if (player.length) {
							player.audioPlayer({
									rootPath: 	settings.rootPath
								});
						}
					},
					
				// Updating the share buttons after a state change
				
				updateShares = function() {
						if (settings.hasOwnProperty('share') && getLightbox().is(':visible')) {
							$(settings.share.hook).trigger('updateLinks');
						}	
					},
										
				// Initializing Social sharing
				
				prepareShares = function(opt) {
						
						$.fn.renderShares.defaults.buttonTheme = opt['buttonTheme'] || 'dark';
						$.fn.renderShares.defaults.indexName = settings.indexName;
						$.fn.renderShares.defaults.facebookAppId = opt['facebookAppId'] || '';
						
						if (opt.hasOwnProperty('callAction')) {
							$.fn.renderShares.defaults.callAction = opt.callAction;
						}
						
						$(opt['hook']).renderShares(opt);
						
						$.fn.lightbox.defaults.share = opt.sites;
					},

				// Preparing map
				
				prepareMap = function(images, settings) {
					
							var	getMarkers = function(images) {
										// Reading out map locations
										var markers = [];
										
										for (var i = 0, l; i < images.length; i++) {
											l = album.getLocation(images[i]);
											if (l) {
												markers.push({
													title:		[((i + 1) + '.'), images[i]['title'], images[i]['comment']].filter(Boolean).join(' '),
													pos: 		l,
													link:		images[i].path
												});
											}
										}
										
										return markers;
									};
							
							mapRoot = $('.map-root').eq(0);
							
							// Change defaults
							if (settings.hasOwnProperty('type')) {
								$.fn.lightbox.defaults.mapType = settings.type;
								$.fn.addMap.defaults.type = settings.type;
							}
							if (settings.hasOwnProperty('zoom')) {
								$.fn.lightbox.defaults.mapZoom = settings.zoom;
								$.fn.addMap.defaults.zoom = settings.zoom;
							}
							if (settings.hasOwnProperty('apiKey')) {
								$.fn.addMap.defaults.apiKey = settings.apiKey;
							}
						
							mapRoot.addMap({
									markers: 			getMarkers(images),
									autoLoad:			true,
									onMarkerClick:		function() {
																if (this.hasOwnProperty('link')) {
																	var el = getCardByName(this.link);
																	if (el.length) {
																		if (!getLightbox().is(':visible')) {
																			getOverlay().trigger('setactivecard', el);
																		}
																		lightboxLoad(el);
																	}
																}
															}
								});
					
					},
				
				// Preparing shopping cart
				
				prepareShoppingCart = function() {
					
						// assigning paypal.js defaults 
						$.fn.paypal.defaults.relPath = settings.relPath;
						$.fn.paypal.defaults.rootPath = settings.rootPath;
						
						// Initializing the cart
						currency = album.getCurrency();
						
						// Set rangeslider's default currency
						if ($.fn.hasOwnProperty('rangeSlider')) {
							$.fn.rangeSlider.defaults.currency = currency;
						}
						
						if (shopRoot.length) {
							// Calling Paypal module
							shopRoot.paypal(album, { 
									
									getSelected: 	function() {
															return getSelectedItems();
														},
														
									selectNone:		function() {
															getOverlay().trigger('selectNone');
														},
														
									getCurrent:		function() {
															return getCurrentItem();
														}
									
								});
							
							// "Add selected" button
							shopRoot.find('.' + settings.addSelectedClass).on('click', function() {
									shopRoot.trigger('addItems', getSelectedItems());
									getOverlay().trigger('selectNone');
									return false;
								});
							
							// Ensuring lightbox has 'shop' property
							if (!$.fn.lightbox.defaults.hasOwnProperty('shop')) {
								$.fn.lightbox.defaults.shop = {};
							}
							
							// Setting lightbox's shop root variable
							$.fn.lightbox.defaults.shop.root = shopRoot;
						}
					
					},
					
				// Preparing feedback
				
				prepareFeedback = function() {
					
						// assigning feedback.js defaults 
						$.fn.feedback.defaults.relPath = settings.relPath;
						$.fn.feedback.defaults.rootPath = settings.rootPath;
						
						// Initializing the feedback cart
						if (feedbackRoot.length) {
							
							// Get selected items
							settings.feedback['getSelected'] = function() {
									return getSelectedItems();
								};
								
							settings.feedback['selectNone'] = function() {
									getOverlay().trigger('selectNone');
								};
								
							settings.feedback['getCurrent'] = function() {
									return getCurrentItem();
								};
								
							//settings.feedback['button'] = controls.find('[data-rel=feedback]');
							
							// Calling Feedback module
							feedbackRoot.feedback(album, settings.feedback);
							/*								
							// "Select all" button
							feedbackRoot.find('.' + settings.selectAllClass).on('click', function() {
								getOverlay().trigger('selectAll');
								return false;
							});
							
							// "Select none" button
							feedbackRoot.find('.' + settings.selectNoneClass).on('click', function() {
								getOverlay().trigger('selectNone');
								return false;
							});
							*/
							// "Add selected" button
							feedbackRoot.find('.' + settings.addSelectedClass).on('click', function() {
									feedbackRoot.trigger('addItems', getSelectedItems());
									getOverlay().trigger('selectNone');
									return false;
								});
							
							// Ensuring lightbox has 'feedback' property
							if (!$.fn.lightbox.defaults.hasOwnProperty('feedback')) {
								$.fn.lightbox.defaults.feedback = {};
							}
							// Setting lightbox's feedback root variable
							$.fn.lightbox.defaults.feedback.root = feedbackRoot;
						}
					},
					
				/********************************************************
				 *
				 *					Index page initialization
				 *
				 ********************************************************/
				
				filterSliderImages = function(images) {
					
						return images.filter(function(img) {
								return !img.hasOwnProperty('sliderImage');
							});
					},
				
				prepareIndex = function() {
					
						if (!album) {
							return;
						}
						
						var images = album.getImages();
						
						if (settings.hasOwnProperty('skipFirst')) {
							images = filterSliderImages(images);
						}
						
						// Removing thumbnail links right away
						getCards().each(function() {
								var a = $(this).find('.' + settings.thumbClass);
								a.data('href', a.attr('href'));
								a.attr('href', '');
							});
			
						// Preapring folders
						prepareFolders();
			
						
						// Shopping cart
						if ($.fn.hasOwnProperty('paypal')) {
							shopRoot = $('#shop-root');
							shopBox = $('#shop-cont,#shop-box').eq(0);
							prepareShoppingCart();				
						}
						
						// Feedback tool
						if (settings.hasOwnProperty('feedback')) {
							feedbackRoot = $('#feedback-root');
							feedbackBox = $('#feedback');
							prepareFeedback();
						}
						
						// "Select all" button
						shopBox.add(feedbackBox).find('.' + settings.selectAllClass).on('click', function() {
								getOverlay().trigger('selectAll');
								return false;
							});
						
						// "Select none" button
						shopBox.add(feedbackBox).find('.' + settings.selectNoneClass).on('click', function() {
								getOverlay().trigger('selectNone');
								return false;
							});
														
						// Rendering thumbnails
						if (images.length) {
							
							// Preparing images on the base overlay
							getBaseOverlay().renderImages(images, prepareImages, true);
							
							// Start slideshow
							$('#start-slideshow').on('click', startAuto);
							
							// Collecting markers and initializing map
							if (settings.hasOwnProperty('map') && settings.map.index) {
								prepareMap(images, settings.map);
							}
		
						} else {
							// no thumbnails, only folders
							if (window.location.hash) {
								// we should check the search / tags / newimages filter
								stateChange();
							}
							
							$('#start-slideshow').hide();
						}
						
						// Collecting tags if gathered only from this folder
						if (settings.hasOwnProperty('tagCloud')) {
							
							if (settings.tagCloud['depth'] === 'current') {
								album.collectTags($.extend(settings.tagCloud, {
										exact:		settings.exactFields,
										ready:		function() {
															renderTagCloud(this);
														}
									}));
							}
							
							if (settings.tagCloud.hasOwnProperty('searchHook')) {
								prepareTagSearch();
							}
						}
						
						// Custom query (eg. new images sample box)
						var queryTarget = $('[data-jalbum-query]');
						
						if (queryTarget.length) {
							// Reding settings from data attribute
							var qsettings = queryTarget.data('jalbum-query');
							
							if (qsettings.hasOwnProperty('fn')) {
								// Calling the jalbum.album function
								album[qsettings.fn]($.extend(qsettings, {
											ready:		function(opt) {
												
																for (var i = 0, a, item, mx = Math.min(this.length, 10); i < mx; i++) {
																	
																	item = this[i];
																	
																	a = $('<a>', {
																			href:	album.getLink(item)
																		}).appendTo(queryTarget);
																	
																	a.append($('<img>', {
																			src:		album.getThumbPath(item),
																			width:		item[J.THUMB][J.WIDTH],
																			height:		item[J.THUMB][J.HEIGHT],
																			alt:		item[J.TITLE] || item[J.NAME]
																		}));
																}
																
																if (this.length > 10) {
																	queryTarget.append($('<a>', {
																			'class':	'more',
																			href:		qsettings.linkMore,
																			text:		'+' + (this.length - 10)
																		}));
																}
																
																if (settings.rightClickProtect) {
																	queryTarget.on('contextmenu', 'img', function(e) {
																			e.preventDefault()
																			return false;
																		});
																}
															}
								
									}));
							}
						}
					},
					
				/********************************************************
				 *
				 *					Custom page initialization
				 *
				 ********************************************************/
				
				preparePage = function() {
						// Page :: initializing functions
						// Search 
						if (settings.hasOwnProperty('search')) {
							prepareSearch($(settings.search.hook));
						}
						
						settings.pageName = window.location.pathname.getFile();
						
						var paths = [];
						
						getCards().each(function() {
								var a = $(this).find('.' + settings.thumbClass);
								a.data('href', a.attr('href'));
								a.attr('href', '');
								paths.push($(this).data(J.PATH));
							});
						
						if (paths.length) {
						
							album.collectByPath({
									paths:		paths,
									ready:		function() {
														// Rendering thumbnails
														if (this.length) {
															
															// Preparing images on the base overlay
															getBaseOverlay().renderImages(this, prepareImages, true);
															
															// Collecting markers and initializing map
															if (settings.hasOwnProperty('map')) {
																prepareMap(this, settings.map);
															}
														}
													}
								});
						}
					},
				
				/********************************************************
				 *
				 *	 Preparing functions that need the full album tree
				 *
				 ********************************************************/
								
				prepareDeepFunctions = function() {
					
						// Collecting tags if gathered from deep tree
						if (settings.hasOwnProperty('tagCloud') && settings.tagCloud['depth'] !== 'current') {
							album.collectTags($.extend(settings.tagCloud, {
									exact:		settings.exactFields,
									ready:		function() {
														renderTagCloud(this);
													}
								}));
						}
						
						// Search 
						if (settings.hasOwnProperty('search')) {
							prepareSearch($(settings.search.hook));
						}
						
						// Search new
						if (settings.hasOwnProperty('searchNew')) {
							prepareSearchNew($(settings.searchNew.hook));
						}
					},
	
				/********************************************************
				 *
				 *					Preparing folders
				 *
				 ********************************************************/
				
				// Marking new folders
				
				markFoldersNew = function() {
						
						if (!album) {
							return;
						}
						
						var folders = album.getFolders(),
							secs = settings.markNew['days'] * ONEDAY_S,
							s,
							d;
						
						getFolders().each(function(i) {
								
								if (i < folders.length) {
									d = folders[i][J.DATES];
								
									if (d) {
										d = d[settings.markNew['reference']] || d[J.DATERANGE][1] || d[J.DATERANGE][0];
									
										if (d && (nows - d) < secs) {
										s = settings.markNew.text?
											$('<span>', {
													'class':	'new-image',
													text:		settings.markNew.text
												})
											:
											$('<span>', {
													'class':	'icon-new-fill new-image'
												});
										
										if (!TOUCHENABLED) {
											s.data('tooltip', (new Date(d * 1000)).toLocaleDateString(LOCALE)).addTooltip();
										}
										
										$(this).find('.' + settings.thumbClass).eq(0).append(s);
										}
									}
								}
							});
					},
					
				// Preparing folders
			
				prepareFolders = function() { 
					
						if (!getFolderCont().length) {
							// No folders
							return;
						}
						
						if (settings.hasOwnProperty('markNew')) {
							// Marking new folders
							setTimeout(markFoldersNew, 300);
						}
						
						setTimeout(function() {
								
								// Start lazy load 
								lazyloadFolders();
								
								// Removing prelaod class on folders already loaded
								getFolders('.' + settings.preloadClass + ':first-child img')
									.waitAllImg(function() {
											getFolders().removeClass(settings.preloadClass);
										});
							}, 500);
					},
				
				/********************************************************
				 *
				 *					Initializing album
				 *
				 ********************************************************/
				
				initAlbum = function(readyFn) {
					
						if (typeof Album === UNDEF) {
							console.log('Critical Error: Missing jalbum.album.js library!');
						}
						
						var params = {
								makeDate:		settings['makeDate'],
								rootPath:		settings['rootPath'],
								relPath:		settings['relPath'],
								ready: 			(settings.pageType === 'index')? prepareIndex : preparePage,
								deepReady:		prepareDeepFunctions
							};
							
						if (settings.indexName !== $.fn.skin.defaults.indexName) {
							params.indexName = settings['indexName'];
						}
						
						if (settings.audioPoster !== $.fn.skin.defaults.audioPoster) {
							params.audioPoster = settings['audioPoster'];
						}
							
						if (settings.videoPoster !== $.fn.skin.defaults.videoPoster) {
							params.videoPoster = settings['videoPoster'];
						}
						
						album = new Album($, params);
					},
					
				// Main initializing function
				
				init = function() {
				
						// Removing search string, e.g. FB data
						if (window.location.search) {
							removeSearch();
						}
			
						// Restoring scroll position, saving on unload
						restoreScrollPosition();
						$(window).on('beforeunload.' + ns, saveScrollPosition);
						
						// Passing along lightbox defaults
						passDefaults(settings, $.fn.lightbox.defaults, 'indexName,level,previousFoldersLast,previousFolderPath,nextFoldersFirst,nextFolderPath,extraSizes,rightClickProtect,enableKeyboard,enableMouseWheel,useRating,visitorRating');
						passDefaults(settings.lightbox, $.fn.lightbox.defaults);
			
						// Creating new Album instance
						initAlbum();
			
						// Right click protection on folders, slider
						if (settings.rightClickProtect) {
							avoidRightClick();
						}
			
						// Preparing share buttons
						if (settings.hasOwnProperty('share')) {
							prepareShares(settings.share);
						}
						
						// Background music
						prepareBackgroundMusic($('[data-audioPlayer]'));
						
						// Hash change listener
						$window.on('hashchange.' + ns, stateChange);
						
						// Focusing overlay
						main.on('overlayAdded.' + ns, function(e, o) {
								moveInView(o);
							});
						
						// Menu button wiring (small screens)
						$('#topnav-toggle').on({
								'click': 		function() {
														$body.toggleClass('menu-on');
														return false;
													},
								'selectstart':	function(e) {
														e.preventDefault();
														return false;
													}
							});
						
						// Scroll to top btn
						if (settings.scrollToTopButton) {
							$body.scrollToTopBtn();
						}
						
						// Handled by external libraries:
						
						// Newsfeed
						if (typeof Newsfeed !== UNDEF && settings.hasOwnProperty('newsfeed')) {
							Newsfeed.init(settings.newsfeed);
						}
						
						// Instafeed
						if (typeof Instafeed !== UNDEF && !LOCAL && settings.hasOwnProperty('instagram')) {
							var feed = new Instafeed({
									get:			'user',
									userId: 		parseInt(settings.instagram.token.split('.')[0], 10),
									accessToken:	settings.instagram.token,
									limit: 			settings.instagram.limit
								});
							feed.run();
						}
								
					};
			
						
			/********************************************************
			 *
			 *					Execution starts here
			 *
			 ********************************************************/
				
			init();
			
			// Utility function for debugging
			this.getAlbum = function() {
					return album;
				};
			
			return this;
		};
	
	/********************************************************
	 *
	 *						Defaults
	 *
	 ********************************************************/
				
	$.fn.skin.defaults = {
			contentHook: 					'article.cont',
			rowClass:						'row',
			contentClass:					'cont',
			hasOverlayClass:				'has-overlay',
			baseClass:						'base-overlay',
			overlayClass:					'overlay',
			overlayTitleClass:				'title',
			overlayHeadLevel:				4,
			sliderClass:					'slider',
			folderContClass:				'folder-cont',
			fodlersClass:					'folders',
			folderClass:					'folder',
			thumbContClass:					'thumb-cont',
			thumbsClass: 					'thumbnails',
			thumbClass:						'thumb',
			thumbLayout:					'fixgrid',
			captionClasses:					'over bottom hover',
			cardClass:						'card',
			captionPlacement:				'below',
			lbableClass:					'lbable',
			captionClass:					'caption',
			commentClass:					'comment',
			hascaptionClass:				'hascaption',
			selectableClass:				'selectable',
			checkedClass:					'checked',
			activeClass:					'active',
			preloadClass:					'preload',
			lazyloadClass:					'lazyload',
			hideImageClass:					'hide-image',
			showImageClass:					'show-image',
			startshowClass:					'start-btn',
			fullscreenClass:				'fullscreen',
			panoClass:						'pano',
			selectAllClass:					'select-all',
			selectNoneClass:				'select-none',
			keepSelectedClass:				'keep-selected',
			addSelectedClass:				'add-selected',
			folderCols:						2,
			indexName:						'index.html',
			audioPoster:					'audio.poster.png',
			videoPoster:					'video.poster.png',
			exactFields:					'creator,keywords',
			rightClickProtect: 				false,
			useRating:						false,
			jalbumRating:					false,
			visitorRating:					false,
			enableKeyboard:					true,
			videoAuto:						true,
			linkToFolder:					true,
			scrollToTopButton:				true
		};
		
})(jQuery, jQuery(window), jQuery(document), jQuery('body'));
