/*jslint browser: true*/
;(function($, window, document, undefined) {
	'use strict';
	
	/*
	 *	Google Maps functions
	 *	canvas is a jQuery element with data('location') and other optional parameters
	 */

			
	$.fn.addMap = function(settings) {
		
		settings = $.extend({}, $.fn.addMap.defaults, settings);
		
		var getLatLng = function(ll) {
					var lat, lng;
					if (typeof ll === 'string') {
						ll = ll.split(',');
						lat = parseFloat(ll[0]);
						lng = (ll.length > 1)? parseFloat(ll[1]) : 0;
					} else {
						lat = ll[0] || ll['lat'] || 0;
						lng = ll[1] || ll['lng'] || 0;
					}
					return {
						lat: lat,
						lng: lng 
					};
				},
			
			processLatLng = function(markers) {
					for (var i = 0; i < markers.length; i++) {
						if (markers[i].hasOwnProperty('pos') && !(markers[i].hasOwnProperty('latLng'))) {
							markers[i]['latLng'] = getLatLng(markers[i]['pos']);
						}
					}
				},
			
			getBounds = function(markers) {
					var	e = -180,
						n = -90,
						s = 90,
						w = 180,
						ll;
						
					for (var i = 0; i < markers.length; i++) {
						ll = markers[i]['latLng'];
						if (ll) {
							if (w > ll.lng) {
								w = ll.lng;
							}
							if (s > ll.lat) {
								s = ll.lat;
							}
							if (e < ll.lng) {
								e = ll.lng;
							}
							if (n < ll.lat) {
								n = ll.lat;
							}
						}
					}
					
					if (e < w) {
						e = 180;
						w = -180;
					}
					
					if (n < s) {
						n = 90;
						s = -90;
					}
					
					return {
						east: 	e,
						west: 	w,
						north: 	n,
						south: 	s
					};
				};
				
				
			
		return $(this).each(function() {

				var canvas 	= 	$(this),						// Map root
					ns		=	canvas.data('map-ns');			// Namespace
				
				if (typeof ns !== UNDEF) {
					// Already instantitated
					return;
				}
				
				var	options =  	$.extend({}, settings, readData(canvas, 'location,type,zoom,label,markers,current')),
					map,
					marker = [],
					active = -1,
					bounds,
					
					getMarkerByName = function(n) {
						
							for (var i = 0; i < marker.length; i++) {
								if (marker[i]['link'] === n) {
									return i;
								}
							}
							
							return -1;
						},
					
					// Animates a marker and centers map around this marker
					
					animateMarker = function(m) {
							m.setAnimation(google.maps.Animation.BOUNCE);
							map.setCenter(m.getPosition());
						},
							
					// Sets a marker as active
					
					setActive = function(e, n) {
							
							if (typeof n !== UNDEF) {
								
								if (active >= 0) {
									marker[active].setAnimation(null);
									active = -1;
								}
								
								if (!$.isNumeric(n)) {
									n = getMarkerByName(n);
								}
								
								if (n >= 0 && n < marker.length) {
									animateMarker(marker[n]);
									active = n;
									return false;
								}
							}
							
							return true;
						},
						
					// Fits map into bounds
					
					fitBounds = function() {
							if (options.fitBounds && (bounds.east !== bounds.west || bounds.north !== bounds.south)) {
								google.maps.event.addListenerOnce(map, 'idle', function() {
										map.fitBounds(bounds);
									});
							}
						},
						
					// Reset marker animations
					
					resetMarkers = function() {
						
							if (marker && marker.length) {
								
								if (active >= 0) {
									if (marker[active].getAnimation() !== null) {
										marker[active].setAnimation(null);
									}
									active = -1;
								}
								
								if (bounds) {
									fitBounds();
								}
							}
						},
								
					// Marker clicked event
						
					markerClicked = function() {
						
							if (marker && marker.length) {
								
								if (active >= 0) {
									if (marker[active].getAnimation() !== null) {
										marker[active].setAnimation(null);
									}
									active = -1;
								}
								
								animateMarker(this);
							}
							
							if ($.isFunction(options.onMarkerClick)) {
								options.onMarkerClick.call(this);
							}
						},
						
					// Loading script

					loadScript = function(doneFn) {
					
							if (typeof google === 'undefined' || !google.maps) {
								// No API is loaded yet
								var script = document.createElement("script");
								script.type = "text/javascript";
								script.src = options.apiUrl + "?key=" + (options['apiKey'] || "");
								
								if ($.isFunction(doneFn)) {
									script.onload = function() {
										setTimeout(doneFn, 40);
									}
								}
								
								document.body.appendChild(script);
								
							} else {
								// API is ready
								if ($.isFunction(doneFn)) {
									doneFn.call(null);
								}
							}
							
							// Firing ready event
							
							if (options.hasOwnProperty('onReady')) {
								options.onReady.call(this);
							}
						},

					// Rendering map

				
					renderMap = function(e, doneFn) {
							
							if (canvas.data('rendered')) {
								return;
							}
							
							var	cnt = options.markers.length,
								i = 0,
								delay = Math.minMax(10, Math.round(1000 / cnt), 100),
								
								addMarker = function() {
										if (i < cnt) {
											if (options.markers[i].hasOwnProperty('latLng')) {
												marker[i] = new google.maps.Marker({
														map: 			map,
														position:		options.markers[i]['latLng'],
														//color:		'hsl(' + (i / cnt * 360) + ',60%,60%)', 
														animation: 		google.maps.Animation.DROP
													});
												
												if (options.markers[i].hasOwnProperty('title')) {
													marker[i].setTitle(options.markers[i]['title']);
												}
					
												if (options.markers[i].hasOwnProperty('link')) {
													marker[i].link = options.markers[i]['link'];
													google.maps.event.addListener(marker[i], 'click', markerClicked);
												}
											}
											
											i++;
											setTimeout(addMarker, delay);
										}
									};
									
							processLatLng(options.markers);
							
							if (options.markers && options.markers.length > 1) {
								
								bounds = getBounds(options.markers);
								
								map = new google.maps.Map(canvas[0], {
											center: 			{
																	lat:	(bounds.north + bounds.south) / 2,
																	lng:	(bounds.east + bounds.west) / 2
																},
											zoom: 				options.zoom,
											mapTypeId: 			options.type,
											fullscreenControl: 	options.fullscreenControl,
											scrollwheel: 		false,
											panControl:			false,
											controlSize:		32
										});
								
								marker = [];
								addMarker();
								fitBounds();
								
							} else {
								
								var pos = options.hasOwnProperty('location')? 
											getLatLng(options.location)
											:
											options.markers[0]['latLng'];
								
								map = new google.maps.Map(canvas[0], {
											center: 			pos, 
											zoom: 				options.zoom,
											mapTypeId: 			options.type,
											fullscreenControl: 	options.fullscreenControl,
											scrollwheel: 		false,
											panControl:			false,
											controlSize:		32
										});
								
								addMarker();
							}
					
					// Events
					
					if (options.hasOwnProperty('onTypeChanged') && $.isFunction(options.onTypeChanged)) {
						google.maps.event.addListener(map, 'maptypeid_changed', function() {
							options.onTypeChanged.call(map.getMapTypeId());
						});
					}
					
					if (options.hasOwnProperty('onZoomChanged') && $.isFunction(options.onZoomChanged)) {
						google.maps.event.addListener(map, 'zoom_changed', function() {
							options.onZoomChanged.call(map.getZoom());
						});
					}
					
					canvas.data('rendered', true);
					
					canvas.on('setActive', setActive);
					
					canvas.on('resetMarkers', resetMarkers);
					
					if ($.isFunction(doneFn)) {
						doneFn.call(null);
					}
				};
				
				if (options.autoLoad) {
					loadScript(renderMap);
				} else {
					loadScript();
				}
				
				canvas.on('render', renderMap);
				
			});
		
	};
					
	$.fn.addMap.defaults = {
			apiUrl:					'https://maps.googleapis.com/maps/api/js',
			zoom:					16,
			fitBounds:				true,
			type:					'roadmap',
			fullscreenControl: 		false,
			autoLoad:				false
		};
	
})(jQuery, window, document);
