/* 
 *	main.js - the skin's javascript functions
 */
	
;(function($, $window, $document, $body, undefined) {
	'use strict';
				
	/********************************************************
	 *
	 *		Implementing select functions on images
	 *
	 ********************************************************/
	
	$.fn.selectable = function(options) {
		
			options = $.extend({
							cardClass:			'card',
							checkboxClass: 		'checkbox',
							checkedClass:		'checked',
							checkmarkClass:		'icon-checkmark',
							hasSelectedClass:	'has-selected',
							allSelectedClass:	'all-selected',
							noneSelected:		null,
							anySelected:		null,
							selectionChange:	null
						}, options);
			
			var self = $(this).eq(0),
				cards,
				
				update = function(n) {
					
						if ($.isFunction(options.selectionChange)) {
							options.selectionChange.call(n);
						}
						
						$body
							.toggleClass(options.hasSelectedClass, n > 0)
							.toggleClass(options.allSelectedClass, n === cards.length);
					},
					
				selectedCnt = function() {
						return cards.filter('.' + options.checkedClass).length;
					},
				
				getImages = function() {
						return options.selector? self.find(options.selector) : self.find('.' + options.cardClass);
					},
					
				selectCard = function(card) {
						card.addClass(options.checkedClass)
							.find('.' + options.checkboxClass)
							.addClass(options.checkmarkClass);
					},
					
				unselectCard = function(card) {
						card.removeClass(options.checkedClass)
							.find('.' + options.checkboxClass)
							.removeClass(options.checkmarkClass);
					},
					
				toggleCard = function(card) {
						if (card.hasClass(options.checkedClass)) {
							unselectCard(card);
						} else {
							selectCard(card);
						}
					},
			
				clicked = function(e) {
						var card = $(e.target).closest('.' + options.cardClass);
						var n = selectedCnt();
						
						if (card.hasClass(options.checkedClass)) {
							unselectCard(card);
							n -= 1;
							
							if (n === 0) {
								if ($.isFunction(options.noneSelected)) {
									options.noneSelected.call();
								}
							} else {
								if ($.isFunction(options.anySelected)) {
									options.anySelected.call(n);
								}
							}
						} else {
							
							selectCard(card);
							n += 1;
							
							if ($.isFunction(options.anySelected)) {
								options.anySelected.call(n);
							}
						}	
						
						update(n);
						return false;
					},
			
				selectAll = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							
							if (n < cards.length) {
								n = cards.length;
								cards.not('.' + options.checkedClass).each(function() {
										selectCard($(this));
									});
										
								if ($.isFunction(options.anySelected)) {
									options.anySelected.call(cards.length);
								}
							}
							update(n);
						}
					},
			
				selectNone = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							
							if (n > 0) {
								n = 0;
								cards.filter('.' + options.checkedClass).each(function() {
										unselectCard($(this));
									});
										
								if ($.isFunction(options.noneSelected)) {
									options.noneSelected.call();
								}
							}
							update(n);
						}
					},
					
				selectInverse = function() {
					
						if (self.is(':visible')) {
							var n = cards.length - selectedCnt();
							
							if (n === 0) {
								// All selected
								cards.each(function() {
										unselectCard($(this));
									});
								
								if ($.isFunction(options.noneSelected)) {
									options.noneSelected.call();
								}
								
							} else {
								// Some selected
								cards.each(function() {
										toggleCard($(this));
									});
								
								if ($.isFunction(options.anySelected)) {
									options.anySelected.call(n);
								}
							}
							update(n);
						}
					},
				
				selectRefresh = function() {
						cards = getImages();
						
						cards.each(function() {
							if (!($(this).children('span.' + options.checkboxClass).length)) {
								$(this).append($('<span>', {
										'class': 	options.checkboxClass
									}).on({
										click: 		clicked
									}));
							}
							
						});
						
					};
			
			selectRefresh();
			
			self.on({
					selectAll: 			selectAll,
					selectNone: 		selectNone,
					selectInverse: 		selectInverse,
					selectRefresh: 		selectRefresh
				});
					
			return this;
		};
			
		
	/********************************************************
	 *
	 *					The main skin code
	 *
	 ********************************************************/
	
	$.fn.skin = function(settings) {
		
			settings = $.extend({}, $.fn.skin.defaults, settings);
			
			var main 				=	$(this),							// Container
				text				= 	getTranslations({					// Translated texts
												foundNTimes:				'found {0} time(s)',
												notFound: 					'not found',
												search: 					'Search',
												newImages:					'New images',
												results:					'Results',
												reset:						'Reset',
												label: 						'Label',
												selectedItems:				'Selection',
												addCart:					'Add to cart',
												'return': 					'return',
												select:						'Select',
												sortBy:						'Sort by',
												sortedBy:					'Sorted by',
												ascending:					'ascending',
												descending:					'descending',
												multipleSelectHint:			'Use SHIFT to select range, CTRL for multiple entries',
												noRating:					'No rating',
												inTheLastDay:				'in the last day',
												inThePastNDays:				'in the past {0} days',
												sinceMyLastVisit:			'since my last visit',
												betweenDays:				'between {0} and {0}',
												onDay:						'on {0}',
												beforeDay:					'before {0}',
												afterDay:					'after {0}',
												imagesAdded:				'Images added',
												imagesModified:				'Images modified',
												imagesTaken:				'Images taken',
												/*
												cookiePolicyText: 			'This album uses cookies to remember user preferences. By using it, you agree to our use of cookies.',
												cookiePolicyAgree: 			'Got it',
												cookiePolicyLearnMore: 		'Learn more',
												*/
												'new':						'New',
												newItem:					'NEW',
												more:						'more',
												less:						'less'
											}),
				
				shopSection 		= $(),									// Whole Shopping section
				shopRoot 			= $(),									// Root element for cart
				feedbackSection 	= $(),									// Whole Feedback section
				feedbackRoot 		= $(),									// Root element for the feedback tool
				album,														// The database
				ns 					= 'projector_skin',						// Namespace
				lastHash 			= '',									// Storing last state
				initByHash 			= window.location.hash !== '',			// Initial URL contains hash
				nows 				= new Date() / 1000,					// Now in seconds
				//today 			= new Date() / 86400000,				// Today (day count since 1970)
				cacheBuster,
			
				/********************************************************
				 *
				 *					Managing overlays
				 *				e.g. after search or filter
				 *
				 ********************************************************/
				/*
					<article class="cont thumbs-top" data-oly_ns="">
						<section class="thumbnails" data-thumbscroll>...</section>
						<section class="lightbox" data-lightbox>...</section>
						<section class="hero">...</section>
					</article>
						
				*/
				
				
				getCurrentOly = function() {
						return main.children(settings.contentHook).last();
					},
				
				// Remove overlay(s) primitive: internal use
				
				removeOlys = function() {
						var o,
							ns,
							b;
						
						main.children(settings.contentHook).each(function() {
								o = $(this);
								
								if (o.data('base')) {
									// Base set :: hide
									b = o;
								} else {
									// Another overlay :: remove 
									ns = o.data('oly_ns');
									if (ns) {
										$window.off('.' + ns);
									}
									
									o.destroyAllTooltips();
									o.trigger('lightboxRemove');
									o.remove();
								}
							});
						
						return b;
					},
					
				// Add primitive: internal use
				
				addOly = function(o) {
					
						var ns = 'oly_' + Math.floor(Math.random() * 10000),				// Namespace
							o = ($('<article>', {
										'class': 	settings.contClass + ' overlay thumbs-' + settings.thumbsPosition
									}).data('oly_ns', ns)).appendTo(main);
						
						// Saving namespace as property
						o[0]['overlay'] = ns;
						
						return o;
						
					},
					
				// Adding new overlay with header
			
				addOverlay = function(options) {
						
						var	b = removeOlys(),
							ths;
						
						main.addClass('has-overlay');
						b.hide();
						
						var o = addOly(),
							h = $('<header>').appendTo(o);
						
						// Title
						if (options && options.hasOwnProperty('title')) {
							h.append($('<h4>', {
									html: 	options.title
								}));
						}
						
						// Close button
						h.append($('<button>', {
								'class':		'close',
								'aria-label':	text.return
							}).on('click', function() {
								removeOverlay();
								return false;
							}).addTooltip(text.return, { 
								pos:	[ 0, 1, 2, 1 ]
							}));
						
						
						return currentOly = o;
					},
			
				// Removing overlay
				
				removeOverlay = function() {
					
						var b = removeOlys();
						
						main.removeClass('has-overlay');
						removeParam(settings.indexName);
						b.show();
						
						setTimeout(function(b) {
								b.trigger('loadImage');
								updateShares();
							}, 100, b);
						
						/*
						$('#selectall').toggleClass('disabled', $('section.' + settings.thumbContClass + ' .' + settings.cardClass).length === 0);
						$('#selectnone, #add-selected').addClass('disabled');
						*/
						
						return currentOly = b;
					},
				
				// Initializing Base overlay
				
				initOverlay = function() {
					
						var o = getCurrentOly();
						
						if (!o.length) {
							// Creating base overlay
							o = addOly();
						}
						
						if (!o.data('oly_ns')) {
							var ns = 'oly_' +  + Math.floor(Math.random() * 10000);
							o.data('oly_ns', ns);
						}
						
						if (!o.data('base')) {
							o.data('base', true);
						}
						
						return o;
					},
								
				currentOly = (settings.pageType === 'index')? initOverlay() : $(),			// Current overlay
				
				// Getting current thumbs array :: optionally selected subelements
				
				getImages = function(selector) {
						return currentOly.find('.' + settings.thumbContClass + (selector? (' ' + selector) : ''));
					},
					
				/********************************************************
				 *
				 *					Get/set thumbnails
				 *
				 ********************************************************/
				 
				// Getting thumbnail by file name
				
				getThumbByName = function(name, decode) {
						var decode = (typeof decode === UNDEF)? false : decode,
							t = $();
						
						getImages('.' + settings.thumbClass).each(function() {
							var l = $(this).attr('href') || $(this).data('href');
							
							if (l) {
								
								if (decode) {
									l = decodeURIComponent(l);
								}
							
								if (l === name || l.endsWith('/' + name)) {
									t = $(this);
									return false;
								}
							}
						});
						
						return t;
					},
					
				// Getting selected thumbs as items
				
				getSelectedItems = function(clear) {
					
						var r = [];
								
						getImages('.' + settings.thumbClass + '.' + settings.checkedClass).each(function() {
								if (clear) {
									$(this).removeClass(settings.checkedClass);
								}
								r.push($(this).data(J.OBJ));
							});
						
						return r;
					},
				
				// Getting all items
				
				getAllItems = function() {
					
						var r = [];
								
						getImages('.' + settings.thumbClass).each(function() {
								r.push($(this).data(J.OBJ));
							});
						
						return r;
					},
				
				// Clicked thumb
				
				thumbClicked = function(e) {
						currentOly.trigger('loadImage', $(e.target).closest('a'));
						return false;
					},
			
				// Setting active thumb by jQ element or N
				
				setActiveThumb = function(e, t) {
						var thumbs = getImages(),
							el = (typeof t === 'number')? thumbs.children('a').eq(t) : $(t),
							ae = thumbs.children('a.' + settings.activeClass);
						
						if (el && el.length && el !== ae) {
							ae.removeClass(settings.activeClass);
							el.addClass(settings.activeClass);
							thumbs.trigger('scrollToActive', el);
						}
					},
					
				// Returns active item
				
				getActiveItem = function(e) {
						var thumbs = getImages(),
							ae = thumbs.children('a.' + settings.activeClass);
						
						return ae.length? ae[J.OBJ] : {};
					},
		
				// Lazyloading
				
				lazyloadThumbs = function(e, p0, doneFn) {
						var thumbs = getImages(),
							proj = thumbs.closest('article'),
							vert = proj.hasClass('thumbs-left') || proj.hasClass('thumbs-right'),
							p0 = (typeof p0 === UNDEF)? thumbs.position()[vert? 'top' : 'left'] : p0,
							pd = (vert? proj.height() : proj.width()),
							t,
							img,
							imgs = $();
						
						thumbs
							.children('a.' + settings.lazyloadClass)
							.filter(function() {
								var p = $(this).position()[vert? 'top' : 'left'] + p0;
								
								// Half screen width backwards, full screen width forwards
								return (p + pd / 2) >= 0 && (p - pd * 2) <= 0;
							})
							.each(function() {
								t = $(this);
								
								if (!t.children('img').length) {
									img = $('<img>', {
											'class': 	settings.hideImageClass
										})
										// Onload action
										.one('load', function() {
											$(this).addClass(settings.showImageClass).removeClass(settings.hideImageClass);
											$(this).parent().removeClass(settings.lazyloadClass);
										})
										.attr('src', album.getThumbPath(t.data(J.OBJ)))
										.appendTo(t);
										
									imgs.append(img);
								}
							});
						
						if ($.isFunction(doneFn)) {
							if (imgs.length) {
								imgs.waitAllImg(doneFn);
							} else {
								doneFn.call();
							}
						}
					
					},
					
				/********************************************************
				 *
				 *					Image overlay rendering 
				 *			or initializing images in the base overlay
				 *
				 ********************************************************/
				
				renderImages = function(items, complete, autoStart, baseSet) {
					
						if (typeof items === UNDEF || !items.length || !$.isArray(items)) {
							return;
						}
						
						var ns = currentOly.data('oly_ns'),
							
							ttpos = [ 1, 2, 1, 0 ],
							
							// cleaning up the old event handlers
							cleanup = function(ns) {	
									$window.off('.' + ns);
								},
							
							// Finding or creating the element to hold the thumbs
							thumbnails = (function() {
									var th = currentOly.children('.' + settings.thumbsClass);
									
									return th.length? th : $('<section>', {
											'class': 	settings.thumbsClass
										}).appendTo(currentOly);
								}()),
							
							// Finding or creating the clip element
							clip = (function() {
									var c = thumbnails.children('.' + settings.clipClass);
									
									return c.length? c : $('<div>', {
											'class': 	settings.clipClass
										}).appendTo(thumbnails);
								}()),
							
							// Finding or creating the thumbs container
							thumbs = (function() {
									var c = clip.children('.' + settings.thumbContClass);
									
									return c.length? c : $('<div>', {
											'class': 	settings.thumbContClass
										}).appendTo(clip);
								}()),
							
							// Rendering thumbs
							initThumbs = function() {
									var ts = thumbs.children('.' + settings.thumbClass),
										card,
										img,
										s,
										st = new Date(),
										cd = settings.hasOwnProperty('markNew')? (nows - settings.markNew.days * ONEDAY_S) : null,
										item;
										
									// Creating new elements for the items with no thumbs
									for (var i = 0; i < items.length; i++) {
										
										item = items[i];
										
										if (i < ts.length) {
											// Already exists in the HTML source
											card = ts.eq(i);
											img = card.find('img').eq(0);
											img.addClass(settings.hideImageClass)
												.on('load.' + ns, function() {
														$(this).addClass(settings.showImageClass)
															.removeClass(settings.hideImageClass)
															.parents('a').removeClass(settings.preloadClass);
													});
											if (img[0].complete) {
												// Cached images
												img.trigger('load');
											}
											if (card.attr('title')) {
												card.addTooltip({ pos: ttpos });
											}
										} else {
											// Not yet exists
											card = $('<a>', {
													'class':	settings.thumbClass + ' ' + settings.lazyloadClass
												})
												.attr('href', album.getItemPath(item))
												.appendTo(thumbs);
											
											if (s = item[J.THUMBCAPTION]) {
												card.addTooltip(s.stripHTML(), { pos: ttpos });
											}
										}
										
										card.data(J.OBJ, item);
			
										if (album.isLightboxable(item)) {
											
											card.addClass(settings.lbableClass);
											
											card.on('click.' + ns, thumbClicked);
											
											if (settings.rightClickProtect) {
												card.on('contextmenu', function(e) {
														e.preventDefault()
														return false;
													});
											}
							
											/*
											if (shopRoot && album.hasShop(item) || feedbackRoot && settings.feedback) {
												card.addClass(settings.selectableClass);
												if (shopRoot) {
													card.append($('<a>', {
															'class': 	'icon-shopping-cart add-cart',
															text: 		' ' + text.addCart
														}).on('click.' + ns, function(e) {
																var card = $(e.target).closest('a');
																if (card.length) {
																	shopRoot.trigger('addItems', card.data(J.OBJ));
																}
																return false;
															})
														);
												}
											}
											*/
											
										} else {
											
											card.addClass(item[J.CATEGORY]);
											card.attr('href', album.getItemPath(item));
										}
										
										if (cd != null) {
											
											var d = item[J.DATES];
											
											if (d && (d = d[settings.markNew.reference]) && d >= cd) {
												s = settings.markNew.text?
													$('<span>', {
															'class':	'new-image',
															text:		settings.markNew.text
														})
													:
													$('<span>', {
															'class':	'icon-new-fill new-image'
														});
													
												card.append(s);
													
												if (!TOUCHENABLED) {
													s.data('tooltip', (new Date(d * 1000)).toLocaleDateString()).addTooltip();
												}
											}
										}
										/*
										if (settings.markNew && item.hasOwnProperty(J.MODIFIEDDAYS)) {
											if ((today - item[J.MODIFIEDDAYS]) < settings.markNew) {
												card.append($('<span>', {
														'class':	'icon-new-fill new-image'
													}).addTooltip((new Date(item[J.MODIFIEDDAYS] * 86400000)).toLocaleDateString(), { pos: ttpos }));
											}
										}
										*/
										if (settings.rightClickProtect) {
											card.on('contextmenu', function(e) {
													e.preventDefault()
													return false;
												});
										}
									}
									
									lazyloadThumbs();
								};
									
					
						// Creating new Unique ID if not yet exists (needed??)
						if (!ns) {
							currentOly.data('oly_ns', ns = 'oly_' + Math.floor(Math.random() * 10000));
						}
						
						if (currentOly.hasClass('thumbs-right')) {
							ttpos = [ 2, 1, 0, 1 ];
						} else if (currentOly.hasClass('thumbs-left')) {
							ttpos = [ 0, 1, 2, 1 ];
						} else if (currentOly.hasClass('thumbs-top')) {
							ttpos = [ 1, 0, 1, 2 ];
						}
							
						initThumbs();
						
						thumbnails.thumbScroll();
						
						// Lazyloading
						thumbnails.on('lazyloadThumbs', lazyloadThumbs);
						
						// Moving the active element into view
						currentOly.on('setActiveThumb', setActiveThumb);
						
						if ($.isFunction(complete)) {
							if (typeof autoStart !== UNDEF) {
								if (typeof baseSet !== UNDEF) {
									complete.call(undefined, thumbnails, autoStart, baseSet);
								} else {
									complete.call(undefined, thumbnails, autoStart);
								}
							} else {
								complete.call(undefined, thumbnails);
							}
						}
						
						return currentOly;
					},
			
				/********************************************************
				 *
				 *					Preparing images
				 *
				 ********************************************************/
				 
				prepareImages = function(thumbs, baseSet) {
					
						if (!album) {
							return;
						}
						/*					
						if (album.hasShop() || settings.feedback) {
							
							if (thumbs.find('.selectable').length) {
								thumbs.selectable({
										cardClass:		'selectable',
										checkedClass:	settings.checkedClass,
										anySelected:	function() {
																$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass).removeClass('disabled');
															},
										noneSelected:	function() {
																$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass).addClass('disabled');
															}
									});
								
								$('#shop,#feedback').show();
								
								$('.' + settings.selectAllClass).removeClass('disabled');
								
							} else {
								
								$('#shop,#feedback').hide();
							}
							
							$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass).addClass('disabled');
						}
						*/
						// $('.' + settings.startshowClass + ' .button').off('click');
						
						/*if (!thumbs.find('.' + settings.lbableClass).length) {
							
							$('.' + settings.startshowClass).fadeOut();
							$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass + ',.' + settings.selectAllClass).addClass('disabled');
							
						} else {
							thumbs.trigger('lightboxRemove');
							*/
							
						currentOly.lightbox(album, {
								
									onLoadStart:	function(n) {
															if (!coverOn()) {
																var item = thumbs.find('.thumb.lbable').eq(n).data(J.OBJ);
																addParam(settings.indexName, {
																		img: 	album.getItemName(item)
																	}, item[J.TITLE]);
															} else {
																removeParam(settings.indexName, 'img');
															}
														},
													
									onLoadEnd: 		function(n) {
															thumbs.trigger('setActiveThumb', n);
														},
													
									onReady:		function() {
										
															if (currentOly.data('base') && settings.autoStart) {
																hideCover();
																startAutoplay();
															} else {
																stopAutoplay();
															}
															
															if (initByHash && window.location.hash) {
																initByHash = false;
																var param = readParam();
																
																currentOly.removeClass(settings.coverDownClass).addClass(settings.coverUpClass);
																
																if (param.hasOwnProperty('label') && param.label !== null) {
																	// Label filter
																	getItemsByLabel(param.label, function() {
																		addParam(settings.indexName, {
																			label:		param.label + '',
																			search:		null,
																			newimages:	null
																		}, text.label + ': ' + param.label);
																		currentOly.trigger('loadImage', param.img? getThumbByName(param.img) : 0);
																	});
																} else if (param.hasOwnProperty('search') && param.search !== null) {
																	// Search
																	getItemsBySearch(param.search, function() {
																		addParam(settings.indexName, {
																			search:		param.search + '',
																			label:		null,
																			newimages: 	null
																		}, text.search + ': ' + param.search);
																		currentOly.trigger('loadImage', param.img? getThumbByName(param.img) : 0);
																	});
																} else if (param.hasOwnProperty('newimages')) {
																	// Search
																	getItemsByDate(param.newimages, function() {
																		addParam(settings.indexName, {
																			newimages:	param.newimages,
																			search:		null,
																			label:		null
																		}, text.searchNew + ': ' + param.searchNew);
																		currentOly.trigger('loadImage', param.img? getThumbByName(param.img) : 0);
																	});
																} else if (param.hasOwnProperty('date')) {
																	var options = getDateRangeOptions(param.date);
																	// Search images in date range
																	getItemsByDate(options, function() {
																		addParam(settings.indexName, {
																				date:			param.date,
																				newimages:		null,
																				search:			null,
																				label:			null
																			}, getDateRangeTitle(options.reference) + ': ' + param.date);
																	});
																} else if (param.hasOwnProperty('img') && param.img !== null) {
																	var el = getThumbByName(param.img);
																	
																	if (el.length) {
																		currentOly.trigger('loadImage', el);
																	} else {
																		removeParam(settings.indexName, 'img');
																	}
																}
																
															} else {
																
																currentOly.trigger('loadImage', 0);
															}
															
															$('.' + settings.startshowClass).fadeIn().children('.button').on('click', startAuto);
														},
														
									onQuit:			function() {
															showCover();
															stopAutoplay();
														},
													
													
									onSlideshowStart:
													function() {
															currentOly.removeClass(settings.pausedClass).addClass(settings.playingClass);
															main.removeClass('paused').addClass('playing');
															if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
																$(settings.backgroundMusic.hook).trigger('startPlayer');
															}
														},
													
									onSlideshowPause:
													function() {
															currentOly.removeClass(settings.playingClass).addClass(settings.pausedClass);
															main.removeClass('playing').addClass('paused');
															if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
																$(settings.backgroundMusic.hook).trigger('pausePlayer');
															}
														}
								});
					},
					
				// Hash change listener :: when the browser's back / forward buttons were pressed
				
				stateChange = function() {
						if (window.location.hash === lastHash) {
							return;
						}
						
						lastHash = window.location.hash;
						var param = readParam();
						
						if (param.hasOwnProperty('label')) {
							// Label filter
							getItemsByLabel(param.label, function() {
								setParam(settings.indexName, {
										label:	param.label
									}, text.label + ': ' + param.label);
								updateShares();
							});
						} else if (param.hasOwnProperty('search')) {
							// Search
							getItemsBySearch(param.search, function() {
								setParam(settings.indexName, {
										search:	param.search
									}, text.search + ': ' + param.search);
								updateShares();
							});
						} else if (param.hasOwnProperty('newimages')) {
							// New images
							getItemsByDate($.extend(settings.newImages, param.newimages), function() {
								setParam(settings.indexName, {
										newimages: param.newimages
									}, text.newImages + ': ' + param.newimages);
								updateShares();	
							});
						} else if (param.hasOwnProperty('date')) {
							// Date range
							var options = getDateRangeOptions(param.date);
							getItemsByDate(options, function() {
								setParam(settings.indexName, {
										date: param.date
									}, getDateRangeTitle(options.reference) + ': ' + param.date);
								updateShares();	
							});
						} else if (param.hasOwnProperty('img')) {
							// Image (no search or label)
							var el = getThumbByName(param.img);
							if (el.length) {
								currentOly.trigger('loadImage', el);
							} else {
								removeParam(settings.indexName, 'img');
							}
						} else {
							// Plain index page
							showCover();
							removeOverlay();
						}
						
					},
					
				// Starts slideshow
				
				startAuto = function(e) {
						var ct = getImages('.' + settings.activeClass + ':first-child');
						
						if (!ct.length) {
							ct = getImages('.' + settings.thumbClass + ':first-child');
						}
						
						if (ct.length) {
							currentOly.trigger('loadImage', [ ct, true ]);
						}
						
						return false;
					},
				
				/********************************************************
				 *
				 *		Functionalities that results in new overlay
				 *
				 ********************************************************/
				
				/********************************************************
				 *
				 *				Keeping only selected items
				 *
				 ********************************************************/
				
				keepSelectedItems = function(tx, ready) {
						var items = getSelectedItems();
						
						if (!items.length) {
							return;
						}
						
						var t = addOverlay({
									className:	'selected-items',
									icon:		'icon-checkbox-checked',
									title:		'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						setTimeout(function() {
								h.find('.progressbar').remove();
								h.append($('<small>', {
										text: 		text.foundNTimes.replace('{0}', items.length)
									}));
								renderImages(items, prepareImages);
								//toggleSelectButtons(0, items.length);
								if (ready) {
									ready.call();
								}
							}, 50);
					},
				
				/********************************************************
				 *
				 *					Search and tag search
				 *
				 ********************************************************/
				 
				// Executing Search
				
				getItemsBySearch = function(tx, ready) {
						var t = addOverlay({
									className:	'search-results',
									icon:		'icon-search',
									title:		'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						setTimeout(function() {
							album.collectItems($.extend(settings.search, {
								exact:		false,
								terms:		tx,
								ready:		function() {
													h.find('.progressbar').remove();
													h.append($('<small>', {
															text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
													renderImages(this, prepareImages);
													if (ready) {
														ready.call();
													}
												}
							}));
						}, 50);
					},
					
				// Preparing Search functionality
				
				prepareSearch = function(f) {
					
						if (!f.length || !album) {
							return;
						}
						
						// Adding defaults if missing
						
						settings.search = $.extend({
								depth:		'tree',
								fields:		'creator,keywords,title,comment'
							}, settings.search);
						
						var startSearch = function(f) {
								
								var tx = f.find('input').eq(0).val();
								
								if (tx && tx.length > 1) {
									f.closest('.panel').add('aside .btn').removeClass('on');
									getItemsBySearch(tx, function() {
											setParam(settings.indexName, {
													search:		tx
												}, text.search + ': ' + tx);
											updateShares();
										});
								}
								return false;
							};
							
						f.find('.search-btn, button').on('click', function() {
								if (f.hasClass(settings.activeClass)) {
									startSearch(f);
									f.removeClass(settings.activeClass);
								} else {
									f.find('input').eq(0).focus();
									f.addClass(settings.activeClass);
								}
								return false;
							});
						
						f.on('submit', function(e) {
								e.preventDefault();
								startSearch(f);
								f.removeClass(settings.activeClass);
								return false;
							});
						
						f.find('input').on({
								'focus': 	function() { 
													f.addClass(settings.activeClass); 
												}
							});
					},
				
				// Preparing Tag Search functionality
				
				prepareTagSearch = function() {
					
						var f = $(settings.tagCloud.searchHook);
						
						if (!f.length || !album) {
							return;
						}
										
						var startSearch = function(f) {
								
								var tx = f.find('input').eq(0).val();
								
								if (tx && tx.length > 1) {
									f.closest('.panel').add('aside .btn').removeClass('on');
									getItemsByLabel(tx, function() {
										setParam(settings.indexName, {
												label: 	tx
											}, text.label + ': ' + tx);
										updateShares();
									});
								}
								return false;
							};
							
						f.find('.search-btn, button').on('click', function() {
							f.find('input').eq(0).focus();
							startSearch(f);
							return false;
						});
						
						f.on('submit', function(e) {
							e.preventDefault();
							startSearch(f);
							return false;
						});
						
					},
				
				
				/********************************************************
				 *
				 *						Tag cloud
				 *
				 ********************************************************/
				 
				// Do a label filtering
				
				getItemsByLabel = function(tx, ready) {
						var t = addOverlay({
									className:		'tags-found',
									icon:			'icon-tag',
									title:			'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
							'class': 'progressbar'
						}));
						
						if (t.offset().top < $window.scrollTop() || t.offset().top > ($window.scrollTop() + $window.height() - 100)) {
							$window.scrollTop(t.offset().top);
						}
						
						album.collectItems($.extend(settings.tagCloud, {
							exact:		settings.exactFields,
							terms:		tx,
							ready: 		function() {
												h.find('.progressbar').remove();
												h.append($('<small>', {
													text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
												}));
												renderImages(this, prepareImages);
												if (ready) {
													ready.call();
												}
											}
						}));
					},
	
				// Collecting and displaying tags
			
				renderTagCloud = function(tags) {
					
						if (!tags.length) {
							return;
						}
						
						var t = $(settings.tagCloud.hook);
						
						if (!t.length) {
							return;
						}
						
						var tagClicked = function() {
								var tx = $(this).children('span').eq(0).text();
								
								$(this).closest('.panel').add('aside .btn').removeClass('on');
								
								if (tx && tx.length > 1) {
									getItemsByLabel(tx, function() {
										setParam(settings.indexName, {
												label: 		tx
											}, text.label + ': ' + tx);
										updateShares();
									});
								}
								
								return false;
							};
						
						for (var a, r, cnt, i = 0, l = tags.length; i < l; i++) {
							
							// create link element
							a = $('<a>', {
									'class': 	'tag'
								}).append($('<span>', {
									html: 		tags[i][0]
								}));
							a.on('click', tagClicked);
							
							// add count
							if (tags[i][1] > 1) {
								if (settings.tagCloud.fontVaries) {
									r = 0;
									cnt = Math.min(64, tags[i][1]);
									// calculate weight
									while (cnt > 1) {
										r++;
										cnt = cnt >> 1;
									}
									a.addClass('size-' + r);
								}
								a.append('&nbsp;').append($('<sup>', {
									text: 		tags[i][1]
								}));
							}
							
							t.append(a);
						}
					},
	
				// Search for new images
				
				getSearchNewTitle = function(r) {
				
						if (r == 'dateTaken') 
							return text.imagesTaken;
						else if (r == 'fileModified')
							return text.imagesModified;
						return text.imagesAdded;
					},
					
				/********************************************************
				 *
				 *						Search new images
				 *
				 ********************************************************/
				 
				// Search for new images
				
				getDateRangeOptions = function(range) {
						var m = range.match(/^([amt])?([cts])?\:?(\d+)\-?(\d+)?/i),
							o = {
									range:		0,
									reference:	'dateTaken',
									depth:		'current'
								};
								
						if (m) {
							if (m[1]) {
								// Reference
								if (m[1] === 'a') {
									o.added = 'added';
								} else if (m[1] === 'm') {
									o.added = 'fileModified';
								} else if (m[1] === 't') {
									o.added = 'dateTaken';
								}
							}
							
							if (m[2]) {
								// Depth
								if (m[2] === 'c') {
									o.depth = 'current';
								} else if (m[2] === 't') {
									o.depth = 'tree';
								} else if (m[2] === 's') {
									o.depth = 'subfolders';
								}
							}
							
							if (m[4]) {
								// Range
								o.start = parseInt(m[3]) || 0;
								o.end = parseInt(m[4]);
							} else if (m[3]) {
								// Single day
								o.start = parseInt(m[3]);
								o.range = 1;
							}
						}
						
						return o;
					},
				
				getDateRangeTitle = function(o) {
						var t1,
							t2,
							start,
							end,  
							today = Math.floor(new Date() / ONEDAY_MS),
							date = function(d) {
									return new Date(d * ONEDAY_MS).toLocaleDateString(LOCALE);
								};
						
						if (o.reference === 'dateTaken') { 
							t1 = text.imagesTaken;
						} else if (o.reference === 'fileModified') {
							t1 = text.imagesModified;
						} else {
							t1 = text.imagesAdded;
						}
						
						if (!o.hasOwnProperty('end') && !o.hasOwnProperty('start')) {
							t2 = getTimespan(o.range);
						} else if (!o.hasOwnProperty('end')) {
							if (o.hasOwnProperty('start')) {
								if (o.hasOwnProperty('range')) {
									if (o['range'] > 1) {
										t2 = text.betweenDays.template(date(o.start), date(o.start + o.range));
									} else {
										t2 = text.onDay.template(date(o.start));
									}
								} else {
									t2 = text.afterDay.template(date(o.start));
								}
							}
						} else {
							t2 = text.beforeDay.template(date(o.end));
						}	
						
						return t1 + ' ' + t2;
					},
				
				getItemsByDate = function(options, ready) {
						var t = addOverlay({
									className:		'search-results',
									icon:			'icon-calendar',
									title:			getDateRangeTitle(options)
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 		'progressbar'
							}));
						
						setTimeout(function() {
								album.collectByDate($.extend(options, {
									ready:		function() {
														h.find('.progressbar').remove();
														h.append($('<small>', {
															text: this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
														renderImages(this, prepareImages);
														//toggleSelectButtons(0, this.length);
														if (ready) {
															ready.call();
														}
													}
								}));
							}, 50);
					},
					
				// Search date range
				
				searchDateRange = function(range) {
						if (range) {
							var options = getDateRangeOptions(range);
							getItemsByDate(options, function() {
									setParam(settings.indexName, {
											date:		range
										}, getDateRangeTitle(options));
									updateShares();
								});
						}
						return false;
					},
				
				// Search new images
				
				searchNew = function(range) {
						
						if (range) {
							
							getItemsByDate($.extend(settings.searchNew, range), function() {
									setParam(settings.indexName, {
											newimages:		range
										}, getDateRangeTitle(options));
									updateShares();
								});
						}
						return false;
					},
				
				// Prepare Search new functionality
				
				prepareSearchNew = function(f) {
					
						if (!f.length || !album) {
							return;
						}
						
						// Setting up defaults if something is missing
						
						settings.searchNew = $.extend({
								days: 				'3,30,90',
								depth: 				'folder',
								reference: 			'dateTaken',
								sinceLastVisit:		true
							}, settings.searchNew);
						
						var	days = settings.searchNew.days.split(',');
						
						f.append($('<label>', {
								text: 	getDateRangeTitle(settings.searchNew.reference)
							}));
								
						for (var i = 0; i < days.length; i++) {
							f.append($('<a>', {
									'class': 	'smallbtn',
									text: 		getTimespan(days[i])
								})
								.data('days', parseInt(days[i], 10))
								.on('click', function() {
									searchNew($(this).data('days'));
								}));
						}
								
						if (settings.searchNew.sinceLastVisit) {
							
							var start = $.cookie('lastVisit'),
								now = Math.round(new Date() / 1000);
							
							if (start && start < (now - ONEDAY_S)) {
								f.append($('<a>', {
										'class': 	'smallbtn',
										text: 		text.sinceMyLastVisit
									})
									.on('click', function() {
										searchNew((now - start) / ODEDAY_S);
									}));
							}
							
							// Storing last visit date for 10 years
							$.cookie('lastVisit', now, 315360000);
						}
						
					},
					
				/********************************************************
				 *
				 *						Misc functions
				 *
				 ********************************************************/
										
				// Updating the share buttons after a state change
				
				updateShares = function() {
						if (settings.hasOwnProperty('share') && getImages().trigger('lightboxVisible')) {
							$(settings.share.hook).trigger('updateLinks');
						}	
					},
					
				/********************************************************
				 *
				 *						Controls
				 *
				 ********************************************************/
				
				// Go fullscreen
				
				goFullscreen = function() {
						
						$(this).trigger('removeTooltip');
						$body.addClass(settings.fullscreenClass);
						$('html').fullScreen(true);
						
					},

				// Exit fullscreen
				
				exitFullscreen = function() {
						
						$(this).trigger('removeTooltip');
						$body.removeClass(settings.fullscreenClass);
						$('html').fullScreen(false);
						
					},
					
				// Controlling the skin states
				
				startAutoplay = function() {
						if (!autoplayOn()) {
							currentOly.trigger('startAuto');
							currentOly.removeClass(settings.pausedClass).addClass(settings.playingClass);
							main.removeClass(settings.pausedClass).addClass(settings.playingClass);
						}
					},
					
				stopAutoplay = function() {
						if (autoplayOn()) {
							currentOly.trigger('stopAuto');
							currentOly.removeClass(settings.playingClass).addClass(settings.pausedClass);
							main.removeClass(settings.playingClass).addClass(settings.pausedClass);
						}
					},
					
				autoplayOn = function() {
						return currentOly.hasClass('playing');
					},
					
				hideCover = function() {
						$('.autoplay').trigger('removeTooltip');
						var t = getImages().find('.thumb.lbable.active');
						if (t.length) {
							var item = t.data(J.OBJ);
							addParam(settings.indexName, {
								img: 	album.getItemName(item)
							}, item[J.TITLE]);
						}
						currentOly.removeClass(settings.coverDownClass).addClass(settings.coverUpClass);				
					},
					
				showCover = function() {
						if (currentOly.data('base')) {
							removeParam(settings.indexName, 'img');
							currentOly.removeClass(settings.coverUpClass).addClass(settings.coverDownClass);
						}
					},
					
				coverOn = function() {
						return currentOly.data('base') && !currentOly.hasClass(settings.coverUpClass);
					},
					
				showPanel = function(p) {
					
						if (!p || !p.length) {
							return;
						}
						
						var btn = $('aside .sidebtn[data-rel=' + p[0].id + ']'),
						
							init = function()  {
									var at = btn.length? btn.parent().position().top : 0,
										bt = btn.length? btn.position().top : 0;
									
									p.addClass('on')
										.css({
												transition:		'none',
												display: 		'flex',
												opacity:		0,
												transform:		'translateX(-16px)'
											});
										 
									window.requestAnimationFrame(function() {
											console.log('top: ' + bt + ' main.height: ' + main.outerHeight() + ' panel.height: ' + p.outerHeight());
											p.css({
													top:			Math.max(0, Math.min(bt, main.outerHeight() - p.outerHeight()))
												});
											
											p.one('transitionend', function() {
													p.css({
															display:		'flex',
															opacity:		1
														});
												})
												.css({
														transition:		'opacity 500ms, transform 500ms ease-out',
														opacity:		1,
														transform:		'translateX(0)'
													});
										});
								};
								
						window.requestAnimationFrame(init);
					},
					
				hidePanel = function(p) {
						
						if (!p || !p.length || p.css('position') === 'static') {
							// don't remove the folder navigation in the sidebar 
							return;
						}
						
						var btn = $('aside .sidebtn[data-rel=' + p[0].id + ']');
						
						$('aside').hideAllTooltips();
						
						window.requestAnimationFrame(function() {
								p.one('transitionend', function() {
											p.removeClass('on')
												.css({
														transition:		'none',
														display: 		'none'
													});
											btn.removeClass('on');
										})
									.css({
											transition:		'opacity 500ms, transform 500ms ease-in',
											opacity:		0,
											transform:		'translateX(-16px)'
										});
							});
					},
					
				hideAllPanels = function() {
					
						$('.panel:visible').each(function() {
								hidePanel($(this));
							});
					},
									
				/********************************************************
				 *
				 *						Initialization
				 *
				 ********************************************************/
				 
				// Reading out map locations
				
				getMarkers = function(images) {
						
						var markers = [];
						
						for (var i = 0; i < images.length; i++) {
							if (images[i].hasOwnProperty('location')) {
								markers.push({
										title:		[(i + '.'), images[i][J.TITLE], images[i][J.COMMENT]].join(' '),
										pos: 		images[i][J.LOCATION],
										link:		images[i][J.NAME]
									});
							}
						}
						
						return markers;
									
					},
					
				// Cookie policy
				/*
				showCookiePolicy = function() {
				
						if (!$.cookie('cookiePolicy')) {
							var	el = $('<div>', { 
										id: 		'cookiepolicy' 
									}).appendTo($body),
								p = $('<p>', { 
										html: 		text.cookiePolicyText 
									}).appendTo(el);
							
							if (settings.cookiePolicyUrl) {
								p.append($('<a>', { 
										text: 		text.cookiePolicyLearnMore, 
										target: 	'_blank', 
										href: 		settings.cookiePolicyUrl 
									}));
							}
							
							p.append($('<a>', {
									'class': 		'smallbtn',
									text: 			text.cookiePolicyAgree 
								}).on('click', function(){ 
									$('#cookiepolicy').fadeOut(500, function() { 
										$(this).remove(); 
									});
									$.cookie('cookiePolicy', true, 36000000); // expires: 10000 days
								}));
							
							el.fadeIn(500);
							setTimeout(function() {
								$('#cookiepolicy').fadeOut(500, function() { 
									$(this).remove(); 
								});
							}, 6000);
						}			
					},
				*/
					
				// Initializing controls
				
				initControls = function() {
					
						// Full screen
						
						$('#fullscreen-btn').on('click', function() {
								if (!document.fullscreenElement) {
									main.addClass('fullscreen');
									document.documentElement.requestFullscreen().catch(function(err) {
										console.log('Error going full-screen: ${err.message} (${err.name})');
									});
								}
							});
						
						$('#fullscreen-exit-btn').on('click', function() {
								if (document.fullscreenElement) {
									main.removeClass('fullscreen');
									document.exitFullscreen().catch(function(err) {
										console.log('Error exiting full-screen: ${err.message} (${err.name})');
									});
								}
							});
						
						// Auto start slideshow button events
						
						if (getImages().length) {
							// Autoplay with various buttons
							$('.btn.play,.btn.autoplay').on('click', function() {
									if (SMALLSCREEN && settings.useFullScreen && !document.fullscreenElement) {
										document.documentElement.requestFullscreen().catch(function(err) {
												console.log('Error going full-screen: ${err.message} (${err.name})');
											});
									}
									hideCover();
									startAutoplay();
								});
							
							$('.album-title .folder-image>.image').on('click', hideCover);
							
							// Pause button
							$('.btn.pause').on('click', function() {
									stopAutoplay();
								});
								
							// Toggling the cover & autoplay when clicking the actual folder
							$('.topnav .' + settings.actualClass).on('click', function(e) {
									if ($(e.target).closest('li').hasClass(settings.actualClass)) {
										if (coverOn()) {
											if (SMALLSCREEN && settings.useFullScreen && !document.fullscreenElement) {
												document.documentElement.requestFullscreen().catch(function(err) {
														console.log('Error going full-screen: ${err.message} (${err.name})');
													});
											}
											hideCover();
											startAutoplay();
										} else {
											stopAutoplay();
											showCover();
										}
										return false;
									}
								});
							
							// Handle
							$('.hero-handle').on('click', function() {
									if (coverOn()) {
										if (SMALLSCREEN && settings.useFullScreen && !document.fullscreenElement) {
											document.documentElement.requestFullscreen().catch(function(err) {
													console.log('Error going full-screen: ${err.message} (${err.name})');
												});
										}
										hideCover();
									} else {
										stopAutoplay();
										showCover();
									}
								});
							
						}
						
						// Closing panels
						
						$('.panel>.close').on('click', function() {
								hidePanel($(this).closest('.panel'));
							});
						
						// Sidebar buttons
						
						$('aside .sidebtn').on('click', function() {
								
								if (this.id.startsWith('fullscreen')) {
									return true;
								}
								
								if ($(this).hasClass('on')) {
									$(this).removeClass('on');
									hidePanel($('#' + $(this).data('rel')));
								} else {
									hideAllPanels();
									$('aside .sidebtn').removeClass('on');
									$(this).addClass('on');
									showPanel($('#' + $(this).data('rel')));
								}
							});
						
						// Tool tips
						$('[data-tooltip]').addTooltip();

					},
					
					
				// Preparing the album upon Album object is loaded
				
				prepareIndex = function() {
					
						if (!album) {
							return;
						}
						
						var images = album.getImages();
						
						// Removing thumbnail links right away
						
						getImages().each(function() {
								var a = $(this).find('.' + settings.imageClass);
								a.data('href', a.attr('href'));
								a.attr('href', '');
							});
						
						// Rendering thumbnails
						if (images.length) {
							
							renderImages(images, prepareImages, true);
							
							// Collecting markers and initializing map
							if (settings.hasOwnProperty('map') && settings.map.index) {
								
								$('.map-root').addMap({
										markers: 			getMarkers(images),
										type:				settings.map['type'] || 'roadmap',
										zoom:				settings.map['zoom'] || 16,
										apiKey:				settings.map['apiKey'],
										fitBounds:			true,
										fullscreenControl:	false,
										onMarkerClick:		function() {
																	if (this.hasOwnProperty('link')) {
																		removeOverlay();
																		currentOly.removeClass(settings.pausedClass + ' ' + settings.coverDownClass).addClass(settings.playingClass + ' ' + settings.coverUpClass);
																		currentOly.trigger('loadImage', getThumbByName(this.link, true));
																	}
																}
									});
								
								// Showing map only if map button clicked
								$('#map-btn').on('click', function() {
										if (!$('.map-root').data('rendered')) {
											setTimeout(function(btn) {
													if (btn.hasClass('on')) {
														$('.map-root').trigger('render');
													}
												}, 20, $(this));
										}
									});
							}
		
						} else if (window.location.hash) {
							// no thumbnails, only folders
							// we should check the search / tags / newimages filter
							stateChange();
						}
						
						// Collecting tags if gathered only from this folder
						if (settings.hasOwnProperty('tagCloud')) {
							
							if (settings.tagCloud['depth'] === 'current') {
								album.collectTags($.extend(settings.tagCloud, {
										exact:		settings.exactFields,
										ready:		function() {
															renderTagCloud(this);
														}
									}));
							}
							
							if (settings.tagCloud.hasOwnProperty('searchHook')) {
								prepareTagSearch();
							}
						}
						
						// Search 
						if (settings.hasOwnProperty('search')) {
							prepareSearch($(settings.search.hook));
						}
						
						// Search new
						if (settings.hasOwnProperty('searchNew')) {
							prepareSearchNew($(settings.searchNew.hook));
						}
								
						// Custom query (eg. new images sample box)
						var queryTarget = $('[data-jalbum-query]');
						
						if (queryTarget.length) {
							var qsettings = queryTarget.data('jalbum-query');
							if (qsettings.hasOwnProperty('fn')) {
								var imgs = album[qsettings.fn](qsettings);
								
							}
						}
					},
				
				// Preparing a custom page
				
				preparePage = function() {
						// Page :: initializing functions
						// Search 
						if (settings.hasOwnProperty('search')) {
							prepareSearch($(settings.search.hook));
						}
						var imgs = getImages();
						
						if (!album || !imgs.length) {
							return;
						}
						
						var paths = [];
						
						for (var i = 0; i < imgs.length; i++) {
							paths.push(imgs.data(J.PATH));
						}
						
						var items = album.collectItemsByPath(paths);
							
						// Removing thumbnail links right away
						
						imgs.each(function() {
								var a = $(this).find('.' + settings.imageClass);
								a.data('href', a.attr('href'));
								a.attr('href', '');
							});
			
					},
				
				// Preparing functions that need the full album tree
				
				prepareDeepFunctions = function() {
					
						// Collecting tags if gathered from deep tree
						if (settings.hasOwnProperty('tagCloud') && settings.tagCloud['depth'] !== 'current') {
							album.collectTags($.extend(settings.tagCloud, {
									exact:		settings.exactFields,
									ready:		function() {
														renderTagCloud(this);
													}
								}));
						}
						
						// Search 
						if (settings.hasOwnProperty('search')) {
							prepareSearch($(settings.search.hook));
						}
						
						// Search new
						if (settings.hasOwnProperty('searchNew')) {
							prepareSearchNew($(settings.searchNew.hook));
						}
					},
					
				// Marking new folders
				
				markFoldersNew = function() {
					
						if (!album) {
							return;
						}
						
						var folders = album.getFolders(),
							secs = settings.markNew['days'] * ONEDAY_S,
							s,
							d;
						
						if (folders.length) {
							getFolders('.' + settings.cardClass).each(function(i) {
									
									if (i < folders.length) {
										d = folders[i][J.DATES];
										
										if (d) {
											d = d[settings.markNew['reference']] || d[J.DATERANGE][1] || d[J.DATERANGE][0];
											
											if (d && (nows - d) < secs) {
												s = settings.markNew.text?
													$('<span>', {
															'class':	'new-image',
															text:		settings.markNew.text
														})
													:
													$('<span>', {
															'class':	'icon-new-fill new-image'
														});
													
												if (!TOUCHENABLED) {
													s.data('tooltip', (new Date(d * 1000)).toLocaleDateString(LOCALE)).addTooltip();
												}
												
												$(this).find('.' + settings.thumbClass).append(s);
											}
										}
									}
								});
						}
					},
					
				// Preparing folders
			
				prepareFolders = function() { 
					
						var scrollTimeout = null,
							folders = getFolders();
						
						if (!folders.children().length) {
							return;
						}
						
						if (settings.hasOwnProperty('markNew')) {
							setTimeout(markFoldersNew, 300);
						}
										
						lazyloadFolders();
					},
				
				// Initializing map
				
				initMap = function() {
						
						if (settings.hasOwnProperty('map')) {
							if (settings.map.hasOwnProperty('type')) {
								$.fn.lightbox.defaults.mapType = settings.map.type;
								$.fn.addMap.defaults.type = settings.map.type;
							}
							if (settings.map.hasOwnProperty('zoom')) {
								$.fn.lightbox.defaults.mapZoom = settings.map.zoom;
								$.fn.addMap.defaults.zoom = settings.map.zoom;
							}
							if (settings.map.hasOwnProperty('apiKey')) {
								$.fn.addMap.defaults.apiKey = settings.map.apiKey;
							}
						}
					},
					
				// Initializing Social sharing
				
				initShares = function() {
						
						if (settings.hasOwnProperty('share')) {
							$.fn.renderShares.defaults.buttonTheme = settings.share['buttonTheme'] || 'dark';
							$.fn.renderShares.defaults.facebookAppId = settings.share['facebookAppId'] || '';
							if (settings.share['callAction']) {
								$.fn.renderShares.defaults.callAction = settings.share['callAction'];
							}
							$(settings.share.hook).renderShares(settings.share);
							if (settings.share.sites !== 'facebook') {
								$.fn.lightbox.defaults.share = settings.share.sites.replace('facebook,', '').replace(',facebook', '');
							}
						}
						
					},
					
				// Initializing background music
				
				initBackgroundMusic = function() {
					
						if ($('[data-audioPlayer]').length) {
							$('[data-audioPlayer]').audioPlayer({
									rootPath: 	settings.rootPath
								});
						}
					},
					
				// Initializing album
				
				initAlbum = function(readyFn) {
					
						if (typeof Album === UNDEF) {
							console.log('Critical Error: Missing jalbum.album.js library!');
						}
						
						var params = {
								makeDate:		settings['makeDate'],
								rootPath:		settings['rootPath'],
								relPath:		settings['relPath'],
								ready: 			readyFn,
								deepReady:		prepareDeepFunctions
							};
							
						if (settings.indexName !== $.fn.skin.defaults.indexName) {
							params.indexName = settings['indexName'];
						}
						
						if (settings.audioPoster !== $.fn.skin.defaults.audioPoster) {
							params.audioPoster = settings['audioPoster'];
						}
							
						if (settings.videoPoster !== $.fn.skin.defaults.videoPoster) {
							params.videoPoster = settings['videoPoster'];
						}
						
						album = new Album($, params);
					},
					
				// Main initializing function
				
				init = function() {
					
						// Removing search string, e.g. FB data
						if (window.location.search) {
							removeSearch();
						}
				
						// Creating new Album
						if (settings.pageType === 'index') {
							// lightbox.js
							passDefaults(settings, $.fn.lightbox.defaults, 'indexName,level,uplink,uplinkText,autoStart,previousFoldersLast,nextFoldersFirst,rightClickProtect,useFullScreen,enableKeyboard,enableMouseWheel,extraSizes');
							passDefaults(settings.lightbox, $.fn.lightbox.defaults);
							
							// thumbscroll.js
							passDefaults(settings, $.fn.thumbScroll.defaults, 'activeClass,disabledClass,thumbContClass,thumbsPosition,enableMouseWheel');
							
							initAlbum(prepareIndex);
						} else if (settings.pageType === 'page') {
							initAlbum(preparePage);
						}
									
						initControls();
						
						initMap();
										
						initShares();
						
						initBackgroundMusic();
						
						// Hash change listener
						$window.on('hashchange.' + ns, stateChange);
					};	
			
			/********************************************************
			 *
			 *					Execution starts here
			 *
			 ********************************************************/
				
			init();
			
			// Utility function for debugging
			this.getAlbum = function() {
					return album;
				};
			
			return this;
		};
	
	$.fn.skin.defaults = {
			contentHook:					'article.cont',
			overlayClass:					'overlay',
			overlayTitleClass:				'title',
			overlayHeadLevel:				3,
			contClass:						'cont',
			coverUpClass:					'cover-up',
			coverDownClass:					'cover-down',
			playingClass:					'playing',
			pausedClass:					'paused',
			folderClass:					'folder',
			thumbsClass:					'thumbnails',
			clipClass:						'clip',
			thumbContClass:					'thumb-cont',
			thumbClass:						'thumb',
			actualClass:					'actual',
			cardClass:						'card',
			lbableClass:					'lbable',
			captionClass:					'caption',
			commentClass:					'comment',
			showImageClass:					'show-image',
			hideImageClass:					'hide-image',
			selectableClass:				'selectable',
			hascaptionClass:				'hascaption',
			checkedClass:					'checked',
			activeClass:					'active',
			preloadClass:					'preload',
			lazyloadClass:					'lazyload',
			startshowClass:					'startshow',
			selectAllClass:					'select-all',
			selectNoneClass:				'select-none',
			addSelectedClass:				'add-selected',
			scrollCheckInterval:			100,
			indexName:						'index.html',
			audioPoster:					'audio.poster.png',
			videoPoster:					'video.poster.png',
			exactFields:					'creator,keywords',
			thumbsPosition:					'top',
			autoStart:						false,
			rightClickProtect: 				false,
			useFullScreen:					true,
			enableKeyboard:					true
		};
		
})(jQuery, jQuery(window), jQuery(document), jQuery('body'));
