/*
 * 	lightbox.js - lightbox for Photoblogger skin
 *	jslint browser: true
 */
 
;(function($, $window, $document, $body, undefined) {	
	'use strict';
	// Requires laza.util, laza.swipe and laza.transform
	
	$.fn.lightbox = function(album, settings) {
		
		if (typeof album === UNDEF) {
			return this;
		}
		
		settings = $.extend({}, $.fn.lightbox.defaults, settings);

		var self = $(this),
			text = getTranslations($.fn.lightbox.text),
			thumbs = $([]),
			thumb = $(),
			main = $(),
			image = $(),
			media = $(),
			caption = $(),
			controls = $(),
			panels = $(),
			el,
			isImage = true,
			isVideo = false,
			isAudio = false,
			isOther = false,
			isExternal = false,
			isVr = false,
			isPdf = false,
			isPanorama = false,
			swiped = false,
			dontRemove = false,
			slideshow = null,
			fitTimeout = null,
			controlsTimeout = null,
			resizeTimeout = null,
			autoPanoTimeout = null,
			
			listening = false,
			
			zoomLevel,
			
			curr = -1,
			loadCounter = 0,
			loadTimer,
			direction = 0,
			mainPadding,
			fitPadding = settings['fitPadding'], 
			
			// Elements
			lightbox,
			lb_overlay,
			lb_activity,
			
			// Controls
			lb_btn_left,
			lb_close,
			lb_zoom,
			lb_play,
			lb_pause,
			lb_btn_right,
			
			// Caption
			lb_caption,
			
			ns = self.data('llb_ns'),
			extraSizes = settings.hasOwnProperty('extraSizes')? settings.extraSizes.split(/,\s*/) : false,
			
			backgroundAudioRoot = $('[data-audioplayer]'),
			backgroundAudioOn = backgroundAudioRoot.length && !backgroundAudioRoot.data('paused'),

			cleanup = function(ns) {	
					// cleaning up the old event handlers
					$window.add(document).off('.' + ns);
					// Removing the lightbox :: you can only have one lightbox per thumb set
					$('#' + ns).remove();
				},
			
			closeModal = function(id) {
					lightbox.find('.modal').trigger('destroy');
				},
			
			// Sets zoom (0 .. 100), where 100 means 1:1 size
			
			setZoom = function(zoom) {
					console.log('Set zoom:' + zoom);
					zoomLevel = zoom;
					
					
				},
				
			// Recalculating image position and size upon window resize
			
			fitImage = function(initial) {
				
					clearTimeout(fitTimeout);
					clearTimeout(resizeTimeout);
					
					if (!main || !main.length) {
						return;
					}
					
					var dim = main.data('dims'),
						w,
						h,
						o = thumbs.eq(curr).data(J.OBJ),
						lbw = lightbox.width(),
						lbh = lightbox.height(),
						lbah,							// Available height
						ratio,
						fillRatio,
						ratioX,
						ratioY,
						isPano = false,
						controlsHeight,
						canZoomIn,
						canZoomOut,
						img = image[0],
						defaultPoster = image.hasClass('default');
					
					if (typeof mainPadding === UNDEF) {
						// Picture padding: border
						mainPadding = parseInt(main.css('paddingTop'));
					}
					
					if (typeof fitPadding === UNDEF) {
						// Image margin defined by "fitRatio" - Photoblogger
						fitPadding = Math.round((1 - settings.fitRatio) * Math.min(lbw, lbh) / 2);
					}
					
					if (!dim) {
						
						// Has no dimensions saved yet
						dim = album.getDimensions(o);
						main.data('dims', dim);
					}						
					
					w = dim[0];
					h = dim[1];
					
					// Discounting padding and image border
					
					lbw -= 2 * (fitPadding + mainPadding);
					lbh -= 2 * (fitPadding + mainPadding);
					
					// Discounting controls height
					
					if (settings.fitImages && settings.fitBelowControls && !slideshow && controls.length && controls.is(':visible')) {
						lbah = lbh - (controlsHeight = Math.ceil(controls.outerHeight()));
					} else {
						controlsHeight = 0;
						lbah = lbh;
					}
						
					if (isVr || isPdf) {							
						// Full screen content
						
						w = Math.min(lbw, w);
						h = Math.min(lbah, h);
						
						ratio = 1;
						
						canZoomIn = canZoomOut = false;
						
					} else if (isExternal) {
						
						ratioX = Math.min(lbw / w, settings.maxZoom);
						ratioY = Math.min(lbah / h, settings.maxZoom);
						
						// Can scale up or max 1:1
						ratio = Math.min(ratioX, ratioY);
						
						w *= ratio;
						h *= ratio;
						
						canZoomIn = w < lbw && h < lbh;
						canZoomOut = !canZoomIn && ratio > 1;
						
					} else if (isAudio && defaultPoster) {			
						// Audio with poster image
						
						w = Math.min(settings.defaultPosterSize[0], lbw);
						h = Math.min(settings.defaultPosterSize[1], lbah);
						
						ratio = 1;
						
						canZoomIn = canZoomOut = false;
						
					} else {										
						// Image, Audio, Video, Other
						
						ratioX = Math.min(lbw / w, settings.maxZoom);
						ratioY = Math.min(lbah / h, settings.maxZoom);
						
						// Fit to screen or 1:1 ?
						
						if (settings.fitImages) {
							// Fit images to screen (normal and panorama)
							
							if (settings.fitBoth || h < lbh) {
								// Fit both mode or small panorama: fit to screen
								ratio = Math.min(ratioX, ratioY);
							} else {
								// Verticaly fit only?
								ratio = settings.fitBoth? Math.min(ratioX, ratioY) : ratioY;
							}
							
							// Can scale up or max 1:1
							ratio = Math.min(settings.scaleUp? settings.maxZoom : 1, ratio);
							
							// Panorama if "fit vertical" mode and wide enough
							isPano = 	isImage 
										&& 
										(	
											(
												settings.autoPano 
												&& 
												!settings.fitBoth 
												&& 
												w * ratio >= lbw * settings.panoramaAutoDetect
											)
											||
											(
												isPanorama
												&&
												w * ratio > lbw
											)
										);
							
							w = Math.round(w * ratio);
							h = Math.round(h * ratio);
							
							canZoomIn = !isPano && ratio < 1 || isPano && !settings.fitBoth;
							
						} else {
							
							// 1:1 or pano
							ratio = 1;
							
							// Checking panorama
							if (w > lbw || h > lbah) {
							
								// Fit for the largest AR (fill screen) to test panorama
								fillRatio = Math.max(ratioX, ratioY);
								
								// Testing panorama
								isPano = 	isImage 
											&& 
											(	
												(
													// panorama set by user
													isPanorama
													&& 
													(
														// hangs over the edges
														w * fillRatio > lbw 
														|| 
														h * fillRatio > lbah
													)
												)
												||
												(
													// ... or auto pano
													settings.autoPano
													&&
													(
														(	
															// ignoring those that doesn't qualify as panorama
															w / h > 2
															||
															w / h < .5625
														)
														&& 
														(	
															// hangs over the edges least 1.5 times the width or height
															(w * fillRatio / lbw) >= settings.panoramaAutoDetect 
															|| 
															(h * fillRatio / lbah) >= settings.panoramaAutoDetect
														)
													)
												)
											);
											
								if (isPano) {
									// display in fill or 1:1 mode?
									if (settings.fitBoth) {
										ratio = (w > h)? ratioY : ratioX;
										w = Math.round(w * ratio);
										h = Math.round(h * ratio);
									}
									
									canZoomIn = false;
									
								} else {
									
									canZoomIn = settings.scaleUp && ratio < 1;
									
								}
								
							} else {
								
								canZoomIn = settings.scaleUp && w < lbw && h < lbah;
								
							}
							
						}
						
						canZoomOut = ratio > 1 || w > lbw || h > (lbah - (settings.fitAboveCaption? caption.outerHeight() : 0));
					}
					
					// Sizing
					
					if (initial === true) {
						
						main.css({
								transition:	'none',
								left: 		fitPadding + Math.round((lbw - w) / 2),
								top: 		fitPadding + controlsHeight + Math.round((lbah - h) / 2),
								width: 		w + 2 * mainPadding,
								height: 	h + 2 * mainPadding,
								opacity: 	0
							}).show();
							
					} else {
						
						main.css({
								transition:	'all 250ms linear', 
								left: 		fitPadding + Math.round((lbw - w) / 2),
								top: 		fitPadding + controlsHeight + Math.round((lbah - h) / 2),
								width: 		w + 2 * mainPadding,
								height: 	h + 2 * mainPadding
							}).on('transitionEnd', function() {
								// Reset center
								if (!isPano) {
									$(this).translateX(0, 500);
								}
							});
						
					}
					
					// Auto panorama attach or detach
					
					if (isImage) {
						
						if (isPano) {
							
							autoPanoTimeout = setTimeout(function() {
									if (isPano) {
										main.autopano({ 
												direction: 	direction 
											});
									} else {
										main.trigger('autopanoRemove');
									}
								}, 550);
								// Delay by transTime = 500ms, necessary to allow time to finish zoom, otherwise pano calculates with wrong dimensions 
							
						} else {
							
							clearTimeout(autoPanoTimeout);
							main.trigger('autopanoRemove');
							
						}
					}
					
					//log('fit:'+lightbox.width() + ':' + lightbox.height());
					//log('window:'+$window.width() + ':' + $window.height());
					
					$.cookie('fitImages', settings.fitImages? (ratio > 1) : (ratio < 1));
					
					if (canZoomIn) {
						lb_zoom.removeClass('icon-contract').addClass('icon-expand');
					} else {
						lb_zoom.removeClass('icon-expand').addClass('icon-contract');
					}
					
					lb_zoom.toggle(canZoomIn || canZoomOut);
							
				},
			
			// Removing the old image and caption
			
			removeOldContent = function() {
				
					var oldMain = lightbox.find('.' + settings.lightboxMainClass),
						oldCaption = lightbox.find('.' + settings.lightboxCaptionClass);
						
					if (oldMain.length) {
						
						oldMain.trigger('autopanoRemove').off('.' + ns);
						
						if (settings.muteBackgroundAudio && backgroundAudioOn) {
							if (isAudio || isVideo) {
								backgroundAudioRoot.trigger('fadeOutPlayer');
							} else {
								backgroundAudioRoot.trigger('fadeInPlayer');
							}
						}
						
						// Removing the old image
						if (!swiped) {
							oldMain.translateXAndFade(-100 * direction, 0, settings.speed, function() {
								oldMain.remove();	
							});
						}
						
						oldMain.off('.' + ns);
					}
					
					if (oldCaption.length) {
						
						// Removing the old caption
						oldCaption.find('.buttons a').trigger('removeTooltip');
						
						if (oldCaption.length > 1) {
							oldCaption.eq(-1).prevAll('.' + settings.lightboxCaptionClass).remove();
							oldCaption = lightbox.find('.' + settings.lightboxCaptionClass);
						}
						
						oldCaption.opacity(0, settings.speed, function() {
							oldCaption.remove();	
						});
					}
				},
			
			// Reading image type
			
			getImageType = function(item) {
					isVideo = isAudio = isOther = isImage = isVr = isPdf = false;
					isExternal = item.hasOwnProperty(J.EXTERNAL);
					isVr = !isExternal && settings.use360Player && item.hasOwnProperty(J.PROJECTIONTYPE) && item[J.PROJECTIONTYPE] === 'equirectangular';
					isPanorama = item.hasOwnProperty(J.PANORAMA) && item[J.PANORAMA];
					if (!isExternal && !isVr) {
						switch (item[J.CATEGORY]) {
							case 'video':
								isVideo = true;
								break;
							case 'audio':
								isAudio = true;
								break;
							case 'other':
								isOther = true;
								isPdf = album.getExtension(item).toLowerCase() === 'pdf';
								break;
							default:
								isImage = true;
						}
					}
				},
			
			// Updates rating (if changed externally)
				
			updateRating = function(r) {
					
					if (curr >= 0) {
						if (typeof r === UNDEF) {
							r = thumbs.eq(curr).data(J.OBJ)[J.VISITORRATING] || 0;
						}
					
						lightbox.find('.' + settings.lightboxCaptionClass + ' .ratingbox span')
							.removeClass('r0 r1 r2 r3 r4 r5')
							.addClass('r' + r);
					}
				},
										
			// Creating caption and buttons
			
			prepareCaption = function(item) {
					
					caption = $('<div>', {
							'class': 	settings.lightboxCaptionClass + ' scrollable ' + (slideshow? ' slideshow' : '')
						})
						.hide()
						.appendTo(lightbox);
					
					var buttons = $('<div>', {
								'class': 	'buttons'
							});
					
					if (settings.useRating) {
						
						var r = item[J.RATING] || 0;
						
						if (settings.visitorRating || r) {
							var s;
						
							el = $('<div>', {
									'class':	'ratingbox'
								}).appendTo(buttons);
								
							el.append($('<label>', {
									html:		text.rating
								}));
								
							s = $('<span>', {
									'class':	'rating',
									html:		STARS
								})
								.appendTo(el);
								
							if (settings.visitorRating) {
								
								if (item.hasOwnProperty(J.VISITORRATING)) {
									// Already exists
									r = item[J.VISITORRATING];
								} else {
									if (!settings.useJalbumRating) {
										// Ignore jAlbum rating
										r = 0;
									}
									// Save the default
									item[J.VISITORRATING] = r;
								}
										
								s.addClass('r' + r + (settings.useJalbumRating? '' : ' icon-close'))
									.on('click.' + ns, function(e) {
											var el = $(this),
												r = Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left - parseFloat(el.css('paddingLeft'))) / 16), 5); 
												//Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left + 4) / el.width()) * 5, 5);
												
											el.removeClass('r0 r1 r2 r3 r4 r5').addClass('r' + r);
											item[J.VISITORRATING] = r;
											
											// Propagate to thumbnails
											self.trigger('refreshRating', thumbs[curr]);
												
											return false;
										});
							} else {
								// Unalterable rating from jAlbum
								s.addClass('r' + r);
							}
						}
					}
					
					// Extra sizes
					/*
					if (extraSizes && item[J.CATEGORY] === 'image') {
						el = ($('<div>', {
								'class': 	'download icon-download'
							})).appendTo(buttons);
						
						var p = album.getImagePath(item);
						
						for (var i = 0; i < extraSizes.length; i++) {
							el.append($('<a>', {
									text: 		extraSizes[i],
									href:  		p.replace('slides/', 'dl/' + extraSizes[i] + '/'),
									download: 	''
								}));
						}
					}
					*/
					// Share buttons
					/*
					if ((DEBUG || !LOCAL) && settings.shareHook) {
						
						$(settings.shareHook).trigger('updateLinks', { href: window.location.href });
					}
					*/
					// Print button
					
					if (settings.printImage && item[J.CATEGORY] === 'image' && !settings.rightClickProtect) {
						controls.find('.print-btn').removeClass('hidden').off('.' + ns).on('click.' + ns, function(e) {
								printImage((item[J.ORIGINAL] && !settings.hideDownload)? item[J.ORIGINAL][J.PATH] : item[J.IMAGE][J.PATH],
									item[J.TITLE] || '',
									item[J.THUMBCAPTION] || ''
								);
							});
					} else {
						controls.find('.print-btn').addClass('hidden');
					}
					
					// Download button
					
					if (settings.showDownload) {
						var href;
						
						if ((isImage || isPdf || settings.allowDownloadOthers) &&
							((href = album.getOriginalPath(item)) ||
							(settings.allowDownloadScaled && (href = album.getImagePath(item))))) {
							controls.find('.download-btn').removeClass('hidden').attr('href', href);
						} else {
							controls.find('.download-btn').addClass('hidden').removeAttr('href');
						}
					}
										
					// Fotomoto button
					
					if (!LOCAL && typeof FOTOMOTO !== UNDEF && settings.fotomoto) {
						controls.find('.fotomoto-btn').off('.' + ns).on('click.' + ns, function(e) {
								FOTOMOTO.API.showWindow(10, 
									album.getOriginalPath(item) || album.getItemPath(item)
								);
							});
					}
					
					// Photo data
					
					if (item[J.PHOTODATA]) {
						controls.find('.photodata-btn').removeClass('hidden');
						panels.find('.photodata').html(item[J.PHOTODATA]);
					} else {
						controls.find('.photodata-btn').addClass('hidden');
						panels.find('.photodata').empty();
					}
					
					// Regions
					
					if (settings.showRegions) {
						
						if (item[J.REGIONS]) {
							controls.find('.regions-btn').removeClass('hidden').off('.' + ns).on('click.' + ns, function() {
									var el = lightbox.find('.' + settings.regionsClass);
									
									if (el.length) {
										el.remove();
									} else {
										var reg = JSON.parse(item[J.REGIONS]);
										
										el = $('<div>', {
											'class':	settings.regionsClass
										}).appendTo(main);
										
										for (var i = 0, r; i < reg.length; i++) {
											r = reg[i].split(';');
											el.append($('<a>').css({
												left:		(100 * parseFloat(r[1])) + '%',
												top:		(100 * parseFloat(r[2])) + '%',
												width:		(100 * parseFloat(r[3])) + '%',
												height:		(100 * parseFloat(r[4])) + '%'
											}).append('<span>' + r[0] + '</span>'));
										}
									}
													
									return false;
								});
							
						} else {
							
							controls.find('.regions-btn').addClass('hidden')
						}
					}
					
					// Map
					
					if (!LOCAL && settings.showMap) {
						
						if (album.hasLocation(item)) {
							setTimeout(function() {
									self.find('[data-map-root]').trigger('setActive', item[J.NAME]);
								}, 100);
						} else {
							self.find('[data-map-root]').trigger('resetMarkers');
						}
					}
					
					if (item[J.SOUNDCLIP]) {
						buttons.append($('<audio>', {
								'class':	'soundclip',
								src:		item[J.SOUNDCLIP],
								controls:	'controls'
							}));
					}
					
					if (item[J.IMAGECAPTION]) {
						caption.append($('<div>', {
								'class': 	'caption',
								html: 		item[J.IMAGECAPTION]
							}));
						caption.find('[data-tooltip]').addTooltip();
					}
					
					if (!buttons.is(':empty')) {
						caption.addClass(settings.hasbuttonsClass);
						caption.append(buttons);
						buttons.children('a').addTooltip();
					}
					
					caption.toggle(!slideshow || caption.children('.caption').length > 0);
					
					if (caption.is(':empty')) {
						lightbox.removeClass(settings.captionVisibleClass);
						caption.remove();
					} else {
						lightbox.addClass(settings.captionVisibleClass);
						caption.show();
					}
						
				},	
			
			// Preloading the next image (direction sensitive)
			
			preloadNext = function() {
					var nextItem;
					
					if (direction < 0 && curr > 0) {
						nextItem = thumbs.eq(curr - 1).data(J.OBJ);
					} else if (curr < thumbs.length - 2) {
						nextItem = thumbs.eq(curr + 1).data(J.OBJ);
					}
					
					if (nextItem) {
						if (nextItem[J.CATEGORY] === 'image') {
							var next = new Image();
							next.src = album.getImagePath(nextItem);
						}
					}
				},
			
			// Image clicked :: toggle controls or audio/video controls
			
			imageClicked = function(e) {
					var touched = $('html').data('whatinput') === 'touch' || e.type === 'touchend';
					
					//log(e.type);
								
					if (isVideo || isAudio) {
						
						var m = media[0];
							
						if (m.paused) {
							m.play();
							if (VEND !== 'ms') {
								m.controls = false;
							}
							if (touched) {
								toggleControls();
								if (lightbox.hasClass(settings.captionVisibleClass)) {
									hideCaption();
								}
							}
						} else {
							m.pause();
							if (VEND !== 'ms') {
								m.controls = true;
							}
							if (touched) {
								showControls();
								showCaption();
							}
						}
						
					} else if (isImage) {
						
						if (touched) {
							toggleControls();
							if (!lightbox.hasClass(settings.captionVisibleClass)) {
								showCaption();
							}
						} else if (settings.clickForNext) {
							// image navigation
							if (((e.pageX || e.originalEvent.pageX) - main.position().left) > (main.width() / 2)) {
								nextImage();
							} else {
								previousImage();
							}
						}
					}
				},
			
			// The image is loaded
			
			imageReady = function() {
				
					loadCounter--;
					
					if (loadCounter > 0) {
						return;
					}
									
					if (DEBUG) {
						var d = new Date();
						if (isExternal || isVr) {
							console.log((isExternal? 'External content' : '360 player') + ' loaded: ' + (d - loadTimer) + 'ms');
						} else if (image.length) {
							console.log(((isVideo || isAudio)? 'Media' : 'Image') + ' [' + curr + '] loaded: ' + (d - loadTimer) + 'ms src="' + image[0].src + '"');
							if (typeof image[0] === UNDEF || typeof media[0] === UNDEF) {
								console.log('Premature ' + (typeof image[0] === UNDEF)? ('loadImage.done(' + image[0].src + ')') : ('loadMedia.done('+ media[0].src + ')'));
							}
						}
					}
					
					main.hide().css('opacity', 0);
					
					// Moving 100px away from center to prepare for moving back
					main.translateX(100 * direction);
					
					if ((isImage || isVideo || isAudio) && settings.rightClickProtect) {
						media.on('contextmenu', function(e) {
								e.preventDefault()
								return false;
							});
					}
												
					if (isVideo || isAudio) {
						media.attr({
							autoplay: 	settings.videoAuto,
							loop:		settings.videoLoop
						});
						media[0].volume = settings.volume;
						media.on('volumechange.' + ns, function() {
								settings.volume = this.volume;
							});
					}
					
					// Leaving time for CSS transform to settle down
					setTimeout(function() {
							
						fitImage(true);
						
						lb_activity.hide();
						
						// Moving back to center
						main.show().translateXAndFade(0, 1, settings.speed);
						
						// Preparing the image after transition
						setTimeout(function() {
								
							if (!isVr && !isPdf && (!isExternal || !main.children('iframe').length)) {
								// Swipe handler
								main.swipe({
										onSwipeStart:	function() {
																$(this).trigger('autopanoStop');
															},
										
										onSwipedLeft: 	function() {
																swiped = true;
																nextImage();
															},
														
										onSwipedRight: 	function() {
																swiped = true;
																previousImage();
															},
														
										onFinished: 	function() {
																if (!dontRemove) {
																	$(this).trigger('removeSwipe');
																	$(this).remove();
																}
															},
														
										onClick:		function(e) {
																if (lightbox.data('panomove')) {
																	$(this).trigger('autopanoStop');
																} else {
																	imageClicked(e);
																}
															}
									});
							}
							/*
							if (settings.autoPano && isImage && image.width() > lightbox.width()) {
								main.autopano({
										autoStart:		true
									});
							}
							*/
							if (settings.onLoadEnd !== false) {
								settings.onLoadEnd(thumb);
							}
							
							if (settings.preloadNext) {
								preloadNext();
							}
							
						}, settings.speed / 2);
						
					}, 50);
					
					// Continuing slideshow
					if (slideshow) {
						if (isAudio || isVideo) {
							suspendAuto();
							media[0].onended = function() {
								resumeAuto();
							};
						} else {
							clearTimeout(slideshow);
							slideshow = setTimeout(nextImage, settings.slideshowDelay);
							if (lb_pause) {
								lb_pause.find('.progress').show();
							}
						}
					} else {
						stopAuto(true);
						if (settings.autohideControls) {
							hideControlsLater();
						}
					}
				},
			
			// Main entry point to load a new image
			
			loadImage = function(n) {
				
					if (typeof n !== 'number') {
						n = thumbs.index(n);
					}
					
					if (n < 0 || n >= thumbs.length) {
						// Out of bounds move
						
						dontRemove = true;
						
						if (n < 0) {
							
							switch (settings.afterLast) {
								case 'donothing':
									n = 0;
									break;
									
								case 'startover':
									n = thumbs.length - 1;
									break;
									
								case 'onelevelup': 
									if (settings.level) {
										window.location.href = '../' + settings.indexName;
									}
									n = 0;
									break;
									
								case 'nextfolder':
									if (settings.baseSet && settings.previousFoldersLast) {
										window.location.href = settings.previousFoldersLast;
									}
									n = 0;
									break;
									
								case 'nextindex':
									if (settings.baseSet && settings.previousFolderPath) {
										window.location.href = settings.previousFolderPath;
									}
									n = 0;
									break;
									
								default:
									n = 0;
									quitLightbox();
									
							}
							
						} else {
							
							switch (settings.afterLast) {
								
								case 'donothing':
									n = thumbs.length - 1;
									break;
									
								case 'startover':
									n = 0;
									break;
									
								case 'onelevelup': 
									if (settings.level) {
										window.location.href = '../' + settings.indexName;
									}
									n = thumbs.length - 1;
									break;
									
								case 'nextfolder':
									if (settings.baseSet && settings.nextFoldersFirst) {
										window.location.href = settings.nextFoldersFirst + (slideshow? '&slideshow' : '');
									}
									n = thumbs.length - 1;
									break;
									
								case 'nextindex':
									if (settings.baseSet && settings.nextFolderPath) {
										window.location.href = settings.nextFolderPath;
									}
									n = thumbs.length - 1;
									break;
									
								case 'ask':
									
									var buttons = new Array();
									
									if (swiped) {
										main.translateXAndFade(0, 1, settings.speed * 2);
										swiped = false;
									}
									
									n = thumbs.length - 1;
						
									if (thumbs.length > 1) {
										// Start over
										buttons.push({
												t: 	text.startOver,
												c:	'icon-loop',
												h: 	function(lb) { 
														loadImage(0);
													}
											});
									}
									
									if (settings.level) {
										// Up one level
										buttons.push({
												t: 	settings.level? text.upOneLevel : (settings.homepageLinkText || text.backToHome), 
												c:	'icon-one-level-up',	
												h: 	function() { 
														window.location.href = '../' + settings.indexName;
													}
											});
									}
									
									// Back to thumbnails
									buttons.push({
											t: 	text.backToIndex,
											c:	'icon-arrow-up',
											h: 	function() { 
													quitLightbox();
												}
										});
									
									if (settings.baseSet) {
										
										if (settings.nextFoldersFirst) {
											// Go to next folder
											buttons.push({
													t: 	text.nextFolder,
													c:	'icon-arrow-right',
													h: 	function() {
															window.location.href = settings.nextFoldersFirst;
														}
												});
										}
										
										if (settings.nextFolderPath) {
											// Go to next index
											buttons.push({
													t: 	text.nextIndex,
													c:	'icon-thumbnails',
													h: 	function() {
															window.location.href = settings.nextFolderPath;
														}
												});
										}
									}
							
									lightbox.modal($('<p>', {
											'class':	'text-center',
											text: 		text.atLastPageQuestion
										}), buttons, {
											onClose:	function() {
															loadImage(thumbs.length - 1);
														},
											'class': 	'secondary',
											title: 		text.atLastPage
										});
									
									return;
										
								default:
									// Back to index
									n = slideshow? 0 : (thumbs.length - 1);
									stopAuto();
									quitLightbox();
							}
						}
						
						if (swiped) {
							main.translateXAndFade(0, 1, settings.speed * 2);
							swiped = false;
						}
						
					} else {
						dontRemove = false;
					}
					
					// Direction is calculated if the current thumb exists
					if (lightbox.is(':visible')) {
						// Lightbox is on
						if ((Math.abs(n - curr) >= thumbs.length - 1) && settings.afterLast === 'startover') {
							// Flip over
							direction = 1;
						} else {
							direction = (curr > n)? -1 : ((curr < n)? 1 : 0);
						}
						
						if (curr === n) {
							// the requested image is already on screen :: nothing to do
							return;
						}
						
						if (settings.autoHideControls) {
							hideControlsLater();
						}
						
					} else {
						// We're on the index page :: show lightbox
						listening = true;
						$body.add('html').css('overflow', 'hidden');
						$body.addClass(settings.lightboxOnClass);
						direction = 0;
						
						if (settings.useRating) {
							updateRating();
						}
						
						lightbox.show();
						backgroundAudioOn = backgroundAudioRoot.length && !backgroundAudioRoot.data('paused');
						
						if (settings.autoStart) {
							startAuto();
						}
					}
					
					closeModal();
					
					var item;
					
					curr = n;
					thumb = thumbs.eq(curr);
					item = thumb.data(J.OBJ);
					
					if (!item) {
						console.log('Fatal error: image (' + curr + ') is missing from the database! (Upload data1.json again!)');
						return;
					}
					
					lb_activity.show();
					if (lb_pause) {
						lb_pause.find('.progress').hide();
					}
					
					getImageType(item);
					
					if (settings.onLoadStart !== false) {
						settings.onLoadStart(thumb);
					}
					
					setTimeout(function() {
						
						// Delayed image loading
											
						removeOldContent(direction);
						swiped = false;
	
						loadCounter = 1;
						loadTimer = new Date();
						
							
						main = $('<div>', {
								'class': 	'lightbox-main ' + (isExternal? 'external' : (isVr? 'vr' : (isPdf? 'pdf' : item[J.CATEGORY])))
							})
							.hide()
							.appendTo(lightbox);
							
						if (isImage) {
							
							// Image
							
							media = image = $('<img>')
								.one('load.' + ns, function(e) {
									imageReady();
								})
								.attr('src', album.getImagePath(item))
								.appendTo(main);
							
							if (DEBUG) {
								console.log('Loading image [' + curr + '] src="' + album.getImagePath(item) + '"');
							}
							
							if (image[0].complete && image[0].naturalWidth > 0) {
								// From cache
								image.off('load.' + ns);
								imageReady();
							}
							
						} else if (isAudio || isVideo) {
							
							if (isVideo) {
								
								// Video
								media = image = $('<video>', {
											preload: 		'auto',
											controls: 		'controls',					// album.getVideoDuration(item) > 1500,
											controlsList:	'nodownload',
											poster: 		album.getPosterPath(item)
										});
								
							} else {
								
								var src = album.getPosterPath(item);
									
								// Audio
								loadCounter = 2;
								
								image = $('<img>', {
										'class': 	'poster' + (src.endsWith('poster.png')? ' default' : '')
									}).one('load', function(e) {
										imageReady();
									})
									.attr('src', src)
									.appendTo(main);
								
								media = $('<audio>', {
										preload: 		'auto',
										controls: 		'true',
										controlsList:	'nofullscreen nodownload'
									});
							}
							
							media.one('loadedmetadata', function(e) {
										imageReady();
									})
								.attr('src', album.getSourcePath(item))
								.appendTo(main);
							
							if (settings.muteBackgroundAudio && backgroundAudioRoot.length) {
								media.on('playing', mediaPlaying)
									.on('paused', mediaPaused)
									.on('ended', mediaEnded);
									
								backgroundAudioOn = !backgroundAudioRoot.data('paused');
							}
							
						} else if (isExternal) {
							
							var cont = item[J.EXTERNAL]['cont'],
								size = item[J.EXTERNAL]['size'];
								
							if (size) {
								size = size.split('x');
								main.data({
									oWidth:		parseInt(size[0]),
									oHeight:	parseInt(size[1])
								})
							}
								
							main.one('DOMReady', function(e) {
									imageReady();
								});
							
							if (cont.match(/^https?\:\/\//i) || cont.match(/\.html?$/i) || cont.match(/^\.\.\//)) {
								main.addClass('iframe')
									.append($('<iframe>', { 
											width: 				'100%',
											height: 			'100%',
											src: 				cont,
											frameborder: 		0,
											allowfullscreen: 	'allowfullscreen'
										}));
							} else {
								if (cont.match(/^<iframe/i)) {
									main.addClass('iframe');
								}
								main.append(cont);
							}
							
							// Considered ready after 200ms or the "ready" event, whatever comes first 
							setTimeout(imageReady, 200);
							
						} else if (isVr) {
							
							var w = Math.round(lightbox.width() * settings.fitRatio),
								h = Math.round(lightbox.height() * settings.fitRatio);
								
							main.css({
										width:		w,
										height:		h
									})
								.attr('id', 'vr' + curr)
								.addClass('vr')
								.show()
								.one('DOMReady', imageReady);
							
							// Photosphere viewer by Jeremy Heleine
							if (typeof PhotoSphereViewer !== UNDEF) {
								setTimeout(function() {
										var psv = new PhotoSphereViewer({
												panorama: 			album.getAbsoluteImagePath(item),
												container: 			main[0],
												usexmpdata:			false,
												zoom_level:			30,
												loading_html:		'<div class="lightbox-loading"><div></div></div>',
												navbar: 			true,
												navbar_style: 		{	
																		autorotateThickness:	2,
																		zoomRangeThickness: 	2,
																		zoomRangeDisk:			12,
																		fullscreenThickness: 	2,
																		backgroundColor: 		'rgba(17, 17, 17, 0.35)'
																	},
											});
									}, settings.speed);
							} else {
								console.log('Fatal Error: Missing "photo-sphere-viewer.min.js"!');
							}
							
							// Considered ready after 200ms or the "ready" event, whatever comes first 
							setTimeout(imageReady, 200);
							
						} else {
							
							// Other
							var target = main;
							
							if (isPdf) {
								
								if (!HASPDFVIEWER || ISIOSDEVICE) {
									
									target = $('<iframe>', {
											src:		'https://docs.google.com/viewer?url=' + window.location.href.getDir() + album.getSourcePath(item) + '&embedded=true'		
										})
										.appendTo(main);
										
								} else {							
									target = $('<object>', {
											type:		'application/pdf'
										})
										.attr('data', album.getSourcePath(item))
										.appendTo(main);
								}
							}
							
							$('<a>', {
									href: 		album.getSourcePath(item),
									target: 	'_blank'
								})
								.append($('<img>', {
									'class': 	'other'
								})
								.one('load', function(e) {
									imageReady();
								})
								.attr('src', album.getImagePath(item)))
								.appendTo(target);
							
							$('<p>', {
									'class': 	'click-hint',
									'html': 	text.clickToOpen
								})
								.appendTo(target);
						}	
						
						prepareCaption(item);
																
					}, settings.speed / 4);
				},
			
			// Hiding the lightbox overlay
			
			quitLightbox = function() {
					
					stopAuto(true);
					
					listening = false;
					$body.add('html').css('overflow', '');
					$body.removeClass(settings.lightboxOnClass);
					
					if (settings.muteBackgroundAudio && backgroundAudioOn) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
					
					if (!main.length) {
						if (settings.onClose !== false) {
							settings.onClose(thumb);
						}
					} else {
						
						if ((isAudio || isVideo) && media) {
							var sv = media[0].volume,
								tm = settings.speed / (sv * 50),
								fade = function() {
										if (media) {
											var v = Math.max(media[0].volume - 0.02, 0);
											if (v > 0.005) {
												media[0].volume = v;
												setTimeout(fade, tm);
											} else {
												media[0].pause();
											}
										}
									};
							
							media.off('.' + ns);
							fade();
						}
						
						lightbox.fadeOut(settings.speed, function() {
							if (main.length) {
								main.remove();
								main = $();
							}
							if (caption.length) {
								caption.remove();
								caption = $();
							}
							//inProgress = false;
							if (settings.onClose !== false) {
								settings.onClose(thumb);
							}
							
							showControls();
							$body.hideAllTooltips();
						});
					}
				},
			
			// Removing the lightbox completely
			
			removeLightbox = function() {
					stopAuto(true);
					listening = false;
					// removing event handlers
					thumbs.off('.' + ns);
					$document.add($window).add(lightbox).off('.' + ns);
					$body.add('html').css('overflow', '');
					$body.removeClass(settings.lightboxOnClass);
					
					if (!main.length) {
						// No image
						lightbox.remove();
					} else {
						// Fade out
						main.animate({ 
								'opacity': 	0 
							}, settings.speed, function() {
								lightbox.remove();
							});
					}
				},
			
			// Starting auto slideshow
			
			startAuto = function(keepMusic) {
					
					clearTimeout(slideshow);
					slideshow = setTimeout(nextImage, settings.slideshowDelay / 4);
					
					if (lb_play) {
						lb_play.hide();
						lb_pause.show();
					}
					$body.add(lightbox).addClass(settings.immerseClass);
					
					if (settings.autohideControls) {
						hideControlsLater();
					}
					
					caption.addClass('slideshow');
					
					if (!(keepMusic === true) && settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
					
					if (settings.onSlideshowStart !== false) {
						settings.onSlideshowStart(thumb);
					}
				},
			
			resumeAuto = function() {
					
					if (lb_play) {
						lb_play.hide();
						lb_pause.show();
					}
					
					if (settings.autohideControls) {
						hideControlsLater();
					}
					
					$body.add(lightbox).addClass(settings.immerseClass);
					slideshow = setTimeout(nextImage, settings.slideshowDelay / 4);
					
					if (settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
				},
	
			stopAuto = function(keepMusic) {
				
					if (settings.onSlideshowPause !== false) {
						settings.onSlideshowPause(thumb);
					}
					
					if (!(keepMusic === true) && settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeOutPlayer');
					}
					
					if (lb_play) {
						lb_play.show();
						lb_pause.hide();
					}
					clearTimeout(slideshow);
					slideshow = null;
					$body.add(lightbox).removeClass(settings.immerseClass);
					
					//showArrows();
					showControls();
					caption.removeClass('slideshow');
					
				},
			
			suspendAuto = function() {
					if (settings.backgroundAudioSlideshowControl) {
						backgroundAudioRoot.trigger('fadeOutPlayer');
					}
					clearTimeout(slideshow);
					slideshow = null;
					//showControls();
				},
				
			mediaPlaying = function() {
				
					if (settings.muteBackgroundAudio) {
						if (backgroundAudioOn = !backgroundAudioRoot.data('paused')) {
							backgroundAudioRoot.trigger('fadeOutPlayer');
						}
					}
				},
				
			mediaPaused = function() {
					if (settings.muteBackgroundAudio && backgroundAudioOn) {
						backgroundAudioRoot.trigger('fadeInPlayer');
					}
				},
				
			mediaEnded = function() {
					backgroundAudioOn = false;
				},
			
			toggleZoom = function() {
					settings.fitImages = !settings.fitImages;
					//savePrefs();
					fitImage();
				},
			
			showControls = function() {
					clearTimeout(controlsTimeout);
					$body.add(lightbox).removeClass(settings.immerseClass);
				},
			
			hideControls = function() {
					var hel;
					clearTimeout(controlsTimeout);
					// Check if mouse is over a panel 
					if (hel = document.querySelectorAll(':hover')) {
						for (var i in hel) {
							if (hel[i].id === 'panels') {
								hideControlsLater();
								return;
							}
						}
					}
					
					$body.hideAllTooltips();
					$body.add(lightbox).addClass(settings.immerseClass);
				},
				
			toggleControls = function() {
					$body.add(lightbox).toggleClass(settings.immerseClass, !controlsHidden());
				},
			
			controlsHidden = function() {
					return $body.hasClass(settings.immerseClass);
				},
			
			hideControlsLater = function() {
					clearTimeout(controlsTimeout);
					controlsTimeout = setTimeout(function() {
							//hideArrows();
							hideControls();
						}, slideshow? Math.min(settings.slideshowDelay / 2, settings.hideControlsDelay) : settings.hideControlsDelay);
				},
				
			showCaption = function() {
					lightbox.addClass(settings.captionVisibleClass);
				},
				
			hideCaption = function() {
					lightbox.removeClass(settings.captionVisibleClass);
				},
			
			previousImage = function() {
					loadImage(curr - 1);
				},
			
			nextImage = function() {
					loadImage(curr + 1);
				},
								
			// Setting up the structure
			
			initLightbox = function(ns) {
				
					lightbox = $('<div>', {
							id: 		ns,
							'class': 	'lightbox'
						}).hide().appendTo(self);
					
					// Darken background
					lb_overlay = $('<div>', {
							'class': 	'lightbox-overlay'
						}).appendTo(lightbox);
					
					// Activity indicator
					lb_activity = $('<div>', {
							'class': 	'lightbox-loading'
						}).append('<div>').appendTo(lightbox);
					
					// Controls group
					var lb_controls = $('<div>', {
								'class':	'controls'
							}).appendTo(lightbox);
						
					if (settings.zoomSlider) {
						var zoom_panel= $('<div>', {
									'class':	'zoom-control'
								}),
							zoom_out_btn = $('<a>', {
									'class':	'icon-zoom-out zoom-out button'
								}).appendTo(zoom_panel),
							zoom_inp = $('<input>', {
									name:		'zoom-level',
									type:		'range'
								})
								.appendTo(zoom_panel),
							zoom_in_btn = $('<a>', {
									'class':	'icon-zoom-in zoom-in button'
								}).appendTo(zoom_panel);
							
						// Zoom toggle button
						lb_zoom = $('<button>', {
									type: 		'button',
									'class': 	'zoom icon-search button'
								})
							.on('click.' + ns, function(){
									if (listening) {
										toggleZoom();
									}
									return false; 
								})
							.addTooltip($('<div>', {
									'class':	'zoom-panel'
								}).append(zoom_panel))
							.appendTo(lb_controls);
							
						zoom_inp.on('change', function() {
								setZoom(parseInt(zoom_inp.val()));
								return true;
							});
						
						zoom_in_btn.on('click', function() {
								var v = Math.min(100, parseInt(zoom_inp.val()) + 20);
								zoom_inp.val(v);
								setZoom(v);
								return false;
							});
						
						zoom_out_btn.on('click', function() {
								var v = Math.max(0, parseInt(zoom_inp.val()) - 20);
								zoom_inp.val(v);
								setZoom(v);
								return false;
							});
						
					} else {
						// Zoom toggle button
						lb_zoom = $('<button>', {
								type: 		'button',
								'class': 	'zoom icon-expand button',
								'title': 	text.toggleZoom
							})
							.on('click.' + ns, function(){
								if (listening) {
									toggleZoom();
								}
								return false; 
							})
							.appendTo(lb_controls);
					}
					
					// Play button
					lb_play = $('<button>', {
							type: 		'button',
							'class': 	'play icon-play button',
							'title': 	text.slideshow
						})
						.on('click.' + ns, function(){ 
							if (listening) {
								startAuto();
							}
							return false; 
						})
						.appendTo(lb_controls);
		
					// Pause button
					lb_pause = $('<button>', {
							type: 		'button',
							'class': 	'pause icon-pause button',
							'title': 	text.pause
						})
						.append($('<span>', {
							'class': 	'progress'
						}))
						.hide()
						.on('click.' + ns, function(){ 
							if (listening) {
								stopAuto();
							}
							return false; 
						})
						.appendTo(lb_controls);
					
					// Close button
					lb_close = $('<button>', {
							type: 		'button',
							'class': 	'close icon-close button',
							'title': 	text.close
						})
						.on('click.' + ns, function() {
							if (listening) {
								quitLightbox();
							}
							return false; 
						})
						.appendTo(lb_controls);
					
					// Buttons
					lb_btn_left = thumbs.length?
						$('<button>', {
								type: 		'button',
								'class': 	'arrow-left button'
							})
							.on('click.' + ns, function(){
								if (listening) {
									previousImage();
								}
								return false;
							})
							.appendTo(lightbox)
						: 
						$();
						
					lb_btn_right = thumbs.length?
						$('<button>', {
								type: 		'button',
								'class': 	'arrow-right button'
							})
							.on('click.' + ns, function(){
								if (listening) {
									nextImage();
								}
								return false;
							})
							.appendTo(lightbox)
						: 
						$();
						
					// Adding tooltips
					lightbox.find('button').not('.icon-search').addTooltip();
		
				};
			
		// Initializing Lightbox
					
		if (ns) {
			// should never happen, but still living lightbox attached to thumb elements
			cleanup(ns);
		}
		
		// Creating new namespace
		self.data('llb_ns', ns = 'llb_' + Math.floor(Math.random() * 10000));		

		// Finding thumbs
		thumbs = self.find(settings.delegate);
		
		// Control strip
		controls = $('#controls');
		
		panels = $('#panels');
		
		// Initializing controls
		initLightbox();	
		
		// Setting up events
				
		// Resize
		$window.on('resize.' + ns /* + ' orientationchange.' + ns*/, function() {
				clearTimeout(resizeTimeout);
				// Executed only if 50ms has passed since the last resize event
				resizeTimeout = setTimeout(function() { 
					clearTimeout(resizeTimeout);
					if (main && main.length && !(isVr && isFullscreen())) {
						fitImage();
					}
				}, 50);
			});
		
		// Quit on document touch
		if (settings.quitOnDocClick) {
			lb_overlay.on('click.' + ns, function(e) {
					
					if (settings.autohideControls && controlsHidden()) {
						// Toggle controls back
						showControls();
						return false;
					}
					
					if (main.length && !$(e.target).is(main)) {
						// Quit
						quitLightbox();
					}
				});
		}
		
		// On mouse-driven devices
		
		if (settings.autohideControls) {
			
			// Auto hide controls
			$document.on('mousemove.' + ns, function() {
					showControls();
					hideControlsLater();
				});
		}
						
		// Keyboard
		if (settings.enableKeyboard) {
			$document.on('keyup.' + ns, function(e) {
				if (!listening || !main.length || lightbox.children('.modal:visible').length || document.activeElement && (document.activeElement.nodeName === 'INPUT' || document.activeElement.nodeName === 'TEXTAREA')) {
					return true;
				}
				
				e.preventDefault();
				
				switch (e.keyCode) {
					
					case 27:
						quitLightbox();
						break;
					
					case 33:
					case 37:
						previousImage();
						break;
						
					case 34:
					case 39:
						nextImage();
						break;
						
					case 97:
					case 35:
						loadImage(thumbs.length - 1);
						break;
						
					case 103:
					case 36:
						loadImage(0);
						break;
					
					case 106:
					case 179:
						if (slideshow) {
							stopAuto();
						} else {
							startAuto();
						}
						break;
						
					case 107:
						toggleZoom();
						break;
						
					case 32:
						if (isVideo || isAudio) {
							if (media[0].paused) {
								media[0].play();
							} else {
								media[0].pause();
							}
						} else {
							if (slideshow) {
								stopAuto();
							} else {
								startAuto();
							}
						}
						break;
						
					default:
						return true;
										
				}
				
				return false;
			});
		}
		
		// Mouse wheel
		if (settings.enableMouseWheel) {
			var wheelTimeout = null;
			lightbox.on('mousewheel.' + ns + ' DOMMouseScroll.' + ns, function(e) {
					if (listening) {
						clearTimeout(wheelTimeout);
						if (	!main.length || 
								$(e.target).closest('.' + settings.lightboxCaptionClass).length ||
								main.hasClass('pdf') || 
								lightbox.children('.modal:visible').length
							) {
							return true;
						}
						wheelTimeout = setTimeout((e.originalEvent.wheelDelta > 0 || e.originalEvent.detail < 0)? previousImage : nextImage, 100);
					}
					return false;
				});
		}
		
		// Events that can be triggered from outside
		
		// External trigger to switch image
		self.on('lightboxLoad.' + ns, function(e, n, auto) {
					stopAuto(true);
					
					if (typeof n === 'number') {
						n = Math.minMax(0, n, thumbs.length);
					} else if (typeof n === UNDEF) {
						n = 0;
					} else {
						n = thumbs.index(n);
					}
					
					if (settings.onStart !== false) {
						settings.onStart((typeof n === 'number')? thumbs.eq(n) : th);
					}
					
					loadImage(n);
					
					if (auto) {
						if (typeof auto === 'number') {
							setTimeout(startAuto, auto);
						} else {
							startAuto();
						}
					}
				})
			// External trigger to quit = does not remove, just hides the lightbox
			.on('lightboxQuit.' + ns, function() {
					quitLightbox();
					return false;
				})
			// Removing the lightbox completely
			.on('lightboxRemove.' + ns, function() {
					removeLightbox();
					return false;
				})
			// Stopping auto hide controls
			.on('stopAutoHide.' + ns, function() {
					clearTimeout(controlsTimeout);
				})
			// Returning the current lightbox container
			.on('lightboxContainer.' + ns, function() {
					return lightbox;
				})
			// External trigger to go to the previous image
			.on('lightboxNext.' + ns, function() {
					stopAuto();
					nextImage();
					return false;
				})
			// External trigger to go to next image
			.on('lightboxPrevious.' + ns, function() {
					stopAuto();
					previousImage();
					return false;
				});
				
		// Ready event
		
		if (settings.hasOwnProperty('onReady') && typeof settings.onReady === 'function') {
			settings.onReady(thumb);
		}
		
		// Auto start
		
		if (settings.autoStart && settings.baseSet) {
			startAuto();
		}

		return this;
	};
	
	$.fn.lightbox.defaults = {
			delegate:						'.card.lbable',
			lightboxMainClass:				'lightbox-main',
			lightboxCaptionClass:			'lightbox-caption',
			lightboxOnClass:				'lightbox-on',
			buttonClass:					'secondary button small',
			activeClass:					'active',
			scrollClass:					'scroll',
			regionsClass:					'regions',
			captionVisibleClass:			'caption-visible',
			thumbsVisibleClass:				'thumbs-visible',
			hasbuttonsClass:				'hasbuttons',
			immerseClass:					'immerse',
			slideshowDelay:					4000,
			hideControlsDelay:				2500,
			fitImages:						true,
			fitBoth:						true,
			fitBelowControls:				true,
			fitRatio:						0.94,
			maxZoom:						1.4,
			scaleUp:						false,
			zoomSlider:						false,
			baseSet:						true,
			autohideControls:				false,
			autoStart:						false,
			clickForNext:					true,
			backgroundAudioSlideshowControl:false,
			muteBackgroundAudio:			true,
			use360Player:					true,
			afterLast:						'donothing',
			mapHeight:						0.8,
			videoAuto:						false,
			videoLoop:						false,
			autoPano:						false,
			volume:							0.5,
			rightClickProtect: 				false,
			useRating:						false,
			jalbumRating:					false,
			visitorRating:					false,
			showNumbers:					false,
			showShare:						false,
			allowDownloadOthers:			false,
			allowDownloadScaled:			false,
			showDownload:					true,
			showMap:						false,
			showRegions:					false,
			printImage:						false,
			speed: 							400,
			panoramaAutoDetect:				1.5,
			panoramaTreshold:				[ 2.5, 2.5 ],
			defaultPosterSize:				[ 628, 360 ],
			preloadNext:					true,
			enableKeyboard: 				true,
			enableMouseWheel:				true,
			quitOnEnd:						true,
			quitOnDocClick: 				true,
			onStart:						false,
			onClose:						false,
			onLoadStart:					false,
			onLoadEnd:						false,
			onAfterLast:					false,
			onBeforeFirst:					false,
			onReady:						false,
			onSlideshowStart:				false,
			onSlideshowPause:				false
		};
	
	$.fn.lightbox.text = {
			startOver:						'Start over',
			upOneLevel:						'Up one level',
			backToHome:						'Back to home',
			backToIndex:					'Back to index page',
			nextFolder:						'Next folder',
			nextIndex:						'Next index page',
			atLastPageQuestion:				'Where to go next?',
			atLastPage:						'At last page',
			atFirstPage:					'At first page',
			close:							'Close',
			slideshow:						'Slideshow',
			pause:							'Pause',
			rating:							'Rating',
			toggleZoom:						'Toggle zoom',
			download: 						'Download',
			print:							'Print',
			printLabel:						'Print out this photo on your printer',
			mapBtn: 						'Map',
			mapLabel:						'Show the photo location on map',
			fotomotoBtn:					'Buy / Share',
			fotomotoLabel:					'Buy prints or digital files, share, send free eCards',
			mostphotosBtn:					'Purchase',
			mostphotosLabel:				'Download this image from <b>mostphotos.com</b>!',
			regionsBtn:						'People',
			regionsLabel:					'Show tagged people', 
			share:							'Share',
			shareLabel:						'Share this photo over social sites',
			shareOn:						'Share on',
			shopBtn: 						'Buy',
			shopLabel:						'Add this item to the shopping cart',
			viewCartLabel:					'View shopping cart',
			feedbackLabel:					'View feedback window',
			metaBtn: 						'Photo data',
			metaLabel:						'Display photographic (Exif/Iptc) data',
			viewCart:						'View cart',
			addCart:						'Add to cart',
			addComment:						'Add comment',
			clickToOpen:					'Click to open this document with the associated viewer!'
		};

	
})(jQuery, jQuery(window), jQuery(document), jQuery('body'));
