/* 
 *	main.js - the skin's javascript functions
 */
	
;(function($, $window, $document, $body, undefined) {
	'use strict';
				
	/********************************************************
	 *
	 *		Implementing select functions on images
	 *
	 ********************************************************/
	
	$.fn.selectable = function(options) {
		
			options = $.extend({
							cardClass:			'card',
							checkboxClass: 		'checkbox',
							checkedClass:		'checked',
							checkmarkClass:		'icon-checkmark',
							hasSelectedClass:	'has-selected',
							allSelectedClass:	'all-selected',
							noneSelected:		null,
							anySelected:		null,
							selectionChange:	null
						}, options);
			
			var self = $(this).eq(0),
				cards,
				
				update = function(n) {
					
						if (typeof options.selectionChange === 'function') {
							options.selectionChange.call(n);
						}
						
						$body
							.toggleClass(options.hasSelectedClass, n > 0)
							.toggleClass(options.allSelectedClass, n === cards.length);
					},
					
				selectedCnt = function() {
						return cards.filter('.' + options.checkedClass).length;
					},
				
				getImages = function() {
						return options.selector? self.find(options.selector) : self.find('.' + options.cardClass);
					},
					
				selectCard = function(card) {
						card.addClass(options.checkedClass)
							.find('.' + options.checkboxClass)
							.addClass(options.checkmarkClass);
					},
					
				unselectCard = function(card) {
						card.removeClass(options.checkedClass)
							.find('.' + options.checkboxClass)
							.removeClass(options.checkmarkClass);
					},
					
				toggleCard = function(card) {
						if (card.hasClass(options.checkedClass)) {
							unselectCard(card);
						} else {
							selectCard(card);
						}
					},
			
				clicked = function(e) {
						var card = $(e.target).closest('.' + options.cardClass);
						var n = selectedCnt();
						
						if (card.hasClass(options.checkedClass)) {
							unselectCard(card);
							n -= 1;
							
							if (n === 0) {
								if (typeof options.noneSelected === 'function') {
									options.noneSelected.call();
								}
							} else {
								if (typeof options.anySelected === 'function') {
									options.anySelected.call(n);
								}
							}
						} else {
							
							selectCard(card);
							n += 1;
							
							if (typeof options.anySelected === 'function') {
								options.anySelected.call(n);
							}
						}	
						
						update(n);
						return false;
					},
			
				selectAll = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							
							if (n < cards.length) {
								n = cards.length;
								cards.not('.' + options.checkedClass).each(function() {
										selectCard($(this));
									});
										
								if (typeof options.anySelected === 'function') {
									options.anySelected.call(cards.length);
								}
							}
							update(n);
						}
					},
			
				selectNone = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							
							if (n > 0) {
								n = 0;
								cards.filter('.' + options.checkedClass).each(function() {
										unselectCard($(this));
									});
										
								if (typeof options.noneSelected === 'function') {
									options.noneSelected.call();
								}
							}
							update(n);
						}
					},
					
				selectInverse = function() {
					
						if (self.is(':visible')) {
							var n = cards.length - selectedCnt();
							
							if (n === 0) {
								// All selected
								cards.each(function() {
										unselectCard($(this));
									});
								
								if (typeof options.noneSelected === 'function') {
									options.noneSelected.call();
								}
								
							} else {
								// Some selected
								cards.each(function() {
										toggleCard($(this));
									});
								
								if (typeof options.anySelected === 'function') {
									options.anySelected.call(n);
								}
							}
							update(n);
						}
					},
				
				selectRefresh = function() {
						cards = getImages();
						
						cards.each(function() {
							if (!($(this).children('span.' + options.checkboxClass).length)) {
								$(this).append($('<span>', {
										'class': 	options.checkboxClass
									}).on({
										click: 		clicked
									}));
							}
							
						});
						
					};
			
			selectRefresh();
			
			self.on({
					selectAll: 			selectAll,
					selectNone: 		selectNone,
					selectInverse: 		selectInverse,
					selectRefresh: 		selectRefresh
				});
					
			return this;
		};
			
		
	/********************************************************
	 *
	 *					The main skin code
	 *
	 ********************************************************/
	
	$.fn.skin = function(settings) {
		
			settings = $.extend({}, $.fn.skin.defaults, settings);
			
			var main 				=	$(this),								// Main / top level container
				mainCont			= 	main,									// Main container is same as Main
				text 				= 	getTranslations({						// Translated texts
												foundNTimes:				'found {0} time(s)',
												notFound: 					'not found',
												search: 					'Search',
												newImages:					'New images',
												results:					'Results',
												reset:						'Reset',
												label: 						'Label',
												selectedItems:				'Selection',
												addCart:					'Add to cart',
												'return': 					'return',
												select:						'Select',
												sortBy:						'Sort by',
												sortedBy:					'Sorted by',
												ascending:					'ascending',
												descending:					'descending',
												multipleSelectHint:			'Use SHIFT to select range, CTRL for multiple entries',
												noRating:					'No rating',
												inTheLastDay:				'in the last day',
												inThePastNDays:				'in the past {0} days',
												sinceMyLastVisit:			'since my last visit',
												betweenDays:				'between {0} and {0}',
												onDay:						'on {0}',
												beforeDay:					'before {0}',
												afterDay:					'after {0}',
												imagesAdded:				'Images added',
												imagesModified:				'Images modified',
												imagesTaken:				'Images taken',
												'new':						'New',
												more:						'more',
												less:						'less'
											}),
				
				panels				=	$('#panels'),							// Panels (container)
				controls			=	$('#controls'),							// Controls (container)
				shopRoot 			=	$('#shop'),								// Root element for cart
				feedbackRoot 		=	$('#feedback'),							// Root element for the feedback tool
				mapRoot				=	$('#map'),								// Root element for map
				album,															// The database
				ns 					= 	'st_skin',								// Namespace
				lastHash 			= 	'',										// Storing last state
				pageExt				= 	settings.indexName.split('.')[1] || 'html',	// Index page extension
				initByHash 			= 	window.location.hash !== "",			// Initial URL contains hash
				nows 				=	new Date() / 1000,						// Now in seconds
				currency 			=	'EUR',
				scrollTop 			= 	$window.scrollTop(),					// initial scroll top
				lazyloadTimeout 	= 	null,									// Lazyload tracking
				lazyloadRunning 	= 	false,									// lock to avoid multiple calls
				
				// Determine card type
				
				cardType = function(card) {
						var s = card.closest('section');
						if (!s.length) {
							return 'image';
						} else if (s.hasClass(settings.folderClass)) {
							return 'folder';
						} else if (s.hasClass(settings.thumbsClass)) {
							return 'thumb';
						}
						
						return '';
					},
																	
				/********************************************************
				 *
				 *				Managing buttons and panels
				 *
				 ********************************************************/
				 
				// Toggle select buttons: selected count, max count
				
				toggleSelectButtons = function(sc, mc) {
						var p = shopRoot.add(feedbackRoot);
						
						if (typeof mc === UNDEF || !mc) {
							p.find('.' + settings.selectBoxClass).hide();					
						} else {
							p.find('.' + settings.selectBoxClass).show();
							p.find('.' + settings.selectNoneClass + 
									',.' + settings.addSelectedClass + 
									',.' + settings.keepSelectedClass).toggleClass('disabled', typeof sc === UNDEF || sc === 0);
							
							p.find('.' + settings.selectAllClass).toggleClass('disabled', typeof mc !== UNDEF && sc === mc);
						}
					},
						
				// Toggle boxes according to item count
				toggleBoxes = function(mc) {
					
						if (mc) {
							shopBox.add(feedbackBox).show();
						} else {
							shopBox.add(feedbackBox).hide();
						}
					},
						
				/********************************************************
				 *
				 *					Managing overlays
				 *				e.g. after search or filter
				 *
				 ********************************************************/
				/*
						<article id="main-cont" class="content base-overlay" >
							<section class="folders">
								<div class="foldercard cont"></div>
							</section>
							<section class="${thumbnailsClass}">
								<div class="cont ${thumbContClass} $captionClasses $layoutClass">
								</div>
							</section>
						</article>
						<article class="content overlay" id="${namespace}" data-oly_ns="${namespace}"> <!-- dynamically generated -->
							<header>
								<div class="cont icon-${icon}">
									<h4 class="${overlayTitleClass}"> Title</h4>
									<a class="close" href>${return}</a>
								</div>
							</header>
							<section class="row ${thumbContClass}">
								<div class="cont $captionClasses $layoutClass">
								</div>
							</section>
						</article>
				*/
				
				OLYTAG = 'article',
				
				// Creating header
				
				createOverlayHeader = function(o, options) {
						var hc 	= $('<div>', {
										'class':	settings.contClass
									});
						
						if (options) {
							// Icon
							if (options.hasOwnProperty('icon')) {
								hc.addClass(options.icon);
							}
							
							// Title
							if (options.hasOwnProperty('title')) {
								hc.append($('<h' + settings.overlayHeadLevel + '>', {
										'class':	settings.overlayTitleClass,
										html: 		' ' + options.title
									}));
							}			
						}
						
						// Close button
						hc.append($('<a>', {
								'class': 	'close',
								href: 		'',
								text: 		text.return
							}).on('click', function() {
								removeOverlay();
								return false;
							}));
						
						// Adding as first element to overlay
						o.prepend($('<header>').append(hc));
					},
					
				// Creating an empty thumbnail container for overlays
				
				createThumbContainer = function(o) {
						var tc = o.children('section.' + settings.thumbContClass);
						
						if (!tc.length) {
							tc = $('<section>', {
									'class':	settings.thumbsClass
								})
								.append($('<div>', {
									'class':	[
													settings.contClass, 
													'caption-' + settings.captionPlacement, 
													settings.thumbContClass, 
													settings.thumbLayout, 
													settings.captionClasses
												].join(' ')
								}))
								.appendTo(o);
						}
							
						return tc;
					},
						
				// Adding new overlay
			
				addOverlay = function(options) {
						
						// Removing previous overlay 
						var	ns = 'oly_' + Math.floor(Math.random() * 10000),			// Namespace
							bo = getBaseOverlay(),										// Base overlay
							o = $('<' + OLYTAG + '>', {
									'class': 	settings.contentClass + ' ' + settings.overlayClass
								}).attr('data-oly_ns', ns);
							
						// Class to signal the page has overlay 
						main.addClass(settings.hasOverlayClass);
						
						// Removing existing overlay(s) besides base
						removeExtraOverlays();
						
						if (bo.length) {
							// Hide Base
							bo.hideAllTooltips();
							bo.hide();
							// ... and its attached elements
							$('[rel=' + bo.data('oly_ns') + ']').hide();
							o = o.insertAfter(bo);
						} else {
							// No base
							o = o.appendTo(main);
						}
						
						// Save namespace ID
						o[0]['overlay'] = ns;
						
						// Creating overlay header
						if (typeof options !== UNDEF) {
							createOverlayHeader(o, options);
							if (options.hasOwnProperty('className')) {
								o.addClass(options['className']);
							}
						}
						
						// Ensure thumb container exists
						createThumbContainer(o);
						
						hideAllPanels();
						
						// Call "added" action if present
						main.trigger('overlayAdded', o);
						
						return o;
					},
					
				// Getting base overlay
				
				getBaseOverlay = function() {
						
						return mainCont.find(OLYTAG + '.' + settings.baseClass).eq(0);
					},
										
				// Getting extra overlays
				
				getExtraOverlays = function(visible) {
						
						return visible?
							mainCont.find(OLYTAG + '.' + settings.overlayClass + ':visible')
							:
							mainCont.find(OLYTAG + '.' + settings.overlayClass);
					},
										
				// Getting all overlays; base included
				
				getAllOverlays = function(visible) {
						return visible?
							mainCont.find(OLYTAG + '.' + settings.baseClass + ':visible, ' + OLYTAG + '.' + settings.overlayClass + ':visible')
							:
							mainCont.find(OLYTAG + '.' + settings.baseClass + ', ' + OLYTAG + '.' + settings.overlayClass);
					},
					
				// Getting active overlay; creating one if missing
				
				getOverlay = function() {
						var o = getAllOverlays(true).last();		// The latest added visible overlay
					
						if (!o.length) {
							o = addOverlay();
							createThumbContainer(o);
						}
						
						// Name space ID
						if (!o.data('oly_ns')) {
							var ns = 'oly_' +  + Math.floor(Math.random() * 10000);
							o.attr('data-oly_ns', ns);
						}
						
						return o;
					},
					
				// Get overlay Namespace
				
				getOverlayNS = function() {
						return getOverlay().data('oly_ns');
					},
			
				// Attaching related boxes to be removed with the overlay
				
				attachToOverlay = function(el) {
						var ns = getOverlayNS();
							
						if (typeof ns !== UNDEF) {
							el.each(function() {
									$(this).attr('rel', ns);
								});
						}
					},
					
				// Removing all extra overlays
				
				removeExtraOverlays = function() {
						var removeOly = function(o) {
									var ns = o.data('oly_ns');
									
									if (typeof ns !== UNDEF) {
										// Event handlers
										$window.off('.' + ns);
										// Attached elements
										$('[rel=' + ns + ']').remove();
									}
									
									o.trigger('lightboxRemove');
									o.remove();
								};
								
						getExtraOverlays().each(function() {
								removeOly($(this));
							});
					},
					
				// Removing currrent overlay; showing base
				
				removeOverlay = function() {
						var bo = getBaseOverlay();
						
						// Class to signal the page has no overlay 
						main.removeClass(settings.hasOverlayClass);
						
						// Removing existing overlay(s) besides base
						removeExtraOverlays();
						
						if (bo.length) {
							// Showing base overlay
							bo.show();
							// Attached elements
							$('[rel=' + bo.data('oly_ns') + ']').show();
							// Refreshing thumbnails
							//bo.find('.' + settings.thumbsClass).trigger('refresh');
							
							// restore original URL
							removeParam(settings.indexName);
								
							//main.trigger('overlayRemoved', main.children('section.' + settings.thumbContClass + ':visible').eq(0));
							updateShares();
							toggleSelectButtons(0, getImages().filter('.' + settings.selectableClass).length); 						
						}
					},

				// Getting folders
				
				getFolders = function(o, sel) {
					
						if (typeof o === UNDEF || typeof o == STRING) {
							sel = (typeof o == STRING)? o : '';
							o = getOverlay();
						}
						
						if (o.length) {
							return o.find('.' + settings.folderClass + (sel? (' ' + sel) : ''));
						}
						 
						return $();
					},
													
				// Getting all images of the overlay
				
				getImages = function(o, sel) {
							
						if (typeof o === UNDEF || typeof o == STRING) {
							sel = (typeof o == STRING)? o : '';
							o = getOverlay();
						}
						
						if (o.length) {
							return o.find('.' + settings.imageClass + (sel? (' ' + sel) : ''));
						}
						
						return $();
					},
					
				// Getting current thumbs array
				
				getBaseImages = function(sel) {
						return getBaseOverlay().find('section.' + settings.imageClass + (sel? (' ' + sel) : ''));
					},
				
				// Getting all the cards of the overlay (image + caption)
				
				getCards = function(o, sel) {
						
						if (typeof o === UNDEF || typeof o == STRING) {
							sel = (typeof o == STRING)? o : '';
							o = getOverlay();
						}
						
						return o.find('.' + settings.cardClass + (sel? sel : '')).filter(function() {
								// Returns cards only with images inside
								return $(this).children('.' + settings.imageClass).length; 
							});
					},
					
				// Getting current card
				
				getActiveCard = function() {
						
						return getOverlay().find('.' + settings.cardClass + '.' + settings.activeClass);
					},
					
				// Getting attached lightbox
				
				getLightbox = function(o) {
					
						return (o || getOverlay()).find('.' + settings.lightboxClass);
					},
					
				// Loading an image in lightbox
				
				lightboxLoad = function(card, auto) {
					
						hideAllPanels();
						getOverlay().trigger('lightboxLoad', (typeof auto === UNDEF)? card : [ card, auto ]);
					},
					
				// Leaving lightbox
				
				lightboxQuit = function() {
					
						hideAllPanels();
						getOverlay().trigger('lightboxQuit');
					},
					
				// Getting image by file name, name is mandatory

				getCardByName = function(o, name) {
						var e = $(),
							n;
						
						if (typeof o === STRING) {
							// no overlay specified
							name = o;
							o = getBaseOverlay();
						}
						
						o.find('.' + settings.cardClass + ' .' + settings.imageClass).each(function() {
								n = this.getAttribute((this.nodeName === 'A')? 'href' : 'src');	
								if (n && (n.slice(0, 7) === 'slides/' && n.substring(7) === name || n === name)) {
									e = $(this).closest('.' + settings.cardClass);
									return false;
								}
							});
						
						return e;
					},
				
				// Getting selected items from current overlay
				
				getSelectedItems = function(o) {
						var r = [];
						
						if (typeof o === UNDEF || !o.length) {
							o = getOverlay();
						}
						
						o.find('.' + settings.cardClass + '.' + settings.checkedClass).each(function() {
								r.push($(this).data(J.OBJ));
							});
						
						return r;
					},

				// Getting active item
				
				getCurrentItem = function(o) {
					
						if (typeof o === UNDEF || !o.length) {
							o = getOverlay();
						}
						
						var i = o.find('.' + settings.cardClass + '.' + settings.activeClass);
						
						return i.length? i.data(J.OBJ) : null;
					},

				// Getting all items from current overlay
				
				getAllItems = function(o) {
					
						if (typeof o === UNDEF || !o.length) {
							o = getOverlay();
						}
						
						var r = [];
								
						o.find('.' + settings.cardClass).each(function() {
								r.push($(this).data(J.OBJ));
							});
						
						return r;
					},
								
				/********************************************************
				 *
				 *						Folder tree
				 *
				 ********************************************************/
				
				getFolderTree = function() {
					
						var getFolderObjects = function(folder) {
								if (folder.hasOwnProperty(J.FOLDERS) && folder[J.FOLDERS].length) {
									var ul, li, i, p, f = folder[J.FOLDERS];
									ul = $('<ul>');
									for (i = 0; i < f.length; i++) {
										p = album.getPath(f[i]);
										li = $('<li>').appendTo(ul);
										li.append($('<a>', {
												href: 	(p? (p + '/') : '') + settings.indexName,
												html: 	f[i][J.TITLE] || f[i][J.NAME] || ''
											})).append(getFolderObjects(f[i]));
										if (!p) {
											li.addClass('actual');
										}
									}
									return ul;
								}
								return $();
							},
							
							getHomeLink = function() {
								var p = settings.rootPath;
								return $('<a>', {
										'class': 	'home-link icon-home',
										href:		(p? (p + '/') : '') + settings.indexName,
										text:		' ' + album.getAlbumTitle()
									});
							};
							
						if (settings.folderTree.type === 'tree') {
							if (settings.level > 0) {
								return getHomeLink().add(getFolderObjects(album.getTree()));
							} else {
								return getFolderObjects(album.getTree());
							}
						} else {
							getFolderObjects(album.getCurrentFolder());
						}
					};
			
			/********************************************************
			 *
			 *					Image overlay rendering 
			 *			or initializing images in the base overlay
			 *
			 ********************************************************/
				 
			$.fn.renderImages = function(items, complete, baseSet) {
				
					if (typeof items === UNDEF || !items.length) {
						return this;
					}
					
					if (!Array.isArray(items)) {
						items = [ items ];
					}
					
					var overlay 			= $(this).eq(0),
						ns 					= overlay.data('oly_ns'),								// Namespace
						thumbContainer		= $(),													// thumbnail container (for overlays)
						animateTimeout 		= null,													// Animate tracking
						animateRunning 		= false,												// lock to avoid multiple calls
												
						// Loading visible images
						
						animateImages = function() {
							
								if (animateRunning) {
									return;
								}
								
								clearTimeout(animateTimeout);
									
								var st = $window.scrollTop();
								
								if (Math.abs(st - scrollTop) > 20) {
									// Scrolling in progress: wait until finishes
									scrollTop = st;
									animateTimeout = setTimeout(animateImages, settings.scrollCheckInterval);
									return;
								}
								
								//$body.toggleClass(settings.belowfoldClass, ($window.outerHeight() - st) <= 0);
								scrollTop = st;
								
								// Animate images only if not in lightbox
								
								if (getLightbox().is(':hidden')) {
									
									animateRunning = true;
									
									// Preloaded images
									getCards().filter('.' + settings.preloadClass).removeClass(settings.preloadClass)
									
									/*.each(function() {
											var card = $(this),
												img = card.find('.' + settings.imageClass + ' img').eq(0);
												
											if (img.length) {
												img.one('load', function() {
														$(this).css('visibility', 'visible');
														if (!$(this).hasClass(settings.appearClass)) {
															$(this).removeClass(settings.disappearClass).addClass(settings.appearClass);
														}
													});
												
												// Cached image?
												if (img[0].complete) {
													img.trigger('load');
												}
											
											}
											
											card.removeClass(settings.preloadClass);
										});
									*/
									var wh = $window.height();
									
									// Only cards with lazyload or disappear class
									getCards().filter('.' + settings.lazyloadClass).each(function() {
											var card = $(this),
												tt = card.offset().top;
												
											if (tt < scrollTop + wh - 20) {
												// Above fold
												var img = card.find('.' + settings.imageClass + ' img').eq(0);
												
												if (!img.length || typeof img.attr('src') !== UNDEF && img.attr('src') !== false) {
													// Not image or has source already: apply appear class
													if (!card.hasClass(settings.appearClass)) {
														card.removeClass(settings.disappearClass).addClass(settings.appearClass);
													}
														
												} else {
													// Unloaded image; apply source and wait until loaded to appear
													img.one('load', function() {
															// Showing image on load
															if (!$(this).hasClass(settings.appearClass)) {
																$(this).removeClass(settings.disappearClass).addClass(settings.appearClass);
															}
														})
														.attr('src', img.data('src') || album.getThumbPath(card.data(J.OBJ)));
												}
												
												card.removeClass(settings.lazyloadClass);
											}
										});
									
								}
								
								// perpetrating animate
								animateTimeout = setTimeout(animateImages, settings.scrollCheckInterval);
								animateRunning = false;
							},
													
						// Rendering images
							
						initImages = function() {
								var cards = getCards(), 
									card,
									image,
									img,
									isThumb,
									isLbable,
									isMedia,
									caption,
									s,
									d,
									cd = settings.hasOwnProperty('markNew')? (nows - settings.markNew.days * ONEDAY_S) : null,
									item;
								
								// Creating new elements for the items with no thumbs
								for (var i = 0; i < items.length; i++) {
									
									item = items[i];
									
									if (i < cards.length) {
										// Already exists in the HTML source
										card = cards.eq(i);
									} else {
										// Not yet exists in the HTML source: happens only on overlays
										card = $('<div>', {
												'class': 	[ settings.cardClass, settings.lazyloadClass, item[J.CATEGORY] ].join(' ')
											})
											.appendTo(thumbContainer);
											
									}
									
									// Finding/creating image's <a> wrap
									image = card.children('.' + settings.imageClass);
									if (!image.length) {
										image = $('<a>', {
												'class':	settings.imageClass
											})
											//.data('href', album.getItemPath(item))
											.appendTo(card);
									}
									
									// Data pointer
									card.data(J.OBJ, item);
									
									isThumb = card.closest('section').hasClass(settings.thumbsClass);
									isLbable = album.isLightboxable(item);
									isMedia = item[J.CATEGORY] === 'video' || item[J.CATEGORY] === 'audio';
									
									// Creating structure
									if (isLbable) {
										
										card.addClass(settings.lbableClass);
										
										if (isMedia) {
											// Video or audio
										} else {
											
											//image.addClass(settings.lazyloadClass);
											
											// Finding/creating image
											img = image.find('img').eq(0);
											if (!img.length) {
												img = $('<img>', {
														'class':	settings.disappearClass
													}).appendTo(image);
											}
											
											// No source yet?
											if (typeof img.attr('src') === UNDEF || img.attr('src') === false) {
												// Saving thumb path for lazy load
												img.data('src', isThumb? 
															album.getThumbPath(card.data(J.OBJ)) 
															:
															album.getImagePath(card.data(J.OBJ))
														);
											}
											
											// Adding width, height
											if (isThumb) {
												img.attr({
														width:		item[J.THUMB][J.WIDTH],
														height:		item[J.THUMB][J.HEIGHT]
													});
											} else {
												img.attr({
														width:		item[J.IMAGE][J.WIDTH],
														height:		item[J.IMAGE][J.HEIGHT]
													});
											}
											
											// Adding extra classes
											if ((item[J.THUMB][J.HEIGHT] / item[J.THUMB][J.WIDTH]) < 0.5) {
												card.addClass(settings.panoClass);
											}

											// Icon thuumbnail?
											if (item[J.THUMB][J.PATH].match(/res\/\w+\.png$/)) {
												image.addClass('iconthumb');
											}
											
											// CLick event
											image.on('click.' + ns, imageClicked);
										}
									}
									
									// Caption
									if (isThumb) {
										if (s = item[J.THUMBCAPTION]) {
										
											caption = card.children('.' + settings.captionClass);
										
											if (!caption.length) {
												// No caption yet
												caption = $('<div>', {
														'class': 	settings.captionClass,
														html: 		s
													}).appendTo(card);
											}
										
											card.addClass(settings.hascaptionClass);
										}
									} else {
										card.toggleClass(settings.hascaptionClass, card.children('.' + settings.captionClass).length);
									}
									
									// Preparing for Lightbox
									if (isLbable) {
										
										if (shopRoot.length && album.hasShop(item) || feedbackRoot.length && settings.feedback) {
											// Select box
											image.addClass(settings.selectableClass);
											
											if (shopRoot.length) {
												d = $('<a>', {
															'class': 	'icon-shopping-cart add-cart',
															text: 		' ' + text.addCart
														})
													.on('click.' + ns, function(e) {
															var card = $(e.target).closest('.' + settings.cardClass);
															if (card.length) {
																shopRoot.trigger('addItems', card.data(J.OBJ));
															}
															return false;
														})
													.appendTo(image);
													
												d.addTooltip(album.getPriceRange(item));
											}
										}
										
										if (!baseSet && settings.linkToFolder) {
											// Folder link
											var p = album.getPath(item);
											
											if (p) {
												image.append($('<a>', {
														'class': 	'icon-folder folder-link',
														href:		 p + settings.indexName
													}));
											}
										}
										/*
										if (baseSet) {
											image.attr('href', '#img=' + item[J.PATH]);
										}
										*/
									} else {
										
										card.addClass(item[J.CATEGORY]);
										image.attr('href', album.getItemPath(item));
									}
									
									// Mark new images
									if (cd != null) {
										
										d = item[J.DATES];
										
										if (d && (d = d[settings.markNew.reference]) && d >= cd) {
											s = settings.markNew.text?
												$('<span>', {
														'class':	'new-image',
														text:		settings.markNew.text
													})
												:
												$('<span>', {
														'class':	'icon-new-fill new-image'
													});
											
											image.append(s);
											
											if (!TOUCHENABLED) {
												s.data('tooltip', (new Date(d * 1000)).toLocaleDateString(LOCALE)).addTooltip();
											}
										}
									}
									
									// Rating
									if (settings.useRating && isLbable) {
										
										var r = item[J.RATING] || 0;
										
										if (settings.visitorRating || r) {
											
											s = $('<span>', {
													'class':	'rating',
													html:		STARS
												})
												.appendTo(image);
												
											if (settings.visitorRating) {
												
												if (item.hasOwnProperty(J.VISITORRATING)) {
													// Already exists
													r = item[J.VISITORRATING];
												} else {
													if (!settings.useJalbumRating) {
														// Ignore jAlbum rating
														r = 0;
													}
													// Save the default
													item[J.VISITORRATING] = r;
												}
												
												s.addClass('r' + r + (settings.useJalbumRating? '' : ' icon-close'))
													.on('click.' + ns, function(e) {
															var el = $(this),
																r = Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left - (parseFloat(el.css('paddingLeft')))) / 16), 5),
																item = el.closest('.' + settings.cardClass).data(J.OBJ); 
																//Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left + 4) / el.width()) * 5, 5);
																
															el.removeClass('r0 r1 r2 r3 r4 r5').addClass('r' + r);
															item[J.VISITORRATING] = r;
															
															if (!baseSet) {
																// Propagate to base overlay
																el = getCardByName(getBaseOverlay(), item[J.NAME]);
																el.find('.rating')
																	.removeClass('r0 r1 r2 r3 r4 r5')
																	.addClass('r' + r);
															}
															
															return false;
														});
											} else {
												// Unalterable rating from jAlbum
												s.addClass('r' + r);
											}
										}
									}
										
									// Prevent copy
									if (settings.rightClickProtect) {
										image.on('contextmenu', function(e) {
												e.preventDefault()
												return false;
											});
									}
								}
								
							},
													
						// Clicked image
						
						imageClicked = function(e) {
								if (e.target.nodeName === 'A' && ($(e.target).closest('.' + settings.commentClass).length || $(e.target).hasClass('folder-link'))) {
									// Link in comment or link to folder
									return true;
								}
								lightboxLoad($(e.target).closest('.' + settings.cardClass));
								return false;
							},
														
						// Marking active card
						
						setActiveCard = function(card) {
							
								if (card) {
									var t = 			$(card),
										item =			t.data(J.OBJ),
										wh = 			$window.height(),
										scrollTop = 	$window.scrollTop(),
										tt = 			t.offset().top,
										th = 			t.outerHeight();
									
									getCards().removeClass(settings.activeClass);
									t.addClass(settings.activeClass);
									
									if (album.hasLocation(item)) {
										mapRoot.trigger('setActive', item[J.NAME]);
									} else {
										mapRoot.trigger('resetMarkers');
									}
									
									// Moving into view
									if (tt + th > scrollTop + wh) {
										$('html, body').stop(true, false).animate({
												scrollTop: 		Math.round(tt - wh + th + 30) + 'px'
											});
									} else if (tt < scrollTop) {
										$('html, body').stop(true, false).animate({
												scrollTop: 		Math.round(Math.max(tt - 30, 0)) + 'px'
											});
									}
								}
							},
							
						// Refreshing the rating
						
						refreshRating = function(e, card, rating) {
							
								if (card) {
									var t = $(card),
										o = t.data(J.OBJ),
										r = ((typeof rating !== UNDEF)? rating : (settings.visitorRating? o[J.VISITORRATING] : o[J.RATING])) || 0;
										
									t.find('.rating')
										.removeClass('r0 r1 r2 r3 r4 r5')
										.addClass('r' + r);
										
									if (!baseSet) {
										var el = getCardByName(o[J.NAME]);
										el.find('.rating')
											.removeClass('r0 r1 r2 r3 r4 r5')
											.addClass('r' + r);
									}
								}
								
							};
					
					// Creating new Unique ID if not yet exists
					if (!ns) {
						overlay.data('oly_ns', ns = 'oly_' + Math.floor(Math.random() * 10000));
					}
					
					thumbContainer = overlay.find('.' + settings.thumbContClass).eq(0);
					
					if (!baseSet) {
						if (!thumbContainer.length) {
							thumbContainer = createThumbContainer(overlay);
						}
						
						if (items.length > settings.columns) {
							thumbContainer.addClass('multi');
						} else {
							thumbContainer.addClass('cnt-' + items.length);
						}
						
						overlay.append($('<div>', {
								'class': 	'progressbar'
							}));
					}
						
					initImages();
										
					animateTimeout = setTimeout(animateImages, settings.scrollCheckInterval / 2);
					
					// Moving the active element into view
					overlay.on('setactivecard', function(e, card) {
							setActiveCard(card);
						});
					
					// Fixing thumbnails for vertical layout
					//overlay.on('refresh', refresh);
					
					// Refreshing the rating
					overlay.on('refreshRating', refreshRating);
					
					if (typeof complete === 'function') {
						if (typeof baseSet !== UNDEF) {
							complete.call(undefined, overlay, baseSet);
						} else {
							complete.call(undefined, overlay);
						}
					}
					
					overlay.find('.progressbar').remove();
					
					return this;
				};
				
			
			var	
				/********************************************************
				 *
				 *					Preparing images
				 *
				 ********************************************************/
										
				prepareImages = function(overlay, baseSet) {
						
						if (!album) {
							return;
						}
						
						var cards = getCards(overlay);
						
						if (cards.length > 1) {
							
							// Filtering and sort only if there are at least 2 items
							
							if ($('.filter-cont').length) {
								// Initializing Filters / Sort
								prepareFilterBox(baseSet);
							}
						}
						
						if (!cards.filter('.' + settings.lbableClass).length) {
							
							$('.' + settings.startshowClass).fadeOut();
							
						} else {
							
							if (album.hasShop() || settings.feedback) {
								
								var ci = cards.find('.' + settings.selectableClass);
								
								toggleSelectButtons(0, ci.length);
								
								if (ci.length) {
									// Make items selectable
									overlay.selectable({
											cardClass:			settings.selectableClass,
											checkedClass:		settings.checkedClass,
											selectionChange:	function() {
																		toggleSelectButtons(this, ci.length); 
																		shopRoot.add(feedbackRoot).trigger('itemsSelected', this);
																	}
										});
									
									
									shopRoot.add(feedbackRoot).find('.' + settings.keepSelectedClass).on('click.' + ns, function() {
											keepSelectedItems(text.selectedItems, function() {
													removeParam();
													updateShares();
												});
											
											return false;
										});
									
								}
							}
							
							/* Waiting for Derek's answer
							if (!LOCAL && typeof FOTOMOTO !== UNDEF && settings.fotomoto) {
								FOTOMOTO.API.checkinImages(getFotomotoImages(cards), function(images) {
										
									});
							}
							*/
							
							overlay.trigger('lightboxRemove');
							
							overlay.lightbox(album, {
									
									baseSet:		(typeof baseSet === UNDEF)? false : baseSet,
									
									onLoadStart:	function(card) {
															var item = card.data(J.OBJ);
															if (item) {
																addParam(settings.pageName || settings.indexName, {
																		img: 		album.getItemName(item)
																	}, item[J.TITLE]);
															}						
														},
														
									onLoadEnd: 		function(card) {
															overlay.trigger('setactivecard', card);
															setTimeout(updateShares, 100);
														},
													
									onClose:		function() {
															removeParam(settings.pageName || settings.indexName, 'img');
															updateShares();
														},
													
									onReady:		function() {
															if (initByHash && window.location.hash) {
																initByHash = false;
																var param = readParam();
																
																if (param.hasOwnProperty('label')) {
																	// Label filter
																	getItemsByLabel(param.label, function() {
																		addParam(settings.indexName, {
																				label:			param.label,
																				search:			null,
																				newimages:		null,
																				date:			null
																			}, text.label + ': ' + param.label);
																	});
																} else if (param.hasOwnProperty('search')) {
																	// Search
																	getItemsBySearch(param.search, function() {
																		addParam(settings.indexName, {
																				search:			param.search,
																				label:			null,
																				newimages: 		null,
																				date:			null
																			}, text.search + ': ' + param.search);
																	});
																} else if (param.hasOwnProperty('newimages')) {
																	// Search new images
																	getItemsByDate(getSearchNewOptions(param.newimages), function() {
																		addParam(settings.indexName, {
																				newimages:		param.newimages,
																				search:			null,
																				label:			null,
																				date:			null
																			}, text.searchNew + ': ' + param.newimages);
																	});
																} else if (param.hasOwnProperty('date')) {
																	var options = getDateRangeOptions(param.date);
																	// Search images in date range
																	getItemsByDate(options, function() {
																		addParam(settings.indexName, {
																				date:			param.date,
																				newimages:		null,
																				search:			null,
																				label:			null
																			}, getDateReference(options.reference) + ': ' + param.date);
																	});
																} else if (param.hasOwnProperty('img')) {
																	var card = getCardByName(param.img);
																	if (card.length) {
																		if (param.hasOwnProperty('slideshow')) {
																			lightboxLoad(card, 2000);
																		} else {
																			lightboxLoad(card);
																		}
																	} else {
																		removeParam(settings.pageName || settings.indexName, 'img');
																	}
																} else if (param.hasOwnProperty('slideshow')) {
																	lightboxLoad(0, 2000);
																}
															}
															
															$('.' + settings.startshowClass).fadeIn().on('click', function() {
																	$(this).trigger('removeTooltip');
																	startAuto();
																	return false;
																});
														},
													
									onSlideshowStart:
													function() {
															if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
																$(settings.backgroundMusic.hook).trigger('startPlayer');
															}
														},
													
									onSlideshowPause:
													function() {
															if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
																$(settings.backgroundMusic.hook).trigger('pausePlayer');
															}
														}
								});
						}
					},
				
				// Hash change listener :: when the browser's back / forward buttons were pressed
				
				stateChange = function() {
						if (window.location.hash === lastHash) {
							return;
						}
						
						lastHash = window.location.hash;
						var param = readParam();
						
						if (param.hasOwnProperty('label')) {
							// Label filter
							getItemsByLabel(param.label, function() {
								setParam(settings.indexName, {
										label:	param.label
									}, text.label + ': ' + param.label);
								updateShares();
							});
						} else if (param.hasOwnProperty('search')) {
							// Search
							getItemsBySearch(param.search, function() {
								setParam(settings.indexName, {
										search:	param.search
									}, text.search + ': ' + param.search);
								updateShares();
							});
						} else if (param.hasOwnProperty('newimages')) {
							// New images
							getItemsByDate(getSearchNewOptions(param.newimages), function() {
								setParam(settings.indexName, {
										newimages: param.newimages
									}, text.newImages + ': ' + param.newimages);
								updateShares();	
							});
						} else if (param.hasOwnProperty('date')) {
							// Date range
							var options = getDateRangeOptions(param.date);
							getItemsByDate(options, function() {
								setParam(settings.indexName, {
										date: param.date
										}, getDateReference(options.reference) + ': ' + param.date);
								updateShares();	
							});
						} else if (param.hasOwnProperty('img')) {
							// Image (no search or label)
							var card = getCardByName(param.img);
							if (card.length) {
								if (param.hasOwnProperty('slideshow')) {
									lightboxLoad(card, 2000);
								} else {
									lightboxLoad(card);
								}
							} else {
								removeParam(settings.indexName, 'img');
							}
						} else if (param.hasOwnProperty('slideshow')) {
							lightboxLoad(0, 2000);
						} else {
							// Plain index page
							lightboxQuit()
							removeOverlay();
						}
						
					},
				
				// Starts slideshow
				
				startAuto = function(e) {
						var card = $();
						
						if (!settings.restartSlideshow) {
							card = getCards('.' + settings.activeClass);
						}
						
						if (!card.length) {
							// Fall back to first
							card = getCards().eq(0);
						}
					
						if (SMALLSCREEN && settings.slideshowFullscreen) {
							requestFullscreen();
						}
						
						lightboxLoad(card, 2000);
						
						return false;
					},
				
				/********************************************************
				 *
				 *		Functionalities that results in new overlay
				 *
				 ********************************************************/
				
				/********************************************************
				 *
				 *				Keeping only selected items
				 *
				 ********************************************************/
				
				keepSelectedItems = function(tx, ready) {
						var items = getSelectedItems();
						
					if (!items.length || items.length === getImages('.' + settings.lbableClass).length) {
							return;
						}
						
						var t = addOverlay({
									className:	'selected-items',
									icon:		'icon-checkbox-checked',
									title:		'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						setTimeout(function() {
								h.find('.progressbar').remove();
								h.append($('<small>', {
										text: 		text.foundNTimes.replace('{0}', items.length)
									}));
								t.renderImages(items, prepareImages);
								toggleSelectButtons(0, items.length);
								//toggleBoxes(items.length);
								if (typeof ready === FUNCTION) {
									ready.call();
								}
							}, 50);
					},
				
				/********************************************************
				 *
				 *					Search and tag search
				 *
				 ********************************************************/
				 
				// Executing Search
				
				getItemsBySearch = function(tx, ready) {
						var t = addOverlay({
									className:	'search-results',
									icon:		'icon-search',
									title:		'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						setTimeout(function() {
							album.collectItems($.extend(settings.search, {
								exact:		false,
								terms:		tx,
								types:		'-folder',
								ready:		function() {
													h.find('.progressbar').remove();
													h.append($('<small>', {
															text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
													t.renderImages(this, prepareImages);
													toggleSelectButtons(0, this.length);
													//toggleBoxes(this.length);
													if (typeof ready === FUNCTION) {
														ready.call();
													}
												}
							}));
						}, 50);
					},
					
				// Preparing Search functionality
				
				prepareSearch = function(f) {
					
						if (!f.length || !album) {
							return;
						}
						
						// Adding defaults if missing
						
						settings.search = $.extend({
								depth:		'tree',
								fields:		'creator,keywords,title,comment'
							}, settings.search);
						
						var startSearch = function(f) {
								
								var tx = f.find('input').eq(0).val();
								
								if (tx && tx.length > 1) {
									getItemsBySearch(tx, function() {
											setParam(settings.indexName, {
													search:		tx
												}, text.search + ': ' + tx);
											updateShares();
										});
								}
								return false;
							};
							
						f.find('.search-btn').on('click', function() {
								if (f.hasClass(settings.activeClass)) {
									startSearch(f);
									f.removeClass(settings.activeClass);
								} else {
									f.find('input').eq(0).focus();
									f.addClass(settings.activeClass);
								}
								return false;
							});
						
						f.on('submit', function(e) {
								e.preventDefault();
								startSearch(f);
								f.removeClass(settings.activeClass);
								return false;
							});
						
						f.find('input').on({
								'focus': 	function() { 
													f.addClass(settings.activeClass); 
												}
							});
					},
				
				// Preparing Tag Search functionality
				
				prepareTagSearch = function() {
					
						var f = $(settings.tagCloud.searchHook);
						
						if (!f.length || !album) {
							return;
						}
										
						var startSearch = function(f) {
								
								var tx = f.find('input').eq(0).val();
								
								if (tx && tx.length > 1) {
									getItemsByLabel(tx, function() {
										setParam(settings.indexName, {
												label: 	tx
											}, text.label + ': ' + tx);
										updateShares();
									});
								}
								return false;
							};
							
						f.find('.search-btn, button').on('click', function() {
							f.find('input').eq(0).focus();
							startSearch(f);
							return false;
						});
						
						f.on('submit', function(e) {
							e.preventDefault();
							startSearch(f);
							return false;
						});
						
					},
				
				/********************************************************
				 *
				 *						Tag cloud
				 *
				 ********************************************************/
				 
				// Do a label filtering
				
				getItemsByLabel = function(tx, caseSensitive, ready) {
						var t = addOverlay({
									className:		'tags-found',
									icon:			'icon-tag',
									title:			'&ldquo;' + tx + '&rdquo;'
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
							'class': 'progressbar'
						}));
						
						if (t.offset().top < $window.scrollTop() || t.offset().top > ($window.scrollTop() + $window.height() - 100)) {
							$window.scrollTop(t.offset().top);
						}
						
						if (typeof caseSensitive === UNDEF) {
							caseSensitive = false;
						} else if (typeof caseSensitive === FUNCTION) {
							ready = caseSensitive;
							caseSensitive = false;
						}
						
						album.collectItems($.extend(settings.tagCloud, {
								exact:			settings.exactFields,
								caseSensitive:	caseSensitive,
								terms:			tx,
								ready: 			function() {
														h.find('.progressbar').remove();
														h.append($('<small>', {
															text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
														t.renderImages(this, prepareImages);
														toggleSelectButtons(0, this.length);
														//toggleBoxes(this.length);
														if (typeof ready === FUNCTION) {
															ready.call();
														}
													}
						}));
					},
	
				// Collecting and displaying tags
			
				renderTagCloud = function(tags) {
					
						if (!tags.length) {
							return;
						}
						
						var t = $(settings.tagCloud.hook);
						
						if (!t.length) {
							return;
						}
						
						var tagClicked = function() {
								var tx = $(this).children('span').eq(0).text();
								
								if (tx && tx.length > 1) {
									getItemsByLabel(tx, true, function() {
										setParam(settings.indexName, {
												label: 		tx
											}, text.label + ': ' + tx);
										updateShares();
									});
								}
								
								return false;
							};
						
						for (var a, r, cnt, i = 0, l = tags.length; i < l; i++) {
							
							// create link element
							a = $('<a>', {
									'class': 	'tag'
								}).append($('<span>', {
									html: 		tags[i][0]
								}));
							a.on('click', tagClicked);
							
							// add count
							if (tags[i][1] > 1) {
								if (settings.tagCloud.fontVaries) {
									r = 0;
									cnt = Math.min(64, tags[i][1]);
									// calculate weight
									while (cnt > 1) {
										r++;
										cnt = cnt >> 1;
									}
									a.addClass('size-' + r);
								}
								a.append('&nbsp;').append($('<sup>', {
									text: 		tags[i][1]
								}));
							}
							
							t.append(a);
						}
						
						if (tags.length) {
							var d;
							
							if (t.outerHeight() > 175) {
								
								t.css({
									maxHeight: 150,
									overflow: 'hidden'
								});
								
								d = $('<div>', {
									'class': 'toggle-height'
								});
								
								d.append($('<a>', {
										'class': 	'icon-caret-down',
										text: 		' ' + text.more
									}).on('click', function() {
										var b = t.parent(),
											toggleHeight = function() {
												var h = t.prop('scrollHeight');
												
												b.css({
														height: 	'auto',
														overflow: 	'visible'
													});
												t.css({
														maxHeight: 	150,
													});
												setTimeout(function() {
														t.css({
																maxHeight: 	h + 20
															});
													}, 25);
												d.addClass(settings.activeClass);
											};
										
										b.css({
												height: 	b.height(),
												overflow: 	'hidden'
											});
										t.css({
												maxHeight: 	'none'
											});
										setTimeout(toggleHeight, 50);
									
								})).append($('<a>', {
										'class': 	'icon-caret-up',
										text: 		' ' + text.less
									}).on('click', function() {
										t.css({
												maxHeight: 	150
											});
										d.removeClass(settings.activeClass);
									}));
								
								t.after(d);
							}
						}
					},
					
				/********************************************************
				 *
				 *						Search new images
				 *
				 ********************************************************/
				 
				getSearchNewOptions = function(days) {
					
						return settings.hasOwnProperty('searchNew')?
							{
								reference:		settings.searchNew['reference'] || 'dateTaken',
								depth:			settings.searchNew['depth'] || 'subfolders',
								start:			nows / ONEDAY_S - days
							}
							:
							{
								reference:		'dateTaken',
								depth:			'subfolders',
								start:			nows / ONEDAY_S - days
							};
						
					},
					
				// Search for new images
				
				getDateRangeOptions = function(range) {
						var m = range.match(/^([amt])?([cts])?\:?(\d+)\-?(\d+)?/i),
							o = {
									range:		0,
									reference:	'dateTaken',
									depth:		'current'
								};
								
						if (m) {
							if (m[1]) {
								// Reference
								if (m[1] === 'a') {
									o.added = 'added';
								} else if (m[1] === 'm') {
									o.added = 'fileModified';
								} else if (m[1] === 't') {
									o.added = 'dateTaken';
								}
							}
							
							if (m[2]) {
								// Depth
								if (m[2] === 'c') {
									o.depth = 'current';
								} else if (m[2] === 't') {
									o.depth = 'tree';
								} else if (m[2] === 's') {
									o.depth = 'subfolders';
								}
							}
							
							if (m[4]) {
								// Range
								o.start = parseInt(m[3]) || 0;
								o.end = parseInt(m[4]);
							} else if (m[3]) {
								// Single day
								o.start = parseInt(m[3]);
								o.range = 1;
							}
						}
						
						return o;
					},
				
				getDateReference = function(r) {
					
						if (r === 'dateTaken') { 
							return text.imagesTaken;
						} else if (r === 'fileModified') {
							return text.imagesModified;
						}
						
						return text.imagesAdded;
					},
					
				getDateRangeTitle = function(o) {
						var t1 = getDateReference(o['reference']),
							t2,
							start,
							end,  
							today = Math.floor(new Date() / ONEDAY_MS),
							date = function(d) {
									return new Date(d * ONEDAY_MS).toLocaleDateString(LOCALE);
								};
						
						if (!o.hasOwnProperty('end') && !o.hasOwnProperty('start')) {
							t2 = getTimespan(o.range);
						} else if (!o.hasOwnProperty('end')) {
							if (o.hasOwnProperty('start')) {
								if (o.hasOwnProperty('range')) {
									if (o['range'] > 1) {
										t2 = text.betweenDays.template(date(o.start), date(o.start + o.range));
									} else {
										t2 = text.onDay.template(date(o.start));
									}
								} else {
									t2 = getTimespan(nows / ONEDAY_S - o.start);
									//t2 = text.afterDay.template(date(o.start));
								}
							}
						} else {
							t2 = text.beforeDay.template(date(o.end));
						}	
						
						return t1 + ' ' + t2;
					},
				
				getItemsByDate = function(options, ready) {
						var t = addOverlay({
									className:		'search-results',
									icon:			'icon-calendar',
									title:			getDateRangeTitle(options)
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.append($('<span>', {
								'class': 		'progressbar'
							}));
						
						setTimeout(function() {
								album.collectByDate($.extend(options, {
									ready:		function() {
														h.find('.progressbar').remove();
														h.append($('<small>', {
															text: this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
														}));
														t.renderImages(this, prepareImages);
														toggleSelectButtons(0, this.length);
														//toggleBoxes(this.length);
														if (typeof ready === FUNCTION) {
															ready.call();
														}
													}
								}));
							}, 50);
					},
					
				// Search date range
				
				searchDateRange = function(range) {
					
						if (range) {
							
							var options = getDateRangeOptions(range);
							
							getItemsByDate(options, function() {
									setParam(settings.indexName, {
											date:		range
										}, getDateRangeTitle(options));
									updateShares();
								});
						}
						return false;
					},
				
				// Search new images
				
				searchNew = function(days) {
						
						if (days) {
							
							getItemsByDate($.extend(settings.searchNew, { 
									start: 		nows / ONEDAY_S - days 
								}), function() {
									setParam(settings.indexName, {
											newimages:		days
										}, getDateReference(settings.searchNew['reference']));
									updateShares();
								});
						}
						return false;
					},
				
				// Prepare Search new functionality
				
				prepareSearchNew = function(f) {
					
						if (!f.length || !album) {
							return;
						}
						
						// Setting up defaults if something is missing
						
						settings.searchNew = $.extend({
								days: 				'3,30,90',
								depth: 				'folder',
								reference: 			'dateTaken',
								sinceLastVisit:		true
							}, settings.searchNew);
						
						var	days = settings.searchNew.days.split(',');
						
						f.append($('<label>', {
								text: 	getDateReference(settings.searchNew.reference)
							}));
								
						for (var i = 0; i < days.length; i++) {
							f.append($('<a>', {
									'class': 	'small button',
									text: 		getTimespan(days[i])
								})
								.data('days', parseInt(days[i], 10))
								.on('click', function() {
									searchNew($(this).data('days'));
								}));
						}
								
						if (settings.searchNew.sinceLastVisit) {
							
							var start = $.cookie('lastVisit'),
								now = Math.round(new Date() / 1000);
							
							if (start && start < (now - ONEDAY_S)) {
								f.append($('<a>', {
										'class': 	'small button',
										text: 		text.sinceMyLastVisit
									})
									.on('click', function() {
										searchNew((now - start) / ODEDAY_S);
									}));
							}
							
							// Storing last visit date for 10 years
							$.cookie('lastVisit', now, 315360000);
						}
						
					},
															
				/********************************************************
				 *
				 *					Filtering & Sort
				 *
				 ********************************************************/
				
				// Dates are objects
				
				getDate = function(item, name) {
						if (item.hasOwnProperty(J.DATES)) {
							if (name === 'addedDate') {
								return item[J.DATES]['added'];
							} else if (name === 'originalDate') {
								return item[J.DATES]['dateTaken'];
							} else {
								return item[J.DATES]['fileModified'];
							}
						}
						
						return null;
					},
					
				// Some variables get remapped by the skin
				
				getMappedVal = function(item, name) {
					
						if (item.hasOwnProperty(name)) {
							// Generic case
							return item[name];
						}
						
						return null;
					},
					
				// Do filtering on current folder
				
				doFilter = function(e, ready) {
						var form = $(e.target).closest('form'),
							items = getAllItems(),
							cnt = items.length,
							tx = [];
						
						form.find('select:visible,input[type=range],input[type=date]').each(function() {
								var name = this.name,
									label = form.find('label[for=' + name + ']').text(),
									type = $(this).data('type'),
									value = $(this).val(),
									display = function(val) {
											if (name ===  'rating') {
												return Math.round(val);
											} else if (name === 'fileSize') {
												return niceByte(val);
											} else if (type === 'date') {
												return (new Date(val * 1000)).toLocaleDateString(LOCALE);
											} else if (type === 'numeric') {
												if ($(this).data('unit') === 'seconds') {
													return niceTime(val);
												} else if ($(this).data('unit') === 'byte') {
													return niceByte(val);
												} else if (val >= 10) {
													return Math.round(val);
												} else if (val >= 1) {
													return val.toFixed(1);
												} else if (val >= 0.1) {
													return val.toFixed(2);
												}
												return val;
											}
											return toCurrency(val, currency);
										};
									
								if (type === 'numeric' || type === 'date' || type === 'financial') {
										
									var low = $(this).data('low'),
										high = $(this).data('high');
										
									if (low > this.min || high < this.max) {
										// Only if the range is narrower than the whole range
										
										if (name.endsWith('Date')) {
											// Date
											for (var i = items.length - 1, val; i >= 0; i--) {
												if ((val = getDate(items[i], name)) !== null && 
													(val >= low && val <= high)) {
													continue;
												}
												items.splice(i, 1);
											}
										} else if (JCAMERA.indexOf(name) !== -1) {
											// Camera data fields
											if (name === 'exposureTime') {
												for (var i = items.length - 1, val; i >= 0; i--) {
													if (items[i].hasOwnProperty(J.CAMERA) &&
														items[i][J.CAMERA].hasOwnProperty(name) &&
														(val = getExposure(items[i][J.CAMERA][name])) &&
														(val >= low && val <= high)) {
														continue;
													}
													items.splice(i, 1);
												}
											} else if (name.startsWith('focalLength')) {
												for (var i = items.length - 1, val; i >= 0; i--) {
													if (items[i].hasOwnProperty(J.CAMERA) &&
														items[i][J.CAMERA].hasOwnProperty(name) &&
														!isNaN(val = parseInt(items[i][J.CAMERA][name])) && 
														(val >= low && val <= high)) {
														continue;
													}
													items.splice(i, 1);
												}
											} else {
												for (var i = items.length - 1, val; i >= 0; i--) {
													if (items[i].hasOwnProperty(J.CAMERA) &&
														items[i][J.CAMERA].hasOwnProperty(name) &&
														(val = items[i][J.CAMERA][name]) !== null &&
														(val >= low && val <= high)) {
														continue;
													}
													items.splice(i, 1);
												}
											}
										} else if (name === 'rating') {
											// Rating
												for (var i = items.length - 1, val, r = settings.visitorRating? J.VISITORRATING : J.RATING; i >= 0; i--) {
													val = items[i][r] || 0;
												if (val >= low && val <= high) {
													continue;
												}
												items.splice(i, 1);
											}
										} else {
											// Generic
											for (var i = items.length - 1, val; i >= 0; i--) {
												if ((val = items[i][name]) !== null && 
													(val >= low && val <= high)) {
													continue;
												}
												items.splice(i, 1);
											}
										}
									
										tx.push(label + '=' + display(low) + '&ndash;' + display(high));
									}
									
								} else if (this.nodeName === 'SELECT' && value !== null && value.length) {
									
									// Only if at least one option is selected
																		
									if (!Array.isArray(value)) {
										value = [ value ];
									}
									
									if (JCAMERA.indexOf(name) !== -1) {
										// Camera data :: no arrays expected
										for (var i = items.length - 1, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(J.CAMERA) &&
												items[i][J.CAMERA].hasOwnProperty(name) &&
												(val = items[i][J.CAMERA][name]) !== null &&
												value.includes(val)) {
												continue;
											}
											items.splice(i, 1);
										}
										
									} else {
										// Generic case
										for (var i = items.length - 1, found, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(name)) {
												val = items[i][name];
												if (Array.isArray(val)) {
													// Array e.g. keywords
													found = false;
													for (var j = 0; j < value.length; j++) {
														if (val.includes(value[j])) {
															console.log(items[i]['name'] + ': ' + val);  
															found = true;
															break;
														}
													}
													if (found) {
														continue;
													}
												} else if (value.includes(val)) {
													continue;
												}
											}
											items.splice(i, 1);
										}
									}
									
									// Header text addition
									if (this['multiple']) {
										tx.push(name + '=[' + value.join(',') + ']');  
									} else {
										tx.push(label + '=' + value);  
									}
								}
							});
						
						if (items.length < cnt) {
							
							// Only show overlay if at least one thumb was filtered out
							
							var t = addOverlay({
										className:	'filtered-items',
										//icon:		'icon-filter',
										title:		text.results
									}),
								h = t.find('header .' + settings.overlayTitleClass);
								
							h.addClass('filter')
								.append($('<span>', {
									'class': 	'progressbar'
								}));
							
							t.find('.close').on('click', resetBaseFilters);
							
							setTimeout(function() {
									h.find('.progressbar').remove();
									h.append($('<small>', {
											html: 		tx.join(', ') + ': ' + text.foundNTimes.replace('{0}', items.length)
										}));
									t.renderImages(items, prepareImages);
									toggleSelectButtons(0, items.length);
										toggleBoxes(items.length);
										if (typeof ready === FUNCTION) {
										ready.call();
									}
								}, 50);
						}
						
						return false;
					},
					
				// Do sorting on current set of thumbs
				
				doSort = function(e, ready) {
						var el = this.selectedOptions[0],
							name = this.value;
						
						if (!el || !name) {
							return true;
						}
						
						var items = getAllItems(),
							cnt = items.length,
							data = $(el).data('sort'),
							type = data['type'],
							asc = data['ascending'],
							tx = [],
							v1,
							v2;
							
						if (type === 'numeric' || type === 'financial' || type === 'date') {
							// Number
							
							if (name.endsWith('Date')) {
								// Date
								items = items.sort(function(i1, i2) {
										v1 = getDate(i1, name);
										v2 = getDate(i2, name);
										return asc? (v1 || 0) - (v2 || 0)
													:
													(v2 || 0) - (v1 || 0);
									});
								
							} else if (JCAMERA.indexOf(name) !== -1) {
								// Camera data
								if (name === 'exposureTime') {
									items = items.sort(function(i1, i2) {
											v1 = (i1.hasOwnProperty(J.CAMERA) && i1[J.CAMERA].hasOwnProperty(name))? 
													i1[J.CAMERA][name] : 0;
											v2 = (i2.hasOwnProperty(J.CAMERA) && i2[J.CAMERA].hasOwnProperty(name))? 
													i2[J.CAMERA][name] : 0;
											return asc? (getExposure(v1 || 0) - getExposure(v2 || 0)) 
														: 
														(getExposure(v2 || 0) - getExposure(v1 || 0));
										});
								} else {
									items = items.sort(function(i1, i2) {
											v1 = (i1.hasOwnProperty(J.CAMERA) && i1[J.CAMERA].hasOwnProperty(name))? 
													parseFloat(i1[J.CAMERA][name]) : 0;
											v2 = (i2.hasOwnProperty(J.CAMERA) && i2[J.CAMERA].hasOwnProperty(name))? 
													parseFloat(i2[J.CAMERA][name]) : 0;
											return asc? ((v1 || 0) - (v2 || 0)) 
														: 
														((v2 || 0) - (v1 || 0));
										});
								}
								
							} else if (name === 'rating') {
								// Rating							
									var r = settings.visitorRating? J.VISITORRATING : J.RATING;
								items = items.sort(function(i1, i2) {
											return asc? (i1[r] || 0) - (i2[r] || 0)
													:
														(i2[r] || 0) - (i1[r] || 0);
									});
								
							} else {
								// Every other numeric
								items = items.sort(function(i1, i2) {
										if (i1.hasOwnProperty(name) && i2.hasOwnProperty(name)) {
											return asc?	parseFloat(i1[name]) - parseFloat(i2[name])
														:
														parseFloat(i2[name]) - parseFloat(i1[name]);
										}
										return asc? (i1.hasOwnProperty(name)? 1 : 0)
													:
													(i2.hasOwnProperty(name)? -1 : 0);
									});
							}
							
						} else {
							// String
							
							items = items.sort(function(i1, i2) {
								if (i1.hasOwnProperty(name) && i2.hasOwnProperty(name)) {
									return asc?	(i1[name] + '').localeCompare(i2[name] + '')
												:
												(i2[name] + '').localeCompare(i1[name] + '');
								}
								return asc? (i1.hasOwnProperty(name)? 1 : 0)
											:
											(i2.hasOwnProperty(name)? -1 : 0);
							});
							
						}
						
						var t = addOverlay({
									className:	'sorted-items',
									//icon:		'icon-ordering',
									title:		text.sortedBy + '<small>' + data.label + ' (' + text[data.ascending? 'ascending' : 'descending'] + ')</small>' 
								}),
							h = t.find('header .' + settings.overlayTitleClass);
							
						h.addClass('icon-ordering sort')
							.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						t.find('.close').on('click', resetBaseSort);
						
						setTimeout(function() {
								h.find('.progressbar').remove();
								//h.after(fh);
								t.renderImages(items, prepareImages);
								toggleSelectButtons(0, items.length);
								//toggleBoxes(items.length);
								if (typeof ready === FUNCTION) {
									ready.call();
								}
							}, 50);
						
						return false;
					},
					
				// Read filter ranges from current overlay
				
				getFilterRanges = function(items, filter) {
						
						if (!filter || !items.length) {
							return;
						}
						
						var setMinMax = function(f, val) {
									if (f.hasOwnProperty('min') && f.hasOwnProperty('min')) {
										if (f.min > val) {
											f.min = val;
										} else if (f.max < val) {
											f.max = val;
										}
									} else {
										f.min = f.max = val;
									}
								};
								
						for (var name in filter) {
							
							var f = filter[name], 
								type = f['type'];
							
							if (type === 'numeric' || type === 'date' || type === 'financial') {
								
								if (name.endsWith('Date')) {
									// Date
									for (var i = items.length - 1, val; i >= 0; i--) {
										if ((val = getDate(items[i], name)) !== null) {
											setMinMax(f, val);
										}
									}
								} else if (JCAMERA.indexOf(name) !== -1) {
									// Camera data fields
									if (name === 'exposureTime') {
										for (var i = items.length - 1, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(J.CAMERA) &&
												items[i][J.CAMERA].hasOwnProperty(name) &&
												(val = getExposure(items[i][J.CAMERA][name]))) {
												setMinMax(f, val);
											}
										}
									} else if (name.startsWith('focalLength')) {
										for (var i = items.length - 1, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(J.CAMERA) &&
												items[i][J.CAMERA].hasOwnProperty(name) &&
												!isNaN(val = parseInt(items[i][J.CAMERA][name]))) {
												setMinMax(f, val);
											}
										}
									} else {
										for (var i = items.length - 1, val; i >= 0; i--) {
											if (items[i].hasOwnProperty(J.CAMERA) &&
												items[i][J.CAMERA].hasOwnProperty(name) &&
												(val = items[i][J.CAMERA][name]) !== null) {
												setMinMax(f, val);
											}
										}
									}
								} else if (name === 'rating') {
									// Rating
									for (var i = items.length - 1, val, r = settings.visitorRating? J.VISITORRATING : J.RATING; i >= 0; i--) {
										val = items[i][r] || 0;
										setMinMax(f, val);
									}
								} else {
									// Generic
									for (var i = items.length - 1, val; i >= 0; i--) {
										if ((val = items[i][name]) !== null) {
											setMinMax(f, val);
										}
									}
								}
								
							} else {
								// String
								if (!f.hasOwnProperty('values')) {
									f.values = [];
								}
								
								if (JCAMERA.indexOf(name) !== -1) {
									// Camera data :: no arrays expected
									for (var i = 0, val; i < items.length; i++) {
										if (items[i].hasOwnProperty(J.CAMERA) &&
											items[i][J.CAMERA].hasOwnProperty(name)) {
											pushNew(f.values, (items[i][J.CAMERA][name] + '').trim());
										}
									}
								} else {
									for (var i = 0, val; i < items.length; i++) {
										if (items[i].hasOwnProperty(name)) {
											pushNew(f.values, items[i][name]);
										}
									}
								}
							}
						}
						
						return filter;
					},
					
				// Enabling / disabling the reset button
				
				enableResetBtn = function(e) {
						var form = (typeof e === UNDEF || e.hasOwnProperty('target'))? $(this).closest('form') : e;
						form.find('button[type=reset]').removeClass('disabled');
					},
					
				disableResetBtn = function(e) {
						var form = (typeof e === UNDEF || e.hasOwnProperty('target'))? $(this).closest('form') : e;
						form.find('button[type=reset]').addClass('disabled');
					},
					
				// Reseting filters to defaults
				
				_resetFilters = function(form) {
					
						form.find('select').each(function() {
								$(this).prop('selectedIndex', this['multiple']? -1 : 0);
							});
						
						form.find('input[type=range]').each(function() {
								$(this).trigger('resetRange');
							});
						
						disableResetBtn(form);
						
					},
					
				// Reset filters after a form elemnt changed
				
				resetFilters = function(e) {
						_resetFilters($(e.target).closest('form'));
						return false;
					},
					
				// Reseting base filters to defaults
				
				resetBaseFilters = function() {
						resetBaseSort();
						_resetFilters($('#filters form.filter'));
					},
					
				// Reseting base filters to defaults
				
				resetBaseSort = function() {
						$('#filters form.sort select').prop('selectedIndex', 0);
					},
					
				// Preparing filters / sort
				
				prepareFilterBox = function(baseSet) {
					
						var items = getAllItems(),
							filters = album.getPropertyObject(album.getCurrentFolder(), J.FILTERS, true),
							sort = album.getPropertyObject(album.getCurrentFolder(), J.SORT, true),
							box = $(),
							cont,
							filterForm = $(),
							sortForm = $(),
							group,
							inp,
							lbl,
							type,
							buttons,
							btn;
							
						if ((filters || sort) && items.length > 1) {
							
							if (baseSet) {
								
								box = $('#filters');
								filterForm = box.find('form.filter');
								
							} else {
								
								box = $('<div>', {
										'class':		'box filter-cont',
										id:				'filters_' + getOverlayNS()
									}).insertBefore($('#filters').hide());
									
								box.append($('<h' + settings.overlayHeadLevel + '>', {
										'class':		'icon-' + (filters? 'filter' : 'ordering'),
										text:			$('#filters').find('.' + settings.overlayTitleClass).text()
									}));
								
								filterForm = $('<form>', {
										'class':		'filter'
									}).appendTo(box);
									
							}
							
							buttons = $('<div>', {
									'class':	'buttons'
								}).appendTo(filterForm);
								
							// Attaching box to current overlay
							attachToOverlay(box);
						
						} else {
							
							if (baseSet) {
								$('#filters').hide();
							}
							
							return;
						}
						
						if (filters) {
							
							// Getting filter range
							filters = getFilterRanges(items, filters);
							
							var m;
							
							for (var f in filters) {
								
								type = filters[f]['type'];
								
								group = $('<div>', {
										'class':		'group ' + ((type === 'multiple_string' || type === 'single_string')? 'select' : 'range') 
									}).insertBefore(buttons);
								
								if (filters[f]['label']) {
									lbl = $('<label>', {
											'for':			f,
											text:			filters[f]['label']
										}).appendTo(group);
								}
									
								if (type === 'multiple_string' || type === 'single_string') {
									
									if (f === 'rating') {
										filters[f].values = [ text.noRating, 1, 2, 3, 4, 5 ];
									}
									
									inp = $('<select>', {
											name:			f
										}).appendTo(group);
									
									if (type === 'multiple_string') {
										inp.attr('multiple', true);
										inp.addTooltip(text.multipleSelectHint);
									}
									
									if (!filters[f].hasOwnProperty('values') || !filters[f].values.length) { // || filters[f].values.length < 2) {
										
										group.remove();
										
									} else {
									
										if (type === 'single_string') {
											inp.prepend($('<option>', {
													text:			text.select,		// + ' ' + filters[f]['label'],
													disabled:		'',
													selected:		''
												}));
										}
										
										if (filters[f].values) {
											for (var i = 0; i < filters[f].values.length; i++) {
												inp.append($('<option>', {
														text:			filters[f].values[i]
													}));
											}
										}
										
										inp.on('change', enableResetBtn);
									}
									
								} else {
									
									inp = $('<input>', {
											name:		f,
											type:		'range'
										})
										.data('type', type)
										.appendTo(group);
									
									if (f === 'rating') {
										filters[f]['step'] = 1;
										filters[f]['min'] = 0;
										filters[f]['max'] = 5;
									}
									
									if (!filters[f].hasOwnProperty('min') || !filters[f].hasOwnProperty('max') || filters[f]['min'] === filters[f]['max']) {
										
										group.remove();
										
									} else {
										
										if (type === 'financial' && (m = filters[f]['label'].match(/\S+\s+[\(\[](\S+)[\)\]]/))) {
										
											filters[f]['currency'] = m[1];
										
										} else if (type === 'numeric') {
											
											if ('exposureTime.shutterSpeed'.indexOf(f) >= 0) {
												filters[f]['unit'] = 'seconds';
											} else if (f === 'fileSize') {
												filters[f]['unit'] = 'byte';
												filters[f]['step'] = 1;
											}
										}
										filters[f].onChanged = enableResetBtn;
										filters[f].onReseted = disableResetBtn;
										
										inp.rangeSlider(filters[f]);
									}
								}
									
								inp.attr('data-type', type || 'single_string');
							}
							
							// Adding buttons
							
							if ((inp = filterForm.find('input,select')).length) {
								
								filterForm.data('single', inp.length === 1 && inp[0].nodeName === 'SELECT' && !inp.attr('multiple'));
								
								if (!(baseSet && filterForm.data('single'))) {
									// Reset button
									btn = $('<button>', {
											type:			'reset',
											'class':		(baseSet? 'disabled secondary' : 'alert') + ' button',
											text:			' ' + text.reset
										}).appendTo(buttons);
									
										if (baseSet) {
											btn.on('click.' + ns, resetFilters);
										} else {
											btn.on('click.' + ns, function() {
													resetBaseFilters();
													removeOverlay();
												});
										}
								}
									
								if (filterForm.data('single')) {
									// Single combo box -> Update instantly on change
									
									inp.on('change.' + ns, doFilter);
									
								} else {
									// Multiple inputs or not single combo box -> Use 'Search' button
									btn = filterForm.find('button[type=submit]');
									
									if (!btn.length) {
										btn = $('<button>', {
												type:			'submit',
												'class':		'button',
												text:			' ' + text.search
											}).appendTo(buttons);
									}
									
									btn.on('click.' + ns, doFilter);
								}
								
								filterForm.on('submit.' + ns, doFilter);
							}
						}
						
						if (buttons.is(':empty')) {
							buttons.remove();
						}
						
						if (sort) {
							
							
							if (baseSet) {
								
								sortForm = box.find('form.sort');
								
							} else {
								
								sortForm = $('<form>', {
											'class':		'sort'
										}).appendTo(cont);
							}
							
							lbl = $('<label>', {
									'class':		'icon-ordering',
									'for':			'sortby',
									text:			' '
								}).appendTo(sortForm);
								
							inp = $('<select>', {
									'class':		'small',
									name:			'sortby'
								}).appendTo(sortForm);
							
							inp.prepend($('<option>', {
									text:			text.sortBy,
									disabled:		'',
									selected:		''
								}));
							
							for (var s in sort) {
								
								inp.append($('<option>', {
										html:			'&#8613; ' + sort[s].label,
										value:			s
									}).data('sort', {
										type:			sort[s].type,
										label:			sort[s].label,
										ascending:		true
									}));
								
								inp.append($('<option>', {
										html:			'&#8615; ' + sort[s].label,
										value:			s
									}).data('sort', {
										type:			sort[s].type,
										label:			sort[s].label,
										asc:			false
									}));
							}
						
							inp.on('change.' + ns, doSort);
						}
						
						if ((!sortForm.length || sortForm.is(':empty')) && (!filterForm.length || filterForm.is(':empty'))) {
							box.hide();
						}
					},
				
				/********************************************************
				 *
				 *						Controls
				 *
				 ********************************************************/
				
				// Initializing ZIP download button
				
				initZip = function() {
				
						panels.find('.zip-cont a.button').each(function() {
								var btn = $(this);
								
								getFileSize(btn.attr('href'), function(size) {
										if (size) {
											btn.append('<span class="badge">' + niceByte(size) + '</span>');
										}
									});
							});
					},

				// Show panel
				
				showPanel = function(p, doneFn) {
						
						clearTimeout(p.data('fade'));
						
						getOverlay().trigger('stopAutoHide');
						
						var btn = controls.find('[data-rel=' + p[0].id + ']'),
							
							init = function() {
									// Preset
									var pos = btn.position(),
										t =	pos.top + btn.outerHeight(),
										l = Math.minMax(0, pos.left, $window.outerWidth() - p.outerWidth());
									
									// Remove tooltip
									btn.trigger('removeTooltip');
									
									p.css({
											transition:		'none',
											display: 		'flex',
											opacity:		0,
											transform:		'translateY(16px)'
										});

									p.css({
											left:			l,
											top:			t,
											maxWidth:		'calc(100vw - ' + l + 'px)',
											maxHeight:		'calc(100vh - ' + t + 'px)'
										});
									
									p.data('finished', false);
									window.requestAnimationFrame(start);
								},
								
							start = function() {
								
									p.one('transitionend', function() {
												if (!p.data('finished')) {
													p.css({
															display: 		'flex',
															opacity:		1
														});
													p.data('finished', true);
													window.requestAnimationFrame(finish);
												}
											})
										.css({
												transition:		'all 500ms ease-out',
												opacity:		1,
												transform:		'translateY(0)'
											});
								
								},
								
							finish = function() {
								
								var ac = btn.data('autoclose');

									if (ac) {
										
										// Do not autoclose while mouse is around
										p.add(btn)
											.on('mouseenter.' + ns + ' mouseover.' + ns, function() {
													p.data('over', true);
													clearTimeout(p.data('fade'));
												})
											.on('mouseleave.' + ns, function() {
													p.data('over', false);
													p.data('fade', setTimeout(function() {
															if (!p.data('over')) {
																clearTimeout(p.data('fade'));
																hidePanel(p);
															}
														}, ac));
												});
											
										// Auto hide
										p.data('fade', setTimeout(hidePanel, ac, p));
									
									}
									
									// Render map on demand
									if (p[0].id === 'map' && !mapRoot.data('rendered')) {
										setTimeout(function(btn) {
												mapRoot.trigger('render');
											}, 20, $(this));
									}
									
									if (typeof doneFn === 'function') {
										doneFn.call(null);
									}
								};
						
						if (btn.is(':visible')) {
							// Add 'on' class
							btn.add(p).addClass('on');
										
							clearTimeout(p.data('fade'));
							window.requestAnimationFrame(init);
						} else {
							hidePanel(p);
						}
								
					},
					
				// Hide panel
				
				hidePanel = function(p, doneFn) {
						
						if (p.is(':hidden')) {
							p.data('over', false);
							return;
						}
						
						if (p.data('over')) {
							clearTimeout(p.data('fade'));
							p.data('fade', setTimeout(hidePanel, 1000, p));
						}
						
						var btn = controls.find('[data-rel=' + p[0].id + ']'),
							finished = false;
						
						// Related button
						btn.add(p).removeClass('on');
						p.hideAllTooltips();
						//console.log('Hiding "' + id + '"');
						//stopTransition(p);
						
						p.one('transitionend', function(e) {
									if (!finished) {
										finished = true;
										$(this).css({
													transition:		'none',
													display: 		'none',
													opacity:		0
												});
											
										$(this).removeClass('on');
											
										if (typeof doneFn === 'function') {
											doneFn.call(null);
										}
									}
								})
							.css({
									transition:		'all 500ms ease-out',
									opacity:		0,
									transform:		'translateY(16px)'
								});
					},
					
				// Hiding all panels
				
				hideAllPanels = function(doneFn) {
					
						panels.children('.on').each(function() {
								hidePanel($(this));
							});
						
						if (typeof doneFn === 'function') {
							doneFn.call(null);
						}
					},
						
				// Initializing controls
				
				initControls = function() {
						var menuTo = null;
						
						if (controls.find('.fullscreen').length) {
							// Wiring buttons
							// Go fullscreen
							controls.find('.fullscreen').on('click.' + ns, function() {
									$(this).trigger('removeTooltip');
									requestFullscreen(function() {
											main.addClass(settings.fullscreenClass);
										})
								});
							
							// Exit fullscreen
							
							controls.find('.windowed').on('click.' + ns, function() {
									
									$(this).trigger('removeTooltip');
									exitFullscreen(function() {
											main.removeClass(settings.fullscreenClass);
										})
									
								});
						}
					
						initZip();
						
						// Click outside a panel to hide
						main.on('click.' + ns, function(e) { 
								var el = $(e.target); 
								
								if (!el.closest('.' + settings.panelClass).length) {
									hideAllPanels();
								}
								
								return true; 
							});
						
						// Track belowfold class
						$window.on('scroll.' + ns, function() {
								$body.toggleClass(settings.belowfoldClass, ($('.hero').outerHeight() - $window.scrollTop()) <= 0);
							});
						
						// Toggle buttons
						controls
							.find('[data-rel]')
							.each(function() {
									var btn = $(this),
										p = $('#' + btn.data('rel'));
										
									if (p.length) {
										// Related panel
										btn
											.addClass('has-panel')
											.on('selectstart.' + ns, function(e) {
													e.preventDefault();
													return false;
												})
											.on('click.' + ns, function() {
													var controls = $(this).closest('.' + settings.controlsClass),
														btn = $(this),
														p = $('#' + btn.data('rel'));
													
													controls.hideAllTooltips();
													p.data('over', false);
													
													if (btn.hasClass('on')) {
														hidePanel(p);
													} else {
														hideAllPanels(function() {
																p.data('over', true);
																showPanel(p);
															});
													}
													
													return false;
												});
	
									} else {
										// No related panel found
										btn.removeAttr('data-rel');
									}
								});
							
					},
					
						
				/********************************************************
				 *
				 *			Initialization of misc functions
				 *
				 ********************************************************/
										
				// Right click blocking
				
				avoidRightClick = function() {
						getImages().add(getFolders()).on('contextmenu', 'img', function(e) {
								e.preventDefault()
								return false;
							});
					},
					
				// Initializing background music
				
				prepareBackgroundMusic = function(player) {
					
						if (player.length) {
							player.audioPlayer({
									rootPath: 	settings.rootPath
								});
						}
					},
					
				// Updating the share buttons after a state change
				
				updateShares = function() {
						if (settings.hasOwnProperty('share')) {
							var opt = { 
									href: 	window.location.href 
								};
								
							if (getLightbox().is(':visible')) {
								var d = getActiveCard().data(J.OBJ);
								if (d) {
									opt.image = 		d[J.IMAGE][J.PATH];
									opt.title = 		d.hasOwnProperty(J.TITLE)? 
															d[J.TITLE] 
															: 
															d[J.NAME].stripExt().replace(/[_-]/g, ' ');
									opt.description = 	d.hasOwnProperty(J.IMAGECAPTION)? 
															d[J.IMAGECAPTION].stripHTML() 
															: 
															(d.hasOwnProperty(J.COMMENT)? d[J.COMMENT].stripHTML() : '');
								}
							}
							
							$(settings.share.hook).trigger('updateLinks', opt);
						}	
					},
									
				// Initializing Social sharing
				
				prepareShares = function(opt) {
						
						$.fn.renderShares.defaults.buttonTheme = opt['buttonTheme'] || 'dark';
						$.fn.renderShares.defaults.indexName = settings.indexName;
						$.fn.renderShares.defaults.facebookAppId = opt['facebookAppId'] || '';
						
						if (opt.hasOwnProperty('callAction')) {
							$.fn.renderShares.defaults.callAction = opt.callAction;
						}
						
						$(opt['hook']).renderShares(opt);
						$.fn.lightbox.defaults.share = opt.sites;
						
						if (opt.hasOwnProperty('sites')) {
							$.fn.lightbox.defaults.shareHook = opt['hook'];
						}
					},
					
				// Preparing map
				
				prepareMap = function(images, settings) {
					
							var	getMarkers = function(images) {
										// Reading out map locations
										var markers = [];
										
										for (var i = 0, l; i < images.length; i++) {
											l = album.getLocation(images[i]);
											if (l) {
												markers.push({
													title:		[((i + 1) + '.'), images[i]['title'], images[i]['comment']].filter(Boolean).join(' '),
													pos: 		l,
													link:		images[i].path
												});
											}
										}
										
										return markers;
									};
							
							// Change defaults
							if (settings.hasOwnProperty('type')) {
								$.fn.lightbox.defaults.mapType = settings.type;
								$.fn.addMap.defaults.type = settings.type;
							}
							if (settings.hasOwnProperty('zoom')) {
								$.fn.lightbox.defaults.mapZoom = settings.zoom;
								$.fn.addMap.defaults.zoom = settings.zoom;
							}
							if (settings.hasOwnProperty('apiKey')) {
								$.fn.addMap.defaults.apiKey = settings.apiKey;
							}
						
							mapRoot.addMap({
									markers: 			getMarkers(images),
									fitBounds:			true,
									onMarkerClick:		function() {
																if (this.hasOwnProperty('link')) {
																	var el = getCardByName(this.link);
																	if (el.length) {
																		if (getLightbox().is(':visible')) {
																			lightboxLoad(el);
																		} else {
																			getOverlay().trigger('setactivecard', el);
																		}
																	}
																}
															}
								});
					
					},
				
				// Preparing shopping cart
				
				prepareShoppingCart = function() {
					
						// assigning paypal.js defaults 
						$.fn.paypal.defaults.relPath = settings.relPath;
						$.fn.paypal.defaults.rootPath = settings.rootPath;
						
						// Initializing the cart
						currency = album.getCurrency();
						
						// Set rangeslider's default currency
						if ($.fn.hasOwnProperty('rangeSlider')) {
							$.fn.rangeSlider.defaults.currency = currency;
						}
						
						if (shopRoot.length) {
							// Calling Paypal module
							shopRoot.paypal(album, { 
									
									getSelected: 	function() {
															return getSelectedItems();
														},
														
									selectNone:		function() {
															getOverlay().trigger('selectNone');
														},
														
									getCurrent:		function() {
															return getCurrentItem();
														},
														
									button:			controls.find('[data-rel=shop]')
									
								});
							
							// "Add selected" button
							shopRoot.find('.' + settings.addSelectedClass).on('click', function() {
									shopRoot.trigger('addItems', getSelectedItems());
									getOverlay().trigger('selectNone');
									return false;
								});
							
							// Ensuring lightbox has 'shop' property
							if (!$.fn.lightbox.defaults.hasOwnProperty('shop')) {
								$.fn.lightbox.defaults.shop = {};
							}
							
							// Setting lightbox's shop root variable
							$.fn.lightbox.defaults.shop.root = shopRoot;
						}
					
					},
					
				// Preparing feedback
				
				prepareFeedback = function() {
					
						// assigning feedback.js defaults 
						$.fn.feedback.defaults.relPath = settings.relPath;
						$.fn.feedback.defaults.rootPath = settings.rootPath;
						
						// Initializing the feedback cart
						if (feedbackRoot.length) {
							
							// Get selected items
							settings.feedback['getSelected'] = function() {
									return getSelectedItems();
								};
								
							settings.feedback['selectNone'] = function() {
									getOverlay().trigger('selectNone');
								};
								
							settings.feedback['getCurrent'] = function() {
									return getCurrentItem();
								};
								
							settings.feedback['button'] = controls.find('[data-rel=feedback]');
							
							// Calling Feedback module
							feedbackRoot.feedback(album, settings.feedback);
															
							// "Add selected" button
							feedbackRoot.find('.' + settings.addSelectedClass).on('click', function() {
									feedbackRoot.trigger('addItems', getSelectedItems());
									getOverlay().trigger('selectNone');
									return false;
								});
							
							// Ensuring lightbox has 'feedback' property
							if (!$.fn.lightbox.defaults.hasOwnProperty('feedback')) {
								$.fn.lightbox.defaults.feedback = {};
							}
							// Setting lightbox's feedback root variable
							$.fn.lightbox.defaults.feedback.root = feedbackRoot;
						}
					},
					
				/********************************************************
				 *
				 *					Index page initialization
				 *
				 ********************************************************/
				
				prepareIndex = function() {
					
						if (!album) {
							return;
						}
						
						var images = album.getImages();

						// Removing thumbnail links right away
						getImages().each(function() {
								var a = $(this).find('.' + settings.imageClass);
								a.data('href', a.attr('href'));
								a.attr('href', '');
							});
			
						// Preapring folders
						prepareFolders();
			
						// Shopping cart
						if ($.fn.hasOwnProperty('paypal')) {
							prepareShoppingCart();				
						}
						
						// Feedback tool
						if (settings.hasOwnProperty('feedback')) {
							prepareFeedback();
						}
						
						// "Select all" button
						shopRoot.add(feedbackRoot).find('.' + settings.selectAllClass).on('click', function() {
								getOverlay().trigger('selectAll');
								return false;
							});
						
						// "Select none" button
						shopRoot.add(feedbackRoot).find('.' + settings.selectNoneClass).on('click', function() {
								getOverlay().trigger('selectNone');
								return false;
							});
								
						// Rendering thumbnails
						if (images.length) {
							
							// Preparing images on the base overlay
							getBaseOverlay().renderImages(images, prepareImages, true);
							
							// Start slideshow
							$('#start-slideshow').on('click', startAuto);
							                                           
							// Collecting markers and initializing map
							if (settings.hasOwnProperty('map')) {
								prepareMap(images, settings.map);
							}
							
						} else {
							// no thumbnails, only folders
							if (window.location.hash) {
								// we should check the search / tags / newimages filter
								stateChange();
							}
							
							$('#start-slideshow').hide();
						}
						
						// Collecting tags if gathered only from this folder
						if (settings.hasOwnProperty('tagCloud')) {
							
							if (settings.tagCloud['depth'] === 'current') {
								album.collectTags($.extend(settings.tagCloud, {
										exact:		settings.exactFields,
										ready:		function() {
															renderTagCloud(this);
														}
									}));
							}
							
							if (settings.tagCloud.hasOwnProperty('searchHook')) {
								prepareTagSearch();
							}
						}
						
						// Custom query (eg. new images sample box)
						var queryTarget = $('[data-jalbum-query]');
						
						if (queryTarget.length) {
							// Reding settings from data attribute
							var qsettings = queryTarget.data('jalbum-query');
							
							if (qsettings.hasOwnProperty('fn')) {
								// Calling the jalbum.album function
								album[qsettings.fn]($.extend(qsettings, {
											ready:		function(opt) {
												
																for (var i = 0, a, item, mx = Math.min(this.length, 10); i < mx; i++) {
																	
																	item = this[i];
																	
																	a = $('<a>', {
																			href:	album.getLink(item)
																		}).appendTo(queryTarget);
																	
																	a.append($('<img>', {
																			src:		album.getThumbPath(item),
																			width:		item[J.THUMB][J.WIDTH],
																			height:		item[J.THUMB][J.HEIGHT],
																			alt:		item[J.TITLE] || item[J.NAME]
																		}));
																}
																
																if (this.length > 10) {
																	queryTarget.append($('<a>', {
																			'class':	'more',
																			href:		qsettings.linkMore,
																			text:		'+' + (this.length - 10)
																		}));
																}
																
																if (settings.rightClickProtect) {
																	queryTarget.on('contextmenu', 'img', function(e) {
																			e.preventDefault()
																			return false;
																		});
																}
															}
								
									}));
							}
						}
					},
				
				/********************************************************
				 *
				 *					Custom page initialization
				 *
				 ********************************************************/
				
				preparePage = function() {
						// Page :: initializing functions
						// Search 
						if (settings.hasOwnProperty('search')) {
							prepareSearch($(settings.search.hook));
						}
						
						settings.pageName = window.location.pathname.getFile();
						
						var paths = [];
						
						getCards().each(function() {
								var a = $(this).find('.' + settings.imageClass);
								a.data('href', a.attr('href'));
								a.attr('href', '');
								paths.push($(this).data(J.PATH));
							});
						
						if (paths.length) {
						
							album.collectByPath({
									paths:		paths,
									ready:		function() {
														// Rendering thumbnails
														if (this.length) {
															
															// Preparing images on the base overlay
															getBaseOverlay().renderImages(this, prepareImages, true);
															
															// Collecting markers and initializing map
															if (settings.hasOwnProperty('map')) {
																prepareMap(this, settings.map);
															}
														}
													}
								});
						}
					},
				
				/********************************************************
				 *
				 *	 Preparing functions that need the full album tree
				 *
				 ********************************************************/
				
				prepareDeepFunctions = function() {
					
						// Collecting tags if gathered from deep tree
						if (settings.hasOwnProperty('tagCloud') && settings.tagCloud['depth'] !== 'current') {
							album.collectTags($.extend(settings.tagCloud, {
									exact:		settings.exactFields,
									ready:		function() {
														renderTagCloud(this);
													}
								}));
						}
						
						// Search 
						if (settings.hasOwnProperty('search')) {
							prepareSearch($(settings.search.hook));
						}
						
						// Search new
						if (settings.hasOwnProperty('searchNew')) {
							prepareSearchNew($(settings.searchNew.hook));
						}
					},
					
				/********************************************************
				 *
				 *					Preparing folders
				 *
				 ********************************************************/
				
				// Marking new folders
				
				markFoldersNew = function() {
						
						if (!album) {
							return;
						}
						
						var folders = album.getFolders(),
							secs 	= settings.markNew['days'] * ONEDAY_S,
							s,
							d;
						
						getFolders().each(function(i) {
								
								if (i < folders.length) {
									d = folders[i][J.DATES];
									
									if (d) {
										d = d[settings.markNew['reference']] || d[J.DATERANGE][1] || d[J.DATERANGE][0];
										
										if (d && (nows - d) < secs) {
											s = settings.markNew.text?
												$('<span>', {
														'class':	'new-image',
														text:		settings.markNew.text
													})
												:
												$('<span>', {
														'class':	'icon-new-fill new-image'
													});
											
											if (!TOUCHENABLED) {
												s.data('tooltip', (new Date(d * 1000)).toLocaleDateString(LOCALE)).addTooltip();
											}
											
											$(this).find('.' + settings.thumbClass).eq(0).append(s);
										}
									}
								}
							});
					},
									
				// Adding one thumb or image
				/*
				loadImage = function(card) {
						
						var onloadFn = function() {
									$(this).addClass(settings.showImageClass).removeClass(settings.hideImageClass);
									card.removeClass(settings.lazyloadClass);
								},
							image;
							
						if (card.length) {
							var type = cardType(card);
							
							if (type === 'folder') {
									
								image = card.find('[data-src]');
								
								if (image.length) {
									image.each(function() {
											var img = $(this).children('img'); 
											if (!img.length) {
												img = $('<img>', {
														'class': 	settings.hideImageClass,
														alt:		album.getAlt(card.data(J.OBJ))
													}).appendTo($(this));
											}
											// Onload action
											img.attr('src', $(this).data('src'));
										});
									
									if (image.length === 1) {
										image.find('img').one('load', onloadFn);
									} else {
										image.find('img').waitAllImg(onloadFn);
									}
								} else {
									card.removeClass(settings.lazyloadClass);
								}
								
							} else {								
								
								var image = card.children('.' + settings.imageClass),
									img;
									
								if (image.length) {
									
									img = image.find('img'); 
									
									if (!img.length) {
										// Not yet created
										img = $('<img>', {
													'class': 	settings.hideImageClass,
													alt:		album.getAlt(card.data(J.OBJ))
												})
											.one('load', onloadFn)
											.prependTo(image);
									}
									
									img.attr('src', (type === 'image')?
												album.getImagePath(card.data(J.OBJ))
												:
												album.getThumbPath(card.data(J.OBJ))
										);
								}
							}		
						}
					},
				*/
				
				// Loading visible images
				
				lazyloadFolders = function() {
					
						if (lazyloadRunning || getLightbox().is(':visible')) {
							return;
						}
						
						if (lazyloadTimeout) {
							clearTimeout(lazyloadTimeout);
							lazyloadTimeout = null;
						}
						
						var st = $window.scrollTop();
						
						if (Math.abs(st - scrollTop) > 20) {
							// Scrolling in progress: wait until finishes
							scrollTop = st;
							lazyloadTimeout = setTimeout(lazyloadFolders, settings.scrollCheckInterval);
							return;
						}
						
						lazyloadRunning = true;
						
						// Lazyload images
						var toLoad = getFolders().children('.' + settings.lazyloadClass),
							wh = $window.height();
						
						toLoad.each(function() {
								var card = $(this),
									tt = card.offset().top,
									th = card.height();
									
								if ((tt + th) > (scrollTop - wh)) {
									// Below 1 screens above scroll pos
									if (tt < (scrollTop + 1 * wh)) {
										// and above 1 screens down
										card.find('[data-src]').each(function() { 
												var thumb = $(this),
													img = thumb.find('img');
												
												if (!img.length) {
													img = $('<img>', {
															'class': 	settings.disappearClass
															//alt:		album.getAlt(card.data(J.OBJ))
														}).appendTo(thumb);
												}
												
												img.one('load', function() {
														$(this).addClass(settings.appearClass).removeClass(settings.disappearClass);
													})
													.attr('src', thumb.data('src'));
													
											});
										
										card.removeClass(settings.lazyloadClass);
										
									} else {
										// Far below: stop loading more
										return false;
									}
								}
							});
						
						// perpetrating lazyload
						scrollTop = st;
						lazyloadTimeout = setTimeout(lazyloadFolders, settings.scrollCheckInterval);
						lazyloadRunning = false;
							
					},
					
				// Preparing folders
			
				prepareFolders = function() { 
					
						var folders = getFolders();
						
						if (!folders.length) {
							return;
						}
						
						if (settings.hasOwnProperty('markNew')) {
							// Marking new folders
							setTimeout(markFoldersNew, 300);
						}
						
						setTimeout(function() {
								
								lazyloadFolders();
								
								// Removing prelaod class on folders already loaded
								folders.filter('.' + settings.preloadClass + ':first-child img')
									.waitAllImg(function() {
											folders.find('.' + settings.preloadClass).removeClass(settings.preloadClass);
										});
							}, 500);
					},
					
				/********************************************************
				 *
				 *					Initializing album
				 *
				 ********************************************************/
				
				initAlbum = function(readyFn) {
					
						if (typeof Album === UNDEF) {
							console.log('Critical Error: Missing jalbum.album.js library!');
						}
						
						var params = {
								makeDate:		settings['makeDate'],
								rootPath:		settings['rootPath'],
								relPath:		settings['relPath'],
								ready: 			(settings.pageType === 'index')? prepareIndex : preparePage,
								deepReady:		prepareDeepFunctions
							};
							
						if (settings.indexName !== $.fn.skin.defaults.indexName) {
							params.indexName = settings['indexName'];
						}
						
						if (settings.audioPoster !== $.fn.skin.defaults.audioPoster) {
							params.audioPoster = settings['audioPoster'];
						}
							
						if (settings.videoPoster !== $.fn.skin.defaults.videoPoster) {
							params.videoPoster = settings['videoPoster'];
						}
						
						album = new Album($, params);
					},
					
				// Main initializing function
				
				init = function() {
					
						// Removing search string, e.g. FB data
						if (window.location.search) {
							removeSearch();
						}
						
						// Restoring scroll position, saving on unload
						restoreScrollPosition();
						$(window).on('beforeunload.' + ns, saveScrollPosition);
						
						// Passing along lightbox defaults
						passDefaults(settings, $.fn.lightbox.defaults, 'indexName,level,previousFoldersLast,previousFolderPath,nextFoldersFirst,nextFolderPath,extraSizes,rightClickProtect,enableKeyboard,enableMouseWheel,useRating,visitorRating');
						passDefaults(settings.lightbox, $.fn.lightbox.defaults);
						
						// Creating new Album instance
						initAlbum();
			
						// Right click protection on folders, slider
						if (settings.rightClickProtect) {
							avoidRightClick();
						}
						
						// Initializing controls
						initControls();
						
						// Preparing share buttons
						if (settings.hasOwnProperty('share')) {
							prepareShares(settings.share);
						}
						
						// Background music
						prepareBackgroundMusic($('[data-audioPlayer]'));
						
						// Hash change listener
						$window.on('hashchange.' + ns, stateChange);
						
						// Focusing overlay
						main.on('overlayAdded.' + ns, function(e, o) {
								moveInView(o);
							});
						
					};	
			
			/********************************************************
			 *
			 *					Execution starts here
			 *
			 ********************************************************/
				
			init();
			
			// Utility function for debugging
			this.getAlbum = function() {
					return album;
				};
			
			return this;
		};
	
	/********************************************************
	 *
	 *						Defaults
	 *
	 ********************************************************/
				
	$.fn.skin.defaults = {
			contentClass: 					'content',
			contClass:						'cont',
			hasOverlayClass:				'has-overlay',
			baseClass:						'base-overlay',
			overlayClass:					'overlay',
			overlayTitleClass:				'title',
			overlayHeadLevel:				3,
			lightboxClass:					'lightbox',
			controlsClass:					'controls',
			panelClass:						'panel',
			panelsClass:					'panels',
			sliderClass:					'slider',
			belowfoldClass:					'below-fold',
			folderContClass:				'folder-cont',
			fodlersClass:					'folders',
			folderClass:					'folder',
			thumbContClass:					'thumb-cont',
			thumbsClass: 					'thumbnails',
			thumbClass:						'thumb',
			contClass:						'cont',
			thumbLayout:					'fixgrid',
			columns:						4,
			imageClass:						'image',
			captionClasses:					'over bottom hover',
			cardClass:						'card',
			appearClass:					'appear',
			disappearClass:					'disappear',
			folderCardClass:				'foldercard',
			mosaicClass:					'mosaic',
			captionPlacement:				'below',
			lbableClass:					'lbable',
			captionClass:					'caption',
			commentClass:					'comment',
			hascaptionClass:				'hascaption',
			selectableClass:				'selectable',
			checkedClass:					'checked',
			activeClass:					'active',
			preloadClass:					'preload',
			lazyloadClass:					'lazyload',
			hideImageClass:					'hide-image',
			showImageClass:					'show-image',
			startshowClass:					'start-btn',
			fullscreenClass:				'fullscreen',
			panoClass:						'pano',
			selectBoxClass:					'select-box',
			selectAllClass:					'select-all',
			selectNoneClass:				'select-none',
			keepSelectedClass:				'keep-selected',
			addSelectedClass:				'add-selected',
			indexName:						'index.html',
			audioPoster:					'audio.poster.png',
			videoPoster:					'video.poster.png',
			exactFields:					'creator,keywords',
			hidePanelDelay:					{
												'topnav':		6000,
												'map':			10000,
												'albuminfo':	10000,
												'player':		3000,
												'social':		6000,
												'filters':		15000,
												'search':		0,
												'tagcloud':		6000
											},
			dateFormat:						'M/d/yyyy',
			rightClickProtect: 				false,
			useRating:						false,
			jalbumRating:					false,
			visitorRating:					false,
			enableKeyboard:					true,
			videoAuto:						true,
			linkToFolder:					true
		};
		
})(jQuery, jQuery(window), jQuery(document), jQuery('body'));
