/* 
 *	main.js - the skin's javascript functions
 */
	
;(function($, $window, $document, $body, undefined) {
	'use strict';
				
	// Implementing select functions on thumbnails
	
	$.fn.selectable = function(options) {
		
			options = $.extend({
							cardClass:			'card',
							checkboxClass: 		'checkbox',
							checkedClass:		'checked',
							noneSelected:		null,
							anySelected:		null
						}, options);
			
			var self = $(this).eq(0),
				cards,
				
				selectedCnt = function() {
						return cards.filter('.' + options.checkedClass).length;
					},
			
				getThumbs = function() {
						return options.selector? self.find(options.selector) : self.find('.' + options.cardClass);
					},
			
				clicked = function(e) {
						var card = $(e.target).closest('.' + options.cardClass);
						
						var n = selectedCnt();
						if (card.hasClass(options.checkedClass)) {
							card.removeClass(options.checkedClass);
							if (n === 1) {
								if (typeof options.noneSelected === FUNCTION) {
									options.noneSelected.call();
								}
							} else {
								if (typeof options.anySelected === FUNCTION) {
									options.anySelected.call(n - 1);
								}
							}
						} else {
							card.addClass(options.checkedClass);
							if (typeof options.anySelected === FUNCTION) {
								options.anySelected.call(n + 1);
							}
						}	
							
						return false;
					},
			
				selectAll = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							if (n < cards.length) {
								cards.not('.' + options.checkedClass).addClass(options.checkedClass);
								if (typeof options.anySelected === FUNCTION) {
									options.anySelected.call(cards.length);
								}
							}
						}
					},
			
				selectNone = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							if (n > 0) {
								cards.filter('.' + options.checkedClass).removeClass(options.checkedClass);
								if (typeof options.noneSelected === FUNCTION) {
									options.noneSelected.call();
								}
							}
						}
					},
					
				selectInverse = function() {
					
						if (self.is(':visible')) {
							var n = selectedCnt();
							if (n === cards.length) {
								cards.removeClass(options.checkedClass);
								if (typeof options.noneSelected === FUNCTION) {
									options.noneSelected.call();
								}
							} else {
								cards.toggleClass(options.checkedClass);
								if (typeof options.anySelected === FUNCTION) {
									options.anySelected.call(cards.length - n);
								}
							} 
						}
					},
				
				selectRefresh = function() {
						cards = getThumbs();
						
						cards.each(function() {
							if (!($(this).children('span.' + options.checkboxClass).length)) {
								$(this).append($('<span>', {
										'class': 	options.checkboxClass
									}).on({
										click: 		clicked
									}));
							}
							
						});
						
					};
			
			selectRefresh();
			
			self.on({
					selectAll: 			selectAll,
					selectNone: 		selectNone,
					selectInverse: 		selectInverse,
					selectRefresh: 		selectRefresh
				});
					
			return this;
		};
			
	// The main skin code
	
	$.fn.skin = function(settings) {
		
		settings = $.extend({}, $.fn.skin.defaults, settings);
		
		var content = 			$(settings.contentHook),				// Main content
			text = 				getTranslations({						// Translated texts
										foundNTimes:			'found {0} time(s)',
										notFound: 				'not found',
										search: 				'Search',
										newImages:				'New images',
										results:				'Results',
										reset:					'Reset',
										label: 					'Label',
										selectedItems:			'Selection',
										addCart:				'Add to cart',
										'return': 				'return',
										select:					'Select',
										sortBy:					'Sort by',
										sortedBy:				'Sorted by',
										ascending:				'ascending',
										descending:				'descending',
										multipleSelectHint:		'Use SHIFT to select range, CTRL for multiple entries',
										noRating:				'No rating',
										inTheLastDay:			'in the last day',
										inThePastNDays:			'in the past {0} days',
										sinceMyLastVisit:		'since my last visit',
										imagesAdded:			'Images added',
										imagesModified:			'Images modified',
										imagesTaken:			'Images taken',
										newItem:				'NEW',
										more:					'more',
										less:					'less'
									}),
			
			shopBox = 			$(),									// Whole Shopping section
			shopRoot =			$(),									// Root element for cart
			feedbackBox =		$(),									// Whole Feedback section
			feedbackRoot =		$(),									// Root element for the feedback tool
			album,														// The database
			lightbox,													// Lightbox element
			ns = 				'tiger_skin',							// Namespace
			lastHash = 			'',										// Storing last state
			initByHash = 		window.location.hash !== "",			// Initial URL contains hash
			cacheBuster,												// Refresh thumbnails?
			nows =				new Date() / 1000,						// Now in seconds
			//today = 			new Date() / 86400000,					// Today (day count since 1970)
			currency =			'EUR',
			lazyloadFoldersTimeout = null,								// Lazyload folders tarcking
			lazyloadFoldersInProgress = false,
			
			// Managing overlays
			
			// Creating header
			
			createOverlayHeader = function(t, options) {
					var h = $('<div>', {
							'class': 	'large-12 columns cont-box'
						}).prependTo(t);
					
					h = $('<header>', {
							 'class':	((options && options.hasOwnProperty('icon'))? options.icon : '')
						}).appendTo(h);
					
					if (options && options.hasOwnProperty('title')) {
						h.append($('<h4>', {
								html: 	options.title
							}));
					}			
							
					h.append($('<a>', {
							'class': 	'close',
							href: 		'',
							text: 		text.return
						}).on('click', function() {
							removeOverlay();
							return false;
						}));
					
					return h;
				},
				
			// Creating thumbnail grid parent
			
			createOverlayContainer = function(t) {
					
					t = $('<div>', {
							'class':	'large-12 columns'
						}).appendTo(t);
						
					t = $('<div>', {
							'class':	[ settings.thumbsClass, settings.thumbHoverClass, 'row', settings.thumbGridClass ].join(' ')
						}).appendTo(t);
						
					return t;
				},
				
			// Adding new overlay
		
			addOverlay = function(options) {
					
					// Removing previous overlay 
					var	ths;
					
					content
						.addClass('has-overlay')
						.children('section.' + settings.thumbContClass)
						.each(function() {				
								if (this.hasOwnProperty('overlay')) {
									removeOly($(this));
								} else {
									ths = $(this).hide();
									// Attached elements
									$('[rel=' + $(this).data('oly_ns') + ']').hide();
								}
							});
					
					var ns = 'oly_' + Math.floor(Math.random() * 10000),	// Namespace
						t = ($('<section>', {
								'class': 	'row overlay ' + settings.thumbContClass + ' ' + options['className'],
								id: 		ns
							}).data('oly_ns', ns));
						
					if (ths) {
						t = t.insertAfter(ths);
					} else {
						t = t.prependTo(content);
					}
					
					t[0]['overlay'] = ns;
					
					createOverlayHeader(t, options);
					createOverlayContainer(t);
					
					content.trigger('overlayAdded', t);
					
					return t;
				},
				
			// Attaching related boxes to be removed with the overlay
			
			attachToOverlay = function(el) {
					var ns = getOverlayNS();
						
					el.each(function() {
							$(this).attr('rel', ns);
						});
				},
				
			// Get overlay Namespace
			
			getOverlayNS = function() {
					return content.children('section.' + settings.thumbContClass).last().data('oly_ns');
				},
		
			// Removing overlay
			
			removeOly = function(t) {
					var ns = t.data('oly_ns');
					
					if (ns) {
						// Event handlers
						$window.off('.' + ns);
						// Attached elements
						$('[rel=' + ns + ']').remove();
					}
					
					t.closest('.' + settings.thumbsClass).trigger('lightboxRemove');
					t.remove();
				},
				
			// Removing overlay
			
			removeOverlay = function() {
				
					content.removeClass('has-overlay').children('section.' + settings.thumbContClass).each(function() {
						var oly = $(this);
						if (this.hasOwnProperty('overlay')) {
							// Thumbnails with lightbox attached
							removeOly(oly);
						} else {
							// Showing hidden overlay(s)
							oly.show();
							// Attached elements
							$('[rel=' + oly.data('oly_ns') + ']').show();
							// Refreshing thumbnails
							oly.find('.' + settings.thumbsClass).trigger('refresh');
						}
					});
					
					removeParam(settings.indexName);
					
					content.trigger('overlayRemoved', content.children('section.' + settings.thumbContClass + ':visible').eq(0));
					updateShares();
					
					if (currentThumbs('.selectable').length) {
						$('.' + settings.selectAllClass).removeClass('disabled');
						if ($('.' + settings.checkedClass).length) {
							$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass + ',.' + settings.keepSelectedClass).removeClass('disabled');
						} else {
							$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass + ',.' + settings.keepSelectedClass).addClass('disabled');
						}
						shopBox.add(feedbackBox).show();
					} else {
						shopBox.add(feedbackBox).hide();
					}
					
					return null;
				},
			
			// Getting active overlay :: creating one there's if no such
			
			getOverlay = function() {
				
					var o = content.children('section.' + settings.thumbContClass + ':visible').eq(0);
						
					if (!o.length) {
						o = $('<section>', {
								'class': 	'row ' + settings.thumbContClass
							}).appendTo(content);
							
						createOverlayContainer(o);
					}
					
					if (!o.data('oly_ns')) {
						var ns = 'oly_' +  + Math.floor(Math.random() * 10000);
						o.data('oly_ns', ns).attr('id', ns);
					}
					
					return o;
				},
				
			// Getting base overlay
			
			getBaseOverlay = function() {
			
					return content.children('section.' + settings.thumbContClass).filter(function(i, o) { return !o.hasOwnProperty('overlay'); }).eq(0);
				},
				
			// Getting base overlay thumbnail by name
			
			getBaseOverlayThumbByName = function(name) {
					var t = $(),
						o = getBaseOverlay();
					
					o.find('.' + settings.thumbClass).each(function() {
						var l = $(this).data('href');
						
						if (l) {
							l = l.substring(l.lastIndexOf('/') + 1);
						} else {
							l = $(this).attr('href').replace('#img=', '');
						}
						
						if (l === name) {
							t = $(this).parent();
							return false;
						}
					});
					
					return t;
				},
				
			// Getting folders
			
			getFolders = function(selector) {
					return content.find('section.' + settings.folderContClass + 
							(selector? (' ' + selector) : ''));
				},
												
			// Getting current thumbs array :: optionally selected subelements
			
			currentThumbs = function(selector) {
					return content.find('section.' + settings.thumbContClass + ':visible .' + settings.thumbsClass + 
							(selector? (' ' + selector) : ''));
				},
			
			// Getting thumbnail by file name
			
			getThumbByName = function(name) {
					var t = $();
					/*
					if (name.match(/%[0-9A-F]{2}/gi)) {
						name = decodeURIComponent(name);
					}
					*/
					name = transcodeJavaURIComponent(name);
					
					currentThumbs('.' + settings.thumbClass).each(function() {
						var l = $(this).data('href');
						
						if (l) {
							l = l.substring(l.lastIndexOf('/') + 1);
						} else {
							l = $(this).attr('href').replace('#img=', '');
						}
						
						if (l === name) {
							t = $(this).parent();
							return false;
						}
					});
					
					return t;
				},
				
			// Getting selected items from current overlay
			
			getSelectedItems = function() {
				
					var r = [];
							
					currentThumbs('.' + settings.thumbClass + '.' + settings.checkedClass).each(function() {
							r.push($(this).closest('.' + settings.cardClass).data(J.OBJ));
						});
					
					return r;
				},
			
			// Getting all items from current overlay
			
			getAllItems = function() {
				
					var r = [];
							
					currentThumbs('.' + settings.thumbClass).each(function() {
							r.push($(this).closest('.' + settings.cardClass).data(J.OBJ));
						});
					
					return r;
				},
				
			// Getting all items from base overlay
			
			getBaseItems = function() {
				
					var r = [];
					
					getBaseOverlay().find('.' + settings.thumbsClass + ' .' + settings.thumbClass).each(function() {
							r.push($(this).closest('.' + settings.cardClass).data(J.OBJ));
						});
					
					return r;
				},
				
			// Adding one folder's thumb
			
			loadFolderThumb = function(thumb) {
					
					thumb.find('a[data-thumb]').each(function() {
						var t = $(this);
						if (!t.children('img').length) {
							t.append($('<img>', {
									alt: 		t.data('alt'),
									'class': 	'hide-image'
								})
								// Onload action
								.one('load', function() {
									thumb.removeClass(settings.lazyloadClass);
									$(this).addClass('show-image').removeClass('hide-image');
								})
								.attr('src', t.data('thumb')));
						}
					});
				},
			
			// Loading visible folder thumbs
			
			lazyloadFolders = function(doneFn) {
					
					if (lazyloadFoldersInProgress) {
						clearTimeout(lazyloadFoldersTimeout);
						return;
					}
					
					var folders = getFolders();
					
					if (!folders.is(':visible')) {
						return;
					}
					
					lazyloadFoldersInProgress = true;
					
					folders = folders.find('.' + settings.lazyloadClass);
					
					if (folders.length) {
						var wh = $window.height(),
							scrollTop = $window.scrollTop();
					
						folders.each(function() {
							var t = $(this),
								tt = t.offset().top,
								th = t.height();
								
							if ((tt < scrollTop + 2 * wh) && (tt + th) > (scrollTop - wh)) {
								loadFolderThumb(t);
							}
							
							if (tt >= scrollTop + 2 * wh) {
								// Below fold
								return false;
							}
						});
						
						clearTimeout(lazyloadFoldersTimeout);
						lazyloadFoldersTimeout = setTimeout(lazyloadFolders, settings.scrollCheckInterval);
					
					} else {
						$window.off('.lazy');
					}
					
					lazyloadFoldersInProgress = false;
					
					if (typeof doneFn === FUNCTION) {
						doneFn.call(this);
					}
					
				};

		
		// Rendering thumbnails in a progressive fashion
		// this = '<section>' thumb-cont
		
		$.fn.renderThumbs = function(items, complete, baseSet) {
			
			if (typeof items === UNDEF || !items.length || !Array.isArray(items)) {
				return this;
			}
			
			var self = $(this).eq(0),
				// Namespace
				ns = self.closest('section').data('oly_ns'),
				
				// Columns
				cols = 0,
				
				// Scroll timeout
				scrollTop = $window.scrollTop(),
				
				// Lazy load timeout
				lazyloadTimeout = null,
				
				cleanup = function(ns) {	
						// cleaning up the old event handlers
						$window.off('.' + ns);
					},
				
				// Finding or creating the element to hold the thumbs
				
				target = (function() {
						var t = self.find('.' + settings.thumbsClass);
						
						if (!t.length) {
							t = $('<section>', {
									'class': 	'row ' + settings.thumbContClass
								}).appendTo(self);
							t = createOverlayContainer(t);
						}
						
						return t;
					}()),
				
				// Rendering thumbs
				
				initThumbs = function() {
						var cards = currentThumbs('.' + settings.cardClass), 
							card,
							thumb,
							img,
							//src,
							caption,
							s,
							d,
							cd = settings.hasOwnProperty('markNew')? (nows - settings.markNew.days * ONEDAY_S) : null,
							tw,
							item;
						
						// Initializing thumbs
						for (var i = 0; i < items.length; i++) {
							
							item = items[i];
							
							if (i < cards.length) {
								// Already exists in the HTML source
								card = cards.eq(i);
								thumb = card.children('.' + settings.thumbClass);
								caption = card.find('.' + settings.captionClass);
								img = thumb.find('img').eq(0);
								img.addClass('hide-image')
									.on('load.' + ns, function() {
											$(this).addClass('show-image')
												.removeClass('hide-image')
												.parents('.' + settings.cardClass).removeClass(settings.preloadClass);
										});
								if (img[0].complete) {
									// Cached images
									img.trigger('load');
								}
							} else {
								// Not yet exists
								card = $('<div>', {
										'class': 	settings.cardClass + ' ' + settings.lazyloadClass
									}).append(thumb = $('<a>', {
										'class':	settings.thumbClass + ' ' + settings.verticalMiddleClass
									}).data('href', album.getHref(item))
									).appendTo($('<div>', {
										'class':	'column'
									}).appendTo(target));
								
								if (s = item[J.THUMBCAPTION]) {
									card.addClass(settings.hascaptionClass);
									caption = $('<div>', {
											'class': 	settings.captionClass,
											html: 		s
										});
									
									if (settings.captionTooltip) {
										caption.append('<b class="nub"></b>');
									}
									
									if (settings.captionAbove) {
										card.prepend(caption);
									} else {
										card.append(caption);
									}
								}
							}
							
							card.data(J.OBJ, item);

							if (album.isLightboxable(item)) {
								
								card.addClass(settings.lbableClass);
								
								thumb.on('click.' + ns, thumbClicked);
									
								if (shopRoot.length && album.hasShop(item) || feedbackRoot.length && settings.feedback) {
									thumb.addClass('selectable');
									if (shopRoot.length) {
										d = $('<a>', {
													'class': 	'icon-shopping-cart add-cart',
													text: 		' ' + text.addCart
												})
											.on('click.' + ns, function(e) {
													var card = $(e.target).closest('.' + settings.cardClass);
													if (card.length) {
														shopRoot.trigger('addItems', card.data(J.OBJ));
													}
													return false;
												})
											.appendTo(thumb);
											
										d.addTooltip(album.getPriceRange(item));
									}
								}
								
								if (!baseSet && settings.linkToFolder) {
									var p = album.getRelativeFolderPath(item);
									
									if (p) {
										thumb.append($('<a>', {
												'class': 	'icon-folder folder-link',
												href:		 p + '/' + settings.indexName
											}));
									}
								}
								
								if (baseSet) {
									thumb.attr('href', '#img=' + item[J.PATH]);
								}
								
							} else {
								
								card.addClass(item[J.CATEGORY]);
								thumb.attr('href', album.getItemPath(item));
							}
							
							if (cd != null) {
								
								d = item[J.DATES];
								
								if (d && (d = d[settings.markNew.reference]) && d >= cd) {
									s = $('<span>', {
											'class':	'icon-star new-image'
										}).appendTo(thumb);
										
									if (!TOUCHENABLED) {
										s.data('tooltip', (new Date(d * 1000)).toLocaleDateString()).addTooltip();
									}
								}
							}
							
							if (settings.useRating) {
								
								var r = item[J.RATING] || 0;
								
								if (settings.visitorRating || r) {
									
									s = $('<span>', {
											'class':	'rating',
											html:		STARS
										})
										.appendTo(thumb);
										
									if (settings.visitorRating) {
										
										if (item.hasOwnProperty(J.VISITORRATING)) {
											// Already exists
											r = item[J.VISITORRATING];
										} else {
											if (!settings.useJalbumRating) {
												// Ignore jAlbum rating
												r = 0;
											}
											// Save the default
											item[J.VISITORRATING] = r;
										}
										
										s.addClass('r' + r + (settings.useJalbumRating? '' : ' icon-cancel'))
											.on('click.' + ns, function(e) {
													var el = $(this),
														r = Math.minMax(0, Math.round((getCoords(e).x - 10 - el.offset().left) / 16), 5),
														item = el.closest('.' + settings.cardClass).data(J.OBJ); 
														//Math.minMax(0, Math.floor((getCoords(e).x - el.offset().left + 4) / el.width()) * 5, 5);
														
													el.removeClass('r0 r1 r2 r3 r4 r5').addClass('r' + r);
													item[J.VISITORRATING] = r;
													
													if (!baseSet) {
														// Propagate to base overlay
														el = getBaseOverlayThumbByName(item[J.NAME]);
														el.find('.rating')
															.removeClass('r0 r1 r2 r3 r4 r5')
															.addClass('r' + r);
													}
													
													return false;
												});
									} else {
										// Unalterable rating from jAlbum
										s.addClass('r' + r);
									}
								}
							}
								
							if (settings.rightClickProtect) {
								thumb.on('contextmenu', function(e) {
										e.preventDefault()
										return false;
									});
							}
						}
						
					},
					
				// Get thumbnail columns
				
				getCols = function() {
						var el = $('<div>', {
									'class':		'columns',
									visibility: 	'hidden',
									height: 		0
								}).appendTo(target),
							cols = Math.floor(0.05 + target.width() / el.outerWidth(true)); 
							
						el.remove();
						return cols;
					},

				// Loading visible thumbs
				
				refresh = function() {
					
						if (!target.is(':visible')) {
							return;
						}
						
						lazyloadTimeout = clearTimeout(lazyloadTimeout);
							
						var st = $window.scrollTop();
						
						if (Math.abs(st - scrollTop) > 20) {
							// Scrolling in progress
							scrollTop = st;
							lazyloadTimeout = setTimeout(refresh, settings.scrollCheckInterval);
							return;
						}
						
						// Lazyload images
						var thumbs = currentThumbs('.' + settings.lazyloadClass),
							wh = $window.height();
						
						thumbs.not(':empty').each(function() {
							var t = $(this),
								tt = t.offset().top,
								th = t.height();
								
							if ((tt + th) > (scrollTop - wh)) {
								
								if (tt < (scrollTop + 2 * wh)) {
									// 2 screens ahead
									loadThumb(t);
								} else {
									return false;
								}
							}
						});
						
						if (thumbs.length) {
							// perpetrating lazyload
							scrollTop = st;
							lazyloadTimeout = setTimeout(refresh, settings.scrollCheckInterval);
							
						} else {
							// No more left
							$window.off('scroll.' + ns);
						}
					},
				
				// Clicked thumb
				
				thumbClicked = function(e) {
						if (e.target.nodeName === 'A' && ($(e.target).closest('.' + settings.commentClass).length || $(e.target).hasClass('folder-link'))) {
							// Link in comment or link to folder
							return true;
						}
						target.trigger('lightboxLoad', $(e.target).closest('.' + settings.cardClass));
						return false;
					},
				
				// Adding one thumb
				
				loadThumb = function(card) {
						
						if (card.length) {
							var thumb = card.children('.' + settings.thumbClass).eq(0);
							if (!thumb.children('img').length) {
								thumb.append($('<img>', {
										'class': 	'hide-image'
									})
									// Onload action
									.one('load', function() {
										$(this).addClass('show-image').removeClass('hide-image');
										card.removeClass(settings.lazyloadClass);
										//currentThumbs().trigger('refresh', true);
									})
									.attr('src', album.getThumbPath(card.data(J.OBJ)) + (cacheBuster || '')));
							}	
						}
											
					},
				
				// Marking active thumb
				
				setActiveThumb = function(e, thumb) {
					
						if (thumb) {
							var t = $(thumb),
								wh = 			$window.height(),
								scrollTop = 	$window.scrollTop(),
								tt = 			t.offset().top,
								th = 			t.outerHeight();
							
							target.find('.' + settings.activeClass).removeClass(settings.activeClass);
							t.addClass(settings.activeClass);
							
							// Moving into view
							if (tt + th > scrollTop + wh) {
								$('html, body').stop(true, false).animate({
										scrollTop: 		Math.round(tt - wh + th + 30) + 'px'
									});
							} else if (tt < scrollTop) {
								$('html, body').stop(true, false).animate({
										scrollTop: 		Math.round(Math.max(tt - 30, 0)) + 'px'
									});
							}
						}
					},
					
				// Refreshing the rating
				
				refreshRating = function(e, thumb, rating) {
					
						if (thumb) {
							var t = $(thumb),
								o = t.data(J.OBJ),
								r = ((typeof rating !== UNDEF)? rating : (settings.visitorRating? o[J.VISITORRATING] : o[J.RATING])) || 0;
								
							t.find('.rating')
								.removeClass('r0 r1 r2 r3 r4 r5')
								.addClass('r' + r);
								
							if (!baseSet) {
								var el = getBaseOverlayThumbByName(o[J.NAME]);
								el.find('.rating')
									.removeClass('r0 r1 r2 r3 r4 r5')
									.addClass('r' + r);
							}
						}
						
					};
			
			// Creating new Unique ID if not yet exists
			if (!ns) {
				$(this).closest('section').data('oly_ns', ns = 'oly_' + Math.floor(Math.random() * 10000));
			}
				
			self.append($('<div>', {
					'class': 	'progressbar'
				}));
				
			initThumbs();

			if (!settings.fixedShapeThumbs || settings.captionPlacement !== 'tooltip') {
				setTimeout(function() {
						var match = [];
						
						if (settings.captionAbove) {
							match.push(settings.captionClass);
						}
						
						if (!settings.fixedShapeThumbs) {
							match.push(settings.thumbClass);
						}
						
						match.push(settings.cardClass);
						
						currentThumbs().matchHeight({
								delegate:	settings.cardClass,
								match:		match
							});
					}, settings.scrollCheckInterval);
			}
			
			lazyloadTimeout = setTimeout(refresh, settings.scrollCheckInterval / 2);
			
			// Moving the active element into view
			self.on('setactivethumb', setActiveThumb);
			
			// Start (restart) loading the thumbs
			self.on('refresh', refresh);
			
			// Refreshing the rating
			self.on('refreshRating', refreshRating);
			
			if (typeof complete === FUNCTION) {
				if (typeof baseSet !== UNDEF) {
					complete.call(undefined, target, baseSet);
				} else {
					complete.call(undefined, target);
				}
			}
			
			self.find('.progressbar').remove();
			
			return this;
		};
		
		
		// Filtering selected items
			
		var	
			// Keeping only selected items
			
			keepSelectedItems = function(tx, ready) {
					var items = getSelectedItems();
					
					if (!items.length || items.length === currentThumbs('.' + settings.lbableClass).length) {
						return;
					}
					
					var t = addOverlay({
								className:	'selected-items',
								icon:		'icon-checkbox-checked',
								title:		'&ldquo;' + tx + '&rdquo;'
							}),
						h = t.find('header h4');
						
					h.append($('<span>', {
							'class': 	'progressbar'
						}));
					
					setTimeout(function() {
						h.find('.progressbar').remove();
						h.append($('<small>', {
								text: 		text.foundNTimes.replace('{0}', items.length)
							}));
						t.renderThumbs(items, prepareThumbs);
						if (ready) {
							ready.call();
						}
						shopBox.add(feedbackBox).show();
					}, 50);
				},
				
			// Dates are objects
			
			getDate = function(item, name) {
					if (item.hasOwnProperty(J.DATES)) {
						if (name === 'addedDate') {
							return item[J.DATES]['added'];
						} else if (name === 'originalDate') {
							return item[J.DATES]['dateTaken'];
						} else {
							return item[J.DATES]['fileModified'];
						}
					}
					
					return null;
				},
				
			// Some variables get remapped by the skin
			
			getMappedVal = function(item, name) {
				
					if (item.hasOwnProperty(name)) {
						// Generic case
						return item[name];
					}
					
					return null;
				},
				
			// Do filtering on current folder
			
			doFilter = function(e) {
					var form = $(e.target).closest('form'),
						items = getAllItems(),
						cnt = items.length,
						tx = [];
					
					form.find('select:visible,input[type=range],input[type=date]').each(function() {
							var name = this.name,
								label = form.find('label[for=' + name + ']').text(),
								type = $(this).data('type'),
								value = $(this).val(),
								display = function(val) {
										if (name ===  'rating') {
											return Math.round(val);
										} else if (type === 'date') {
											return (new Date(val * 1000)).toLocaleDateString();
										} else if (type === 'numeric') {
											if ($(this).data('unit') === 'seconds') {
												return niceTime(val);
											} else if ($(this).data('unit') === 'byte') {
												return niceByte(val);
											} else if (val >= 10) {
												return Math.round(val);
											} else if (val >= 1) {
												return val.toFixed(1);
											} else if (val >= 0.1) {
												return val.toFixed(2);
											}
											return val;
										}
										return toCurrency(val, currency);
									};
								
							if (type === 'numeric' || type === 'date' || type === 'financial') {
									
								var low = $(this).data('low'),
									high = $(this).data('high');
									
								if (low > this.min || high < this.max) {
									// Only if the range is narrower than the whole range
									
									if (name.endsWith('Date')) {
										// Date
										for (var i = items.length - 1, val; i >= 0; i--) {
											if ((val = getDate(items[i], name)) !== null && 
												(val >= low && val <= high)) {
												continue;
											}
											items.splice(i, 1);
										}
									} else if (JCAMERA.indexOf(name) !== -1) {
										// Camera data fields
										if (name === 'exposureTime') {
											for (var i = items.length - 1, val; i >= 0; i--) {
												if (items[i].hasOwnProperty(J.CAMERA) &&
													items[i][J.CAMERA].hasOwnProperty(name) &&
													(val = getExposure(items[i][J.CAMERA][name])) &&
													(val >= low && val <= high)) {
													continue;
												}
												items.splice(i, 1);
											}
										} else if (name.startsWith('focalLength')) {
											for (var i = items.length - 1, val; i >= 0; i--) {
												if (items[i].hasOwnProperty(J.CAMERA) &&
													items[i][J.CAMERA].hasOwnProperty(name) &&
													!isNaN(val = parseInt(items[i][J.CAMERA][name])) && 
													(val >= low && val <= high)) {
													continue;
												}
												items.splice(i, 1);
											}
										} else {
											for (var i = items.length - 1, val; i >= 0; i--) {
												if (items[i].hasOwnProperty(J.CAMERA) &&
													items[i][J.CAMERA].hasOwnProperty(name) &&
													(val = items[i][J.CAMERA][name]) !== null &&
													(val >= low && val <= high)) {
													continue;
												}
												items.splice(i, 1);
											}
										}
									} else if (name === 'rating') {
										// Rating
										for (var i = items.length - 1, val, r = settings.visitorRating? J.VISITORRATING : J.RATING; i >= 0; i--) {
											val = items[i][r] || 0;
											if (val >= low && val <= high) {
												continue;
											}
											items.splice(i, 1);
										}
									} else {
										// Generic
										for (var i = items.length - 1, val; i >= 0; i--) {
											if ((val = items[i][name]) !== null && 
												(val >= low && val <= high)) {
												continue;
											}
											items.splice(i, 1);
										}
									}
								
									tx.push(label + '=' + display(low) + '&ndash;' + display(high));
								}
								
							} else if (this.nodeName === 'SELECT' && value !== null && value.length) {
								
								// Only if at least one option is selected
																	
								if (!Array.isArray(value)) {
									value = [ value ];
								}
								
								if (JCAMERA.indexOf(name) !== -1) {
									// Camera data :: no arrays expected
									for (var i = items.length - 1, val; i >= 0; i--) {
										if (items[i].hasOwnProperty(J.CAMERA) &&
											items[i][J.CAMERA].hasOwnProperty(name) &&
											(val = items[i][J.CAMERA][name]) !== null &&
											value.includes(val)) {
											continue;
										}
										items.splice(i, 1);
									}
									
								} else {
									// Generic case
									for (var i = items.length - 1, found, val; i >= 0; i--) {
										if (items[i].hasOwnProperty(name)) {
											val = items[i][name];
											if (Array.isArray(val)) {
												// Array e.g. keywords
												found = false;
												for (var j = 0; j < value.length; j++) {
													if (val.includes(value[j])) {
														console.log(items[i]['name'] + ': ' + val);  
														found = true;
														break;
													}
												}
												if (found) {
													continue;
												}
											} else if (value.includes(val)) {
												continue;
											}
										}
										items.splice(i, 1);
									}
								}
								
								// Header text addition
								if (this['multiple']) {
									tx.push(name + '=[' + value.join(',') + ']');  
								} else {
									tx.push(label + '=' + value);  
								}
							}
						});
					
					if (items.length < cnt) {
						
						// Only show overlay if at least one thumb was filtered out
						
						var sh = content.children('section.' + settings.thumbContClass + ':visible').eq(0).find('header h4.sort'),
							t = addOverlay({
									className:	'filtered-items',
									//icon:		'icon-filter',
									title:		text.results
								}),
							h = t.find('header h4');
							
						h.addClass('icon-filter filter')
							.append($('<span>', {
								'class': 	'progressbar'
							}));
						
						t.find('.close').on('click', resetBaseFilters);
						
						setTimeout(function() {
								h.find('.progressbar').remove();
								h.append($('<small>', {
										html: 		tx.join(', ') + ': ' + text.foundNTimes.replace('{0}', items.length)
									}));
								h.after(sh);
								t.renderThumbs(items, prepareThumbs);
								if (items.length) {
									shopBox.add(feedbackBox).show();
								}
							}, 50);
					}
					
					return false;
				},
				
			// Do sorting on current set of thumbs
			
			doSort = function(e) {
					var el = this.selectedOptions[0],
						name = this.value;
					
					if (!el || !name) {
						return true;
					}
					
					var items = getAllItems(),
						cnt = items.length,
						data = $(el).data('sort'),
						type = data['type'],
						asc = data['ascending'],
						tx = [],
						v1,
						v2;
						
					if (type === 'numeric' || type === 'financial' || type === 'date') {
						// Number
						
						if (name.endsWith('Date')) {
							// Date
							items = items.sort(function(i1, i2) {
									v1 = getDate(i1, name);
									v2 = getDate(i2, name);
									return asc? (v1 || 0) - (v2 || 0)
												:
												(v2 || 0) - (v1 || 0);
								});
							
						} else if (JCAMERA.indexOf(name) !== -1) {
							// Camera data
							if (name === 'exposureTime') {
								items = items.sort(function(i1, i2) {
										v1 = (i1.hasOwnProperty(J.CAMERA) && i1[J.CAMERA].hasOwnProperty(name))? 
												i1[J.CAMERA][name] : 0;
										v2 = (i2.hasOwnProperty(J.CAMERA) && i2[J.CAMERA].hasOwnProperty(name))? 
												i2[J.CAMERA][name] : 0;
										return asc? (getExposure(v1 || 0) - getExposure(v2 || 0)) 
													: 
													(getExposure(v2 || 0) - getExposure(v1 || 0));
									});
							} else {
								items = items.sort(function(i1, i2) {
										v1 = (i1.hasOwnProperty(J.CAMERA) && i1[J.CAMERA].hasOwnProperty(name))? 
												parseFloat(i1[J.CAMERA][name]) : 0;
										v2 = (i2.hasOwnProperty(J.CAMERA) && i2[J.CAMERA].hasOwnProperty(name))? 
												parseFloat(i2[J.CAMERA][name]) : 0;
										return asc? ((v1 || 0) - (v2 || 0)) 
													: 
													((v2 || 0) - (v1 || 0));
									});
							}
							
						} else if (name === 'rating') {
							// Rating
							var r = settings.visitorRating? J.VISITORRATING : J.RATING;
							items = items.sort(function(i1, i2) {
									return asc? (i1[r] || 0) - (i2[r] || 0)
												:
												(i2[r] || 0) - (i1[r] || 0);
								});
							
						} else {
							// Every other numeric
							items = items.sort(function(i1, i2) {
									if (i1.hasOwnProperty(name) && i2.hasOwnProperty(name)) {
										return asc?	parseFloat(i1[name]) - parseFloat(i2[name])
													:
													parseFloat(i2[name]) - parseFloat(i1[name]);
									}
									return asc? (i1.hasOwnProperty(name)? 1 : 0)
												:
												(i2.hasOwnProperty(name)? -1 : 0);
								});
						}
						
					} else {
						// String
						
						items = items.sort(function(i1, i2) {
							if (i1.hasOwnProperty(name) && i2.hasOwnProperty(name)) {
								return asc?	(i1[name] + '').localeCompare(i2[name] + '')
											:
											(i2[name] + '').localeCompare(i1[name] + '');
							}
							return asc? (i1.hasOwnProperty(name)? 1 : 0)
										:
										(i2.hasOwnProperty(name)? -1 : 0);
						});
						
					}
					
					var fh = content.children('section.' + settings.thumbContClass + ':visible').eq(0).find('header h4.filter'),
						t = addOverlay({
								className:	'sorted-items',
								//icon:		'icon-ordering',
								title:		text.sortedBy + '<small>' + data.label + ' (' + text[data.ascending? 'ascending' : 'descending'] + ')</small>' 
							}),
						h = t.find('header h4');
						
					h.addClass('icon-ordering sort')
						.append($('<span>', {
							'class': 	'progressbar'
						}));
					
					t.find('.close').on('click', resetBaseSort);
					
					setTimeout(function() {
							h.find('.progressbar').remove();
							h.after(fh);
							t.renderThumbs(items, prepareThumbs);
							if (items.length) {
								shopBox.add(feedbackBox).show();
							}
						}, 50);
					
					return false;
				},
			
			// Do a label filtering
		
			getItemsByLabel = function(tx, caseSensitive, ready) {
					var t = addOverlay({
								className:	'tags-found',
								icon:		'icon-label',
								title:		'&ldquo;' + tx + '&rdquo;'
							}),
						h = t.find('header h4');
						
					h.append($('<span>', {
						'class': 'progressbar'
					}));
					
					if (t.offset().top < $window.scrollTop() || t.offset().top > ($window.scrollTop() + $window.height() - 100)) {
						$window.scrollTop(t.offset().top);
					}
					
					if (typeof caseSensitive === UNDEF) {
						caseSensitive = false;
					} else if (typeof caseSensitive === FUNCTION) {
						ready = caseSensitive;
						caseSensitive = false;
					}
						
					album.collectItems($.extend(settings.tagCloud, {
						exact:			settings.exactFields,
						terms:			tx,
						caseSensitive:	caseSensitive,
						ready: 			function() {
												h.find('.progressbar').remove();
												h.append($('<small>', {
													text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
												}));
												t.renderThumbs(this, prepareThumbs);
												if (ready) {
													ready.call();
												}
												if (this.length) {
													shopBox.add(feedbackBox).show();
												}
											}
					}));
				},
				

			// Collecting and displaying tags
		
			renderTagCloud = function(tags) {
				
					if (!tags.length) {
						return;
					}
					
					var t = $(settings.tagCloud.hook);
					
					if (!t.length) {
						return;
					}
					
					var tagClicked = function() {
							var tx = $(this).children('span').eq(0).text();
							
							if (tx && tx.length > 1) {
								getItemsByLabel(tx, true, function() {
									setParam(settings.indexName, {
											label: 		tx
										}, text.label + ': ' + tx);
									updateShares();
								});
							}
							
							return false;
						};
						
					for (var a, r, cnt, i = 0, l = tags.length; i < l; i++) {
						
						// create link element
						a = $('<a>', {
								'class': 	'tag'
							}).append($('<span>', {
								html: 		tags[i][0]
							}));
						a.on('click', tagClicked);
						
						// add count
						if (tags[i][1] > 1) {
							if (settings.tagCloud.fontVaries) {
								r = 0;
								cnt = Math.min(64, tags[i][1]);
								// calculate weight
								while (cnt > 1) {
									r++;
									cnt = cnt >> 1;
								}
								a.addClass('size-' + r);
							}
							a.append('&nbsp;').append($('<sup>', {
								text: 		tags[i][1]
							}));
						}
						
						t.append(a);
					}
					
					if (tags.length) {
						var d;
						
						if (t.outerHeight() > 175) {
							
							t.css({
								maxHeight: 150,
								overflow: 'hidden'
							});
							
							d = $('<div>', {
								'class': 'toggle-height'
							});
							
							d.append($('<a>', {
									'class': 	'icon-drop-down',
									text: 		' ' + text.more
								}).on('click', function() {
									var b = t.parent(),
										toggleHeight = function() {
											var h = t.prop('scrollHeight');
											
											b.css({
													height: 	'auto',
													overflow: 	'visible'
												});
											t.css({
													maxHeight: 	150,
												});
											setTimeout(function() {
													t.css({
															maxHeight: 	h + 20
														});
												}, 25);
											d.addClass(settings.activeClass);
										};
									
									b.css({
											height: 	b.height(),
											overflow: 	'hidden'
										});
									t.css({
											maxHeight: 	'none'
										});
									setTimeout(toggleHeight, 50);
								
							})).append($('<a>', {
									'class': 	'icon-drop-up',
									text: 		' ' + text.less
								}).on('click', function() {
									t.css({
											maxHeight: 	150
										});
									d.removeClass(settings.activeClass);
								}));
							
							t.after(d);
						}
					}
				},
				
			// Producing folder tree HTML
			
			getFolderTree = function() {
					var getFolderObjects = function(folder) {
							if (folder.hasOwnProperty(J.FOLDERS) && folder[J.FOLDERS].length) {
								var ul, li, i, p, f = folder[J.FOLDERS];
								ul = $('<ul>');
								for (i = 0; i < f.length; i++) {
									p = album.getRelativeFolderPath(f[i]);
									li = $('<li>').appendTo(ul);
									li.append($('<a>', {
											href: 	(p? (p + '/') : '') + settings.indexName,
											html: 	f[i][J.TITLE] || f[i][J.NAME] || ''
										})).append(getFolderObjects(f[i]));
									if (!p) {
										li.addClass('actual');
									}
								}
								return ul;
							}
							return $();
						},
						
						getHomeLink = function() {
							var p = settings.rootPath;
							return $('<a>', {
									'class': 	'home-link icon-home',
									href:		(p? (p + '/') : '') + settings.indexName,
									text:		' ' + album.getAlbumTitle()
								});
						};
						
					if (settings.folderTree.type === 'tree') {
						if (settings.level > 0) {
							return getHomeLink().add(getFolderObjects(album.getTree()));
						} else {
							return getFolderObjects(album.getTree());
						}
					} else {
						getFolderObjects(album.getCurrentFolder());
					}
				},
		
			// Executing Search
			
			getItemsBySearch = function(tx, ready) {
					var t = addOverlay({
								className:	'search-results',
								icon:		'icon-search',
								title:		'&ldquo;' + tx + '&rdquo;'
							}),
						h = t.find('header h4');
						
					h.append($('<span>', {
							'class': 	'progressbar'
						}));
					
					setTimeout(function() {
						album.collectItems($.extend(settings.search, {
							exact:		false,
							terms:		tx,
							ready:		function() {
												h.find('.progressbar').remove();
												h.append($('<small>', {
														text: 	this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
													}));
												t.renderThumbs(this, prepareThumbs);
												if (ready) {
													ready.call();
												}
												if (this.length) {
													shopBox.add(feedbackBox).show();
												}
											}
						}));
					}, 50);
				},
	
		
			// Preparing Search functionality
			
			prepareSearch = function(f) {
				
					if (!f.length || !album) {
						return;
					}
					
					// Adding defaults if missing
					
					settings.search = $.extend({
						depth:		'tree',
						fields:		'creator,keywords,title,comment'
					}, settings.search);
					
					var startSearch = function(f) {
							
							var tx = f.find('input').eq(0).val();
							
							if (tx && tx.length > 1) {
								getItemsBySearch(tx, function() {
									setParam(settings.indexName, {
											search:		tx
										}, text.search + ': ' + tx);
									updateShares();
								});
							}
							return false;
						};
						
					f.find('.search-btn, button').on('click', function() {
						if (f.hasClass(settings.activeClass)) {
							startSearch(f);
							f.removeClass(settings.activeClass);
						} else {
							f.find('input').eq(0).focus();
							f.addClass(settings.activeClass);
						}
						return false;
					});
					
					f.on('submit', function(e) {
						e.preventDefault();
						startSearch(f);
						f.removeClass(settings.activeClass);
						return false;
					});
					
					f.find('input').on({
						'focus': 	function() { 
										f.addClass(settings.activeClass); 
									}
					});
				},
			
			// Preparing Tag Search functionality
			
			prepareTagSearch = function() {
				
					var f = $(settings.tagCloud.searchHook);
					
					if (!f.length || !album) {
						return;
					}
									
					var startSearch = function(f) {
							
							var tx = f.find('input').eq(0).val();
							
							if (tx && tx.length > 1) {
								getItemsByLabel(tx, true, function() {
									setParam(settings.indexName, {
											label: 	tx
										}, text.label + ': ' + tx);
									updateShares();
								});
							}
							return false;
						};
						
					f.find('.search-btn, button').on('click', function() {
						f.find('input').eq(0).focus();
						startSearch(f);
						return false;
					});
					
					f.on('submit', function(e) {
						e.preventDefault();
						startSearch(f);
						return false;
					});
					
				},
			
			// Search for new images
			
			getSearchNewTitle = function(r) {
			
					if (r === 'dateTaken') 
						return text.imagesTaken;
					else if (r === 'fileModified')
						return text.imagesModified;
					return text.imagesAdded;
				},
			
			getItemsByDate = function(range, ready) {
					var t = addOverlay({
								className:		'search-results',
								icon:			'icon-star',
								title:			getSearchNewTitle(settings.searchNew.reference) + ' ' + getTimespan(range)
							}),
						h = t.find('header h4');
						
					h.append($('<span>', {
							'class': 'progressbar'
						}));
					
					setTimeout(function() {
						album.collectByDate($.extend(settings.searchNew, {
							range:		range,
							ready:		function() {
												h.find('.progressbar').remove();
												h.append($('<small>', {
													text: this.length? text.foundNTimes.replace('{0}', this.length) : text.notFound
												}));
												t.renderThumbs(this, prepareThumbs);
												if (ready) {
													ready.call();
												}
												if (this.length) {
													shopBox.add(feedbackBox).show();
												}
											}
						}));
					}, 50);
				},
	
			// Search new images
			
			searchNew = function(range) {
					
					if (range) {
						getItemsByDate(range, function() {
							setParam(settings.indexName, {
									newimages:		range
								}, getSearchNewTitle(settings.searchNew.reference) + ' ' + getTimespan(range));
							updateShares();
						});
					}
					return false;
				},
			
			// Prepare Search new functionality
			
			prepareSearchNew = function(f) {
				
					if (!f.length || !album) {
						return;
					}
					
					// Setting up defaults if something is missing
					
					settings.searchNew = $.extend({
							days: 				'3,30,90',
							depth: 				'folder',
							reference: 			'dateTaken',
							sinceLastVisit:		true
						}, settings.searchNew);
					
					var	days = settings.searchNew.days.split(',');
					
					f.append($('<label>', {
							text: 	getSearchNewTitle(settings.searchNew.reference)
						}));
							
					for (var i = 0; i < days.length; i++) {
						f.append($('<a>', {
								'class': 	'smallbtn',
								text: 		getTimespan(days[i])
							})
							.data('days', parseInt(days[i], 10))
							.on('click', function() {
								searchNew($(this).data('days'));
							}));
					}
							
					if (settings.searchNew.sinceLastVisit) {
						
						var start = $.cookie('lastVisit'),
							now = Math.round(new Date() / 1000);
						
						if (start && start < (now - ONEDAY_S)) {
							f.append($('<a>', {
									'class': 	'smallbtn',
									text: 		text.sinceMyLastVisit
								})
								.on('click', function() {
									searchNew((now - start) / ODEDAY_S);
								}));
						}
						
						// Storing last visit date for 10 years
						$.cookie('lastVisit', now, 315360000);
					}
					
				},
				
			// Preparing thumbnails
					
			prepareThumbs = function(thumbs, baseSet) {
				
					if (!album) {
						return;
					}
					
					if (album.hasShop() || settings.feedback) {
						
						if (thumbs.find('.selectable').length) {
							thumbs.selectable({
									cardClass:			'selectable',
									checkedClass:		settings.checkedClass,
									anySelected:		function() {
																$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass + ',.' + settings.keepSelectedClass).removeClass('disabled');
																shopRoot.add(feedbackRoot).trigger('itemsSelected', this);
															},
									noneSelected:		function() {
																$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass + ',.' + settings.keepSelectedClass).addClass('disabled');
																shopRoot.add(feedbackRoot).trigger('itemsSelected', 0);
															}
								});
							
							$('#shop,#feedback').show();
							
							$('.' + settings.selectAllClass).removeClass('disabled');
							
							$('.' + settings.keepSelectedClass).addClass('disabled').on('click.' + ns, function() {
									keepSelectedItems(text.selectedItems, function() {
											removeParam();
											updateShares();
										});
									
									return false;
								});
							
							
							
						} else {
							
							$('.' + settings.selectAllClass).addClass('disabled');
							$('#shop,#feedback').hide();
						}
						
						$('.' + settings.selectNoneClass + ',.' + settings.addSelectedClass).addClass('disabled');
					}
					
					if (thumbs.children().length > 1) {
						
						// Filtering and sort only if there are at least 2 items
						
						if ($('.filter-cont').length) {
							// Initializing Filters / Sort
							prepareFilterBox(baseSet);
						}
					}
					
					if (!thumbs.find('.' + settings.lbableClass).length) {
						
						$('.' + settings.startshowClass).fadeOut();
						
					} else {
						
						thumbs.trigger('lightboxRemove');
						
						thumbs.lightbox(album, {
								
								baseSet:		(typeof baseSet === UNDEF)? false : baseSet,
								
								onLoadStart:	function(el) {
														var item = el.data(J.OBJ);
														if (item) {
															addParam(settings.indexName, {
																	img: 	album.getItemName(item)
																}, item[J.TITLE]);
														}						
													},
													
								onLoadEnd: 		function(el) {
														thumbs.trigger('setactivethumb', el);
													},
												
								onClose:		function() {
														removeParam(settings.indexName, 'img');
													},
												
								onReady:		function() {
														if (initByHash && window.location.hash) {
															initByHash = false;
															var param = readParam();
															
															if (param.hasOwnProperty('label')) {
																// Label filter
																getItemsByLabel(param.label, function() {
																	addParam(settings.indexName, {
																			label:			param.label,
																			search:			null,
																			newimages:		null
																		}, text.label + ': ' + param.label);
																});
															} else if (param.hasOwnProperty('search')) {
																// Search
																getItemsBySearch(param.search, function() {
																	addParam(settings.indexName, {
																			search:			param.search,
																			label:			null,
																			newimages: 		null
																		}, text.search + ': ' + param.search);
																});
															} else if (param.hasOwnProperty('newimages')) {
																// Search
																getItemsByDate(param.newimages, function() {
																	addParam(settings.indexName, {
																			newimages:		param.newimages,
																			search:			null,
																			label:			null
																		}, text.searchNew + ': ' + param.searchNew);
																});
															} else if (param.hasOwnProperty('img')) {
																var el = getThumbByName(param.img);
																if (el.length) {
																	currentThumbs().trigger('lightboxLoad', el);
																} else {
																	removeParam(settings.indexName, 'img');
																}
															}
														}
														
														$('.' + settings.startshowClass).fadeIn().children('.button').on('click', startAuto);
													},
												
								onSlideshowStart:
												function() {
														if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
															$(settings.backgroundMusic.hook).trigger('startPlayer');
														}
													},
												
								onSlideshowPause:
												function() {
														if (settings.hasOwnProperty('backgroundMusic') && settings.backgroundMusic.slideshowControl) {
															$(settings.backgroundMusic.hook).trigger('pausePlayer');
														}
													}
							});
					}
				},
			
			// Starts slideshow
			
			startAuto = function(e) {
					var t = $();
					
					if (!settings.restartSlideshow) {
						t = currentThumbs('.' + settings.cardClass + '.' + settings.activeClass + ':first-child');
					}
					
					if (!t.length) {
						// Fall back to first
						t = currentThumbs('.' + settings.cardClass + ':first-child');
					}
				
					currentThumbs().trigger('lightboxLoad', [ t, 2000 ]);
					
					return false;
				},
			
			// Reading out map locations
			
			getMarkers = function(images) {
					
					var markers = [];
					
					for (var i = 0; i < images.length; i++) {
						if (images[i].hasOwnProperty('location')) {
							markers.push({
								title:		[((i + 1) + '.'), images[i]['title'], images[i]['comment']].join(' '),
								pos: 		images[i].location,
								link:		images[i].path
							});
						}
					}
					
					return markers;
								
				},
			
			// Preparing functions that need the full album tree
			
			deepReadyFunction = function() {
				
					// Collecting tags if gathered from deep tree
					if (settings.hasOwnProperty('tagCloud') && settings.tagCloud['depth'] !== 'current') {
						album.collectTags($.extend(settings.tagCloud, {
								exact:		settings.exactFields,
								ready:		function() {
													renderTagCloud(this);
												}
							}));
					}
					
				},

			// Preparing the album upon Album object is loaded
			
			prepareAlbum = function() {
				
					if (!album) {
						return;
					}
					
					var images = album.getImages();
					
					// Removing thumbnail links right away
					
					currentThumbs('.' + settings.cardClass + '>a').each(function() {
							$(this).data('href', this.href).attr('href', '');
						});
		
					// Preapring folders
					
					prepareFolders();
		
					// Shopping cart
					
					if ($.fn.hasOwnProperty('paypal')) {
												
						// assigning paypal.js defaults 
						$.fn.paypal.defaults.relPath = settings.relPath;
						$.fn.paypal.defaults.rootPath = settings.rootPath;
						
						// Initializing the cart
						shopBox = $('section#shop');
						shopRoot = $('#shop-root').paypal(album, { 
								'getSelected': 		function() {
														return getSelectedItems(true);
													},
								'selectNone':		function() {
														currentThumbs().trigger('selectNone');
													}
							});
						
						currency = album.getCurrency();
						
						if ($.fn.hasOwnProperty('rangeSlider')) {
							$.fn.rangeSlider.defaults.currency = currency;
						}
						
						if (shopRoot.length) {
							
							// "Select all" button
							shopBox.find('.' + settings.selectAllClass).on('click', function() {
								currentThumbs().trigger('selectAll');
								return false;
							});
							
							// "Select none" button
							shopBox.find('.' + settings.selectNoneClass).on('click', function() {
								currentThumbs().trigger('selectNone');
								return false;
							});
							
							// "Add selected" button
							shopBox.find('.' + settings.addSelectedClass).on('click', function() {
								shopRoot.trigger('addItems', getSelectedItems());
								currentThumbs().trigger('selectNone');
								return false;
							});
							
							// Ensuring lightbox has 'shop' property
							if (!$.fn.lightbox.defaults.hasOwnProperty('shop')) {
								$.fn.lightbox.defaults.shop = {};
							}
							// Setting lightbox's shop root variable
							$.fn.lightbox.defaults.shop.root = shopRoot;
						}
					}
					
					// Feedback tool
					
					if (settings.hasOwnProperty('feedback')) {
						
						// assigning paypal.js defaults 
						$.fn.feedback.defaults.relPath = settings.relPath;
						$.fn.feedback.defaults.rootPath = settings.rootPath;
						
						// Initializing the cart
						feedbackBox = $('#feedback');
						
						// Get selected items
						settings.feedback['getSelected'] = function() {
								return getSelectedItems(true);
							}
						settings.feedback['selectNone'] = function() {
								currentThumbs().trigger('selectNone');
							}
							
						feedbackRoot = $('#feedback-root').feedback(album, settings.feedback);
						
						if (feedbackRoot.length) {
							
							// "Select all" button
							feedbackBox.find('.' + settings.selectAllClass).on('click', function() {
								currentThumbs().trigger('selectAll');
								return false;
							});
							
							// "Select none" button
							feedbackBox.find('.' + settings.selectNoneClass).on('click', function() {
								currentThumbs().trigger('selectNone');
								return false;
							});
							
							// "Add selected" button
							feedbackBox.find('.' + settings.addSelectedClass).on('click', function() {
								feedbackRoot.trigger('addItems', getSelectedItems());
								currentThumbs().trigger('selectNone');
								return false;
							});
							
							// Ensuring lightbox has 'feedback' property
							if (!$.fn.lightbox.defaults.hasOwnProperty('feedback')) {
								$.fn.lightbox.defaults.feedback = {};
							}
							// Setting lightbox's feedback root variable
							$.fn.lightbox.defaults.feedback.root = feedbackRoot;
						}
					}
					
					// Rendering thumbnails
					if (images.length) {
						getOverlay().renderThumbs(images, prepareThumbs, true);
						
						// Collecting markers and initializing map
						if (settings.hasOwnProperty('map') && settings.map.index) {
							$('.map-root').addMap({
								markers: 			getMarkers(images),
								type:				settings.map['type'] || 'roadmap',
								zoom:				settings.map['zoom'] || 16,
								fitBounds:			true,
								fullscreenControl:	true,
								click:				function() {
														if (this.link) {
															var el = getThumbByName(this.link);
															if (el.length) {
																removeOverlay();
																currentThumbs().trigger('lightboxLoad', el);
															}
														}
													}
							});
						}
	
					} else if (window.location.hash) {
						// no thumbnails, only folders
						// we should check the search / tags / newimages filter
						stateChange();
					}
					
					// Collecting tags if gathered only from this folder
					if (settings.hasOwnProperty('tagCloud')) {
						
						if (settings.tagCloud['depth'] === 'current') {
							album.collectTags($.extend(settings.tagCloud, {
									exact:		settings.exactFields,
									ready:		function() {
														renderTagCloud(this);
													}
								}));
						}
						
						if (settings.tagCloud.hasOwnProperty('searchHook')) {
							prepareTagSearch();
						}
					}
					
					// Search 
					if (settings.hasOwnProperty('search')) {
						prepareSearch($(settings.search.hook));
					}
					
					// Search new
					if (settings.hasOwnProperty('searchNew')) {
						prepareSearchNew($(settings.searchNew.hook));
					}
				},
			
			 
			// Read filter ranges from current overlay
			
			getFilterRanges = function(items, filter) {
					
					if (!filter || !items.length) {
						return;
					}
					
					var setMinMax = function(f, val) {
								if (f.hasOwnProperty('min') && f.hasOwnProperty('min')) {
									if (f.min > val) {
										f.min = val;
									} else if (f.max < val) {
										f.max = val;
									}
								} else {
									f.min = f.max = val;
								}
							};
							
					for (var name in filter) {
						
						var f = filter[name], 
							type = f['type'];
						
						if (type === 'numeric' || type === 'date' || type === 'financial') {
							
							if (name.endsWith('Date')) {
								// Date
								for (var i = items.length - 1, val; i >= 0; i--) {
									if ((val = getDate(items[i], name)) !== null) {
										setMinMax(f, val);
									}
								}
							} else if (JCAMERA.indexOf(name) !== -1) {
								// Camera data fields
								if (name === 'exposureTime') {
									for (var i = items.length - 1, val; i >= 0; i--) {
										if (items[i].hasOwnProperty(J.CAMERA) &&
											items[i][J.CAMERA].hasOwnProperty(name) &&
											(val = getExposure(items[i][J.CAMERA][name]))) {
											setMinMax(f, val);
										}
									}
								} else if (name.startsWith('focalLength')) {
									for (var i = items.length - 1, val; i >= 0; i--) {
										if (items[i].hasOwnProperty(J.CAMERA) &&
											items[i][J.CAMERA].hasOwnProperty(name) &&
											!isNaN(val = parseInt(items[i][J.CAMERA][name]))) {
											setMinMax(f, val);
										}
									}
								} else {
									for (var i = items.length - 1, val; i >= 0; i--) {
										if (items[i].hasOwnProperty(J.CAMERA) &&
											items[i][J.CAMERA].hasOwnProperty(name) &&
											(val = items[i][J.CAMERA][name]) !== null) {
											setMinMax(f, val);
										}
									}
								}
							} else if (name === 'rating') {
								// Rating
								for (var i = items.length - 1, val, r = settings.visitorRating? J.VISITORRATING : J.RATING; i >= 0; i--) {
									val = items[i][r] || 0;
									setMinMax(f, val);
								}
							} else {
								// Generic
								for (var i = items.length - 1, val; i >= 0; i--) {
									if ((val = items[i][name]) !== null) {
										setMinMax(f, val);
									}
								}
							}
							
						} else {
							// String
							if (!f.hasOwnProperty('values')) {
								f.values = [];
							}
							
							if (JCAMERA.indexOf(name) !== -1) {
								// Camera data :: no arrays expected
								for (var i = 0, val; i < items.length; i++) {
									if (items[i].hasOwnProperty(J.CAMERA) &&
										items[i][J.CAMERA].hasOwnProperty(name)) {
										pushNew(f.values, (items[i][J.CAMERA][name] + '').trim());
									}
								}
							} else {
								for (var i = 0, val; i < items.length; i++) {
									if (items[i].hasOwnProperty(name)) {
										pushNew(f.values, items[i][name]);
									}
								}
							}
						}
					}
					
					return filter;
				},
				
			// Enabling / disabling the reset button
			
			enableResetBtn = function(e) {
					var form = (typeof e === UNDEF || e.hasOwnProperty('target'))? $(this).closest('form') : e;
					form.find('button[type=reset]').removeClass('disabled');
				},
				
			disableResetBtn = function(e) {
					var form = (typeof e === UNDEF || e.hasOwnProperty('target'))? $(this).closest('form') : e;
					form.find('button[type=reset]').addClass('disabled');
				},
				
			// Reseting filters to defaults
			
			_resetFilters = function(form) {
				
					form.find('select').each(function() {
							$(this).prop('selectedIndex', this['multiple']? -1 : 0);
						});
					
					form.find('input[type=range]').each(function() {
							$(this).trigger('resetRange');
						});
					
					disableResetBtn(form);
					
				},
				
			// Reset filters after a form elemnt changed
			
			resetFilters = function(e) {
					_resetFilters($(e.target).closest('form'));
					return false;
				},
				
			// Reseting base filters to defaults
			
			resetBaseFilters = function() {
					resetBaseSort();
					_resetFilters($('#filters form.filter'));
				},
				
			// Reseting base filters to defaults
			
			resetBaseSort = function() {
					$('#filters form.sort select').prop('selectedIndex', 0);
				},
				
			// Preparing filters / sort
			
			prepareFilterBox = function(baseSet) {
				
					var items = getAllItems(),
						filters = album.getPropertyObject(album.getCurrentFolder(), J.FILTERS, true),
						sort = album.getPropertyObject(album.getCurrentFolder(), J.SORT, true),
						box = $(),
						cont,
						filterForm = $(),
						sortForm = $(),
						group,
						inp,
						lbl,
						type,
						buttons,
						btn;
						
					if ((filters || sort) && items.length > 1) {
						
						if (baseSet) {
							
							box = $('#filters');
							
						} else {
							
							box = $('<div>', {
									'class':		'row filter-cont cont-box',
									id:				'filters_' + getOverlayNS()
								}).insertBefore($('#filters').hide());
								
							cont = $('<div>', {
									'class':		'columns'
								}).appendTo(box);
								
							cont.append($('<h4>', {
									'class':		'icon-' + (filters? 'filter' : 'sort'),
									text:			$('#filters').find('h4').text()
								}));
															
						}
						
						// Attaching box to current overlay
						attachToOverlay(box);
					
					} else {
						
						if (baseSet) {
							$('#filters').hide();
						}
						
						return;
					}
					
					if (filters) {
						
						if (baseSet) {
							
							filterForm = box.find('form.filter');
							
						} else {
							
							filterForm = $('<form>', {
									'class':		'filter'
								}).appendTo(cont);
						}
						
						buttons = $('<div>', {
								'class':	'buttons'
							}).appendTo(filterForm);
							
						// Getting filter range
						filters = getFilterRanges(items, filters);
						
						var m;
						
						for (var f in filters) {
							
							type = filters[f]['type'];
							
							group = $('<div>', {
									'class':		'group ' + ((type === 'multiple_string' || type === 'single_string')? 'select' : 'range') 
								}).insertBefore(buttons);
							
							if (filters[f]['label']) {
								lbl = $('<label>', {
										'for':			f,
										text:			filters[f]['label']
									}).appendTo(group);
							}
								
							if (type === 'multiple_string' || type === 'single_string') {
								
								if (f === 'rating') {
									filters[f].values = [ text.noRating, 1, 2, 3, 4, 5 ];
								}
								
								inp = $('<select>', {
										name:			f
									}).appendTo(group);
								
								if (type === 'multiple_string') {
									inp.attr('multiple', true);
									inp.addTooltip(text.multipleSelectHint);
								}
								
								if (!filters[f].hasOwnProperty('values') || !filters[f].values.length) { // || filters[f].values.length < 2) {
									
									group.remove();
									
								} else {
								
									if (type === 'single_string') {
										inp.prepend($('<option>', {
												text:			text.select,		// + ' ' + filters[f]['label'],
												disabled:		'',
												selected:		''
											}));
									}
									
									if (filters[f].values) {
										for (var i = 0; i < filters[f].values.length; i++) {
											inp.append($('<option>', {
													text:			filters[f].values[i]
												}));
										}
									}
									
									inp.on('change', enableResetBtn);
								}
								
							} else {
								
								inp = $('<input>', {
										name:		f,
										type:		'range'
									})
									.data('type', type)
									.appendTo(group);
								
								if (f === 'rating') {
									filters[f]['step'] = 1;
									filters[f]['min'] = 0;
									filters[f]['max'] = 5;
								}
								
								if (!filters[f].hasOwnProperty('min') || !filters[f].hasOwnProperty('max') || filters[f]['min'] === filters[f]['max']) {
									
									group.remove();
									
								} else {
									
									if (type === 'financial' && (m = filters[f]['label'].match(/\S+\s+[\(\[](\S+)[\)\]]/))) {
									
										filters[f]['currency'] = m[1];
									
									} else if (type === 'numeric') {
										
										if ('exposureTime.shutterSpeed'.indexOf(f) >= 0) {
											filters[f]['unit'] = 'seconds';
										} else if (f === 'fileSize') {
											filters[f]['unit'] = 'byte';
											filters[f]['step'] = 1;
										}
									}
									filters[f].onChanged = enableResetBtn;
									filters[f].onReseted = disableResetBtn;
									
									inp.rangeSlider(filters[f]);
								}
							}
								
							inp.attr('data-type', type || 'single_string');
						}
						
						// Adding buttons
						
						if ((inp = filterForm.find('input,select')).length) {
							
							filterForm.addClass((inp.length === 1)? 'single':'multi');
							filterForm.data('single', inp.length === 1 && inp[0].nodeName === 'SELECT' && !inp.attr('multiple'));
							
							if (!(baseSet && filterForm.data('single'))) {
								// Reset button
								btn = $('<button>', {
										type:			'reset',
										'class':		(baseSet? 'disabled secondary' : 'alert') + ' button',
										text:			' ' + text.reset
									}).appendTo(buttons);
								
									if (baseSet) {
										btn.on('click.' + ns, resetFilters);
									} else {
										btn.on('click.' + ns, function() {
												resetBaseFilters();
												removeOverlay();
											});
									}
							}
								
							if (filterForm.data('single')) {
								// Single combo box -> Update instantly on change
								
								inp.on('change.' + ns, doFilter);
								
							} else {
								// Multiple inputs or not single combo box -> Use 'Search' button
								btn = filterForm.find('button[type=submit]');
								
								if (!btn.length) {
									btn = $('<button>', {
											type:			'submit',
											'class':		'button',
											text:			' ' + text.search
										}).appendTo(buttons);
								}
								
								btn.on('click.' + ns, doFilter);
							}
							
							filterForm.on('submit.' + ns, doFilter);
						}
					}
					
					if (buttons.is(':empty')) {
						buttons.remove();
					}
					
					if (sort) {
						
						
						if (baseSet) {
							
							sortForm = box.find('form.sort');
							
						} else {
							
							sortForm = $('<form>', {
										'class':		'sort'
									}).appendTo(cont);
						}
						
						lbl = $('<label>', {
								'class':		'icon-ordering',
								'for':			'sortby',
								text:			' '
							}).appendTo(sortForm);
							
						inp = $('<select>', {
								'class':		'small',
								name:			'sortby'
							}).appendTo(sortForm);
						
						inp.prepend($('<option>', {
								text:			text.sortBy,
								disabled:		'',
								selected:		''
							}));
						
						for (var s in sort) {
							
							inp.append($('<option>', {
									html:			'&#8613; ' + /*'&#8593; ' +*/ sort[s].label,
									value:			s
								}).data('sort', {
									type:			sort[s].type,
									label:			sort[s].label,
									ascending:		true
								}));
							
							inp.append($('<option>', {
									html:			'&#8615; ' + /*'&#8595; ' +*/ sort[s].label,
									value:			s
								}).data('sort', {
									type:			sort[s].type,
									label:			sort[s].label,
									asc:			false
								}));
						}
					
						inp.on('change.' + ns, doSort);
					}
					
					if ((!sortForm.length || sortForm.is(':empty')) && (!filterForm.length || filterForm.is(':empty'))) {
						box.hide();
					}
				},
			
			// Preparing a custom page
			
			preparePage = function() {
					// Page :: initializing functions
					// Search 
					if (settings.hasOwnProperty('search')) {
						prepareSearch($(settings.search.hook));
					}
				},
			
			// Marking new folders
			
			markFoldersNew = function() {
					if (!album) {
						return;
					}
					
					var folders = album.getFolders(),
						secs = settings.markNew['days'] * ONEDAY_S,
						d;
					
					if (folders.length) {
						getFolders('.' + settings.cardClass).each(function(i) {
								if (i < folders.length) {
									d = folders[i][J.DATES];
									if (d) {
										d = d[settings.markNew['reference']] || d[J.DATERANGE][1] || d[J.DATERANGE][0];
										if (d && (nows - d) < secs) {
											$(this).find('.' + settings.thumbClass).append($('<span>', {
													'class':	'icon-star new-image'
												}).data('tooltip', (new Date(d * 1000)).toLocaleDateString()).addTooltip());
										}
									}
								}
							});
					}
				},
				
			// Preparing folders
		
			prepareFolders = function() { 
				
					var scrollTimeout = null,
						folders = getFolders();
					
					if (!folders.children().length) {
						return;
					}
					
					if (settings.hasOwnProperty('markNew')) {
						setTimeout(markFoldersNew, 300);
					}
					
					setTimeout(function() {
							
							// Automatic height matching by row
							if (!settings.fixedShapeFolderThumbs || settings.folderCaptionPlacement !== 'over') {
								folders.matchHeight({
										delegate:		settings.cardClass,
										match:			settings.fixedShapeFolderThumbs?
															[ settings.cardClass ]
															:
															[ settings.thumbClass, settings.cardClass ]
									});
							}
							
							lazyloadFolders();
							
							folders.find('.' + settings.preloadClass + ':first-child img')
								.waitAllImg(function() {
										folders.find('.' + settings.preloadClass).removeClass(settings.preloadClass);
										folders.trigger('refresh');
									});
						}, 500);
				},
			
			// Cookie policy
			
			showCookiePolicy = function() {
			
					if (!$.cookie('cookiePolicy')) {
						var	el = $('<div>', { 
									id: 		'cookiepolicy' 
								}).appendTo($body),
							p = $('<p>', { 
									html: 		text.cookiePolicyText 
								}).appendTo(el);
						
						if (settings.cookiePolicyUrl) {
							p.append($('<a>', { 
									text: 		text.cookiePolicyLearnMore, 
									target: 	'_blank', 
									href: 		settings.cookiePolicyUrl 
								}));
						}
						
						p.append($('<a>', {
								'class': 		'smallbtn',
								text: 			text.cookiePolicyAgree 
							}).on('click', 	function(){ 
								$('#cookiepolicy').fadeOut(500, function() { 
									$(this).remove(); 
								});
								$.cookie('cookiePolicy', true, 36000000); // expires: 10000 days
							}));
						
						el.fadeIn(500);
						setTimeout(function() {
							$('#cookiepolicy').fadeOut(500, function() { 
								$(this).remove(); 
							});
						}, 6000);
					}			
				},
			
			// Hash change listener :: when the browser's back / forward buttons were pressed
			
			stateChange = function() {
					if (window.location.hash === lastHash) {
						return;
					}
					
					lastHash = window.location.hash;
					var param = readParam();
					
					if (param.hasOwnProperty('label')) {
						// Label filter
						getItemsByLabel(param.label, function() {
							setParam(settings.indexName, {
									label:	param.label
								}, text.label + ': ' + param.label);
							updateShares();
						});
					} else if (param.hasOwnProperty('search')) {
						// Search
						getItemsBySearch(param.search, function() {
							setParam(settings.indexName, {
									search:	param.search
								}, text.search + ': ' + param.search);
							updateShares();
						});
					} else if (param.hasOwnProperty('newimages')) {
						// New images
						getItemsByDate(param.newimages, function() {
							setParam(settings.indexName, {
									newimages: param.newimages
								}, text.newImages + ': ' + param.newimages);
							updateShares();	
						});
					} else if (param.hasOwnProperty('img')) {
						// Image (no search or label)
						var el = getThumbByName(param.img);
						if (el.length) {
							currentThumbs().trigger('lightboxLoad', el);
						} else {
							removeParam(settings.indexName, 'img');
						}
					} else {
						// Plain index page
						currentThumbs().trigger('lightboxQuit');
						removeOverlay();
					}
					
				},
			
			// Updating the share buttons after a state change
			
			updateShares = function() {
					if (settings.hasOwnProperty('share') && currentThumbs().trigger('lightboxContainer').is(':visible')) {
						$(settings.share.hook).trigger('updateLinks');
					}	
				};
			
		
		/*
		 *	START of Skin code
		 */
		
		// Removing search string
		
		if (window.location.search) {
			removeSearch();
		}
		
		// Creating new Album
		
		if (settings.pageType === 'index') {
			
			if (typeof Album === UNDEF) {
				console.log('Critical Error: Missing jalbum.album.js library!');
			}
			
			var params = {
					makeDate:		settings['makeDate'],
					rootPath:		settings['rootPath'],
					relPath:		settings['relPath'],
					ready: 			prepareAlbum,
					deepReady:		deepReadyFunction
				};
				
			if (settings.indexName !== $.fn.skin.defaults.indexName) {
				params.indexName = settings['indexName'];
			}
			
			if (settings.audioPoster !== $.fn.skin.defaults.audioPoster) {
				params.audioPoster = settings['audioPoster'];
			}
				
			if (settings.videoPoster !== $.fn.skin.defaults.videoPoster) {
				params.videoPoster = settings['videoPoster'];
			}
			
			album = new Album($, params);
		}
		
		// Cache buster if thumbnail dimensions has changed
		
		if ((cacheBuster = $.cookie($('title').text() + '_thumbDims')) && settings.hasOwnProperty('thumbDims') && cacheBuster === settings.thumbDims) {
			cacheBuster = '';
		} else {
			cacheBuster = '?' + (cacheBuster? cacheBuster : settings.thumbDims);
			$.cookie($('title').text() + '_thumbDims', settings['thumbDims']);
		}
		
		// Right click protection on folders, slider
		
		if (settings.rightClickProtect) {
			$('.' + settings.folderContClass + ',.' + settings.sliderClass).on('contextmenu', 'img', function(e) {
					e.preventDefault()
					return false;
				});
		}
		
		// Initializing map
		
		if (settings.hasOwnProperty('map')) {
			if (settings.map.hasOwnProperty('type')) {
				$.fn.lightbox.defaults.mapType = settings.map.type;
				$.fn.addMap.defaults.type = settings.map.type;
			}
			if (settings.map.hasOwnProperty('zoom')) {
				$.fn.lightbox.defaults.mapZoom = settings.map.zoom;
				$.fn.addMap.defaults.zoom = settings.map.zoom;
			}
			if (settings.map.hasOwnProperty('apiKey')) {
				$.fn.addMap.defaults.apiKey = settings.map.apiKey;
			}
		}
					
		// Passing along lightbox defaults
		
		passDefaults(settings, $.fn.lightbox.defaults, 'indexName,level,previousFoldersLast,nextFoldersFirst,rightClickProtect,enableKeyboard,enableMouseWheel,extraSizes,useRating,jalbumRating,visitorRating,lbableClass');
		passDefaults(settings.lightbox, $.fn.lightbox.defaults);
		
		// Social sharing
		
		if (settings.hasOwnProperty('share')) {
			$.fn.renderShares.defaults.buttonTheme = settings.share['buttonTheme'] || 'dark';
			$.fn.renderShares.defaults.facebookAppId = settings.share['facebookAppId'] || '';
			if (settings.share['callAction']) {
				$.fn.renderShares.defaults.callAction = settings.share['callAction'];
			}
			$(settings.share.hook).renderShares(settings.share);
			if (settings.share.sites !== 'facebook') {
				$.fn.lightbox.defaults.share = settings.share.sites.replace('facebook,', '').replace(',facebook', '');
			}
		}
		
		// Background music
		
		if ($('[data-audioPlayer]').length) {
			$('[data-audioPlayer]').audioPlayer({
					rootPath: 	settings.rootPath
				});
		}
				
		// Hash change listener
		
		$window.on('hashchange.' + ns, stateChange);
		
		// Utility function for debugging
		
		this.getAlbum = function() {
				return album;
			};
		
	};
	
	$.fn.skin.defaults = {
			contentHook:				'article.cont',
			sliderClass:				'slider',
			folderContClass:			'folder-cont',
			fodlersClass:				'folders',
			folderHoverClass:			'hover-scale',
			folderClass:				'folder',
			thumbContClass:				'thumb-cont',
			thumbGridClass:				'caption-below small-up-2 medium-up-3 large-up-4',
			thumbHoverClass:			'hover-scale',
			thumbsClass:				'thumbnails',
			thumbClass:					'thumb',
			verticalMiddleClass:		'vertical-middle',
			cardClass:					'card',
			lbableClass:				'lbable',
			captionClass:				'caption',
			commentClass:				'comment',
			hascaptionClass:			'hascaption',
			checkedClass:				'checked',
			activeClass:				'active',
			preloadClass:				'preload',
			lazyloadClass:				'lazyload',
			startshowClass:				'startshow',
			panoClass:					'pano',
			selectAllClass:				'select-all',
			selectNoneClass:			'select-none',
			keepSelectedClass:			'keep-selected',
			addSelectedClass:			'add-selected',
			fixedShapeThumbs:			true,
			fixedShapeFolderThumbs:		true,
			folderCols:					2,
			scrollCheckInterval:		250,
			indexName:					'index.html',
			audioPoster:				'audio.poster.png',
			videoPoster:				'video.poster.png',
			exactFields:				'creator,keywords',
			fixGrid:					false,
			autoStart:					false,
			scaleUp:					false,
			rightClickProtect: 			false,
			useRating:					false,
			jalbumRating:				false,
			visitorRating:				false,
			enableKeyboard:				true,
			videoAuto:					true,
			linkToFolder:				true
		};
		
})(jQuery, jQuery(window), jQuery(document), jQuery('body'));
